/*
 *	Interface to matcher
 */
#include "defs.h"
#include "term_types.h"
#include "free_types.h"
#include "pure_types.h"
#include "state_types.h"
#include "functions.h"

#if defined(DEBUG)
#include <stdio.h>
#endif

static BOOL find_match();
static TERM *ac_uncompress();

/*
 *	Build a match object
 */
void *build_match(p, s, tot_var)
TERM *p, *s;
int tot_var;
{
  MATCH_OBJECT *m = MALLOC(MATCH_OBJECT);
  int i;
  BINDING *b;

/*
Flattening supressed for use with CiME:
- terms are already flattened w.r.t. AC operators,
- trick: commutative operators are sent here as non flattened AC operators
 
#if defined(DEBUG)
  printf("flattenning p=");
  print_term_eker(p);
  printf("...\n");
#endif
  flatten(p);
#if defined(DEBUG)
  printf("flattenning s=");
  print_term_eker(s);
  printf("...\n");
#endif
  flatten(s);
*/


#if defined(DEBUG)
  printf("ac_compressing p=");
  print_term_eker(p);
  printf("...\n");
#endif
  ac_compress(p);
#if defined(DEBUG)
  printf("ac_compressing s=");
  print_term_eker(s);
  printf("...\n");
#endif
  ac_compress(s);
#if defined(DEBUG)
  printf("filling fields of m...\n");
#endif
/*
  m->orig_pat = p;
  m->orig_sub = s;
*/
  m->total_var = tot_var;
#if defined(DEBUG)
  printf("allocating m->bind_tab...\n");
#endif
  m->bind_tab = b = CALLOC(tot_var, BINDING);
  for(i = 0; i < tot_var; i++){
    b[i].value = NULL;
    b[i].n_times = 0;
  }
  m->pure_stack = NULL;
  m->pure_prob = NULL; /* user might destroy_match() before solve_pure() */
  m->first = TRUE;
#if defined(DEBUG)
  printf("calling build_free()...\n");
#endif
  if(!build_free(m, p, s, &(m->free_prob))){
/*
    destroy_term(p);
    destroy_term(s);
*/
    FREE(m->bind_tab);
    FREE(m);
#if defined(DEBUG)
    printf("build_match returned NULL because build_free() returned FALSE\n");
#endif
    return((void *) NULL);
  }
  return((void *) m);
}

/*
 *	Extract a match
 */
static BOOL find_match(m)
MATCH_OBJECT *m;
{
  if (m == NULL)
    {
#if defined(DEBUG)
      printf("find_match(m) return FALSE because m was NULL\n");
#endif
      return FALSE;
    }
  if (m->first)
    {
#if defined(DEBUG)
      printf("m->first was true\n");
#endif
      m->first = FALSE;
      if (m->free_prob == NULL)
	{
#if defined(DEBUG)
	  printf("find_match(m) return TRUE because m->free-prob is NULL\n");
#endif
	  return TRUE;
	}
      if (!solve_free(m, m->free_prob, TRUE))
	{
#if defined(DEBUG)
	  printf("find_match(m) return FALSE because m->free_prob has no solution\n");
#endif
	  return FALSE;
	}
    if (build_pure(m))
      {
	if (m->pure_prob == NULL || solve_pure(m->pure_prob, TRUE))
	  {
#if defined(DEBUG)
	    printf("find_match(m) return TRUE because m->pure_prob has a solution\n");
#endif
	    return TRUE;
	  }
#if defined(DEBUG)
	printf("find_match(m) continues because m->pure_prob has no solution\n");
#endif
	destroy_pure(m->pure_prob);
      }
#if defined(DEBUG)
      printf("build_pure(m) returned FALSE\n");
#endif
    }
  else
    {
#if defined(DEBUG)
      printf("m->first was false\n");
#endif
      if (m->free_prob == NULL)
	{
	  return(FALSE);
	}
      if (m->pure_prob != NULL && solve_pure(m->pure_prob, FALSE))
        {
	  return(TRUE);
	}
      destroy_pure(m->pure_prob);
    }
  while (solve_free(m, m->free_prob, FALSE))
    {
      if (build_pure(m))
	{
	  if (m->pure_prob == NULL || solve_pure(m->pure_prob, TRUE))
	    {
	      return(TRUE);
	    }
	  destroy_pure(m->pure_prob);
	}
    }
  m->pure_prob = NULL; /* don't want to free the same pure_prob twice */
  return(FALSE);
}

BOOL extract_match(vm, assignment)
void *vm;
TERM *assignment[];
{
  MATCH_OBJECT *m = (MATCH_OBJECT *) vm;
  int i, j, t, n_var, n_term, a;
  VAR_HEADER *var_tab;
  TERM_HEADER *term_tab;
  TERM *p;
  TERM_LIST *head, *tail;

  if (m == NULL)
    {
#if defined(DEBUG)
      printf("extract_match(m,...) exited because m was NULL\n");
#endif
      return FALSE;
    }

  if (!find_match(m))
    {
#if defined(DEBUG)
      printf("extract_match(m,...) exited because find_match(m) was FALSE\n");
#endif
      return FALSE;
    }

  for (i = 0; i < m->total_var; i++)
    {
#if defined(DEBUG)
      printf("m->bind_tab[%d].value = ",i);
      print_term_eker(m->bind_tab[i].value); 
      printf("\n");
#endif
/* Claude Marche
      assignment[i] = m->bind_tab[i].value; 
*/
      if (m->bind_tab[i].value != NULL)
	{
	  assignment[i] = ac_uncompress(m->bind_tab[i].value);
	}
      else
	{
	  assignment[i] = NULL;
	}
    }
  if (m->pure_prob == NULL)
    {
#if defined(DEBUG)
      printf("extract_match(m,...) return TRUE because m->pure_prob was NULL\n");
#endif

      return TRUE; /*******  Claude :  was FALSE */
    }

  n_var = m->pure_prob->n_var;
  n_term = m->pure_prob->n_term;
  var_tab = m->pure_prob->var_tab;
  term_tab = m->pure_prob->term_tab;
  for(j = 0; j < n_var; j++){
    if(var_tab[j].owner == SHARED){
      assignment[var_tab[j].var] =
        ac_uncompress(term_tab[var_tab[j].ass.s.cur_ass].term);
    }
    else if(var_tab[j].ass.o.cur_size == 1){
      for(i = 0; i < n_term; i++){
        if(var_tab[j].ass.o.assign[i].cur != 0)
          break;
      }
      assignment[var_tab[j].var] = ac_uncompress(term_tab[i].term);
    }
    else{
      head = tail = NULL;
      a = 0;
      for(i = 0; i < n_term; i++){
        for(t = var_tab[j].ass.o.assign[i].cur; t; t--){
          if(tail == NULL)
            tail = head = MALLOC(TERM_LIST);
          else
            tail = tail->next_arg = MALLOC(TERM_LIST);
          tail->arg = ac_uncompress(term_tab[i].term);
          a++;
        }
      }
      tail->next_arg = NULL;
      p = MALLOC(TERM);
      p->type = AC_NORMAL;
      p->sym = var_tab[j].owner;
      p->rest.f.list_len = a;
      p->rest.f.arg_list = head;
      p->rest.f.arg_tail = tail;
      assignment[var_tab[j].var] = p;
    }
  }
  return TRUE;
}
   
static TERM *ac_uncompress(t)
TERM *t;
{
  TERM *p = MALLOC(TERM);
  TERM_LIST *q, *head, *tail;
  AC_LIST *r;
  int i;

  switch(t->type){
  case VARIABLE:
  case CONSTANT:
    *p = *t;
    break;
  case FUNCTION:
  case AC_NORMAL:
  case COMMUTATIVE:
    p->type = t->type;
    p->sym = t->sym;
    head = tail = NULL;
    for (q = t->rest.f.arg_list; q; q = q->next_arg)
      {
	if (tail == NULL)
	  tail = head = MALLOC(TERM_LIST);
	else
	  tail = tail->next_arg = MALLOC(TERM_LIST);
	tail->arg = ac_uncompress(q->arg);
      }
    tail->next_arg = NULL;
    p->rest.f.list_len = t->rest.f.list_len;
    p->rest.f.arg_list = head;
    p->rest.f.arg_tail = tail;
    break;
  case AC_COMPRESSED:
  case COM_COMPRESSED:
    p->type = AC_NORMAL;
    p->sym = t->sym;
    head = tail = NULL;
    for (r = t->rest.a.ac_list; r; r = r->next_ac)
      {
	for (i = r->mult; i; i--)
	  {
	    if(tail == NULL)
	      tail = head = MALLOC(TERM_LIST);
	    else
	      tail = tail->next_arg = MALLOC(TERM_LIST);
	    tail->arg = ac_uncompress(r->arg);
	  }
      }
    tail->next_arg = NULL;
    p->rest.f.list_len = t->rest.a.arg_count;
    p->rest.f.arg_list = head;
    p->rest.f.arg_tail = tail;
    break;
  }
  return(p);
}

/*
 *	destroy object and free storage
 */
void destroy_match(MATCH_OBJECT *m)
{
  if (m == NULL) return;
  destroy_free(m->free_prob);
  destroy_pure(m->pure_prob);
/*
  destroy_term(m->orig_pat);
  destroy_term(m->orig_sub);
*/
  FREE(m->bind_tab);
  FREE(m);
}

