(***************************************************************************

  Implementation of some basic tools on bottom-up tree automata
  
  CiME Project - Dmons research team - LRI - Universit Paris XI
  
  $Id: automaton.ml,v 1.2 2001/04/20 13:42:08 marche Exp $

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: automaton.ml,v 1.2 2001/04/20 13:42:08 marche Exp $

***************************************************************************)

(*

Obsolete header:

***************************************************************************)

(* 
   Les automates ascendants d'arbres avec condition sur les sous-termes
   immediats pour des termes sans symbole AC
*)

type t = int

open Types


type var_id = int
exception VARIABLE of var_id
		
type 'SYMBOL internal_term = 
    Var of var_id 
  | App of ('SYMBOL * ('SYMBOL internal_term list))

type 'SYMBOL formula = 
    OR of 'SYMBOL formula * 'SYMBOL formula  
  | AND of 'SYMBOL formula * 'SYMBOL formula
  | EQ of (('SYMBOL internal_term list) * ('SYMBOL internal_term list))
  | EXIST of (('SYMBOL internal_term list) * 'SYMBOL formula)
  | TRUE 
  | FALSE



type 'STATE epsilon_rule = 
    { 
      initial_state : 'STATE ;
      final_state   : 'STATE 
    } 



type ('SYMBOL, 'TERM,  'STATE) conditional_rewrite_rule = 
    {
      tete : 'SYMBOL;
      etats_initiateurs : 'STATE list;
      conditions : (int * ('TERM -> 'TERM -> bool) * int) list ;
      etat_final : 'STATE 
    }


type ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata = 
    {
      alphabet : 'SYMBOL list ;
      typage : ('SYMBOL -> 'TYPE) ;
      tete_de_terme : 'TERM -> 'SYMBOL ;
      liste_des_sous_termes : 'TERM -> ('TERM list);
      etats : 'STATE list ;
      etats_finaux : 'STATE list ;
      etat_poubelle : 'STATE ;
      epsilon : ( 'STATE) epsilon_rule list ;
      regles : (('SYMBOL, 'TERM, 'STATE) conditional_rewrite_rule ) list ;
      internal_names : (('SYMBOL internal_term) list)  ;
      internal_formula : ( ( 'SYMBOL formula) list )
    }




  
let bien_type () = (* Il faut verifier ici que les regles ont des types corrects *) true ;;


let make_automata a b c d e f g h i =  
  if bien_type () 
  then 
    { 
      alphabet = a;
      typage = b  ;
      tete_de_terme = c ;
      liste_des_sous_termes = d ; 
      etats = e ;
      etats_finaux = f ;
      etat_poubelle = g ;
      epsilon = h ;
      regles = i ; 
      internal_names = [] ; 
      internal_formula = []} 
  else  failwith "Bad automatum's type in tree_automata.ml"  ;;


let make_rule a b c d = 
  {
    tete =a ;
    etats_initiateurs = b ;
    conditions = c ;
    etat_final = d 
  }

let get_alphabet auto =  auto.alphabet;;

let get_states auto = auto.etats;;

let get_final_states auto = auto.etats_finaux ;;

let get_rules auto = auto.regles ;;

let get_head_of_rule rule = rule.tete;;

let get_constraints_of_rule rule = rule.conditions ;;

let get_initiator_of_rule rule = rule.etats_initiateurs ;;

let get_final_state_of_rule rule = rule.etat_final ;;
 
let add_rule a r = 
  make_automata 
    a.alphabet 
    a.typage 
    a.tete_de_terme 
    a.liste_des_sous_termes 
    a.etats 
    a.etats_finaux 
    a.etat_poubelle 
    a.epsilon 
    (r::a.regles);;

let add_epsilon_rule a r = 
  make_automata 
    a.alphabet 
    a.typage 
    a.tete_de_terme 
    a.liste_des_sous_termes 
    a.etats a.etats_finaux 
    a.etat_poubelle 
    (r::a.epsilon) 
    a.regles;;

let get_epsilon_rules auto = auto.epsilon;;

let present_dans_liste = List.mem

let rec replace_ieme_in_list_with i list new_item = 
  match i, list with
      0, (r::l) -> new_item::l
    | _, (r::l) -> r::(replace_ieme_in_list_with (i-1) l new_item)
    | _,[] -> 
	failwith "List too short to make replacement in tree_automata.ml"

    
let nieme = List.nth      	       
	      
let rec run_automata auto t =
  let sstl = auto.liste_des_sous_termes t in
  let rec apply_rule x y z = match (x,y,z) with
      (r1::reste),sydetete,l  -> 
	if ((r1.tete=sydetete) & ( r1.etats_initiateurs = l) 
	    & ( List.for_all 
		  (fun (a,b,c) -> b (nieme sstl a ) (nieme sstl c)) 
		  r1.conditions ))
	then r1.etat_final 
	else apply_rule reste sydetete l 
    | [],_,_ -> auto.etat_poubelle
  in  
    apply_rule auto.regles 
      (auto.tete_de_terme t) (List.map (run_automata auto) (sstl) );;


let rec is_present x y = match (x,y) with
  (a::b),(l::ll) -> (List.mem a l) & (is_present b ll)
| [],[] -> true
| _,_ -> failwith "Bad arity in tree_automata.ml" ;;

let rec apply_epsilon_rules s epsilon =
  let rec create q = function
      (eps::l) -> 
	if (List.mem eps.initial_state q) 
      	then 
	  if (eps.final_state=eps.initial_state) 
	  then 
	    create q l
	  else (Listutils.union [eps.final_state] (create q l)) 
	else (create q l) 
    | [] -> []
  in let created = create s epsilon in
    if (created =[]) 
    then s 
    else (Listutils.union created (apply_epsilon_rules created epsilon ) )
    ;;   

   
(*run de l'automate non deterministe*)
let rec nd_run_automata auto t =
  let sstl = auto.liste_des_sous_termes t in
  let rec apply_rule x y z = match (x,y,z) with
      (r1::reste),sydetete,l  -> 
	
	if ((r1.tete=sydetete) & (is_present r1.etats_initiateurs  l) 
      	    & ( List.for_all (fun (a,b,c) -> b (nieme sstl a ) (nieme sstl c)) r1.conditions ))
	then (Listutils.union (apply_epsilon_rules [r1.etat_final] auto.epsilon ) (Listutils.union [r1.etat_final] (apply_rule reste sydetete l ))) 
	else apply_rule reste sydetete l 
    | [],_,_ -> [auto.etat_poubelle]
  in  
    apply_rule auto.regles (auto.tete_de_terme t) (List.map (nd_run_automata auto) (sstl) );;





let triv_leads_to auto q1 q2 = 
  present_dans_liste q2 (apply_epsilon_rules [q1] auto.epsilon) ;;

let liste_des_ancetres auto s = 
  let rec recancester = function
      (r::rr) -> 
  	if (triv_leads_to auto r s) 
  	then (Listutils.union [r] (recancester rr)) 
  	else (recancester rr) 
    | [] -> []
  in recancester auto.etats_finaux  ;;


(* 
   Permet un run d'un automate nd avec epsilon sur un terme 
   avec des variables sortees.
   L'Environnemnt courant (Current_Env doit etre initialise)
*)

let rec nd_run_varautomata auto env t = 
  let sstl = auto.liste_des_sous_termes t in
  let rec apply_rule  x y z = match (x,y,z) with
      (r1::reste),sydetete,l  -> 
	
	if ((r1.tete=sydetete) & (is_present r1.etats_initiateurs  l) 
      	    & ( List.for_all 
		  (fun (a,b,c) -> b (nieme sstl a ) (nieme sstl c)) 
		  r1.conditions ))
	then (Listutils.union 
		(apply_epsilon_rules [r1.etat_final] auto.epsilon) 
		(Listutils.union [r1.etat_final] (apply_rule reste sydetete l))) 
	else (Listutils.union 
		(apply_epsilon_rules [r1.etat_final] auto.epsilon) 
		(apply_rule reste sydetete l))
    | [],_,_ -> [auto.etat_poubelle]
  in  
    try 
      let tet = (auto.tete_de_terme t)
      in 
(* print_string "Applying rule to a functional sy\n"; *)
	apply_rule auto.regles tet (List.map (nd_run_varautomata auto env) sstl )
    with 
	(VARIABLE(x)) -> 
(* print_string "Variable! Nb of ancester : "; print_int (List.length (liste_des_ancetres auto s)); print_string "\n";*) 
	  
	  let s = (State([VS( (*sort_head (List.hd(get_sorts_of_variable x))*)
			    try 
			      List.assoc x env
			    with 
				Not_found -> failwith "Bad Environement error in Specif make_signature_automata..."
			    )]))
	  in
	    liste_des_ancetres auto s  
;;


let test_acceptance a b =  
  let qf = run_automata a b in 
  (qf, present_dans_liste qf a.etats_finaux)


let nd_test_acceptance a b =  
  let qf_list = nd_run_automata a b in 
  (qf_list, 
   List.exists (function qf -> present_dans_liste qf a.etats_finaux) qf_list) 

let nd_test_var_acceptance env a b =  
  let qf_list = nd_run_varautomata a env b in
  (qf_list,
   List.exists (function qf -> present_dans_liste qf a.etats_finaux) qf_list)

type ('a,'b) u = U1 of 'a | U2 of 'b ;;

let mku1 x = U1 x
and mku2 x = U2 x  

let union_of_automata a1 a2 =

  let mkru1 { tete = t; 
	      etats_initiateurs = ei;
	      conditions = c;
	      etat_final = ef 
	    } 
    = { tete = t;
	etats_initiateurs = List.map mku1 ei;
	conditions = c;
	etat_final = mku1 ef
      }

  and mkru2 { tete = t; 
	      etats_initiateurs = ei;
	      conditions = c;
	      etat_final = ef
	    }
    = { tete = t;
	etats_initiateurs = List.map mku2 ei;
	conditions = c;
	etat_final = mku2 ef
      }

  and mkre1 { initial_state=inis;
	      final_state=fis
	    } 
    = { initial_state = mku1 inis;
	final_state = mku1 fis
      }
	
  and mkre2 { initial_state = inis;
	      final_state = fis} 
    = { initial_state = mku2 inis;
	final_state = mku2 fis}
  in
    { 
      alphabet = Listutils.union a1.alphabet a2.alphabet ;
      typage = a1.typage ;
      tete_de_terme = a1.tete_de_terme ;
      liste_des_sous_termes =a1.liste_des_sous_termes ;
      etats = (List.map mku1 a1.etats) @ (List.map mku2 a2.etats);
      etats_finaux = (List.map mku1 a1.etats_finaux) @ 
		     (List.map mku2 a2.etats_finaux) ;
      etat_poubelle = mku1 a1.etat_poubelle;
      epsilon = (List.map mkre1 a1.epsilon) @ (List.map mkre2 a2.epsilon) ;
      regles = (List.map mkru1 a1.regles) @ (List.map mkru2 a2.regles) ;
      internal_names = [] ;
      internal_formula = [] 
    }   

let count = ref 0

let initialize_count () = count:=0;;

let fresh_new_var _ = 
  let _ = incr count in
    Var (!count)

let internalize_automata auto = 
  { 
    alphabet = auto.alphabet ;
    typage = auto.typage ;
    tete_de_terme = auto.tete_de_terme ;
    liste_des_sous_termes = auto.liste_des_sous_termes ;
    etats = auto.etats  ;
    etats_finaux =auto.etats_finaux ;
    etat_poubelle = auto.etat_poubelle;
    epsilon = auto.epsilon ;
    regles = auto.regles   ;
    internal_names = List.map fresh_new_var auto.etats;
    internal_formula = auto.internal_formula 
  }

let e2i ext intern q =
  try nieme intern (Listutils.index q ext) with 
      _ -> failwith "Fatal Error : your automatum must be internalized just before applying create_internal_formula in tree_automata.ml" ;;


let rec build_formula i_names states epsi x y = match (x,y) with
  (r1::l),q ->  
    if (r1.etat_final<>q) 
    then build_formula i_names states epsi l q 
    else 
      let qi = e2i states i_names q 
      in 
      let fresh_list = List.map fresh_new_var r1.etats_initiateurs 
      in 
      ( OR((EXIST(fresh_list,AND(EQ([qi],[App(r1.tete,fresh_list )] ),EQ(fresh_list,List.map (e2i states i_names) r1.etats_initiateurs)))),build_formula i_names states  epsi l q) )
| [],q -> 
    let rec treat_epsilon i_names states x y = match (x,y) with
      (e1::rst),q -> 
	if (e1.final_state<>q) 
      	then treat_epsilon i_names states rst q 
      	else 
	  let myvar = fresh_new_var () 
	  in 
      	  (OR(EXIST([myvar],AND(EQ([e2i states i_names q],[myvar]),EQ([myvar],[e2i states i_names q]))),treat_epsilon i_names states rst q) ) 
    | [],_ -> FALSE 
    in treat_epsilon i_names states epsi q ;; 


let create_internal_formula auto =
  { alphabet = auto.alphabet ;
    typage = auto.typage ;
    tete_de_terme = auto.tete_de_terme ;
    liste_des_sous_termes = auto.liste_des_sous_termes ;
    etats = auto.etats  ;
    etats_finaux =auto.etats_finaux ;
    etat_poubelle = auto.etat_poubelle;
    epsilon = auto.epsilon ;
    regles = auto.regles   ;
    internal_names = auto.internal_names ;
    internal_formula = 
    List.map 
      (build_formula auto.internal_names auto.etats auto.epsilon auto.regles ) 
      auto.etats };;



(* let complement = fun auto -> { alphabet = auto.alphabet ;
      	       	       	       typage = auto.typage ;
			       tete_de_terme = auto.tete_de_terme ;
			       liste_des_sous_termes = auto.liste_des_sous_termes ;
			       etats =auto.etats ;
			       etats_finaux= Listutils.subtract auto.etats auto.etats_finaux ;
			       etat_poubelle = auto.etat_poubelle ;
			       epsilon =auto.epsilon ;
			       regles =auto.regles  ;
      	       	       	       internal_names = auto.internal_names  ;
      	       	       	       internal_formula = auto.internal_formula
			     };;
*)

let nd_test_complement = 
  fun a t -> let resu = nd_test_acceptance a t
              in (fst resu,not (snd resu)) ;;

let nd_test_intersect a1 a2 t = (snd (nd_test_acceptance a1 t))&(snd (nd_test_acceptance a2 t));;


let reach_with qlist rlist elist =

  let rec recreachr = function
      (r::rr) ->
	if (List.for_all (fun x -> List.mem x qlist ) r.etats_initiateurs)
	then Listutils.union [r.etat_final] (recreachr rr)
	else recreachr rr
    | [] -> []
  and recreache = function
      (e::ee) ->
	if (List.mem e.initial_state qlist)
	then Listutils.union [e.final_state] (recreache ee)
	else recreache ee
    |  [] -> []
  in
  Listutils.union (recreache elist) (recreachr rlist);;



let clean_automata (auto,delt) =   

  let rec build_necessary qlist =
    let cree=reach_with qlist auto.regles auto.epsilon 
    in 
    if ((Listutils.subtract cree qlist)=[]) 
    then qlist
    else build_necessary (Listutils.union cree qlist)
  in
  let new_states = build_necessary []
  in
  let rec avirer = function
      [] -> []		       
    | (r::rr) -> 
	if (List.exists  
	      (fun x -> not (List.mem x new_states)) 
	      r.etats_initiateurs)
	then (r::(avirer rr))
	else (avirer rr)
	    
  in
  let rec epsibad = function
      [] -> []
    | (r::rr) ->
	if (not (List.mem r.initial_state new_states))
            or (not (List.mem r.final_state new_states))
	then (r::(epsibad rr))
	else epsibad rr
	    
  in 
  ({ alphabet = auto.alphabet ;
     typage = auto.typage ;
     tete_de_terme = auto.tete_de_terme ;
     liste_des_sous_termes = auto.liste_des_sous_termes ;
     etats = new_states ;
     etats_finaux= Listutils.intersect new_states auto.etats_finaux ;
     etat_poubelle = auto.etat_poubelle ;
     epsilon =Listutils.subtract auto.epsilon (epsibad auto.epsilon);
     regles =Listutils.subtract auto.regles  (avirer auto.regles);
     internal_names = []  ;
     internal_formula = []},
   (Listutils.subtract delt (epsibad delt))) 
  ;;


