(***************************************************************************

 Interface of some basic tools on bottom-up tree automata

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: automaton.mli,v 1.2 2001/04/20 13:42:09 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI
  
  $Id: automaton.mli,v 1.2 2001/04/20 13:42:09 marche Exp $

***************************************************************************)

(* 
   Les automates ascendants d'arbres avec condition sur les sous-termes 
   immediats pour des termes sans symboles AC. Les conditions sont de la 
   forme : 
   un entier = le numero de l'argument (de 0 a (arite f)-1 
   la comparaison a effectuer entre les sous-termes 
   un entier = le numero de l'autre sous-terme a comparer 
   ex : (2,=,3) verifie que les 2eme et 3eme arguments sont egaux 
 *)

type t = int

open Types;;


type var_id = int ;;

exception VARIABLE of var_id (*state*);;


type ('a, 'b) u ;;

type 'SYMBOL internal_term = 
    Var of var_id 
  | App of ('SYMBOL * ( 'SYMBOL internal_term list));;

type 'SYMBOL formula = OR of 'SYMBOL formula * 'SYMBOL formula  
      	       	     | AND of 'SYMBOL formula * 'SYMBOL formula
      	       	     | EQ of ( ('SYMBOL internal_term) list* ('SYMBOL internal_term) list) 
      	       	     | EXIST of (( 'SYMBOL internal_term) list * 'SYMBOL formula ) 
      	       	     | TRUE | FALSE ;;



type ( 'STATE) epsilon_rule = { initial_state : 'STATE ;
      	       	       	       	final_state   : 'STATE; } ;;



type ('SYMBOL, 'TERM,  'STATE) conditional_rewrite_rule = {tete : 'SYMBOL;
      	       	       	       	       	       	       	     etats_initiateurs : 'STATE list ;
							     conditions : ( int * ( 'TERM -> 'TERM -> bool ) * int ) list ;
      	       	       	       	       	       	       	     etat_final : 'STATE };;


type ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata = { alphabet : 'SYMBOL list ;
      	       	       	       	       	       	  typage : ('SYMBOL -> 'TYPE) ;
						  tete_de_terme : 'TERM -> 'SYMBOL ;
						  liste_des_sous_termes : 'TERM -> ('TERM list);
						  etats : 'STATE list ;
					          etats_finaux : 'STATE list ;
					          etat_poubelle : 'STATE ;
					          epsilon : ( 'STATE) epsilon_rule list ;
					          regles : (('SYMBOL, 'TERM, 'STATE) conditional_rewrite_rule ) list ;
      	       	       	       	       	     	  internal_names : (('SYMBOL internal_term) list)  ;
      	       	       	       	       	       	  internal_formula : ( ( 'SYMBOL formula) list )
      	       	       	       	       	       	 };;




val make_automata : 'SYMBOL list -> ('SYMBOL -> 'TYPE)  -> ('TERM -> 'SYMBOL ) ->('TERM -> ('TERM list))  -> 'STATE list -> 'STATE list -> 'STATE -> ( 'STATE) epsilon_rule list  -> (('SYMBOL, 'TERM,   'STATE) conditional_rewrite_rule ) list -> ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata ;;


val make_rule : 'SYMBOL -> 'STATE list -> ((int * ( 'TERM -> 'TERM -> bool ) * int ) list ) -> 'STATE ->  ('SYMBOL, 'TERM,   'STATE) conditional_rewrite_rule ;;


val get_alphabet :     ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> 'SYMBOL list;;

val get_states : ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> 'STATE list ;;

val get_final_states :   ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> 'STATE list ;;

val get_rules :  ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata ->  ((('SYMBOL, 'TERM,   'STATE) conditional_rewrite_rule ) list  ) ;;

val add_rule : ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> ('SYMBOL, 'TERM,  'STATE) conditional_rewrite_rule  -> ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata ;;

val add_epsilon_rule : ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata ->  'STATE epsilon_rule  -> ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata ;;

val get_epsilon_rules : ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata ->  (('STATE) epsilon_rule  list);;



val get_head_of_rule : (('SYMBOL, 'TERM,  'STATE) conditional_rewrite_rule )  -> 'SYMBOL ;;

val get_constraints_of_rule : (('SYMBOL, 'TERM,  'STATE) conditional_rewrite_rule )  
      	                              -> ( (int* ( 'TERM -> 'TERM -> bool ) * int ) list) ;;

val get_initiator_of_rule : (('SYMBOL, 'TERM, 'STATE) conditional_rewrite_rule )  
      	       	       	       	      -> 'STATE list ;;

val get_final_state_of_rule :(('SYMBOL, 'TERM, 'STATE) conditional_rewrite_rule ) -> 'STATE;;




(* Deterministe et sans epsilon-transitions termes clos *)

val run_automata  :  ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> 'TERM -> 'STATE ;;

val test_acceptance :  ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> 'TERM -> ( 'STATE * bool) ;;


(* Non Deterministe et avec  epsilon-transitions termes clos *)

val nd_run_automata  :  ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> 'TERM -> 'STATE list ;;


val nd_test_acceptance :  ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> 'TERM -> ( 'STATE list * bool) ;;

(* nd et epsilon terme avec variable *)
val nd_test_var_acceptance :  (var_id * sorte_de_base) list -> ('SYMBOL, 'TERM, 'TYPE,  Types.state ) automata -> 'TERM -> ( Types.state list * bool) ;;

val nd_test_complement : ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> 'TERM -> ( 'STATE list * bool) ;;

val nd_test_intersect : ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata ->('SYMBOL, 'TERM, 'TYPE,  'STATE1) automata  -> 'TERM -> bool ;;


(*

Functions that should be done in the future

\begin{verbatim}

val nd_test_final_state : ('a, term, 'b, state) automata -> term ->
state list * bool ;;

val nd_test_final_state : ('a, 'b, 'c, 'd) automata -> 'b -> 'd list *
bool;;

val remove_epsilons : ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata ->
('SYMBOL, 'TERM, 'TYPE, 'STATE) automata ;;

 val determinize_automata : ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata
 -> ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata ;;

\end{verbatim}

*)


val union_of_automata : ('SYMBOL, 'TERM, 'TYPE,  'STATE) automata -> ('SYMBOL, 'TERM, 'TYPE1,  'STATE1 ) automata -> ('SYMBOL, 'TERM, 'TYPE,  ('STATE,'STATE1) u) automata ;;


(* Donne un etiquetage generique a l'automate sans oublier le premier etiquetage *)
val internalize_automata : ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata -> ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata  ;;

(* Ne s'applique que si l'automate est internalize !! *)
val create_internal_formula : ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata -> ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata  ;;

val triv_leads_to : ('SYMBOL, 'TERM, 'TYPE, 'STATE) automata -> 'STATE -> 'STATE -> bool ;;


val clean_automata : ('a, 'b, 'c, 'd) automata * 'd epsilon_rule list -> ('a, 'b, 'c, 'd) automata * 'd epsilon_rule list;;
