(***************************************************************************

Class Maps over ordered sets

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: class_maps.ml,v 1.2 2000/05/16 14:55:35 marche Exp $

***************************************************************************)


type ('a,'b) node_info =
    {
      key : 'a;
      data : 'b;
      height : int;
      left : ('a,'b) t;
      right : ('a,'b) t
    }	

and ('a,'b) t =
    Empty
  | Node of ('a,'b) node_info


let height = function
    Empty -> 0
  | Node(i) -> i.height
	
let create l x d r =
  let hl = height l 
  and hr = height r 
  in
  Node
    {
      key = x;
      data = d;
      height = if hl >= hr then hl + 1 else hr + 1;
      left = l;
      right = r
    } 
    
let bal l x d r =
  let hl = height l
  and hr = height r
  in 
  if hl > hr + 2 
  then 
    begin
      match l with
          Empty -> invalid_arg "Class_maps.bal"
        | Node(il) -> 
            if height il.left >= height il.right 
	    then
              create il.left il.key il.data (create il.right x d r)
            else 
	      begin
              	match il.right with
                    Empty -> invalid_arg "Class_maps.bal"
              	  | Node(ilr) -> 
                      create 
			(create il.left il.key il.data ilr.left) 
			ilr.key 
			ilr.data 
			(create ilr.right x d r)
              end
    end 
  else 
    if hr > hl + 2 
    then 
      begin
        match r with
            Empty -> invalid_arg "Class_maps.bal"
          | Node(ir) -> 
              if height ir.right >= height ir.left 
	      then
		create (create l x d ir.left) ir.key ir.data ir.right
              else 
		begin
		  match ir.left with
                      Empty -> invalid_arg "Class_maps.bal"
		    | Node(irl) ->
                      	create 
			  (create l x d irl.left) 
			  irl.key 
			  irl.data 
			  (create irl.right ir.key ir.data ir.right)
            	  end
	  end 
	else
          Node
	    {
	      key = x;
	      data = d;
	      height = if hl >= hr then hl + 1 else hr + 1;
	      left = l;
              right = r
   	    }  

let rec add order x data table =
  match table with
      Empty ->
        Node
	  {
	    key = x;
	    data = data;
	    height = 1;
	    left = Empty;
	    right = Empty
	  } 
    | Node(i) ->
        let c = order x i.key in
        if c = 0 
	then
          Node
	    {
	      key = x;
	      data = data;
	      height = i.height;
	      left = i.left;
	      right = i.right
	    }
        else 
	  if c < 0 
	  then
            bal (add order x data i.left) i.key i.data i.right
          else
            bal i.left i.key i.data (add order x data i.right)
;;


let rec find order x table =
  match table with
      Empty ->
        raise Not_found
    | Node(i) ->
        let c = order x i.key in
        if c = 0 then i.data
        else find order x (if c < 0 then i.left else i.right)	
;;
      
let rec iter (f : 'a -> 'b -> unit) table =
  match table with
      Empty ->
        ()
    | Node(i) ->
	iter f i.left;
	f i.key i.data;
	iter f i.right
;;
    


class ['a,'b] map comp =
  object

    val mutable table = (Empty : ('a,'b) t)
		  
    val order = (comp : 'a -> 'a -> int)
		  
    method add x data = 
      table <- add order x data table

    method find x = find order x table

    method iter f = iter f table
	  
end



let empty_map comp = new map comp;;
