(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: ac_critical_pairs.ml,v 1.4 2002/07/17 15:58:49 contejea Exp $

***************************************************************************)

open Positions
open Gen_terms
open Rewrite_rules
open Controle

module CpSet =
  Ordered_sets.MakePoly
    (struct
       type 'symbol t = 'symbol term * 'symbol term
       let compare (s1,t1) (s2,t2) =
	 if ((s1.Hcons.tag = s2.Hcons.tag) && (t1.Hcons.tag = t2.Hcons.tag)) ||
	   ((s1.Hcons.tag = t2.Hcons.tag) && (t1.Hcons.tag = s2.Hcons.tag))
	 then 0
	 else Pervasives.compare (s1.Hcons.tag,t1.Hcons.tag) (s2.Hcons.tag,t2.Hcons.tag)
     end)

let head_critical_pairs sign hct lhs1 rhs1 lhs2 rhs2 =
  let unifiers = 
    Ac_unification.set_of_solutions sign hct lhs1 lhs2 
  in
  List.fold_left
    (fun acc sigma ->
       CpSet.add (Substitution.substitute sign hct sigma rhs1,
		  Substitution.substitute sign hct sigma rhs2)
	 acc)
    CpSet.empty unifiers
;;

let deep_critical_pairs sign hct lhs1 rhs1 lhs2 rhs2 =
  let rec rec_deep_cp p sub_term_rule1 (* l|_p *) = 
    match sub_term_rule1.Hcons.node with
	Var _ -> CpSet.empty
      | Term(_,l) -> 
	  Listutils.foldi
	    (fun i ti acc -> match ti.Hcons.node with
		 Var _ -> acc
	       | Term _ -> 
		   let pi = build_pos p i in
		   let deeper_cps = rec_deep_cp pi ti in
		   let unifiers = 
		     Ac_unification.set_of_solutions sign hct
		       ti (* l|_{p.i} *) lhs2 (* d *) 
		   in
		   let new_pairs =
		     List.fold_left 
		       (fun new_acc sigma ->
			  let rule1_lhs_with_rule2_rhs_at_pos_pi =
			    replace_subterm_at_position_in_term sign hct
			      rhs2 pi lhs1 
			  in
			  CpSet.add 
			    (Substitution.substitute 
			       sign hct sigma
			       rule1_lhs_with_rule2_rhs_at_pos_pi,
			       Substitution.substitute sign hct sigma rhs1)
			    new_acc)
		       deeper_cps unifiers
		   in
		   CpSet.union new_pairs acc)
	    l CpSet.empty 
  in
    rec_deep_cp Top lhs1

let critical_pairs sign hct rule1 rule2 =
  let lhs1 = leftify_term hct rule1.lhs
  and rhs1 = leftify_term hct rule1.rhs
  and lhs2 = rightify_term hct rule2.lhs 
  and rhs2 = rightify_term hct rule2.rhs 
  in
  CpSet.elements
    (CpSet.union
       (deep_critical_pairs sign hct lhs1 rhs1 lhs2 rhs2) 
       (CpSet.union
	  (deep_critical_pairs sign hct lhs2 rhs2 lhs1 rhs1) 
	  (CpSet.union 
	     (head_critical_pairs sign hct lhs1 rhs1 lhs2 rhs2)
	     (match rule1.ext,rule2.ext with 
		| None,None -> CpSet.empty
		| Some(elhs1,erhs1),None -> 
		    let elhs1 = leftify_term hct elhs1
		    and erhs1 = leftify_term hct erhs1
		    in
		      CpSet.union
			(deep_critical_pairs sign hct elhs1 erhs1 lhs2 rhs2) 
			(CpSet.union
			   (deep_critical_pairs sign hct lhs2 rhs2 elhs1 erhs1)
			   (head_critical_pairs sign hct elhs1 erhs1 lhs2 rhs2))
		| None,Some(elhs2,erhs2) -> 
		    let elhs2 = rightify_term hct elhs2
		    and erhs2 = rightify_term hct erhs2
		    in
		      CpSet.union
			(deep_critical_pairs sign hct lhs1 rhs1 elhs2 erhs2) 
			(CpSet.union
			   (deep_critical_pairs sign hct elhs2 erhs2 lhs1 rhs1) 
			   (head_critical_pairs sign hct lhs1 rhs1 elhs2 erhs2)) 
		| Some(elhs1,erhs1),Some(elhs2,erhs2) -> 
		    let elhs1 = leftify_term hct elhs1
		    and erhs1 = leftify_term hct erhs1
		    and elhs2 = rightify_term hct elhs2
		    and erhs2 = rightify_term hct erhs2
		    in
		      CpSet.union
			(deep_critical_pairs sign hct elhs1 erhs1 lhs2 rhs2) 
			(CpSet.union
			   (deep_critical_pairs sign hct lhs2 rhs2 elhs1 erhs1)
			   (CpSet.union
			      (head_critical_pairs sign hct elhs1 erhs1 lhs2 rhs2) 
			      (CpSet.union
				 (deep_critical_pairs sign hct lhs1 rhs1 elhs2 erhs2) 
				 (CpSet.union
				    (deep_critical_pairs sign hct elhs2 erhs2 lhs1 rhs1)
				    (CpSet.union
				       (head_critical_pairs sign hct lhs1 rhs1 elhs2 erhs2) 
				       (CpSet.union
					  (deep_critical_pairs sign hct elhs1 erhs1 elhs2 erhs2) 
					  (CpSet.union
					     (deep_critical_pairs sign hct elhs2 erhs2 elhs1 erhs1)
					     (head_critical_pairs sign hct elhs1 erhs1 elhs2 erhs2)
					  )))))))))))


let self_critical_pairs sign hct rule =
  let lhs1 = leftify_term hct rule.lhs
  and rhs1 = leftify_term hct rule.rhs
  and lhs2 = rightify_term hct rule.lhs
  and rhs2 = rightify_term hct rule.rhs
  in
  CpSet.elements
    (CpSet.union
       (deep_critical_pairs sign hct lhs1 rhs1 lhs2 rhs2) 
       (CpSet.union
	  (head_critical_pairs sign hct lhs1 rhs1 lhs2 rhs2) 
	  (match rule.ext with
	     | None -> CpSet.empty
	     | Some(elhs,erhs) ->
		 let elhs1 = leftify_term hct elhs
		 and erhs1 = leftify_term hct erhs
		 and elhs2 = rightify_term hct elhs
		 and erhs2 = rightify_term hct erhs
		 in
		   CpSet.union
		     (deep_critical_pairs sign hct elhs1 erhs1 lhs2 rhs2) 
		     (CpSet.union
			(deep_critical_pairs sign hct lhs1 rhs1 elhs2 erhs2) 
			(CpSet.union
			   (deep_critical_pairs sign hct elhs1 erhs1 elhs2 erhs2) 
			   (CpSet.union
			      (head_critical_pairs sign hct lhs1 rhs1 elhs2 erhs2) 
			      (head_critical_pairs sign hct elhs1 erhs1 elhs2 erhs2)))))))
       
       
       






