(***************************************************************************

  This module provides parameterized functions to complete a rewriting
  system. They are intended to apply as well on terms and on words.

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: kb.mli,v 1.17 2002/07/19 15:52:19 contejea Exp $

***************************************************************************)
open Orderings_generalities

val coeff_size : int ref 
val coeff_age : int ref 

module type KB = 
sig
  type symbol
  type signature 
  type variables 
  type t
  type simple_rule 
  exception KB_failed
  exception KB_aborted
  exception KB_conjecture_proved of (t * t) list
  val verbose : int ref
  val complete : 
    signature -> variables -> t ordering -> 
     (t * t) list -> ( t * t) list -> simple_rule list
   val prove_conj_by_comp : 
     signature -> variables -> t ordering -> 
       (t * t) list -> (t * t) list -> bool
end

module Make (R : Abstract_rewriting.AbstractRewriting) :
  (KB 
   with type symbol = R.symbol
   and type signature = R.signature
   and type variables = R.variables
   and type t = R.t
   and type simple_rule = R.rule)


module StandardKBCompletion : 
  (KB 
   with type symbol = User_signatures.symbol_id
   and type signature = User_signatures.symbol_id Signatures.signature
   and type variables = Variables.user_variables
   and type t = User_signatures.symbol_id Gen_terms.term
   and type simple_rule = User_signatures.symbol_id Rewrite_rules.t)

module ACCompletion : 
  (KB 
   with type symbol = User_signatures.symbol_id
   and type signature = User_signatures.symbol_id Signatures.signature
   and type variables = Variables.user_variables
   and type t = User_signatures.symbol_id Gen_terms.term
   and type simple_rule = User_signatures.symbol_id Rewrite_rules.t)

module OrderedKBCompletion : 
  functor (TO : Abstract_rewriting.TermOrdering
	   with type t =  User_signatures.symbol_id Gen_terms.term) ->
    (KB 
     with type symbol =  User_signatures.symbol_id
     and type signature =  User_signatures.symbol_id Signatures.signature
     and type variables = Variables.user_variables
     and type t = User_signatures.symbol_id Gen_terms.term
     and type simple_rule = 
       User_signatures.symbol_id Rewrite_rules.t)

module OrderedACCompletion : 
  functor (TO : Abstract_rewriting.TermOrdering
	   with type t =  User_signatures.symbol_id Gen_terms.term) ->
    (KB 
     with type symbol =  User_signatures.symbol_id
     and type signature =  User_signatures.symbol_id Signatures.signature
     and type variables = Variables.user_variables
     and type t = User_signatures.symbol_id Gen_terms.term
     and type simple_rule = 
       User_signatures.symbol_id Rewrite_rules.t)
    
module WordCompletion : 
  (KB
   with type symbol = String_signatures.symbol_id
   and type signature = 
     String_signatures.symbol_id String_signatures.word_signature
   and type variables = unit
   and type t = String_signatures.symbol_id Words.word
   and type simple_rule = 
     String_signatures.symbol_id String_rewriting.rewrite_rule)

module PWordCompletion :
  (KB
   with 
     type signature = Parameterized_signatures.parameterized_signature
   and type variables = unit
   and type t = Parameterized_words.word
   and type simple_rule = Parameterized_rewriting.rewrite_rule)





(*i
Local Variables:
compile-command: "make -C .. opt"
End:
i*)
