(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: kbrewriting.ml,v 1.13 2001/12/03 14:07:08 contejea Exp $

***************************************************************************)

open Gen_terms
open Orderings_generalities
open Substitution
open Kb;;

module StandardRewriting =
  struct
    type 'symbol signature = 'symbol Signatures.signature
    type variables = Variables.user_variables 
    type 'symbol t = 'symbol term
    type 'symbol rule = 'symbol Rewrite_rules.rewrite_rule
    type 'symbol compiled_rules = 'symbol Naive_dnet.dnet
    let size = Gen_terms.size 
    let equals sigma t1 t2 = t1=t2
    let make_rule = Rewrite_rules.make_rule 
    let lhs_of_rule rule = rule.Rewrite_rules.lhs 
    let rhs_of_rule rule = rule.Rewrite_rules.rhs
    let self_critical_pairs sigma r = 
      Standard_critical_pairs.self_critical_pairs sigma r
    let critical_pairs sigma r1 r2 = 
      Standard_critical_pairs.critical_pairs sigma r1 r2
    let compile = Naive_dnet.compile
    let normalize sigma compiled_rules t = 
      Innermost.innermost_normalize sigma
	(Standard_rewriting.compiled_rewrite_at_top compiled_rules) t 
    exception Irreducible
    let force_normalize sigma compiled_rules t = 
      let t' = 
	Standard_innermost.innermost_normalize 
	  (Standard_rewriting.compiled_rewrite_at_top compiled_rules) t in
	if t = t'
	then raise Irreducible
	else t'

    let canonize_pairs l =
      List.map
	(function (s,t) -> 
	   let vars_of_s_and_t =
	     Variables.VarSet.union (set_of_variables s) 
	       (set_of_variables t) in
	   let sigma = 
	     Variables.VarMap.map 
	       (function v -> Var v)
	       (Variables.canonical_renaming vars_of_s_and_t) in
	     (apply_term s sigma, apply_term t sigma))
	l
      
    let print_t = Gen_terms.print_term
    let print_rewrite_rule = Rewrite_rules.print_rewrite_rule
  end

module ACRewriting =
  struct
    type 'symbol signature = 'symbol Signatures.signature
    type variables = Variables.user_variables 
    type 'symbol t = 'symbol term
    type 'symbol rule = 'symbol Rewrite_rules.rewrite_rule
    type 'symbol compiled_rules = 'symbol Naive_dnet.dnet
    let size = Gen_terms.size
    let equals = Gen_terms.equals
    let make_rule = Rewrite_rules.make_rule
    let lhs_of_rule rule = rule.Rewrite_rules.lhs 
    let rhs_of_rule rule = rule.Rewrite_rules.rhs
    let self_critical_pairs sigma r = 
      Ac_critical_pairs.self_critical_pairs sigma r
    let critical_pairs sigma r1 r2 = 
      Ac_critical_pairs.critical_pairs sigma r1 r2
    let compile = Naive_dnet.compile
    let normalize sigma compiled_rules t = 
      Innermost.innermost_normalize sigma 
	(Rewriting.compiled_ac_rewrite_at_top sigma compiled_rules) t 
    exception Irreducible
    let force_normalize sigma compiled_rules t = 
      let t' =
	Innermost.innermost_normalize sigma 
	  (Rewriting.compiled_ac_rewrite_at_top sigma compiled_rules) t in
      if t = t'
      then raise Irreducible
      else t'

    let canonize_pairs l =
      List.map
	(function (s,t) -> 
	   let vars_of_s_and_t =
	     Variables.VarSet.union (set_of_variables s) 
	       (set_of_variables t) in
	   let sigma = 
	     Variables.VarMap.map 
	       (function v -> Var v)
	       (Variables.canonical_renaming vars_of_s_and_t) in
	     (apply_term s sigma, apply_term t sigma))
	l
      
    let print_t = Gen_terms.print_term
    let print_rewrite_rule = Rewrite_rules.print_rewrite_rule
  end


module StandardKBCompletion = Make (Config10) (StandardRewriting)

module ACCompletion = Make (Config10) (ACRewriting)



module WordRewriting =
  struct
    type 'symbol signature = 'symbol String_signatures.word_signature
    type variables = unit 
    type 'symbol t = 'symbol Words.word
    type 'symbol rule = 'symbol String_rewriting.rewrite_rule
    type 'symbol compiled_rules = 'symbol String_rewriting.compiled_srs
    let size sigma t = Words.length t
    let equals sigma = (=)
    let make_rule sigma s t = (s,t)
    let lhs_of_rule = fst
    let rhs_of_rule = snd
    let self_critical_pairs sigma r = 
      String_unification.superpose r r
    let critical_pairs sigma r1 r2 = 
      (String_unification.superpose r1 r2) @
      (String_unification.superpose r2 r1)

    let compile sigma = String_rewriting.compile_srs
    let normalize sigma compiled_rules t = 
      String_rewriting.compiled_normalize compiled_rules t 
    exception Irreducible
    let force_normalize sigma compiled_rules t = 
      let t' = String_rewriting.compiled_normalize compiled_rules t in
      if t = t'
      then raise Irreducible
      else t'

    let canonize_pairs l = l
      
    let print_t sigma () t = Words.pretty_print sigma t
    let print_rewrite_rule sigma () r = 
      String_rewriting.pretty_print sigma [r]

  end

module WordCompletion = Make (Config11) (WordRewriting)

