(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: standard_critical_pairs.ml,v 1.4 2002/07/17 15:58:49 contejea Exp $

***************************************************************************)

open Positions
open Gen_terms
open Rewrite_rules
open Controle
open Unification

module CpSet =
  Ordered_sets.MakePoly
    (struct
       type 'symbol t = 'symbol term * 'symbol term
       let compare (s1,t1) (s2,t2) =
	 if ((s1.Hcons.tag = s2.Hcons.tag) && (t1.Hcons.tag = t2.Hcons.tag)) ||
	   ((s1.Hcons.tag = t2.Hcons.tag) && (t1.Hcons.tag = s2.Hcons.tag))
	 then 0
	 else Pervasives.compare (s1.Hcons.tag,t1.Hcons.tag) (s2.Hcons.tag,t2.Hcons.tag)
     end)

let head_critical_pair sign hct (*i vars i*) rule1 rule2 =
  let lhs1 = leftify_term hct rule1.lhs
  and lhs2 = rightify_term hct rule2.lhs in
    try
      let sigma = free_unification sign hct (*i vars i*) lhs1 lhs2 in
	(Substitution.apply_term hct (leftify_term hct rule1.rhs) sigma,
	 Substitution.apply_term hct (rightify_term hct rule2.rhs) sigma)
    with
	No_solution -> raise No_solution

let deep_critical_pairs sign hct (*i vars i*) rule1 (* l -> r *) rule2 (* g -> d *) =
  let lhs1 = leftify_term hct rule1.lhs
  and rhs1 = leftify_term hct rule1.rhs
  and lhs2 = rightify_term hct rule2.lhs 
  and rhs2 = rightify_term hct rule2.rhs in
  let rec rec_deep_cp p sub_term_rule1 (* l|_p *) = 
    match sub_term_rule1.Hcons.node with
      | Var _ -> CpSet.empty
      | Term(_,l) -> 
	  Listutils.foldi
	    (fun i ti acc -> match ti.Hcons.node with
		 Var _ -> acc
	       | Term _ -> 
		   let pi = build_pos p i in
		   let deeper_cps =
		     rec_deep_cp pi ti in
		     try
		       let current_unifier = 
			 free_unification sign hct (*i vars i*) 
			   ti (* l|_{p.i} *) lhs2 (* d *) in
		       let rule1_lhs_with_rule2_rhs_at_pos_pi =
			 replace_subterm_at_position_in_term sign hct
			   rhs2 pi lhs1 in
		       let current_cp =
			 (Substitution.apply_term hct
			    rule1_lhs_with_rule2_rhs_at_pos_pi current_unifier,
			    Substitution.apply_term hct rhs1 current_unifier) in
		     
			 CpSet.add current_cp (CpSet.union deeper_cps acc)
		     with No_solution -> (CpSet.union deeper_cps acc))
	    l CpSet.empty in
    
    rec_deep_cp Top lhs1

let critical_pairs sign hct (*i vars i*) rule1 rule2 =
  let deep_cps = CpSet.union (deep_critical_pairs sign hct (*i vars i*) rule1 rule2) 
		 (deep_critical_pairs sign hct (*i vars i*) rule2 rule1) in
  try
    let head_cp = head_critical_pair sign hct (*i vars i*) rule1 rule2 in
    CpSet.elements (CpSet.add head_cp deep_cps)
  with 
      No_solution -> CpSet.elements deep_cps
	

let self_critical_pairs sign hct (*i vars i*) rule =
  CpSet.elements (deep_critical_pairs sign hct (*i vars i*) rule rule)
    
