open Variables
open Rewrite_rules
open User_signatures
open Gen_terms
open Traced_rewriting
open Naive_dnet

exception Quit
exception Client_error of string
exception Syntax_error of string;;

class operators = object
  val ops : (string, unit) Hashtbl.t = Hashtbl.create 10
  method union = 
    let rec aux = function
      | [] -> ()
      | h::t -> Hashtbl.add ops h (); aux t in
      aux
  method contains o =
    try
      Hashtbl.find ops o;
      true
    with Not_found -> false
end

class vars = object
  val var_list : (var_id, string) Hashtbl.t =
    init_for_unif VarSet.empty;
    Hashtbl.create 5
  val string_list = Hashtbl.create 5
  method var_of_string s =
    try
      Hashtbl.find string_list s
    with Not_found ->
      let v = fresh_var_for_unif () in
	Hashtbl.add string_list s v;
	Hashtbl.add var_list v s;
	v
  method string_of_var = Hashtbl.find var_list
end

type theory = user_signature * string array * symbol_id dnet

class set_of_theories = object (self)
  val theory_list : (string, theory) Hashtbl.t = Hashtbl.create 3
  method find = Hashtbl.find theory_list
  method add (name, sg, rs) =
    let sigma = from_list sg in
    let (rn, sys) =
      List.split
	(Listutils.mapi
	   (fun i (tn, u, v) ->
	      (tn, make_numbered_rule
		 i
		 sigma
		 User_terms.user_hct
		 (make_term_from_string_term sigma User_terms.user_hct u)
		 (make_term_from_string_term sigma  User_terms.user_hct v)))
	   rs) in
    let names = Array.of_list rn
    and net = compile sigma sys in
      Hashtbl.add theory_list name (sigma, names, net);
      (sigma, names, net)
end

let tr_norm (sigma, _, net) t =
  snd (
    traced_innermost_normalize
	 User_terms.user_hct
	 (new_compiled_rewrite_at_top net)
	 (make_term_from_string_term sigma User_terms.user_hct t))

let str_norm (sigma, rn, sys) t =
  (tr_norm (sigma, rn, sys) t)#to_string rn
