
(*Exceptions related to the Coq client
*)
(*[Quit] is raised when the client shuts the communication down
*)
(*[Client_error] is raised when the client experiments an error
*)
(*[Syntax_error] is raised when a line sent by the client does not fit the ERP
*)
exception Quit
exception Client_error of string
exception Syntax_error of string;;

(* [operators] is the list of the names of the operators in the current theory
*)
class operators : object
  method union : string list -> unit
  method contains : string -> bool
end

(* [vars] is the current set of variables (that grows progressively)
*)
class vars : Variables.user_variables

(* [theory] is the type of a theory with named rules
*)
type theory

(* [set_of_theories] remembers theories that have already been sent to CiME,
   in order not to compile several times the same theory
*)
class set_of_theories : object
  method add : string * (string list * int) list * (string * string Gen_terms.term * string Gen_terms.term) list -> theory
  method find : string -> theory
end

(* [tr_norm (theo#search tn) t] normalizes with the innermost strategy the term [t] in the theory named [tn] in the set of theories [theo] and returns the trace of that normalization
   [str_norm (theo#search tn) t] is the same except it directly returns a printable string instead of the trace
*)
val tr_norm : theory -> string Gen_terms.term -> User_signatures.symbol_id Trace.t

val str_norm : theory -> string Gen_terms.term -> string
