open Communication
open Coq_syntax
open Coq_parser

let theo = new set_of_theories

let erp_server port =
  let ss = new server_socket port 1 in
  let send = ss#write_line 0
  and rcv parsef = parsef Coq_lexer.token (ss#lexing 0) in
    try
      send ("HELLO cime.lri.fr ERP server "^now()^"\tREADY SEARCH Welcome");
      let to_search = rcv search in
	send ("WAIT Searching theory "^to_search);
	let th =
	  (try
	     let u = theo#find to_search in
	     send ("OK SEARCH Theory "^to_search^" found");
	     u
	   with Not_found ->
	     send ("OK SEARCH Theory "^to_search^" not found");
	     send ("READY THEORY Send theory "^to_search);
	     let (name,_,_) as t = rcv theory_oe in
	     send ("WAIT Compiling theory "^name);
	     let u = theo#add t in
	     send ("OK THEORY Compile theory "^name^" finished");
	     u) in
	  send "READY TERM";
        while true do
	  let t = rcv term_oe in
	    send "WAIT Computing trace";
	    send "TRACE Using leftmost innermost";
	    send ((str_norm th t)^"END TRACE\tOK TERM\tREADY TERM")
        done
    with e ->
      let (s, c) = match e with
	| Parsing.Parse_error -> ("Parsing error", 5)
	| Failure "lexing: empty token" -> ("Invalid character", 4)
	| IO_error fd -> ("There has been a communication error with the client", 2)
	| Client_error s -> ("Client error: "^s, 1)
	| Quit -> ("End of communication required by the client.", 0)
	| _ -> ("Unknown error", -1) in
	send ("CLOSE "^s);
	ss#close 0;
	exit c

let _ = erp_server 7315
