(***************************************************************************

Main program for testing integer solving methods.

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: main.ml,v 1.1 2000/05/16 17:36:32 monate Exp $

***************************************************************************)


open Finite_domains;;
open Non_linear_solving;;
open Poly_syntax;;
open Format;;

  
let print_solution = 
  Fd_var_map.iter
    (fun x n ->
       print_string (string_of_fd_var_id x);
       print_string " = ";
       print_string (Num.string_of_num n);
       print_newline())
;;

let solve bound c = 
  begin
    print_string "Resolution des contraintes :\n";
    List.iter
      (fun c ->
	 print_constraint c;
	 print_newline())
      c;
    print_string "pour des variables entre 0 et ";
    print_string (Num.string_of_num bound);
    print_newline();
    let time_before = Unix.times ()
    in
      begin
      	try
      	  let sol = non_linear_solve bound c
      	  in
      	    print_string "Solution : \n";
	    print_solution sol
      	with
	    No_solution -> 
	      print_string "Pas de solution.";
	      print_newline()
      end;
      let time_after = Unix.times ()
      in
	print_string "temps de resolution : ";
	print_float (time_after.Unix.tms_utime -. time_before.Unix.tms_utime);
	print_string " sec.";
	print_newline()
  end
;;

let constr0 = non_linear_constraints_of_string "x = y+1; x = z" ;;

let constr0' = non_linear_constraints_of_string "x = y+1; x = y" ;;

(*

test1 : xy = 6  x+y =5

*)

let constr1 = non_linear_constraints_of_string "x.y = 6; x+y = 5" ;;


(*

 test 2 : xy = 66048, x+y = 514 

*)

let constr2 = non_linear_constraints_of_string 
		"x.y = 66048; x+y = 514";;

(*

 test 3 : x^3+xy^2 = 62935530, x^2y+x+y = 28166807

*)

let constr3 = non_linear_constraints_of_string 
		"x^3 + x.y^2 = 62935530; x^2.y + x + y = 28166807";;


let constr4 = non_linear_constraints_of_string 
		"x^2 + y^2 = z^2; x > 0; y > 0; z > 0";;

let constr5 = non_linear_constraints_of_string 
		"x^3 + y^3 = z^3; x > 0; y > 0; z > 0";;

let constr6 = non_linear_constraints_of_string 
		"x - y - 1 >= 0";;

let constr7 = non_linear_constraints_of_string 
		"2.x + y = 3.z + 1 ; 3.x + 4.z = 5.y + 3";;

let constr8 = non_linear_constraints_of_string 
		"x.y.x.y = 1225 ; x > 1 ; y > 1";;


let constr_bug = non_linear_constraints_of_string "-1 >= 0";;

let main () = 
  Non_linear_solving.verbose := 0;
  Finite_domains.verbose := 0;
  for i=0 to 3 do
  begin
    abstraction_method:=i;
(**)
    solve (Num.Int 1000) constr0;
    solve (Num.Int 1000) constr0';
    solve (Num.Int 1000) constr1;
    solve (Num.Int 1000) constr2;
    solve (Num.Int 1000) constr3;
(**)
    solve (Num.Int 100) constr4;
(**)
    if i >= 1 then solve (Num.Int 100) constr5;
    solve (Num.Int 100) constr6;
    solve (Num.Int 100) constr7;
    solve (Num.Int 1000) constr8;
    solve (Num.Int 1000) constr3;
    solve (Num.Int 1000) constr_bug;
(**)
    Time_profiler.print_profile();
    Time_profiler.reset_profile();
  end
  done
;;

let _ = 
  Printexc.catch main ();
  Time_profiler.print_profile()
;;

