(***************************************************************************

Solving non linear Diophantine constraints

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: non_linear_solving.mli,v 1.15 2002/06/24 08:54:34 marche Exp $

***************************************************************************)


val verbose : int ref;;

type constraint_op = Nul | Pos_or_nul ;;

module NumRing : Polynomials.RingType with type coef = Numbers.t;;

module Fd_polynomials : 
  Polynomials.PolynomialType with 
                               module Base_ring = NumRing
			     and
			       type variable = Finite_domains.fd_var_id
			     and
                               module Var_map = Finite_domains.Fd_var_map
;;

val print_polynomial : Fd_polynomials.poly -> unit;;

type non_linear_constraint = 
    {
      expr : Fd_polynomials.poly;
      cond : constraint_op;
    };;

val print_constraint : non_linear_constraint -> unit;;

val print_non_linear_constraints : non_linear_constraint list -> unit;;

val from_abstract_formula : 
  Abstract_constraint.formula -> non_linear_constraint list;;

(*

  [(non_linear_solve M C)] returns a solution of the constraints [C] in
  the domain [0..M] for each variable. Raises exception [Not_found] if
  no solution exists.

*)

val non_linear_solve : 
    Numbers.t -> non_linear_constraint list 
      -> (Finite_domains.fd_var_id,Numbers.t) Finite_domains.Fd_var_map.t;;

(*
  temporaire
*)

(*
val abstraction_method : int ref;;
*)

val translate_constraints :
    (Finite_domains.fd_var_id,Numbers.t) Finite_domains.Fd_var_map.t ->
      non_linear_constraint list ->
    	Finite_domains.fd_constraint list * 
	(Finite_domains.fd_var_id,Numbers.t) Finite_domains.Fd_var_map.t
;;
