#load "nums.cma";;
#directory "../compat";;
#load "numbers.cmo";;
#load "inttagset.cmo";;
#load "inttagmap.cmo";;
#load "balanced_trees.cmo";;
#load "ordered_sets.cmo";;
#load "hashcons.cmo";;
#load "ptset.cmo";;

#load "abstract_constraint.cmo";;
#load "poly_parser.cmo";;
#load "poly_lexer.cmo";;
#load "poly_syntax.cmo";;
#load "linear_constraints.cmo";;

#use "presburger.ml";;

open Format;;

let print_set s =
  printf "{ ";
  Ptset.iter (fun f -> Linear_constraints.print f;printf ", ") s;
  printf "}";
;;

#install_printer Linear_constraints.print_expr;;
#install_printer Linear_constraints.print;;
#install_printer print_set;;

(*

  test of remove_denominators 

*)

let f = 
  Poly_syntax.constraint_of_string 
    "x+1/2 >= 0 "
;;

let _,f =  from_abstract_formula [] f;;
 
let f = 
  Poly_syntax.constraint_of_string 
    "x+1/2 >= 0 and (3.x - 4)/5 = 0 and x/2 - y/3 < 1/5 and 3 | x/2"
;;

let _,f =  from_abstract_formula [] f;;
 
remove_denominators f;;

(*
 

test of simplify
 
*)

let vx=make_var "x";;
let env = [ "x",vx ];;

let test s = 
  snd (from_abstract_formula env (Poly_syntax.constraint_of_string s));;

let e1 = test "x > 1";;
let e1 = simplify e1;; (* doit donner x-2 >= 0 *)


let e2 = test "x > 0";;
let e2 = simplify e2;; (* doit donner x-1 >= 0 *)


let e3 = test "x < 0";;
let e3 = simplify e3;; (* doit donner -x-1 >= 0 *)


let e4 = test "x < -1";;
let e4 = simplify e4;;  (* doit donner -x-2 >= 0 *)


(* x > 1 and x > 0  doit donner x > 1, cad x-2 >= 0 *)
conj e1 e2;;
conj e2 e1;;

(* x > 1 and x < 0 doit donner false *)
conj e1 e3;;
conj e3 e1;;

(* x > 1 and x < -1 doit donner false *)
conj e1 e4;;
conj e4 e1;;

(* x > 0 and x < 0 doit donner false *)
conj e2 e3;;
conj e3 e2;;

(* x > 0 and x < -1 doit donner false *)
conj e2 e4;;
conj e4 e2;;

(* x < 0 and x < -1 doit donner x < -1 cad -x-2 >= 0*)
conj e3 e4;;
conj e4 e3;;

let x = var vx;;

(* x >= 0 and x <= 0 doit donner x = 0 *)
conj (ge x zero) (le x zero);;
conj (le x zero) (ge x zero);;

(* x >= 0 and x < 0 doit donner False *)
conj (ge x zero) (simplify (lt x zero));;
conj (simplify (lt x zero)) (ge x zero);;

(* x >= 0 and x = 0 doit donner x=0 *)
conj (ge x zero) (eq x zero);;
conj (eq x zero) (ge x zero);;

(* x < 0 and x = 0 doit donner False *)
conj (simplify (lt x zero)) (eq x zero);;
conj (eq x zero) (simplify (lt x zero));;

(* x >= 0 and x > 0 doit donner x > 0 cad x-1 >= 0*)
conj (ge x zero) (simplify (gt x zero));;
conj (simplify (gt x zero)) (ge x zero);;

(* x >= 0 and -x >= 0 doit donner x = 0 *)
conj (ge x zero) (ge (minus x) zero);;
conj (ge (minus x) zero) (ge x zero);;

(* x >= 0 and -x+1 >= 0 doit donner x >= 0 and -x+1 >= 0 *)
conj (ge x zero) (ge (add (minus x) one)  zero);;
conj (ge (add (minus x) one) zero) (ge x zero);;

(* x >= 0 and -x-1 >= 0 doit False *)
conj (ge x zero) (ge (minus (add x one))  zero);;
conj (ge (minus (add x one)) zero) (ge x zero);;







(* x > 1 or x > 0  doit donner x > 0 cad x-1 >= 0*)
disj e1 e2;;
disj e2 e1;;

(* x > 1 or x < 0 doit donner x > 1 or x < 0 cad x-2>=0 or -x-1>=0*)
disj e1 e3;;
disj e3 e1;;

(* x > 1 or x < -1 doit donner x > 1 or x < -1 cad x-2>=0 or -x-2>=0*)
disj e1 e4;;
disj e4 e1;;

(* x > 0 or x < 0 doit donner x > 0 or x < 0  cad x-1 >=0 or -x-1 >=0*)
disj e2 e3;;
disj e3 e2;;

(* x > 0 or x < -1 doit donner x > 0 or x < -1 cad x-1>=0 or -x-2>=0*)
disj e2 e4;;
disj e4 e2;;

(* x < 0 or x < -1 doit donner x < 0 cad -x-1>=0*)
disj e3 e4;;
disj e4 e3;;


(* x >= -1 or x <= 2 doit donner true *)
disj (ge x (cte Numbers.minus_one)) (le x (cte Numbers.two));;

(* x >= 0 or x < 0 doit donner true *)
disj (ge x zero) (simplify (lt x zero));;

(* x >= 0 and x < 0 doit donner false *)
conj (ge x zero) (simplify (lt x zero));;


(*

test of dnf

*)

let vx = make_var "x";;
let x = var vx;;
let vy = make_var "y";;
let y = var vy;;
let vz = make_var "z";;
let z = var vz;;

let f1 = (gt x one);;
let f2 = (gt y one);;
let f3 = (gt z one);;

let g=disj f1 f2;;
dnf g;;
naive_equivalence (dnf g) g 10 100;;

let g = conj f1 f2;;
dnf g;;
naive_equivalence (dnf g) g 10 100;;

let g = neg(conj f1 f2);; 
dnf g;;
naive_equivalence (dnf g) g 10 100;;

let g = neg(disj f1 f2);; 
dnf g;;
naive_equivalence (dnf g) g 10 100;;

let g = conj (disj f1 f2) f3;; 
dnf g;;
naive_equivalence (dnf g) g 10 100;;

let g = conj (disj f1 f2) (neg f1);; 
dnf g;;
naive_equivalence (dnf g) g 10 100;;

let g = disj (conj f1 f2) (neg f1);; 
dnf g;;
naive_equivalence (dnf g) g 10 100;;

let g = conj (disj f1 f2) (disj (neg f1) f2);; 
dnf g;;
naive_equivalence (dnf g) g 10 100;;

let g = conj (disj f1 f2) (disj (neg f1) f2);; 
dnf g;;
naive_equivalence (dnf g) g 10 100;;



(*

  Ralf's example

*)

let ralf_example = 
  "not 6.x < 2.x+w and -2.x < -u and not v < 3.x and 3 | (x-t)";;

let env,ralf_example =
  from_abstract_formula [] (Poly_syntax.constraint_of_string ralf_example);;

let ralf_dnf = dnf ralf_example;;

(*
let ralf_split = split x ralf_dnf;;

multiply_ppcm ralf_split;;
*)

exists_elim (List.assoc "x" env) ralf_example;;






(*  1 <= i <= n-1 *)

let i = make_var "i";;
let n = make_var "n";;

let f =
  conj (ge (var i) one) (le (var i) (sub (var n) one))
(* should give i-1 >=0 and n-i-1 >=0 *)
;;

let l = dnf f;;

(*i
split i l;;

elim_in_conj i (List.hd (dnf f)) (* should give n-2>=0 *)
;;
i*)

let g = exists_elim i f (* should give n-2>=0 *)
;;

let prem = (ge (var n) (cte (Numbers.of_int 3)));;

neg prem;;

let h = implies prem g;;
let h' = implies (ge (var n) zero) g;;




is_satisfiable h,true;;
is_satisfiable h',true;;

is_valid h,true;;
is_valid h',false;;


(*

  1 <= i <= 3

*)

let i = make_var "i";;

let f =
  conj_list
    [ ge (var i) (cte Numbers.one) ;
      le (var i) (cte (Numbers.of_int 3)) ] 
;;

let l = get_all_solutions f in List.length l=3,l;;

(*

  1 <= i < j <= 8

*)

let i = make_var "i";;
let j = make_var "j";;

let f =
  conj_list
    [ ge (var i) (cte Numbers.one) ;
      gt (var j) (var i) ;
      le (var j) (cte (Numbers.of_int 8)) ] 
;;


let l = get_all_solutions f in List.length l=28,l;;

(*

  et i + j == 1 mod 3

*)


let f =
  conj f 
    (divides (Numbers.of_int 3) (sub (add (var i) (var j)) (cte Numbers.one)) )
;;

let l = get_all_solutions f in List.length l=9,l;;

(*

  1 <= i < j 

*)

let i = make_var "i";;
let j = make_var "j";;

let f =
  conj_list
    [ ge (var i) (cte Numbers.one) ;
      gt (var j) (var i) ] 
;;

get_all_solutions f;; (* Should give Infinite *)

(*

  et i + j == 1 mod 3

*)


let f =
  conj f 
    (divides (Numbers.of_int 3) (sub (add (var i) (var j)) (cte Numbers.one)))
;;

get_all_solutions f;; (* Should give Infinite *)

(*

  et i + j == 2 mod 3 

*)


let f =
  conj f 
    (divides (Numbers.of_int 3) (sub (add (var i) (var j)) (cte Numbers.two)))
;;

get_all_solutions f;;(* Should give no solution *)





(*

((1 <= k and k <= n + - 2) ->
  exists i,j
  ((k = j and k + 1 = i and k <= n + - 2 and 1 <= k and
   (i + - j = 1 or i + - j = - 1) and i <= n + - 1 and 1 <= i and 1 <= j and
   j <= n + - 1)))

*)

let vi = make_var "i";;
let vj = make_var "j";;
let vk = make_var "k";;
let vn = make_var "n";;

let i = var vi;;
let j = var vj;;
let k = var vk;;
let n = var vn;;

let eq1 = eq k j;;
let eq2 = eq (add k one) i;;
let eq3 = le k (sub (sub n one) one);;
let eq4 = le one k;;
let eq5 = disj (eq (sub i j) one) (eq (sub j i) one);;
let eq6 = le i (sub n one);;
let eq7 = le one i;;
let eq8 = le one j;;
let eq9 = le j (sub n one);;

let c1 = conj_list [eq1;eq2;eq3;eq4;eq5;eq6;eq7;eq8;eq9];;

let c2 = exists vj c1;;
eliminate_quantifiers c2;;

let c3 = exists vi c2;;
eliminate_quantifiers c3;;

let eq10 = le one k;;
let eq11 = le k (sub (sub n one) one);;

let c4 = conj eq10 eq11;;

let c5 = implies c4 c3;;

let c6 = forall vk c5;;

forall vn c6;;




(*

forall i1 forall i2 forall i3  exists r3 

	(i2+i3=r3 /\ r3> 0 /\ i1=i2 /\ i1 >=0 /\ i2>=0) <-> 
	(i1=i2 /\ i1>= 0 /\ i2 >= 0 /\ (i1>0\/i2>0))

*)

let vi1 = make_var "i1";;
let vi2 = make_var "i2";;
let vi3 = make_var "i3";;
let vr3 = make_var "r3";;

let i1 = var vi1;;
let i2 = var vi2;;
let i3 = var vi3;;
let r3 = var vr3;;

let e1 = eq (add i2 i3) r3;;
let e2 = gt r3 zero;;
let e3 = eq i1 i2;;
let e4 = ge i1 zero;;
let e5 = ge i2 zero;;
let e6 = gt i1 zero;;
let e7 = gt i2 zero;;

let f = equiv (conj_list [e1 ; e2 ; e3 ; e4 ; e5])
	  (conj_list [e3 ; e4 ; e5 ; disj e6 e7]);;

let f1 = exists vr3 f;;

let f1 = eliminate_quantifiers f1;;

let f2 = forall_s [vi1;vi2;vi3] f1;;

let f3 = eliminate_quantifiers f2;;


(*

  a big formula to simplify

*)

let f = "
((-i_39 + 2 = 0 and -j_40 + 4 = 0 and -j_38 + 3 = 0 and N - 15 >= 0 and i_39 >= 0 and i_39 - i_41 = 0 and N - j_42 >= 0 and i_41 >= 0 and (not i_43 - j_42 >= 0) and -i_43 + j_38 = 0 and i_43 >= 0 and  N - j_40 >= 0 and -j_42 + j_40 = 0 and (not i_39 - j_38 >= 0) and (not i_41 - j_40 >= 0) and N - j_38 >= 0))

implies

(exists j_44 i_45, (N - j_44 >= 0 and
                              -i_39 + 2 = 0 and
                             -j_40 + 4 = 0 and
                              (not i_45 - j_44 >=
0) and                     -j_38 + 3 = 0
and                          N - 15 >= 0 and
                              i_39 >= 0 and
                                    i_45 >= 0 and
                                 i_39 - i_41 = 0 and
                                   N - j_42 >= 0 and  
                                 i_41 >= 0 and
                                   i_41 - j_44 = 0 and
                                             (not i_43 - j_42 >=
0) and                                          -i_43 + j_38 =
0 and                                            i_43 >= 0 and
                                                     N - j_40 >= 0 and
                                                     -j_42 + j_40 = 0 and
                                                     (not i_39 - j_38 >=
0) and                                                     (not i_41 -
j_40 >= 0) and                                                     i_43 -
i_45 = 0 and                                                     N - j_38
>= 0))"
;;





let _,f = from_abstract_formula [] (Poly_syntax.constraint_of_string f);;

let f = Linear_constraints.equality_simplification f;;

let f = simplify f;;

is_valid f;;

is_satisfiable f;;


(*

  from diedral group

*)

let c = "
(
 (q_258 >= 0 and
  n - 2 >= 0 and
  p_257 >= 0 and
  r_259 >= 0 and
  ( (n - q_258 + 1 = 0 and (not -n - 2 >= 0) and (not n + 1 = 0)) or
    (q_258 = 0 and n + 1 = 0)) and
  r_259 + p_257 - 2.n + q_258 = 0) 
 and 
  (p_344 >= 0 and
   n - 2 >= 0 and
   (not 2.n + 1 = 0) and
   q_345 >= 0 and
   ((-q_345 + n + 1 = 0 and (not -n - 2 >= 0) and (not n + 1 = 0)) or
     (n + 1 = 0 and q_345 = 0)) and
   q_345 - 2.n + r_346 + p_344 = 0 and
   r_346 >= 0)) 
implies (p_257 = p_344 and r_259=r_346) ";;


let env,f = from_abstract_formula [] (Poly_syntax.constraint_of_string c);;

is_valid f;;

let f = forall (List.assoc "q_345" env) f;;

let g = eliminate_quantifiers f;;

let g = forall (List.assoc "q_258" env) g;;

let h = eliminate_quantifiers g;;

let h = simplify h;;

(*

 bug a corriger

*)

let f = "((N - 1) >= 0 -> ((N + 1) >= 0 or N = 1))";;

let env,f =   from_abstract_formula [] (Poly_syntax.constraint_of_string f);;

(* f should be valid *)
unsafe_is_valid f;;

let g = neg f;;
dnf g;;

(* g should not be satisfiable *)
unsafe_is_satisfiable g;;

let h = exists_s (free_vars g) g;;

(* should return false *)
eliminate_quantifiers h;;

eliminate_quantifiers g;;

let f1 = "N + 1 >= 0";;
let f2 = "N - 1 = 0";;

let env = [];;
let env,f1 = from_abstract_formula env (Poly_syntax.constraint_of_string f1);;
let env,f2 = from_abstract_formula env (Poly_syntax.constraint_of_string f2);;

f1,f2,disj f1 f2;;

(*

a big formula to check

*)

(*
(((not (p_13 = 0 and q_14 = N + 1 and N - 1 >= 0 and N + 1 >= 0)) and
 (not (p_13 = N + 1 and q_14 = 0 and N - 1 >= 0 and N + 1 >= 0)) and
 (not (p_11 = 0 and q_12 = N + 1 and N - 1 >= 0 and N + 1 >= 0)) and
 ((p_11 = N + 1 and p_13 = 0 and q_14 = N + 1 and q_12 = 0 and N - 1 >= 0 and
  N + 1 >= 0) or
 (p_13 = q_12 and (not q_12 = 0) and p_11 - N + q_12 - 1 = 0 and
 N - 1 >= 0 and p_11 >= 0 and -N + q_14 + q_12 - 1 = 0 and q_14 >= 0 and
 q_12 >= 0)) and
 (not (p_11 = N + 1 and q_12 = 0 and N - 1 >= 0 and N + 1 >= 0))) equiv
(exists p_330 q_331 r_332 p_327 q_328 r_329,
*)

let f = "(not (p_13 = 0 and q_14 = N + 1 and N - 1 >= 0 and N + 1 >= 0)) and
r_329 >= 0 and
(not (p_13 = N + 1 and q_14 = 0 and N - 1 >= 0 and N + 1 >= 0)) and
(not (p_11 = 0 and q_12 = N + 1 and N - 1 >= 0 and N + 1 >= 0)) and
q_328 >= 0 and
((p_11 = N + 1 and p_13 = 0 and q_14 = N + 1 and q_12 = 0 and N - 1 >= 0 and
 N + 1 >= 0) or
(p_13 = q_12 and (not q_12 = 0) and p_11 - N + q_12 - 1 = 0 and
N - 1 >= 0 and p_11 >= 0 and -N + q_14 + q_12 - 1 = 0 and q_14 >= 0 and
q_12 >= 0)) and
(not (p_11 = N + 1 and q_12 = 0 and N - 1 >= 0 and N + 1 >= 0)) and
p_327 + r_329 - q_14 + q_328 + 2 = 0 and p_327 >= 0";;

let env = [];;
let env,f = from_abstract_formula env (Poly_syntax.constraint_of_string f);;

let f1 = exists (List.assoc "r_329" env) f;;

let f1 = eliminate_quantifiers f1;;

let f2 = exists (List.assoc "q_328" env) f1;;

let f2 = eliminate_quantifiers f2;;

let f3 = exists (List.assoc "p_327" env) f2;;

let f3 = eliminate_quantifiers f3;;

let toot = 0;;

let fe = "
(((not (p_13 = 0 and q_14 = N + 1 and N - 1 >= 0 and N + 1 >= 0)) and
 (not (p_13 = N + 1 and q_14 = 0 and N - 1 >= 0 and N + 1 >= 0)) and
 (not (p_11 = 0 and q_12 = N + 1 and N - 1 >= 0 and N + 1 >= 0)) and
 ((p_11 = N + 1 and p_13 = 0 and q_14 = N + 1 and q_12 = 0 and N - 1 >= 0 and
  N + 1 >= 0) or
 (p_13 = q_12 and (not q_12 = 0) and p_11 - N + q_12 - 1 = 0 and
 N - 1 >= 0 and p_11 >= 0 and -N + q_14 + q_12 - 1 = 0 and q_14 >= 0 and
 q_12 >= 0)) and
 (not (p_11 = N + 1 and q_12 = 0 and N - 1 >= 0 and N + 1 >= 0)))) 
";;

let env,fe = from_abstract_formula env (Poly_syntax.constraint_of_string fe);;



let f4 = neg(equiv f3 fe);;

let f5 = exists (List.assoc "p_11" env) f4;;

let f5 = eliminate_quantifiers f5;;


let g = neg f;;

(*

  sentinelle

*)


let toto = 0;;

(*

1000*s+100*e+10*n+d + 1000*m+100*o+10*r+e = 10000*m+1000*o+100*n+10*e+y
/\ s <> e /\ s <> n .....
/\ 1<=s<=9 /\ 1<=m<=9 /\ 0<=e<=9 /\ ....

*)

(*i
let sv = make_var "s";;
let ev = make_var "e";;
let nv = make_var "n";;
let dv = make_var "d";;
let mv = make_var "m";;
let ov = make_var "o";;
let rv = make_var "r";;
let yv = make_var "y";;
let s = var sv;;
let e = var ev;;
let n = var nv;;
let d = var dv;;
let m = var mv;;
let o = var ov;;
let r = var rv;;
let y = var yv;;


let send_more_money =
  conj
    (eq 
       (add 
	  (times (Numbers.of_int 1000) s)
	  (add 
	     (times (Numbers.of_int 100) e)
	     (add 
		(times (Numbers.of_int 10) n)
		(add 
		   d
		   (add 
		      (times (Numbers.of_int 1000) m)
		      (add 
			 (times (Numbers.of_int 100) o)
			 (add 		
			    (times (Numbers.of_int 10) r)
			    e)))))))
       (add 
	  (times (Numbers.of_int 10000) m)
	  (add 
	     (times (Numbers.of_int 1000) o)
	     (add 
		(times (Numbers.of_int 100) n)
		(add 
		   (times (Numbers.of_int 10) e)
		   y)))))

    (conj_list
       [
	 ne s e;
	 ne s n;
	 ne s d;
	 ne s m;
	 ne s o;
	 ne s r;
	 ne s y;
	 
	 ne e n;
	 ne e d;
	 ne e m;
	 ne e o;
	 ne e r;
	 ne e y;
	 
	 ne n d;
	 ne n m;
	 ne n o;
	 ne n r;
	 ne n y;
	 
	 ne d m;
	 ne d o;
	 ne d r;
	 ne d y;
	 
	 ne m o;
	 ne m r;
	 ne m y;
	 
	 ne o r;
	 ne o y;
	 
	 ne r y;
	
	 ge s one ;
	 ge m one ;
	 ge e zero ;
	 ge n zero ;
	 ge d zero ;
	 ge o zero ;
	 ge r zero ;
	 ge y zero ;
	 
	 le s (cte (Numbers.of_int 9)) ;
	 le m (cte (Numbers.of_int 9)) ;
	 le e (cte (Numbers.of_int 9)) ;
	 le n (cte (Numbers.of_int 9)) ;
	 le d (cte (Numbers.of_int 9)) ;
	 le o (cte (Numbers.of_int 9)) ;
	 le r (cte (Numbers.of_int 9)) ;
	 le y (cte (Numbers.of_int 9)) ;
       ]
       

    ) 
    
;;

send_more_money;;

let f1 = exists yv send_more_money;;
let f1 = eliminate_quantifiers f1;;

let f2 = exists dv f1;;
let f2 = eliminate_quantifiers f2;;

let f3 = exists rv f2;;
let f3 = eliminate_quantifiers f3;;

let f4 = exists ov f3;;
let f4 = eliminate_quantifiers f4;;

let f5 = exists ev f4;;
let f5 = eliminate_quantifiers f5;;

let f6 = exists ov f5;;

let f7 = exists mv f6;;

exists sv f7, True;;

is_satisfiable send_more_money;;
i*)
