(***************************************************************************

  Discrimination nets for many-to-one matching: 
  how to create them, add or remove a rule, and use them.

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: dnet.ml,v 1.7 2001/04/20 13:42:13 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: dnet.ml,v 1.7 2001/04/20 13:42:13 marche Exp $

***************************************************************************)


open Positions
open Symbols
open Terms
open Equations
open Non_linearity
open Bipartite_graph


module OrderedSymb =
  struct
    type t = Symbols.symbol_id
    let compare = Symbols.compare_symbol_ids
  end


module MapSymb = Map.Make (OrderedSymb)
 

type oriente = LEFT | RIGHT

type regle_etendue = 
    {
      o : oriente;
      regle : regle;
      t : term;
      vp : (var_id * position list) list
    }


type 'a dnode = 
    {
      p : position; (* position de discrimination courante *)
      m : 'a list  (* les termes sont des membres gauches de regles 
		      representes par leur numero *)
    }



type 'a dnet = 
    LEAF of 'a list
  | DNODE of 'a dnode * 'a son MapSymb.t
  | FULLDNODE of 'a dnode * 'a son MapSymb.t * 'a dnet
	
and 'a son =
    Free_son of 'a dnet
  | AC_son of 'a dnet * 'a dnet 


let empty_dnet () = LEAF []


let is_an_empty_dnet = function
    LEAF [] -> true
  | _ -> false


let function_symbols = 
  ref ( [] : symbol_id list)


let is_an_empty_map m =
  try
    MapSymb.iter (fun f f_son -> raise Exit) m;
    true
  with Exit -> false


let rec top_layer = function
    VAR _ as v -> v
  | TERM (f,l) -> 
      if is_free f
      then TERM (f, List.map top_layer l)
      else TERM (f, [])


let fabrique_regle_etendue o r t =
  {
    o = o;
    regle = r;
    t = t;
    vp = non_linearity (top_layer t)
  }



let rec my_print sep print_string = function
    [] -> Printf.printf "[]"
  | [h] -> 
      Printf.printf "[";
      print_string h;
      Printf.printf "]"
  | h::t -> 
      Printf.printf "[";
      print_string h;
      List.iter 
	(function x -> 
	  Printf.printf sep;
	  print_string x)
	t;
      Printf.printf "]"
      	

let print_rule_for_dnet re = 
  Printf.printf "%d" re.regle.num (*;					   
  Printf.printf "(";
  Printf.printf "%s" (string_of_term re.t);
  Printf.printf ")"*)

let rec print_dnet = function
    LEAF ln -> 
      Printf.printf "LEAF ";
      my_print ";" print_rule_for_dnet ln
  | DNODE (u,lv) -> 
      Printf.printf "({p=%s" (string_of_position u.p);
      Printf.printf ";m=";
      my_print ";" print_rule_for_dnet u.m;
      Printf.printf "},";
      MapSymb.iter
	(fun f son ->
	   match son with
	       Free_son (dnet) -> 
		 Printf.printf "(";
		 Printf.printf "%s," (string_of_symbol_id f);
		 Printf.printf "Arc = ";
		 print_dnet dnet;
		 Printf.printf ")"
	     | AC_son (dnet1,dnet2) ->
		 Printf.printf "(";
		 Printf.printf "AC (%s), Sous_termes = " 
		   (string_of_symbol_id f);
		 print_dnet dnet1;
		 Printf.printf ",";
		 Printf.printf "Arc = ";
		 print_dnet dnet2;
		 Printf.printf ")")
	lv;
      Printf.printf ")"
  | FULLDNODE (u,lv,v) -> 
      Printf.printf "({p=%s" (string_of_position u.p);
      Printf.printf ";m=";
      my_print ";" print_rule_for_dnet u.m;
      Printf.printf "},";
      MapSymb.iter
	(fun f son -> 
	   match son with
	       Free_son (dnet) -> 
		 Printf.printf "(";
		 Printf.printf "%s," (string_of_symbol_id f);
		 Printf.printf "Arc = ";
		 print_dnet dnet;
		 Printf.printf ")"
	     | AC_son (dnet1,dnet2) ->
		 Printf.printf "(";
		 Printf.printf "AC (%s), Sous_termes = " (string_of_symbol_id f);
		 print_dnet dnet1;
		 Printf.printf "Arc = ";
		 print_dnet dnet2;
		 Printf.printf ")")
	lv;
      Printf.printf " ; autres symboles :";
      print_dnet v;
      Printf.printf ")"
		      

let rec liste_des_indices_du_dnet = function
    LEAF ln -> ln
  | DNODE (u,_) -> u.m
  | FULLDNODE (u,_,_) -> u.m


let rec add_to_m n = function
    LEAF ln -> LEAF (n::ln)
  | DNODE (u,lv) -> 
      DNODE 
	({p=u.p;m=n::u.m}, 
	 MapSymb.map 
	   (function
	       	Free_son (d) -> Free_son (add_to_m n d)
	      | AC_son (d1,d2) -> AC_son (d1,add_to_m n d2))
	   lv)
  | FULLDNODE (u,lv,v) ->
      FULLDNODE
	({p=u.p;m=n::u.m},
	 MapSymb.map
	   (function
		Free_son (d) -> Free_son (add_to_m n d)
	      | AC_son (d1,d2) -> AC_son (d1,add_to_m n d2))
	   lv,
	   add_to_m n v)


(* 
   [create_pattern_dnet_pos n pattern pos] fabrique un "discrimination
   net" correspondant au seul motif [pattern], ou les positions sont
   decalees de (ont pour prefixe) [pos]. ([n] est le numero de la regle qui
   a [pattern] comme membre gauche.)

  [add_pattern_to_dnet_pos n pattern pos dnet] construit le
  "discrimination net" obtenu en ajoutant le motif [pattern] a la
  position [pos] au dnet [dnet].
*)

let rec create_pattern_dnet_pos n t pos =
  match (sous_terme_a_la_position t pos) with
      VAR _ ->
	begin
	  try
	    let pos' = next_pos_p t pos in
	      create_pattern_dnet_pos n t pos'
	  with No_more_pos -> LEAF [n]
      	end
    | TERM (f,l) -> 
      	if not (is_free f) (* f est C ou AC *)
      	then 
	  begin
	    let dnet1 = 
	      List.fold_left 
		(fun res t -> 
		   let n' = fabrique_regle_etendue n.o n.regle t in
		     add_pattern_to_dnet_pos n' t Top res)
		(LEAF []) l
	    and dnet2 =
	      try 
		let top_layer_t = top_layer t in
		let pos' = next_pos_p top_layer_t pos in
	      	  create_pattern_dnet_pos n top_layer_t pos'
	      with No_more_pos -> LEAF [n]
	    in
	      DNODE({p=pos;m=[n]}, 
		    MapSymb.add f (AC_son (dnet1,dnet2)) MapSymb.empty)
	  end
      	else 
	  begin
	    let dnet =
	      try 
	      	let pos' = next_pos_p t pos in
		  create_pattern_dnet_pos n t pos' 
	      with No_more_pos -> LEAF [n]
	    in
	      DNODE({p=pos;m=[n]}, 
		    MapSymb.add f (Free_son (dnet)) MapSymb.empty)
	  end
	  

and add_pattern_var_to_dnet_pos n = function
    LEAF ln -> LEAF (n::ln)
  | DNODE (u,lv) as dnet ->
      let u' = {p=u.p;m=n::u.m} 
      and lv' = 
	MapSymb.map 
	  (function
	       Free_son (dnet') ->
		 Free_son (add_pattern_var_to_dnet_pos n dnet')
	     | AC_son (dnet1,dnet2) -> 
		 AC_son (dnet1, add_pattern_var_to_dnet_pos n dnet2))
	  lv 
      in
	FULLDNODE (u', lv', LEAF[n])
  | FULLDNODE (u,lv,v) as dnet -> add_to_m n dnet
	
	
	
and add_pattern_term_to_dnet_leaf_pos n t pos = function
    LEAF [] -> create_pattern_dnet_pos n t pos 
  | LEAF ln as dnet ->
      begin
	let t' = sous_terme_a_la_position t pos in
	  match t' with
	      VAR _ -> 
	    	begin
		  try 
		    add_pattern_term_to_dnet_leaf_pos n t 
		      (next_pos_p t pos) dnet
		  with No_more_pos -> LEAF (n::ln)
	    	end
	    | TERM (f,l) ->
	    	begin
		  let u' = {p=pos;m=n::ln} in
		  let f_son = 
		    if not (is_free f) (* f est C ou AC *)
	      	    then 
		      let top_layer_t = top_layer t in
		      let dnet1 = 
			List.fold_left 
			  (fun res t -> 
			     let n' = fabrique_regle_etendue n.o n.regle t
			     in
			       add_pattern_to_dnet_pos n' t Top res)
			  (LEAF []) l
		      and dnet2 = 
		    	try 
			  add_pattern_term_to_dnet_leaf_pos n top_layer_t 
			    (next_pos_p top_layer_t pos) dnet
		    	with No_more_pos -> LEAF (n::ln) in
		  	AC_son (dnet1, dnet2)
		    else
		      let dnet' = 
		    	try 
			  add_pattern_term_to_dnet_leaf_pos n t 
			    (next_pos_p t pos) dnet
		    	with No_more_pos -> LEAF (n::ln) in
		  	Free_son (dnet') 
		  in
		    FULLDNODE (u', MapSymb.add f f_son MapSymb.empty, dnet)
	    	end
      end
  | _ -> failwith "this case should never happen in add_pattern_term_to_dnet_leaf_pos"



and add_pattern_term_to_dnet_dnode_pos n t pos = function
    DNODE (u,lv) as dnet ->
      let u' = {p=pos;m=n::u.m}
      and t' = sous_terme_a_la_position t pos in
      	if pos = u.p
      	then 
	  begin
	    match t' with
	    	VAR _ -> 
		  begin
		    try 
		      let pos' = next_pos_p t pos in
		      let lv' = 
		      	MapSymb.map 
			  (function 
		      	       Free_son (dnet') -> 
				 Free_son (add_pattern_term_to_dnet_pos 
					     n t pos' dnet')
		      	     | AC_son (dnet1,dnet2) ->
				 AC_son (dnet1,
					 add_pattern_term_to_dnet_pos 
					   n t pos' dnet2))
			  lv
		      and reste_son = 
		      	add_pattern_term_to_dnet_leaf_pos n t pos' (LEAF []) in
		      	FULLDNODE (u', lv', reste_son)
		    with No_more_pos ->
		      let lv' = 
		      	MapSymb.map 
			  (function 
		      	       Free_son (dnet') -> 
				 Free_son (add_to_m n dnet')
		      	     | AC_son (dnet1,dnet2) -> 
				 AC_son (dnet1,add_to_m n dnet2))
			  lv 
		      and reste_son = LEAF [n] in 
		      	FULLDNODE (u',lv',reste_son)
		  end
		  
	      | TERM (g,l) ->
	      	  try
		    let g_son = MapSymb.find g lv in
		      match g_son with
			  AC_son (dnet1,dnet2) ->
			    begin
			      let top_layer_t = top_layer t in
			      let dnet1' =  
				List.fold_left 
				  (fun res t -> 
				     let n' = 
				       fabrique_regle_etendue n.o n.regle t 
				     in
				       add_pattern_to_dnet_pos n' t Top res)
				  dnet1 l
			      and dnet2' = 
				try 
				  add_pattern_term_to_dnet_pos n top_layer_t 
				    (next_pos_p top_layer_t pos) dnet2
				with No_more_pos ->
				  add_to_m n dnet2
			      in
			      let lv' = MapSymb.add g (AC_son (dnet1',dnet2'))
					  lv in
				DNODE (u', lv')
			    end
			| Free_son (dnet') ->
		      	    begin
			      let dnet'' =
				try
				  add_pattern_term_to_dnet_pos 
                                    n t (next_pos_p t pos) dnet'
				with No_more_pos -> 
				  add_to_m n dnet'
			      in
			      let lv' = MapSymb.add g (Free_son (dnet'')) lv in
				DNODE (u', lv')
			    end
		  with Not_found ->
		    if not (is_free g) (* g est C ou AC *)
		    then 
		      begin
			let top_layer_t = top_layer t in
			let dnet1 = List.fold_left 
				      (fun res t ->
					 let n' = fabrique_regle_etendue 
						    n.o n.regle t in
					   add_pattern_to_dnet_pos 
					     n' t Top res)
				      (LEAF []) l
			and dnet2 =
			  try
			    create_pattern_dnet_pos 
			      n top_layer_t (next_pos_p top_layer_t pos)
			  with No_more_pos -> LEAF[n] in
			let lv' = MapSymb.add g (AC_son (dnet1,dnet2)) lv in
			  DNODE (u', lv')
		      end
		    else 
		      begin
			let dnet' =
			  try
			    create_pattern_dnet_pos n t (next_pos_p t pos)
			  with No_more_pos -> LEAF [n] in
			let lv' = MapSymb.add g (Free_son (dnet')) lv in
		      	  DNODE (u', lv')
		      end
	  end
      	else
	  begin
	    if (is_smaller_pos pos u.p) (* pos < u.p *)
	    then
	      begin
		match t' with
	    	    VAR _ -> 
		      add_pattern_term_to_dnet_dnode_pos 
			n t (next_pos_p t pos) dnet 
		  | TERM (g,l) ->
		      let g_son =
		      	if not (is_free g) (* g est C ou AC *)
		      	then 
			  let top_layer_t = top_layer t in
			  let dnet1 = 
			    List.fold_left 
			      (fun res t -> 
				 let n' = fabrique_regle_etendue n.o n.regle t 
				 in
				   add_pattern_to_dnet_pos n' t Top res)
			      (LEAF []) l
			  and dnet2 = 
			    add_pattern_term_to_dnet_dnode_pos 
			      n top_layer_t (next_pos_p top_layer_t pos) dnet
			  in 
			    AC_son (dnet1,dnet2)
			else 
			  let dnet' = add_pattern_term_to_dnet_dnode_pos 
					n t (next_pos_p t pos) dnet in
			    Free_son dnet' in
			FULLDNODE (u', MapSymb.add g g_son MapSymb.empty, dnet)
	      end
	    else (* pos > u.p *)
	      begin
		let lv' = 
		  MapSymb.map 
		    (function 
			 Free_son dx ->
			   Free_son (add_pattern_term_to_dnet_pos 
				     n t pos dx)
		     | AC_son (dx1,dx2) ->
			 AC_son (dx1, add_pattern_term_to_dnet_pos 
				   n t pos dx2))
		    lv
		and v' = add_pattern_term_to_dnet_pos n t pos (LEAF []) in
	      	  FULLDNODE ({p=u.p;m=n::u.m}, lv', v')
	      end
	  end
	  
  | FULLDNODE (u,lv,v) as dnet ->
      let u' = {p=pos;m=n::u.m}
      and t' = sous_terme_a_la_position t pos in
      	if pos = u.p
      	then 
	  begin
	    match t' with
	    	VAR _ -> 
		  begin
		    try 
		      let pos' = next_pos_p t pos in
		      let lv' = 
		      	MapSymb.map 
			  (function 
		      	       Free_son (dnet') -> 
				 Free_son (add_pattern_term_to_dnet_pos 
					     n t pos' dnet')
		      	     | AC_son (dnet1,dnet2) ->
				 AC_son (dnet1,
					 add_pattern_term_to_dnet_pos 
					   n t pos' dnet2))
			  lv 
		      and v' = add_pattern_term_to_dnet_pos n t pos' v in
			FULLDNODE (u', lv', v')
		    with No_more_pos ->
		      let lv' = 
		    	MapSymb.map 
			  (function 
		      	       Free_son (dnet') -> 
				 Free_son (add_to_m n dnet')
		      	     | AC_son (dnet1,dnet2) -> 
				 AC_son (dnet1,add_to_m n dnet2))
			  lv 
		      and v' = add_to_m n v 
		      in
		    	FULLDNODE (u', lv', v')
		  end
		  
	      | TERM (g,l) ->
		  try
		    let g_son = MapSymb.find g lv in
		      match g_son with
			  AC_son (dnet1,dnet2) ->
			    let top_layer_t = top_layer t in
			    let dnet1' =
			      List.fold_left 
			      	(fun res t -> 
				   let n' = fabrique_regle_etendue 
					      n.o n.regle t in
				     add_pattern_to_dnet_pos n' t Top res)
			      	dnet1 l
			    and dnet2' =
			      try 
				add_pattern_term_to_dnet_pos n top_layer_t 
				  (next_pos_p top_layer_t pos) dnet2
			      with 
				  No_more_pos -> add_to_m n dnet2 in
			    let lv' = MapSymb.add g (AC_son (dnet1',dnet2')) lv
			    in
			      FULLDNODE (u', lv', v)
			| Free_son (dnet') ->
			    let dnet'' = 
			      try 
				add_pattern_term_to_dnet_pos 
                                  n t (next_pos_p t pos) dnet'
			      with 
				  No_more_pos -> add_to_m n dnet' in
			    let lv' = MapSymb.add g (Free_son (dnet'')) lv in
			      FULLDNODE (u', lv', v)
		  with 
		      Not_found ->
			if not (is_free g) (* g est C ou AC *)
			then 
			  begin
			    let top_layer_t = top_layer t in
			    let dnet1 =
			      List.fold_left 
				(fun res t -> 
				   let n' = fabrique_regle_etendue n.o n.
					      regle t in
				     add_pattern_to_dnet_pos n' t Top res)
				(LEAF []) l
			    and dnet2 = 
			      try 
				add_pattern_term_to_dnet_pos 
				  n top_layer_t (next_pos_p top_layer_t pos) v 
			      with 
				  No_more_pos -> add_to_m n v in
			    let lv' = MapSymb.add g (AC_son (dnet1,dnet2)) lv 
			    in
			      FULLDNODE (u', lv', v)
			  end
			else
			  begin
			    let dnet' = 
			      try
				add_pattern_term_to_dnet_pos
				  n t (next_pos_p t pos) v
			      with 
			      No_more_pos -> add_to_m n v in
			    let lv' = MapSymb.add g (Free_son dnet') lv in
			      FULLDNODE (u', lv', v)
			  end
	  end
	else
	  begin
	    if (is_smaller_pos pos u.p) (* pos < u.p *)
	    then
	      begin
	      let pos' = next_pos_p t pos in
	      	match t' with
	    	    VAR _ -> 
		      add_pattern_term_to_dnet_dnode_pos 
			n t (next_pos_p t pos) dnet 
		  | TERM (g,l) ->
		      let g_son = 
		      	if not (is_free g) (* g est C ou AC *)
		      	then
			  let top_layer_t = top_layer t in
			  let dnet1 =
			    List.fold_left 
			      (fun res t -> 
				 let n' = fabrique_regle_etendue n.o n.regle t
				 in
				   add_pattern_to_dnet_pos n' t Top res)
			      (LEAF []) l
			  and dnet2 = 
			    add_pattern_term_to_dnet_dnode_pos n top_layer_t 
			      (next_pos_p top_layer_t pos) dnet
			  in 
			    AC_son (dnet1,dnet2)
		      	else 
			  let dnet' = 
			    add_pattern_term_to_dnet_dnode_pos n t pos' dnet
			  in
			    Free_son dnet' 
		      in
		    	FULLDNODE (u', MapSymb.add g g_son MapSymb.empty, dnet)
	      end
	    else (* pos > u.p *)
	      begin
		let lv' = 
		  MapSymb.map 
		    (function 
			 Free_son (dx) ->
			   Free_son (add_pattern_term_to_dnet_pos 
				       n t pos dx)
		       | AC_son (dx1,dx2) ->
			   AC_son (dx1,
				   add_pattern_term_to_dnet_pos 
				     n t pos dx2))
		    lv
		and v' = add_pattern_term_to_dnet_pos n t pos v 
		in
	      	  FULLDNODE ({p=u.p;m=n::u.m}, lv', v')
	      end
	  end
  | LEAF _ -> failwith "this case should not happen in Dnet.add_pattern_to_dnet"

and add_pattern_term_to_dnet_pos n t pos = function
    LEAF _ as dnet -> add_pattern_term_to_dnet_leaf_pos n t pos dnet
  | DNODE _ as dnet -> add_pattern_term_to_dnet_dnode_pos n t pos dnet
  | FULLDNODE _ as dnet -> add_pattern_term_to_dnet_dnode_pos n t pos dnet

      
and add_pattern_to_dnet_pos n t pos = 
	match t with
	    VAR _ -> add_pattern_var_to_dnet_pos n 
	  | TERM _ -> add_pattern_term_to_dnet_pos n t pos



let create_pattern_dnet n t =
  create_pattern_dnet_pos n t Top



let add_pattern_to_dnet n t dnet = 
  add_pattern_to_dnet_pos n t Top dnet



let rec enleve n = function
    [] -> []
  | re::lre -> 
      if re.regle.num = n
      then lre
      else re::(enleve n lre)



let rec remove_pattern_from_dnet n = function
    LEAF l -> LEAF (enleve n l)
  | DNODE (u,lv) as dnet ->
      if is_an_empty_map lv
      then LEAF []
      else
	begin
	  if List.exists (function re -> re.regle.num = n) u.m
	  then
	    begin
      	      let lv' = 
	    	MapSymb.fold
		  (fun f f_son res ->
		     match f_son with
			 Free_son(dnet) -> 
			   begin
	      		     let dnet' = remove_pattern_from_dnet n dnet 
			     in
			       match dnet' with
				   LEAF [] -> res
				 | _ -> MapSymb.add f (Free_son (dnet')) res
			   end
		       | AC_son(dnet1,dnet2) ->
			   begin
	      		     let dnet1' = remove_pattern_from_dnet n dnet1
			     and dnet2' = remove_pattern_from_dnet n dnet2 
			     in
			       match dnet2' with
				   LEAF [] -> res
				 | _ -> MapSymb.add f (AC_son (dnet1',dnet2')) res
			   end)
		  lv MapSymb.empty 
	      in
		if is_an_empty_map lv'
		then LEAF []
		else DNODE ({p=u.p;m=enleve n u.m}, lv')
	    end
	  else dnet
	end
  | FULLDNODE (u,lv,v) as dnet ->
      if is_an_empty_map lv
      then remove_pattern_from_dnet n v
      else 
	begin
	  if List.exists (function re -> re.regle.num = n) u.m
	  then
	    begin
      	      let lv' = 
	    	MapSymb.fold
		  (fun f f_son res ->
		     match f_son with
			 Free_son(dnet) -> 
			   begin
	      		     let dnet' = remove_pattern_from_dnet n dnet 
			     in
			       match dnet' with
				   LEAF [] -> res
				 | _ -> MapSymb.add f (Free_son (dnet')) res
			   end
		       | AC_son(dnet1,dnet2) ->
			   begin
	      		     let dnet1' = remove_pattern_from_dnet n dnet1
			     and dnet2' = remove_pattern_from_dnet n dnet2 
			     in
			       match dnet2' with
				   LEAF [] -> res
				 | _ -> MapSymb.add f (AC_son (dnet1',dnet2')) res
			   end)
		  lv MapSymb.empty
	      and v' = remove_pattern_from_dnet n v
	      in
      	    	if (match v' with LEAF [] -> true | _ -> false) &
		  is_an_empty_map lv'
		then LEAF []
		else
		  try 
		    MapSymb.iter
		      (fun f f_son ->
			 match f_son with
			 Free_son (d) -> if (d<>v') then raise Exit
			   | AC_son (_, d) -> if (d<>v') then raise Exit)
		      lv';
		    v'
		  with Exit -> FULLDNODE ({p=u.p;m=enleve n u.m}, lv',v')
	    end
	  else dnet
	end



let rec is_a_possible_match_list lp ls dnet =
  let lp_var, lp_true_pat = 
    List.fold_left 
      (fun (resvar,resnonv) t ->
	 match t with
	     VAR _ -> (t::resvar,resnonv)
	   | TERM _  -> (resvar,t::resnonv))
      ([],[]) lp
  in
  let nb_var = List.length lp_var
  and nb_true_pat = List.length lp_true_pat
  and nb_sub = List.length ls in
  let capacite_residuelle = nb_sub - nb_true_pat in 
    if ((nb_var = 0) & (capacite_residuelle <> 0)) or
      (capacite_residuelle < nb_var)
    then false
    else
      let nb_pat = 
	if nb_var = 0
	then nb_true_pat
	else nb_true_pat+1 in
      let capa_s = Array.create nb_sub 1
      and capa_p = Array.create nb_pat 1
      and edge_matrix = Array.create_matrix nb_sub nb_pat 0 in
	if nb_var <> 0
	then 
	  begin
	    capa_p.(nb_true_pat) <- capacite_residuelle;
	    for i = 0 to (pred nb_sub) do
	      edge_matrix.(i).(nb_true_pat) <- 1
	    done
	  end;
	for i = 0 to (pred nb_sub) do
	  let si = List.nth ls i in
	  let list_for_si = find_patterns si dnet in
	    for j = 0 to (pred nb_true_pat) do
	      let pj = List.nth lp j in
		if List.exists (function re -> re.t = pj) list_for_si
		then edge_matrix.(i).(j) <- 1		
	    done
	done;
(* trop couteux !
   try 
   couplage_max capa_s capa_p edge_matrix;
   true
   with _ -> false
*)
	try
	  for i = 0 to (pred nb_sub) do
	    try
	      for j = 0 to (pred nb_pat) do
	      	if edge_matrix.(i).(j) > 0
	      	then failwith "has_a_corresponding_pattern"
	      done;
	      failwith "has_no_corresponding_pattern"
	    with Failure "has_a_corresponding_pattern" -> ()
	  done;
	  for j = 0 to (pred nb_pat) do
	    try
	      for i = 0 to (pred nb_sub) do
		if edge_matrix.(i).(j) > 0
		then failwith "has_a_corresponding_subject"
	      done;
	      failwith "has_no_corresponding_subject"
	    with Failure "has_a_corresponding_subject" -> ()
	  done;
	  true
	with 
	    Failure "has_no_corresponding_pattern" -> false
	  | Failure "has_no_corresponding_subject" -> false
		
and find_patterns t = function
    LEAF ln -> 
      List.fold_left 
	(fun res re -> 
	   if check_for_non_linearity t re.vp
	   then re::res
	   else res) 
	[] ln
  | DNODE (u,lv) -> 
      begin
      	let t' = sous_terme_a_la_position t u.p in
	  match t' with
	      VAR _ -> raise Not_found
	    | TERM (f,l) -> 
		let f_son = MapSymb.find f lv in
		  match f_son with
		      Free_son (dnet) ->
			find_patterns t dnet
		    | AC_son (dnet1,dnet2) ->
			begin
			  let ln = find_patterns t dnet2 in
			    List.fold_left 
			      (fun res r ->
				 try
				   let p = sous_terme_a_la_position r.t u.p in
				   let lp = liste_des_sous_termes p in
				     (**) if is_a_possible_match_list lp l dnet1
				     then (**) r::res
				     (**) else res (**)
				 with _ -> r::res)
			      [] ln
			end
      end
  | FULLDNODE (u,lv,v) ->
      begin
      	let t' = sous_terme_a_la_position t u.p in
	  match t' with
	      VAR _ -> find_patterns t v
	    | TERM (f,l) -> 
		try
		  let f_son = MapSymb.find f lv in
		    match f_son with
			Free_son (dnet) ->
			  find_patterns t dnet
		      | AC_son (dnet1,dnet2) ->
			  begin
			    let ln = find_patterns t dnet2 in
			      List.fold_left 
				(fun res r ->
				   try
				     let p = sous_terme_a_la_position 
					       r.t u.p in
				     let lp = liste_des_sous_termes p in
				       (**) if is_a_possible_match_list lp l dnet1
				       then (**) r::res
				       (**) else res (**)
				   with _ -> r::res)
				[] ln
			  end
		with Not_found ->
		  find_patterns t v
      end



let create_dnet o r =
  match o with
      LEFT ->
	let n = taille_terme r.l
	and re = fabrique_regle_etendue o r r.l in
	let dnet' = create_pattern_dnet re r.l in
	  if is_a_var_id r.x
	  then 
	    match r.l with 
	  	TERM (f,l) ->
		  let t' = TERM (f,(VAR r.x)::l) in
		  let re' = fabrique_regle_etendue o r t' in
		    (n,add_pattern_to_dnet re' t' dnet')
	      | _ -> failwith "create_dnet"
	  else (n, dnet')
    | RIGHT ->
	let n = taille_terme r.r 
	and re = fabrique_regle_etendue o r r.r in
	let dnet' = create_pattern_dnet re r.r in
	  if is_a_var_id r.y
	  then 
	    match r.r with 
	  	TERM (f,l) -> 
		  let t' = TERM (f,(VAR r.y)::l) in
		  let re' = fabrique_regle_etendue o r t' in
		    (n, add_pattern_to_dnet re' t' dnet')
	      | _ -> failwith "create_dnet"
	  else (n, dnet')



let add_rule_to_dnet o r (n,dnet) =
  match o with
      LEFT ->
	let n' = min n (taille_terme r.l) 
  	and re = fabrique_regle_etendue o r r.l in
  	let dnet' = add_pattern_to_dnet re r.l dnet in
	  if is_a_var_id r.x
	  then 
	    match r.l with 
	  	TERM (f,l) -> 
		  let t' = TERM (f,(VAR r.x)::l) in
		  let re' = fabrique_regle_etendue o r t' in
		    (n', add_pattern_to_dnet re' t' dnet')
	      | _ -> failwith "add_rule_to_dnet"
	  else (n', dnet')
    | RIGHT ->
	let n' = min n (taille_terme r.r)
  	and re = fabrique_regle_etendue  o r r.r in
  	let dnet' = add_pattern_to_dnet re r.r dnet in
	  if is_a_var_id r.y
	  then 
	    match r.r with 
	  	TERM (f,l) -> 
		  let t' = TERM (f,(VAR r.y)::l) in
		  let re' = fabrique_regle_etendue o r t' in
		    (n', add_pattern_to_dnet re' t' dnet')
	      | _ -> failwith "add_rule_to_dnet"
	  else (n', dnet')



let remove_rule_from_dnet m (n,dnet) =
  let dnet' = remove_pattern_from_dnet m dnet 
  and dnet_rule_list = liste_des_indices_du_dnet dnet in
  let re = 
    let rec find_re = function 
      	[] -> failwith "remove_rule_from_dnet"
      | r::lr -> if r.regle.num = m then r else find_re lr
    in find_re dnet_rule_list
  in
  let taille_re = 
    match re.o with
	LEFT -> taille_terme re.regle.l
      | RIGHT -> taille_terme re.regle.r
  in
    if taille_re > n
    then (n,dnet')
    else 
      let n' = 
	List.fold_left 
	  (fun res r -> 
	     let taille_r =
	       match r.o with
		   LEFT -> taille_terme r.regle.l
		 | RIGHT -> taille_terme r.regle.r
	     in
	       if res > taille_r
	       then taille_r
	       else res)
	  max_int dnet_rule_list
      in
	(n',dnet')



let find_rules t (n,dnet) =
  if (taille_terme t) < n
  then []
  else 
    try 
      find_patterns t dnet
    with Not_found -> []


(*
let is_a_possible_match_list = 
  Time_profiler.profile3 "is_a_possible_match_list" is_a_possible_match_list
*)

let find_rules = Time_profiler.profile2 "find_rules" find_rules
