(***************************************************************************

  Finding and checking non linearity for the many-to-one matching.
  
  CiME Project - Dmons research team - LRI - Universit Paris XI
  
  $Id: non_linearity.ml,v 1.5 2001/04/20 13:42:14 marche Exp $
  
***************************************************************************)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: non_linearity.ml,v 1.5 2001/04/20 13:42:14 marche Exp $

***************************************************************************)

(*

Obsolete header:

open Positions;;
open Symbols;;
open Terms;;



type match_conditions =
    NO_COND
  | SUFFICIENT_COND of (var_id * position list) list
  | NECESSARY_COND of (var_id * position list) list



(*
  non-linearity $t$ returns the list of couples $(x,pos_x)$ such that
  $x$ is a variable and pos_x is the list of all its positions in $t$.
*)

let rec non_linearity_pos pos t res = 
  match (sous_terme_a_la_position t pos) with
      TERM _ -> 
	begin
	  try 
      	    let pos' = next_pos_p t pos 
      	    in non_linearity_pos pos' t res
      	  with No_more_pos -> res
	end
	
    | VAR x -> 
	try 
 	  let pos_x = List.assoc x res in
	  let res' = (x,pos::pos_x)::(Listutils.remove x res) in 
	    try 
	      let pos' = next_pos_p t pos in
		non_linearity_pos pos' t res'
	    with No_more_pos -> res'
	with Not_found ->
	  let res' = (x,[pos])::res in
	    try 
	      let pos' = next_pos_p t pos in
		non_linearity_pos pos' t res'
	    with No_more_pos -> res'


let non_linearity t = non_linearity_pos Top t []



(*
  [(check_for_non_linearity t list_of_non_linearities)] checks if for
  all couples (x,pos_x) of [list_of_non_linearities], the subterms of
  [t], [t|_p] are AC-equal when for all [p]s in the same [pos_x].
*)

let rec check_for_non_linearity t = function
    [] -> true
  | (_,(_::[]))::nl -> check_for_non_linearity t nl
  | (_,(p1::lp))::nl -> 
      let t1 = sous_terme_a_la_position t p1 in 
	if List.for_all 
	  (function p -> egaux_ac t1 (sous_terme_a_la_position t p)) lp
	then check_for_non_linearity t nl
	else false
  | (_,[])::_ -> 
      failwith "this case should never happen : check_for_non_linearity"
	;;



(*
  [(extract_substitution t list_of_non_linearities)] checks if for
  all couples (x,pos_x) of [list_of_non_linearities], the subterms of
  [t], [t|_p] are AC-equal when for all [p]s in the same [pos_x], and
  when this is the case, returns the list of [(x,t|_p)] where [p]
  belongs to [pos_x]. Otherwise it fails with "extract_substitution".
*)

let rec extract_substitution t = function
    [] -> []
  | (x,[p])::nl -> 
      (x,sous_terme_a_la_position t p)::(extract_substitution t nl)
  | (x,p::other_pos)::nl ->
      let val_x = sous_terme_a_la_position t p in
	if List.for_all 
	  (function p -> egaux_ac val_x (sous_terme_a_la_position t p)) 
	  other_pos
	then (x,val_x)::(extract_substitution t nl)
	else failwith "extract_substitution"
  | (_,[])::_ -> failwith "this case should never happen : extract_substitution"
	 
