(***************************************************************************

AC critical pairs: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: ac_critical_pairs.ml,v 1.1 2000/05/16 17:36:33 monate Exp $

***************************************************************************)

(*

Obsolete header:

Last modified by: $Author: monate $
File name:        $Source: /users/demons/demons/master-sources-repository//cime/ocaml_src/oldcimroot/ac_critical_pairs.ml,v $
Revision:         $Revision: 1.1 $ 
Last modified on: $Date: 2000/05/16 17:36:33 $

***************************************************************************)

open Symbols;;
open Positions;;
open Terms;;
open Ac_unifiers;;
open Equations;;
open Messages;;
open Format;;
open Substitution;;

(*===========================================

AC critical pair computation

[paires_critiques_ac_etendues (l1->r1) (l2->r2)]
(resp. [auto_paires_critiques_ac_etendues (l->r)]) calcule les paires
critiques ETENDUES (c'est-a-dire tenant compte des extensions
eventuelles) entre l1->r1 et l2->r2 (resp l->r sur elle-meme) et
retourne le resultat sous la forme d'un couple de listes, la premiere
formee de triplets (r1,r2,theta) representant la paire critique en
tete r1\theta=r2\theta, la deuxieme de quintuplets (r1,l2,p,r2,theta)
representant la paire critique r1\theta=l2[r2]_p\theta

===========================================*)

(*
open Pretty_printing;;
*)

let paires_critiques_ac_en_tete l1 r1 l2 r2 = 
(*
  printf "paires_critiques_ac_en_tete\n";	
  flush stdout;
*)

let build_eq theta = (r1,r2,theta) in  
  List.map build_eq (ac_unifiers l1 l2)
;;

(*
let paires_critiques_ac_en_tete_partage f l1 r1 l2 r2 =
(*
  printf "paires_critiques_ac_en_tete_partage\n";	
  flush stdout;
*)
let build_eq theta = 
  begin
(*
  printf "substitution : %s\n" (string_of_substitution theta);
  flush stdout;
*)
  (r1,r2,theta)
  end in List.map build_eq (ac_unifiers_with_sharing f l1 l2)
;;
*)

(*===========================================

Calcul des paires critiques AC

(paires_critiques_ac_etendues l1 r1 l2 r2) renvoie une liste
d'\'equations qui sont les paires critiques AC des r\`egles $l_1\ra
r_1$ et $l_2\ra r_2$ c'est-\`a-dire~:

\[ 
\bigcup_{\sigma\in\CSU_{AC}(l_1|_p,l_2)} r_1\sigma = l1\sigma[r_2\sigma]_p 
\]

===========================================*)

(*=====

Fonctions auxiliaires

=====*)


let paires_critiques_ac_en_tete_avec_partage_eventuel l1 r1 l2 r2 =

   try let f1 = top_symbol_id l1 in
       if f1=(top_symbol_id l2)
       then begin
            (* if is_ac f1
      	     then paires_critiques_ac_en_tete_partage f1 
                            (sous_termes_partages (liste_des_sous_termes l1))
                            r1
                            (sous_termes_partages (liste_des_sous_termes l2))
                            r2
             else *)  
(*
  print_string "paires_critiques_ac_en_tete_avec_partage_eventuel entre les deux regles";	
  force_newline();
  print_term l1;
  print_string " -> ";
  print_term r1;
  force_newline();
  print_term l2;
  print_string " -> ";
  print_term r2;
  force_newline();
  print_flush();
*)
                   let res = (paires_critiques_ac_en_tete l1 r1 l2 r2) in
                   begin
(*
print_string "Paires obtenues:";
force_newline ();
List.iter (fun (r1,r2,theta) -> print_term (Substitution.substitute theta r1); 
                        print_string " <- ";
                        print_term (Substitution.substitute theta l1);
                        print_string " -> ";
                        print_term (Substitution.substitute theta r2);
                        force_newline()) res;
print_flush();
*)
                  res 
                  end
       end
       else []
  with Invalid_argument "Terms.top_symbol_id" -> 
           begin
           message "Warning: a rule has a variable as left-hand side";
           []
	   end
;;




let paires_critiques_ac_strictes l1 r1 l2 r2 =
(*
  print_string "paires_critiques_ac_strictes entre les deux regles";	
  force_newline();
  print_term l1;
  print_string " -> ";
  print_term r1;
  force_newline();
  print_term l2;
  print_string " -> ";
  print_term r2;
  force_newline();
  print_flush();
*)

let rec descendre_en_profondeur l1 p =
  match l1 with
    VAR _ -> []
  | TERM(_,m)   -> parcourir_sous_termes m p 1

and parcourir_sous_termes x y z= match (x,y,z) with
    []   ,  _, _ -> []
  | (s::m) ,p, i -> 
         (if is_a_variable s 
      	    then []
      	    else let sols = (ac_unifiers s l2) 
	     in 
      	       	(List.map (construit_paires (build_pos p i)) sols)
            @ (descendre_en_profondeur s (build_pos p i)))
       @ (parcourir_sous_termes m p (i+1))
and construit_paires p theta = 
      (r1,l1,p,r2,theta)

in let res = descendre_en_profondeur l1 Top in

(*
print_string "Paires obtenues:";
force_newline ();
List.iter (fun (r1,l2,p,r2,theta) -> print_term (Substitution.substitute theta r1); 
                        print_string " <- ";
                        print_term (Substitution.substitute theta l1);
                        print_string " -> ";
                        print_term (Substitution.substitute theta (replace l2 r2 p));
                        force_newline()) res;
print_flush();
*)
  res;;
 



let paires_critiques_ac_larges l1 r1 l2 r2 =
  (paires_critiques_ac_en_tete_avec_partage_eventuel l1 r1 l2 r2),
  (paires_critiques_ac_strictes l1 r1 l2 r2);;



(*=====

Calcul des paires critiques entre deux r\`egles distinctes

=====*)

let paires_critiques_ac_etendues_avec_orientation r1 r2 =
  let lr1 = leftify_rule r1
  and rr2 = rightify_rule r2 in

let compute_pairs rule1 rule2 =
  let x1 = rule1.x
  and x2 = rule2.x
  and l1 = rule1.l
  and l2 = rule2.l
  and r1 = rule1.r
  and r2 = rule2.r in
  let f1=(top_symbol_id l1) 
  and f2=(top_symbol_id l2) 
  and m1=(liste_des_sous_termes l1) 
  and m2=(liste_des_sous_termes l2) in
     

let hext1,pext1 = (if is_a_var_id x1 
        then paires_critiques_ac_larges 
		       l2 
		       r2
		       (TERM(f1,(VAR x1)::m1))
		       (add_left_var f1 x1 r1)
        else [],[])
and hext2,pext2 = (if is_a_var_id x2 
      	then paires_critiques_ac_larges 
		       l1
		       r1
		       (TERM(f2,(VAR x2)::m2))
		       (add_left_var f2 x2 r2)
        else [],[])
and hext3 = (if (f1=f2) & (is_a_var_id x1) & (is_a_var_id x2) 
        then paires_critiques_ac_en_tete_avec_partage_eventuel 
		       (TERM(f1,(VAR x1)::m1))
		       (add_left_var f1 x1 r1)
		       (TERM(f2,(VAR x2)::m2))
		       (add_left_var f2 x2 r2)
        else [])
in
  (paires_critiques_ac_en_tete_avec_partage_eventuel l1 r1 l2 r2) @ 
  hext1 @ hext2 @ hext3,
  (paires_critiques_ac_strictes l1 r1 l2 r2)
  @ (paires_critiques_ac_strictes l2 r2 l1 r1)
  @ pext1 @ pext2  
in  try compute_pairs (leftify_rule r1) (rightify_rule r2)
    with Invalid_argument "Terms.top_symbol_id" -> 
    begin
    message "Warning: a rule has a variable as left-hand side";
    [],[]
    end
;;


let paires_critiques_ac_etendues r1 r2 =
  if r1.orientation
  then begin
       if r2.orientation
       then paires_critiques_ac_etendues_avec_orientation r1 r2
       else let r2' = twist_rule r2 in
            let (f1,s1) = paires_critiques_ac_etendues_avec_orientation r1 r2
            and (f2,s2) = paires_critiques_ac_etendues_avec_orientation r1 r2' in
            (f1 @ f2,s1 @ s2)
       end
  else begin
       let r1' = twist_rule r1 in
       if r2.orientation
       then let (f1,s1) = paires_critiques_ac_etendues_avec_orientation r1 r2
            and (f2,s2) = paires_critiques_ac_etendues_avec_orientation r1' r2 in
            (f1 @ f2,s1 @ s2)
       else let r2' = twist_rule r2 in
            let (f1,s1) = paires_critiques_ac_etendues_avec_orientation r1 r2
            and (f2,s2) = paires_critiques_ac_etendues_avec_orientation r1' r2
            and (f3,s3) = paires_critiques_ac_etendues_avec_orientation r1 r2'
            and (f4,s4) = paires_critiques_ac_etendues_avec_orientation r1' r2' in
            (f1 @ f2 @ f3 @ f4,s1 @ s2 @ s3 @ s4)            
       end
;;

(*=====

Calcul des paires critiques engendr\'ees par une r\`egle sur
elle-m\^eme

=====*)

let auto_paires_critiques_ac_etendues_avec_orientation r =
  let rule1 = (leftify_rule r) 
  and rule2 = (rightify_rule r) in
 
  let x1 = rule1.x
  and x2 = rule2.x
  and l1 = rule1.l
  and l2 = rule2.l
  and r1 = rule1.r
  and r2 = rule2.r in

  try

  let f=(top_symbol_id l1) 
  and m1=(liste_des_sous_termes l1) 
  and m2=(liste_des_sous_termes l2) in

  let hext,pext = (if is_a_var_id x1 
       then let xl1 = TERM(f,(VAR x1)::m1)
       	    and xr1 = (add_left_var f x1 r1)
            and xl2 = TERM(f,(VAR x2)::m2)
       	    and xr2 = (add_left_var f x2 r2) in
            let hext1,pext1 = (paires_critiques_ac_larges xl1 xr1 l2 r2)
	    in hext1 @ (paires_critiques_ac_en_tete_avec_partage_eventuel
		       xl1 xr1 xl2 xr2), pext1
        else [],[])
  in 
   hext @ (paires_critiques_ac_en_tete_avec_partage_eventuel l1 r1 l2 r2),
   pext @ (paires_critiques_ac_strictes l1 r1 l2 r2)

  with Invalid_argument "Terms.top_symbol_id" -> 
    begin
    message "Warning: a rule has a variable as left-hand side";
    [],[]
    end
;;

let auto_paires_critiques_ac_etendues r = 
  if r.orientation
  then auto_paires_critiques_ac_etendues_avec_orientation r
  else let r' = twist_rule r in
       let (f1,s1) = auto_paires_critiques_ac_etendues_avec_orientation r
       and (f2,s2) = auto_paires_critiques_ac_etendues_avec_orientation r'
       and (f3,s3) = paires_critiques_ac_etendues_avec_orientation r r' in
       (f1 @ f2 @ f3,s1 @ s2 @s3)
;;

