/***************************************************************************

Le type terme de CAML : corps

Project:                Cime
Author(s):             Claude Marche'

File name:        camlterm.c
Created on:              27 jan 93
Last modified on: 05 fev 93

***************************************************************************/


#include "camlterm.h"

#if defined (OCAML)
#include <caml/alloc.h> 
#include <caml/memory.h>
#else
#include <alloc.h> 
#include <memory.h>
#endif

#include <stdio.h>
#include <malloc.h>

void failwith(char *c);



static int nombre_d_operateurs = 0;
static type_operateur *table_operateurs = 0;

value clear_operator_table(value unit)
{
  if (table_operateurs != 0)
    {
      free(table_operateurs);
      table_operateurs = 0;
    }
  nombre_d_operateurs = 0;
  /*
  printf ("C operator table has been cleared\n");
  fflush(stdout);
  */
  return Val_unit; 
}

value declare_operator(value vt,value vf)
{
  type_operateur *p;
  int f;
  type_operateur t;

  /*
  printf ("Coucou, on est dans declare_operator\n");
  fflush(stdout);
  */

  f = Int_val(vf);
  /*
  printf ("0\n");
  fflush(stdout);
  printf ("vt = %ld\n",vt);
  fflush(stdout);
  */
#if defined(OCAML)
  t = Int_val(vt);
#else
  t = Tag_val(vt);
#endif

  /*
  printf ("1\n");
  fflush(stdout);
  */
    if (f >= nombre_d_operateurs)
      {
	p = table_operateurs;
	nombre_d_operateurs = f+1;
	table_operateurs = (type_operateur *)calloc(nombre_d_operateurs,sizeof(type_operateur));
	/*
	printf ("2\n");
	fflush(stdout);
	*/
	if ( p != 0)
	  {
	    memcpy(table_operateurs,p,f*sizeof(type_operateur));
	    free(p);
	  }
      }
  table_operateurs[f] = t;
  /*
  printf("operateur : type %d numero %d\n",t,f);
  fflush(stdout);
  */
  return Val_unit;
}

/*=============

Constructeurs

=============*/

#define EQUATION_tag 0

terme build_term(term_tag t,int f,liste l)
{
  value s;
  Push_roots(r,1);
  
  r[0] = l;
  switch (t)
    {
    case CAML_VAR:
      {
	s = alloc(1,CAML_VAR);
	Field(s,0) = Val_int(f);
	break;
      }
    case CAML_TERM:
      {
	s = alloc(2,CAML_TERM);
	Field(s,0) = Val_int(f);
	Field(s,1) = r[0];
	break;
      }
    default:
      failwith("Bug in C function construit_terme()");
      s = alloc(1,CAML_VAR);
      Field(s,0) = Val_int(-1);
    }
  Pop_roots();
  return s;
}

terme construit_terme(type_operateur t,int f,liste l)
{
  return (t == TYPE_CAML_VARIABLE) ?
    build_term(CAML_VAR,f,l) :
    build_term(CAML_TERM,f,l);
}


term_tag tag_of_term(terme t)
{
  return Tag_val(t);
}


type_operateur type_de_l_operateur(terme t)
{
  int f;
  switch (Tag_val(t))
    {
    case CAML_VAR: 
      {
	return TYPE_CAML_VARIABLE;
      }
    case CAML_TERM: 
      {
	f = Int_val(Field(t,0));
/*
	if (f>=0 && f<nombre_d_operateurs)
	  {
	    printf ("Lecture operateur numero %d : type %d\n",f,table_operateurs[f]);
	    fflush(stdout);
*/
	return table_operateurs[f];
/*
          }
*/
      }
  }
  failwith("Bug in C function type_de_l_operateur()");
  return -1;
}

int numero_de_l_operateur(terme t)
{
/*
switch (Tag_val(t))
  {
  case CAML_VAR: return Int_val(Field(t,0));
  case CAML_TERM:*/
 return Int_val(Field(t,0));
/*
  default : return -1;
  }
*/
}

liste liste_des_sous_termes(terme t)
{
switch (Tag_val(t))
  {
  case CAML_VAR: 
    return liste_vide();
  case CAML_TERM: 
    return Field(t,1);
  default : 
    failwith("Bug in C function liste_des_sous_termes()");
    return t;
  }
}

/*
equation construit_equation(terme t1, terme t2)
{
  value e;
  Push_roots(r,2);

  r[0] = t1;
  r[1] = t2;

  e = alloc(3,EQUATION_tag);
  Field(e,0) = Val_int(-1);
  Field(e,1) = r[0];
  Field(e,2) = r[1];
  Pop_roots();
  return e;  
}
*/

/*============

Observateurs

============*/


/*========================

Fonctions sur les termes

========================*/



/*=================================

Fonction d'aplatissement des termes:

Exemple:

aplatir_term <<a+(b+c)>>

donne <<a+b+c>>

=================================*/

/*
liste aplatir_liste_de_termes(liste l)
{
  value result;
  Push_roots(r,3);

  r[0] = l;
  if (N_EST_PAS_VIDE(r[0]))
    {
      r[1] = aplatir_terme(TETE_DE_LISTE(r[0]));
      r[2] = aplatir_liste_de_termes(QUEUE_DE_LISTE(r[0]));
      result = construit_liste(r[1],r[2]);
    }
  else
    {
      result = liste_vide();
    }
  Pop_roots();
  return result;
}


liste aplatir_par_rapport_a(liste l,int f)
{
  value result;
  Push_roots(r,2);

  r[1] = l;
  if (N_EST_PAS_VIDE(r[1]))
    {
      r[0] = aplatir_par_rapport_a(QUEUE_DE_LISTE(r[1]),f);
      if (tag_of_term(TETE_DE_LISTE(r[1])) == CAML_TERM 
	  && numero_de_l_operateur(TETE_DE_LISTE(r[1])) == f)
	{
	  result = concatene_listes(liste_des_sous_termes(TETE_DE_LISTE(r[1])),
				    r[0]);
	}
      else
	{
	  result = construit_liste(TETE_DE_LISTE(r[1]),r[0]);
	}
    }
  else
    {
      result = liste_vide();
    }
  Pop_roots();
  return result;
}

terme aplatir_terme(terme t)
{
  value result;
  Push_roots(r,2);

  r[1] = t;
  r[0] = aplatir_liste_de_termes(liste_des_sous_termes(r[1]));

  if (type_de_l_operateur(r[1]) == TYPE_CAML_AC)
    {
      r[0] = aplatir_par_rapport_a(r[0],numero_de_l_operateur(r[1]));
    }
  result = build_term(tag_of_term(r[1]),
		      numero_de_l_operateur(r[1]),
		      r[0]);
  Pop_roots();
  return result;
}
*/



