(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: completion_state.ml,v 1.2 2001/04/20 13:42:19 marche Exp $

***************************************************************************)

open Args;;
open Orderings;;
open Content;;
open Symbols;;
open Terms;;
open Equations;;
open Equations_as_obj;;
open Dnet;;
open Narrowing;;
open Encompassment;;
open Simplifications;;
open Format;;
open Messages;;

module type COMPLETION_STATE = 
  sig
    type equation
    type equation_set
    type rule_set
    type conjectures_set
    type etat_completion = 
    	{
	  mutable eqs_to_process : equation_set;
	  mutable postponed_eqs : equation list; 
	  mutable preoriented_eqs : equation_set;
	  mutable pre_rules : equation_set;
	  mutable rules : rule_set;
	  mutable disc_net : int * regle_etendue dnet;
	  mutable min_rules_interest_wrt_g : i3;
	  mutable major_deduce_index : int;
	  mutable minor_deduce_index : int;
	  mutable date : int;
	  mutable conjectures : conjectures_set;
	  mutable prove_conjectures : bool
    	}
    exception Completion_failed
    exception Completion_aborted
    val etat : etat_completion
    val afficher_etat : unit -> unit
    val copy_state : unit -> etat_completion
    val restore_state : etat_completion -> unit
    val choice_stack : etat_completion Stack.t
    val current_rules : unit -> int * regle_etendue dnet
    val interest_of_date : int -> int -> int
    val interest_of : int -> int
    val compute_interest_wrt_goal : int -> int -> term -> term -> int array
    val add_new_eq :  equation -> unit
    val add_new_eqs : equation list -> unit
    val add_new_preoriented_eq : equation -> unit
    val add_postponed_eq : equation -> unit
    val add_new_pre_rule : int -> equation -> unit
    val get_rule_of_index : int -> Equations.regle
    val get_major_deduce_index : unit -> int
    val get_minor_deduce_index : unit -> int
    val increment_minor_deduce_index : unit -> unit
    val increment_major_deduce_index : unit -> unit
    val auto_deduce : unit -> bool
    val config_completion_state : unit -> unit
    val update_state_wrt_conj : unit -> unit
  end
;;


module Completion_state =
  functor (Eqt : EQUATION) ->
  functor (EqtSet : EQUATION_SET with type equation = Eqt.t) ->
  functor (RlSet : EQUATION_SET with type equation = regle) ->
  functor (Conj : EQUATION) ->
  functor (ConjSet : EQUATION_SET with type equation = Conj.t) ->
  functor (Nar : USEFUL_FOR_NARROWING with type conjectures_set = ConjSet.eq_set) ->
    struct

open Eqt;;
open EqtSet;;

type equation = t;;
type equation_set = eq_set;;
type rule_set = Rules_set.eq_set;;
type conjectures = Conj.t;;
type conjectures_set = ConjSet.eq_set;;

type etat_completion = 
    {
      mutable eqs_to_process : equation_set;
      mutable postponed_eqs : equation list; 
      mutable preoriented_eqs : equation_set;
      mutable pre_rules : equation_set;
      mutable rules : RlSet.eq_set;
      mutable disc_net : int * regle_etendue dnet;
      mutable min_rules_interest_wrt_g : i3;
      mutable major_deduce_index : int;
      mutable minor_deduce_index : int;
      mutable date : int;
      mutable conjectures : conjectures_set;
      mutable prove_conjectures : bool
    }
;;

exception Completion_failed;;
exception Completion_aborted;;

let choice_stack = Stack.create ();;

let etat = 
  { 
    eqs_to_process = make_empty_equation_set (); 
    postponed_eqs = []; 
    preoriented_eqs = make_empty_equation_set (); 
    pre_rules = make_empty_equation_set ();
    rules = RlSet.make_empty_equation_set ();
    disc_net = max_int, empty_dnet ();
    min_rules_interest_wrt_g = Max;
    major_deduce_index = max_int;
    minor_deduce_index = max_int;
    date = 0;
    conjectures = ConjSet.make_empty_equation_set ();
    prove_conjectures = false
  }
;;


let afficher_etat () =

      	 open_vbox 0;
         print_string " ----------------------";
         print_string (sprintf "State of completion, step %d" etat.date);
         print_string " ----------------------";
         force_newline ();
      	 print_string "-- Equations to orient -- (min complexity = ";
         print_min_interest etat.eqs_to_process;
	 print_string ")";
         force_newline ();
      	 print_equation_set etat.eqs_to_process;
      	 force_newline ();
	 if etat.postponed_eqs <> []
	 then
	   begin
       	     print_string "-- Unorientable equations --";
             force_newline ();
             print_list_of_equations etat.postponed_eqs;
      	     force_newline ()
	   end;
      	 print_string "-- Pre-oriented (symmetrized) equations --";
         print_string " (min complexities = ";
	 print_min_interest etat.preoriented_eqs;
	 print_string ")";
         force_newline ();
      	 print_equation_set etat.preoriented_eqs;
      	 force_newline ();
      	 print_string "-- Pre-rules -- (min complexities = ";
         print_min_interest etat.pre_rules;
	 print_string ")";
         force_newline ();
      	 print_equation_set etat.pre_rules;
      	 force_newline ();
      	 print_string "-- Rules -- (min complexities = ";
         RlSet.print_min_interest etat.rules;
	 print_string ")";
         force_newline ();
      	 RlSet.print_equation_set etat.rules;
      	 force_newline ();
      	 print_string (sprintf "Deduction indices : %d %d" 
                         etat.major_deduce_index 
			 etat.minor_deduce_index);
         force_newline ();
(*
      	 print_string (sprintf "-- Compiled Rules -- ");
         force_newline ();
      	 print_dnet (snd etat.disc_net);
      	 force_newline ();
*)
      	 print_string (sprintf "-- conjectures --");
         force_newline ();
         ConjSet.print_equation_set etat.conjectures;
         force_newline ();
	 close_box()
;;

let copy_state () =
  { 
    eqs_to_process = etat.eqs_to_process;
    postponed_eqs = etat.postponed_eqs;
    preoriented_eqs = etat.preoriented_eqs;
    pre_rules = etat.pre_rules;
    rules = etat.rules;
    disc_net = etat.disc_net;
    min_rules_interest_wrt_g = etat.min_rules_interest_wrt_g;
    major_deduce_index = etat.major_deduce_index;
    minor_deduce_index = etat.minor_deduce_index;
    date = etat.date;
    conjectures = etat.conjectures;
    prove_conjectures = etat.prove_conjectures
  } 
;;


let restore_state old_state =
  etat.eqs_to_process <- old_state.eqs_to_process;
  etat.postponed_eqs <- old_state.postponed_eqs; 
  etat.preoriented_eqs <- old_state.preoriented_eqs;
  etat.pre_rules <- old_state.pre_rules;
  etat.rules <- old_state.rules;
  etat.disc_net <- old_state.disc_net;
  etat.min_rules_interest_wrt_g <- old_state.min_rules_interest_wrt_g;
  etat.major_deduce_index <- old_state.major_deduce_index;
  etat.minor_deduce_index <- old_state.minor_deduce_index;
  etat.date <- old_state.date;
  etat.conjectures <- old_state.conjectures;
  etat.prove_conjectures <- old_state.prove_conjectures
;;

let current_rules () = etat.disc_net;;

let coef_size = ref 1;;
let coef_date = ref 0;;
let coef_nar = ref 0;;


let interest_of_date size date =
  (!coef_size * size) + (!coef_date * date);;

let interest_of size = interest_of_date size etat.date;;

let compute_interest_of_date date u v = 
    (!coef_size * (size_eq u v)) + (!coef_date * date)
;;

let compute_interest_of u v = compute_interest_of_date etat.date u v
;;

let compute_interest_wrt_goal i1 i2 u v =
  
  let occ_nest = 
    let theta = fun x y -> x*y
    and psi_1 = function x -> if x<=0 then 1 else x+1
    and psi_2 = function x -> if x<=0 then 1 else x+1
    and list_of_functions = listof_all_non_top_functions ()
    in
    let interest_wrt s t =
      List.fold_left
	(fun res f -> 
	   let ofu = symb_measure_sum f u
	   and ofv = symb_measure_sum f v
	   and ofs = symb_measure_sum f s
	   and oft = symb_measure_sum f t
	   and nfu = nesting f u
	   and nfv = nesting f v
	   and nfs = nesting f s
	   and nft = nesting f t
	   in
	   let mf = theta (psi_1 ((max ofu ofv) - (max ofs oft)))
		      (psi_2 ((max nfu nfv) - (max nfs nft)))
	   in
	     mf * res)
	1 list_of_functions
    in
      ConjSet.fold_left
	(fun res c ->
	   min res (interest_wrt (Conj.first_term c) 
		      (Conj.second_term c)))
	max_int
    	
    in
      
      match options_courantes.goal_oriented with
      	  0 -> [|i1;i2|]
      	| 1 -> 
	    [|(1+(Nar.is_a_useful_eq_for_nar u v etat.conjectures)) * i1;
	      i2|]
      	| 2 -> [|(occ_nest etat.conjectures) * i1;i2|]
      	| 3 -> 
	    [|((Nar.is_a_useful_eq_for_nar u v etat.conjectures) + 1) *
	     (occ_nest etat.conjectures) *  i1;
	      i2|]
	| 4 -> 
	    [|Nar.is_a_useful_eq_for_nar u v etat.conjectures; 
	     occ_nest etat.conjectures;
	     i1;i2|]
	| 5 -> 
	    [|occ_nest etat.conjectures;
	     Nar.is_a_useful_eq_for_nar u v etat.conjectures;
	     i1;i2|]
	| _ -> failwith "This option is not available for goal"
;;

let compute_if_nec_interest1_of eq =
  if (interest1_of eq) = Uncomputed
  then set_interest1 
    (Some (interest_of_date 
	     (size_eq (first_term eq) (second_term eq)) 
	     (get_age eq))) 
    eq;;

let compute_if_nec_interest2_of eq =
  if (interest2_of eq) = Uncomputed
  then 
    let u = first_term eq
    and v = second_term eq in
      set_interest2
      	(Some (interest_of_date 
		 (if (is_oriented eq) 
		  then (size_rule u v) 
		  else (min (size_rule u v) (size_rule v u)))
		 (get_age eq)))
      	eq;;

let compute_if_nec_interest3_of eq =
  if (interest3_of eq) = Uncomputed
  then 
    let u = first_term eq
    and v = second_term eq in
      set_interest3
	(Some (compute_interest_wrt_goal 
		 (match (interest1_of eq) with
		      Some i1 -> i1
		    | Uncomputed ->
			let i1 = interest_of_date (size_eq u v) (get_age eq) in
			  set_interest1 (Some i1) eq;
			  i1
		    | _ -> failwith "compute_if_nec_interest3_of")
		 (match (interest2_of eq) with 
		      Some i2 -> i2
		    | Uncomputed ->
			let i2 = 
			  interest_of_date
			    (if (is_oriented eq) 
			     then (size_rule u v) 
			     else (min (size_rule u v) (size_rule v u)))
			    (get_age eq) in
			  set_interest2 (Some i2) eq;
			  i2
		    | No_value -> 0
		    | _ -> failwith "compute_if_nec_interest3_of")
		 u v))
	eq;;

let compute_interest3_of eq =
  let u = first_term eq
  and v = second_term eq in
    set_interest3
      (Some (compute_interest_wrt_goal 
	       (match (interest1_of eq) with
		    Some i1 -> i1
		  | Uncomputed ->
		      let i1 = interest_of_date (size_eq u v) (get_age eq) in
			set_interest1 (Some i1) eq;
			i1
		  | _ -> failwith "compute_interest3_of")
	       (match (interest2_of eq) with 
		    Some i2 -> i2
		  | Uncomputed ->
		      let i2 = 
			interest_of_date
			  (if (is_oriented eq) 
			   then (size_rule u v) 
			   else (min (size_rule u v) (size_rule v u)))
			  (get_age eq) in
			set_interest2 (Some i2) eq;
			i2
		  | No_value -> 0
		  | _ -> failwith "compute_interest3_of")
	       u v))
      eq;;

let update_state_wrt_conj () =
  let recompute_goal_interest eq_set = 
    EqtSet.set_min_interest3 Max eq_set;
    EqtSet.iter
      (function eq -> 
	 compute_interest3_of eq;
	 let eq_i3 = Eqt.interest3_of eq
	 and eq_set_i3 = EqtSet.min_interest3 eq_set in
	   if compare_i3 eq_i3 eq_set_i3 = Less_than
	   then EqtSet.set_min_interest3 eq_i3 eq_set)
      eq_set in

    recompute_goal_interest etat.eqs_to_process;
    recompute_goal_interest etat.preoriented_eqs;
    recompute_goal_interest etat.pre_rules;
    
  RlSet.iter
    (function rule -> 
       rule.rule_interest_wrt_goal <-
	 (Some (compute_interest_wrt_goal 
		  (match  rule.rule_interest with
		       Some i1 -> i1
		     | _ -> failwith "update_state_wrt_conj")
		  (match rule.rule_lhs_interest with 
		       Some i2 -> i2
		     | _ -> failwith "update_state_wrt_conj")
		  rule.l rule.r));
       if rule.num = etat.major_deduce_index
       then
	 begin
	   RlSet.set_min_interest3 rule.rule_interest_wrt_goal etat.rules;
	 end)
    etat.rules

;;


let add_new_eq eq =
  let u',v' = rename_pair (first_term eq, second_term eq) in
  let u = sort_term u'
  and v = sort_term v' in
    set_first_term u eq;
    set_second_term v eq;
    compute_if_nec_interest1_of eq;
(*
    compute_if_nec_interest2_of eq;
*)
    set_interest2 No_value eq;
    compute_if_nec_interest3_of eq;
    carefully_insert_equation eq etat.eqs_to_process
;;

let add_new_eqs = List.iter add_new_eq
;;

let add_new_preoriented_eq eq = 
  let u',v' = rename_pair (first_term eq, second_term eq) in
  let u = sort_term u'
  and v = sort_term v' in
    set_first_term u eq;
    set_second_term v eq;
    compute_if_nec_interest1_of eq;
    compute_if_nec_interest2_of eq;
    compute_if_nec_interest3_of eq;
    insert_equation eq etat.preoriented_eqs
;;


let add_postponed_eq eq =
  etat.postponed_eqs <- eq::etat.postponed_eqs
;;


let add_new_pre_rule date eq = 
  compute_if_nec_interest1_of eq;
  compute_if_nec_interest2_of eq;
  compute_if_nec_interest3_of eq;
  insert_equation eq etat.pre_rules
;;


let get_rule_of_index ind =
  RlSet.find (function rule -> rule.num = ind) etat.rules;;

let get_major_deduce_index () = etat.major_deduce_index;;

let get_minor_deduce_index () = etat.minor_deduce_index;;


let increment_minor_deduce_index () = 
  try
    RlSet.iter
      (function rule -> 
	 let num_rule = rule.num in
           if num_rule > etat.minor_deduce_index
           then 
	     begin
	       etat.minor_deduce_index <- num_rule;
	       raise Exit
	     end)
      etat.rules;
    etat.minor_deduce_index <- max_int
  with Exit -> ()
;;


let increment_major_deduce_index () = 
  try
    RlSet.iter
      (function rule ->
     	 let num_rule = rule.num in
	   if num_rule > etat.major_deduce_index 
	   then 
	     begin
      	       etat.major_deduce_index <- num_rule;
	       RlSet.set_min_interest1 (Some (size_eq rule.l rule.r)) 
		 etat.rules;
	       RlSet.set_min_interest2
		 (Some (if rule.orientation 
		       	then size_rule rule.l rule.r 
		       	else  
			  max (size_rule rule.l rule.r) 
			    (size_rule rule.r rule.l)))
		 etat.rules;
	       RlSet.set_min_interest3 rule.rule_interest_wrt_goal etat.rules;
               etat.minor_deduce_index <- 0;
	       increment_minor_deduce_index ();
	       raise Exit
	     end)
      etat.rules;
    etat.major_deduce_index <- max_int;
    RlSet.set_min_interest1 Max etat.rules;
    RlSet.set_min_interest2 Max etat.rules;
    RlSet.set_min_interest3 Max etat.rules;
    etat.min_rules_interest_wrt_g <- Max;
  with Exit -> ()
;;


let auto_deduce () =
  etat.minor_deduce_index = etat.major_deduce_index
;;

let config_completion_state () =
  let strat = options_courantes.strategy_number in
    if strat = 0
    then 
      begin
      	coef_size := 1;
      	coef_date := 0
      end
    else 
      if strat < 0
      then 
	begin
      	  coef_size := 1;
      	  coef_date := -strat
	end
      else 
	begin
      	  coef_size := strat;
      	  coef_date := 1
	end;
    coef_nar := 1024 * options_courantes.goal_oriented
;;


  end;;


module Standard_completion = 
  Completion_state (Equation1) (Equation_set1) (Rules_set)
    (Conjectures) (Conjectures_set) (Standard_useful_for_narrowing);;

module G_completion = 
  Completion_state (Equation_with_goal) (Equation_with_goal_set) 
    (Rules_set) (Conjectures) (Conjectures_set) 
    (Standard_useful_for_narrowing);;


