(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: completion_state.mli,v 1.2 2001/04/20 13:42:19 marche Exp $

***************************************************************************)

open Content;;
open Terms;;
open Equations;;
open Equations_as_obj;;
open Dnet;;

(*=====================================

L'\'etat courant de la compl\'etion est repr\'esent\'e par un
enregistrement contenant:

. L'ensemble des equations a orienter
. l'ensemble des equations inorientables
. l'ensemble des equations preorientees
. l'ensemble des preregles
. l'ensemble des regles

=======================================*)

module type COMPLETION_STATE = 
  sig
    type equation
    type equation_set
    type rule_set
    type conjectures_set
    type etat_completion = 
    	{
	  mutable eqs_to_process : equation_set;
	  mutable postponed_eqs : equation list; 
	  mutable preoriented_eqs : equation_set;
	  mutable pre_rules : equation_set;
	  mutable rules : rule_set;
	  mutable disc_net : int * regle_etendue dnet;
	  mutable min_rules_interest_wrt_g : i3;
	  mutable major_deduce_index : int;
	  mutable minor_deduce_index : int;
	  mutable date : int;
	  mutable conjectures : conjectures_set;
	  mutable prove_conjectures : bool
    	}
    exception Completion_failed
    exception Completion_aborted
    val etat : etat_completion
    val afficher_etat : unit -> unit
    val copy_state : unit -> etat_completion
    val restore_state : etat_completion -> unit
    val choice_stack : etat_completion Stack.t
    val current_rules : unit -> int * regle_etendue dnet
    val interest_of_date : int -> int -> int
    val interest_of : int -> int
    val compute_interest_wrt_goal : int -> int -> term -> term -> int array
    val add_new_eq : equation -> unit
    val add_new_eqs : equation list -> unit
    val add_new_preoriented_eq : equation -> unit
    val add_postponed_eq : equation -> unit
    val add_new_pre_rule : int -> equation -> unit
    val get_rule_of_index : int -> Equations.regle
    val get_major_deduce_index : unit -> int
    val get_minor_deduce_index : unit -> int
    val increment_minor_deduce_index : unit -> unit
    val increment_major_deduce_index : unit -> unit
    val auto_deduce : unit -> bool
    val config_completion_state : unit -> unit
    val update_state_wrt_conj : unit -> unit
  end
;;

module Standard_completion : 
  (COMPLETION_STATE with type equation = Equation1.t
		    and type equation_set = Equation_set1.eq_set
		    and type rule_set = Rules_set.eq_set
		    and type conjectures_set = Conjectures_set.eq_set);;

module G_completion : 
  (COMPLETION_STATE with type equation = Equation_with_goal.t
		    and type equation_set = Equation_with_goal_set.eq_set
		    and type rule_set = Rules_set.eq_set
		    and type conjectures_set = Conjectures_set.eq_set);;
