(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: deduce.ml,v 1.2 2001/04/20 13:42:19 marche Exp $

***************************************************************************)

(***************************************************************************
  
  Completion rules : deduce
  
  Last modified by: $Author: marche $
  File name:        $Source: /users/demons/demons/master-sources-repository//cime/ocaml_src/oldcimroot/deduce.ml,v $
  Revision:         $Revision: 1.2 $ 
  Last modified on: $Date: 2001/04/20 13:42:19 $

***************************************************************************)

open Args;;
open Orderings;;
open Terms;;
open Completion_state;;
open Equations;;
open Equations_as_obj;;
open Ac_critical_pairs;;
open Critical_pair_criteria;;
open Simplifications;;
open Dnet;;
open Format;;
open Messages;;
open Completion_state;;

module type DEDUCE =
  sig 
    val deduce : term ordering -> unit 
  end
;;

module Deduce =
  functor (Eqt : EQUATION) -> 
  functor (Compl : COMPLETION_STATE with type equation = Eqt.t) ->
  functor (Simpl : SIMPLIFICATIONS with type equation = Eqt.t) ->
  functor (Crit_pairs : CRITICAL_PAIR_CRITERIA with type equation = Eqt.t) ->
    
struct

open Eqt;;
open Compl;;
open Simpl;;
open Crit_pairs;;

let deduce order =

  let rules = current_rules () in

    if (get_minor_deduce_index ()) > (get_major_deduce_index ())
    then increment_major_deduce_index ();
    try 
      let major_rule = get_rule_of_index (get_major_deduce_index ()) in
      	if auto_deduce ()
      	then (* Auto-deduce *)
	  begin
    	    if (options_courantes.niveau_de_verbosite >= 1)
            then 
	      begin
      	      	print_string " ---- SELF-DEDUCTION with ";
              	print_regle major_rule;
              	force_newline ();
              	print_flush ()
	      end;
	    let crit_sets = (auto_paires_critiques_ac_etendues major_rule) in
	    let crit_pairs = (make_essential_critical_pairs 
			      	order rules crit_sets) in
	      List.iter (function eq -> set_age etat.date eq) crit_pairs; 
	      if options_courantes.niveau_de_verbosite >= 2
              then 
	      	begin
      	    	  print_string "set of essential critical pairs:";
                  print_break 1 0;
	          print_list_of_equations crit_pairs;
                  force_newline ();
                  print_flush ()
      	      	end;
              if options_courantes.niveau_de_verbosite >= 1
              then message (sprintf "%d essential critical pairs." 
			      (List.length crit_pairs));
              if options_courantes.niveau_de_verbosite >= 1
              then message 
	      	" ---- SIMPLIFY & DELETE ------------------------------";
              let simpl_crit_pairs = 
	      	simplify 
		(if options_courantes.ordered_completion
		 then order
		 else (fun _ _ -> Greater)) crit_pairs rules in 
	      	if options_courantes.niveau_de_verbosite >= 2
	      	then 
		  begin
      	    	    print_string "set of simplified critical pairs:";
                    print_break 1 0;
	            print_list_of_equations simpl_crit_pairs;
                    force_newline ();
                    print_flush ()		  
		  end;
	      	add_new_eqs simpl_crit_pairs
	  end
      	else
          begin
	    try
      	      let minor_rule = 
	      	get_rule_of_index (get_minor_deduce_index ()) in
              	if (options_courantes.niveau_de_verbosite >= 1)
              	then 
		  begin
      	            print_string " ---- DEDUCTION with";
                    print_break 1 0;
                    print_regle major_rule;
                    print_string " and ";
                    print_regle minor_rule;
                    force_newline ();
                    print_flush ()
		  end;
	      	let crit_sets = (paires_critiques_ac_etendues 
				   major_rule minor_rule) in
	      	let crit_pairs = make_essential_critical_pairs 
				   order rules crit_sets in
  	   	  List.iter (function eq -> set_age etat.date eq) crit_pairs;
		  if (options_courantes.niveau_de_verbosite >= 2)
             	  then 
		    begin
      	              print_string "set of essential critical pairs:";
                      print_break 1 0;
	              print_list_of_equations crit_pairs;
                      force_newline ();
                      print_flush ()
                    end;
           	  if (options_courantes.niveau_de_verbosite >= 1)
             	  then 
		    begin
                      print_string (sprintf "%d essential critical pairs." 
				      (List.length crit_pairs));
                      force_newline ();
                      print_flush ()
                    end;
           	  if (options_courantes.niveau_de_verbosite >= 1)
               	  then message 
		    "----- SIMPLIFY & DELETE --------------------------------";
           	  let simpl_crit_pairs = 
		    simplify 
		    (if options_courantes.ordered_completion
		     then order
		     else (fun _ _ -> Greater))
		    crit_pairs rules in
		  if (options_courantes.niveau_de_verbosite >= 1)
		  then message 
		    "----- END of SIMPLIFY & DELETE -------------------------";
		  if options_courantes.niveau_de_verbosite >= 2
		  then 
		    begin
      	    	      print_string "set of simplified critical pairs:";
                      print_break 1 0;
	              print_list_of_equations simpl_crit_pairs;
                      force_newline ();
                      print_flush ()		  
		    end;
		  add_new_eqs simpl_crit_pairs
	  with Not_found -> ()
      	end;
      increment_minor_deduce_index ()
  with Not_found -> increment_major_deduce_index ()
;;

end;;

module Standard_deduce = 
  Deduce (Equation1) (Standard_completion) (Standard_simplifications)
    (Standard_critical_pair_criteria);;

module G_deduce = 
  Deduce (Equation_with_goal) (G_completion) (G_simplifications)
    (G_critical_pair_criteria);;
