(***************************************************************************

Equations and rules, basic definitions: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: equations.ml,v 1.3 2001/04/20 13:42:20 marche Exp $

***************************************************************************)

(*

Obsolete header:

Last modified by: $Author: marche $
File name:        $Source: /users/demons/demons/master-sources-repository//cime/ocaml_src/oldcimroot/equations.ml,v $
Revision:         $Revision: 1.3 $ 
Last modified on: $Date: 2001/04/20 13:42:20 $

***************************************************************************)

open Symbols;;
open Content;;
open Terms;;
open Orderings_generalities;;
open Args;;
open Errors;;


(*===========================
Le type regle

REGLE(n,x,l,r) represente la nieme regle l -> r, et si
x est different de not_a_symbol_id, cette regle doit etre etendue
en x+l -> x+r pour le filtrage
(ou + est le symbole de tete de l)

num est le numero de la regle

0 = regle basique
type regle = REGLE of int * symbol_id * term * term;;
===========================*)

type regle = 
    {
      num : int;
      orientation : bool;
      x   : var_id;
      y   : var_id;
      l   : term;
      r   : term;
      mutable rule_interest : i1;
      mutable rule_interest_wrt_goal : i3;
      mutable rule_lhs_interest : i2 (*;
      mutable rule_rhs_interest : int*)
    }
;;

type equation = 
    {
      mutable n             : int;
      mutable is_oriented   : bool;
      mutable interest      : int;
      mutable interest_wrt_goal : int array;
      mutable lhs_interest  : int;
      u             : term;
      v             : term;
      a             : int
    }
;;

type unif_type =
    Unif_type_AC_only 
  | Unif_type_AC_complete 
  | Unif_type_plain 
  | Unif_type_default
;;

type probleme = 
    EQUATION_A_PROUVER of term * term
  | EQUATION_A_RESOUDRE of unif_type * term * term
  | EQUATION_A_MATCHER of term * term
  | TERMES_A_COMPARER of term * term
(*
  | POLYS_A_COMPARER of polynome * polynome
*)
(*
  | EQUATION_A_RESOUDRE_MODULO_AC of term * term
  | EQUATION_LIBRE_A_RESOUDRE of term * term
*)
  | TERME_A_REDUIRE of term
(*
  |	TERMINAISON_A_PROUVER of regle list
  |	TERMINAISON_DP of regle list;;
*)

(*

  conversions vers chaines

*)

let make_equation u v = 
  {
    n = -1;
    is_oriented = false;
    interest = -1;
    interest_wrt_goal = [||];
    lhs_interest = -1;
    u=u;
    v=v;
    a= -1}
;;

let make_aged_equation a u v = 
  {
    n = -1;
    is_oriented = false;
    interest = -1;
    interest_wrt_goal = [||];
    lhs_interest = -1;
    u=u;
    v=v;
    a= a
  }
;;

let maxmin m n =
  if m > n 
  then 1024*m + n
  else 1024*n + m
;;

let size_eq u v =
  match options_courantes.size_number with
      0 -> (taille_terme_variables_legeres u)+
        (taille_terme_variables_legeres v)
    | 1 -> max (taille_terme_variables_legeres u) 
          (taille_terme_variables_legeres v)
    | 2 -> maxmin (taille_terme_variables_legeres u) 
          (taille_terme_variables_legeres v)
    | n -> 
	if n > 0
	then semantical_error "This size option is not available"
	else 
	  let n' = -n
	  in max (taille_terme_var_size n' u) (taille_terme_var_size n' v)
;;


let size_rule u _ =
  match options_courantes.size_number with
      0 -> (taille_terme_variables_legeres u)
    | 1 -> (taille_terme_variables_legeres u) 
    | 2 -> (taille_terme_variables_legeres u)
    | n -> 
	if n > 0
	then semantical_error "This size option is not available"
	else 
	  let n' = -n
	  in taille_terme_var_size n' u
;;



(*
let flatten_equation (EQUATION(num_eq,size,u,v)) =
   (EQUATION(num_eq,size,flatten_term u,flatten_term v));;
*)



(*======================================================

fabrication d'une regle : on regarde s'il y a besoin d'une extension.
il y a besoin d'une extension lorsque le symbole de tete est AC et
qu'il n'apparait pas de variable lineaire a profondeur 1

======================================================*)
     

let rec incremente_occ x = function
    [] -> [(x,true)]
  | (y,n as yn)::l -> 
      if x=y 
      then (y,false)::l
      else yn::(incremente_occ x l)
;;

let rec cherche_occurrences o = function
    VAR(x) -> (incremente_occ x o)
  | TERM(_,l)   -> List.fold_left cherche_occurrences o l
;;
  
let rec recherche_variables = function
    [] -> [],[]
  | s::l -> 
      let (v,o) = recherche_variables l 
      in
     	match s with
            VAR(x) -> 
	      if List.mem x v 
              then v,(incremente_occ x o)
              else x::v,(incremente_occ x o)
	  | _  ->  v,(cherche_occurrences o s)
;;


let rec cherche_variable_lineaire m = function
    [] -> false
  | (x::l) -> 
      if (List.assoc x m)
      then true
      else cherche_variable_lineaire m l
;;


let il_y_a_une_variable_lineaire l = 
  let vars_prof_un, occurrences = recherche_variables l 
  in
    cherche_variable_lineaire occurrences vars_prof_un
;;
     

let var_for_extension t s =
  try 
    if is_ac (top_symbol_id t)
    then 
      begin
      	let l = (liste_des_sous_termes t) 
       	in
          if il_y_a_une_variable_lineaire l
          then not_a_var_id
          else new_var (variables_de_liste_de_termes (s::l))
      end
    else not_a_var_id
  with Invalid_argument _ -> not_a_var_id
;;


let make_basic_rule (u,v) = 
  {
    num = 0;
    orientation = true;
    x = var_for_extension u v;
    y = not_a_var_id;
    l = u;
    r = v;
    rule_interest = Uncomputed;
    rule_interest_wrt_goal = Uncomputed;
    rule_lhs_interest = Uncomputed (*;
    rule_rhs_interest = -1*)
  }
;;

let make_numbered_rule order num_rule (u,v) =
  let o,left,right = 
    match order u v with
        Greater_than -> true,u,v
      | Less_than -> true,v,u
      | _ -> false,u,v in
  
  {
    num = num_rule;
    orientation = o;
    x = var_for_extension left right;
    y = if o then not_a_var_id else var_for_extension right left;
    l = left;
    r = right;
    rule_interest = Uncomputed;
    rule_interest_wrt_goal = Uncomputed;
    rule_lhs_interest = Uncomputed (*;
    rule_rhs_interest = -1*)
  }
;;

let max_num_rules = ref 0;;

let reset_rule_counter () =  
  max_num_rules := 0
;;

let make_rule order x =
  max_num_rules := !max_num_rules + 1;
  make_numbered_rule order !max_num_rules x
;;

let leftify_rule rule =
  {
    num = rule.num;
    orientation = rule.orientation;
    x = leftify_var rule.x;
    y = leftify_var rule.y;
    l = leftify_term rule.l;
    r = leftify_term rule.r;
    rule_interest = rule.rule_interest;
    rule_interest_wrt_goal = rule.rule_interest_wrt_goal;
    rule_lhs_interest = rule.rule_lhs_interest 
  }
;;

let rightify_rule rule =
  {
    num = rule.num;
    orientation = rule.orientation;
    x = rightify_var rule.x;
    y = rightify_var rule.y;
    l = rightify_term rule.l;
    r = rightify_term rule.r;
    rule_interest = rule.rule_interest;
    rule_interest_wrt_goal = rule.rule_interest_wrt_goal;
    rule_lhs_interest = rule.rule_lhs_interest (*;
    rule_rhs_interest = rule.rule_rhs_interest*)
  }
;;


let twist_rule rule = 
  if rule.orientation
  then failwith "twist_rule ne doit pas etre appele sur une regle orientee"
  else 
    {
      num = rule.num;
      orientation = false;
      x = rule.y;
      y = rule.x;
      l = rule.r;
      r = rule.l;
      rule_interest = rule.rule_interest;
      rule_interest_wrt_goal = rule.rule_interest_wrt_goal;
      rule_lhs_interest = rule.rule_lhs_interest(*;
      rule_rhs_interest = rule.rule_rhs_interest*)
    }
;;




let string_of_rule rule =
  "["^ (string_of_int rule.num)^ "] "^
  (string_of_term rule.l)^
  (if rule.orientation
   then " -> "
   else " <-> ")^
  (string_of_term rule.r)
;;



let string_of_equation e =
  (string_of_term e.u)^
  " = "^
  (string_of_term e.v)
;;



open Format;;

let print_equation e = 
  begin
    open_hovbox 0;
(*
  print_string "[max=";
  print_int e.n;
  print_string "] ";
*)
    print_term e.u;
    print_string " =";
    print_break 1 0;
    print_term e.v;
(*
  (fun(x,y)->print_break x y) (1,0);
  print_string "[";
  print_int e.interest;
  print_string ",";
  print_int e.lhs_interest;
  print_string "]";
*)
    close_box ()
  end
;;



let print_liste_d_equations =  function
    []   -> (print_string "{}")
  | e::l ->
(* 
      let rec print_fin_de_liste_d_equations n = function
	  []  -> 
	    begin
              print_string "}";
              print_break 1 0;
              print_string "(";
              print_int n;
              print_string " equations)"
            end
   	| (e::l) -> 
	    begin
              print_string  ",";
              print_break 0 0;
              print_equation e;
              print_fin_de_liste_d_equations (n+1) l
            end 
      in 
	begin 
          print_string "{";
          open_hovbox 0;
          print_equation e;
          print_fin_de_liste_d_equations 1 l;
          close_box ()
        end
*)
      print_string "{";
      open_hovbox 0;
      print_equation e;
      let nb_eq = 
	List.fold_left 
	  (fun n e' -> 
	     print_string ",";
	     print_break 0 0;
	     print_equation e';
	     succ n)
	  1 l 
      in
      print_string "}";
      print_break 1 0;
      print_string "(";
      print_int nb_eq;
      print_string " equations)"
;;


let print_regle rule =
  print_string "["; 
  print_int rule.num;
  print_string "] ";
  open_hovbox 0;
  print_term rule.l;
  if rule.orientation
  then print_string " ->"
  else print_string " <->";
  print_break 1 0;
  print_term rule.r;
(*
  (fun(x,y)->print_break x y) (1,0);
  print_string "[";
  print_int rule.rule_interest;
  print_string ",";
  print_int rule.rule_lhs_interest;
  print_string "]";
*)
  close_box ()
;;





let print_liste_de_regles = function
   []   -> (print_string "{}")
 | r::l ->
   let rec print_fin_de_liste_de_regles n = function
       [] -> 
	 begin
           print_string " }";
           print_break 1 0;
           print_string "(";
           print_int n;
           print_string " rules)"                 
	 end
     | (r::l) -> 
	 begin
           print_string  ",";
           force_newline ();
           print_regle r;
           print_fin_de_liste_de_regles (n+1) l
         end
   in 
     begin
       print_string "{ ";
       open_hovbox 0;
       print_regle r;
       print_fin_de_liste_de_regles 1 l;
       close_box ()
     end
;;


