(***************************************************************************

  Equations and rules, basic definitions: specifications

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: equations.mli,v 1.3 2001/04/20 13:42:20 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  Last modified by: $Author: marche $
  File name:        equations.mli
  Revision:         $Revision: 1.3 $ 
  Last modified on: $Date: 2001/04/20 13:42:20 $

***************************************************************************)


open Symbols;;
open Content;;
open Terms;;
open Orderings_generalities;;

(*===========================
Le type regle

REGLE(n,x,l,r) represente la nieme regle l -> r, et si
x est different de not_a_symbol_id, cette regle doit etre etendue
en x+l -> x+r pour le filtrage
(ou + est le symbole de tete de l)

num est le numero de la regle

0 = regle basique
type regle = REGLE of int * symbol_id * term * term;;
===========================*)

type regle = 
    {
      num : int;
      orientation : bool;
      x   : var_id;
      y   : var_id;
      l   : term;
      r   : term;
      mutable rule_interest : i1;
      mutable rule_interest_wrt_goal : i3;
      mutable rule_lhs_interest : i2 
    }
;;

val reset_rule_counter : unit -> unit;;

(*==============================

Le type equation est un enregistrement
{n,t1,t2,u,v,a,i} qui represente l'equation u=v et
n est un entier tel que u et v sont irreductibles
pour toutes les regles de numero <= n

Si n vaut -1, u et v n'on pas encore ete reecrit du tout
Si n vaut 0, numero des regles de S, u et v sont S-normalises

t1 est la taille de l'equation
t2 est la taille du membre gauche si deja pre-orientee (-1 sinon)
a est l'age de l'equation
i est l'interet de l'equation
===============================*)


type equation = 
    {
      mutable n             : int;
      mutable is_oriented   : bool;
      mutable interest      : int;
      mutable interest_wrt_goal : int array;
      mutable lhs_interest  : int;
      u             : term;
      v             : term;
      a             : int
    }
;;

val size_eq : term -> term -> int;;

val size_rule : term -> term -> int;;

(*====================

(make_rule (l,r))

Fabrique une regle a partir du membre gauche l
et du membre droit r, en calculant si la regle doit
etre etendue ou non.

=====================*)

val make_numbered_rule : term ordering -> int -> term * term -> regle;;

val make_rule : term ordering -> term * term -> regle;;

val make_basic_rule : term * term -> regle;;

val leftify_rule : regle -> regle;;

val rightify_rule : regle -> regle;;

val twist_rule : regle -> regle;;

(*========================================

Fonction d'aplatissement.

========================================*)
(*
val flatten_equation : equation -> equation;;
*)



(*====================

Ensemble des types de problemes a resoudre

====================*)

type unif_type =
  Unif_type_AC_only 
| Unif_type_AC_complete 
| Unif_type_plain 
| Unif_type_default
;;

type probleme = EQUATION_A_PROUVER of term * term
              | EQUATION_A_RESOUDRE of unif_type * term * term
              | EQUATION_A_MATCHER of term * term
	      | TERMES_A_COMPARER of term * term
(*
	      | POLYS_A_COMPARER of polynome * polynome
*)
(*
              | EQUATION_A_RESOUDRE_MODULO_AC of term * term
              | EQUATION_LIBRE_A_RESOUDRE of term * term
*)
              | TERME_A_REDUIRE of term
(*
	      |	TERMINAISON_A_PROUVER of regle list
	      |	TERMINAISON_DP of regle list;;
*)

(*

  conversions vers chaines

*)

val string_of_rule : regle -> string;;

(*
val string_of_equation : equation -> string;;
*)

(*

  affichages dans des boites

*)

val print_regle : regle -> unit;;

val print_liste_de_regles : regle list -> unit;;


