(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: equations_as_obj.ml,v 1.2 2001/04/20 13:42:20 marche Exp $

***************************************************************************)

(***************************************************************************
  
  Several types of equations for completion.
  
  CiME Project - Dmons research team - LRI - Universit Paris XI
  
  $Id: equations_as_obj.ml,v 1.2 2001/04/20 13:42:20 marche Exp $
  
***************************************************************************)

open Orderings
open Content
open Positions
open Symbols
open Terms
open Substitution
open Equations
open Matching
open Encompassment
open Dnet
open Args
open Format

(*
  EQUATION is the type of the modules of equations used by the completion
*)

module type EQUATION =
sig
  type t 
  val interest1_of : t -> i1 
  val interest2_of : t -> i2
  val interest3_of : t -> i3
  val set_interest1 : i1 -> t -> unit
  val set_interest2 : i2 -> t -> unit
  val set_interest3 : i3 -> t -> unit
  val compare_equation : t ordering
  val first_term : t -> term
  val second_term : t -> term
  val set_first_term : term -> t -> unit
  val set_second_term : term -> t -> unit
  val make_equation : term -> term -> t
  val make_aged_equation : int -> term -> term -> t
  val is_oriented : t -> bool
  val set_orientation : bool -> t -> unit
  val reset_orientation : t -> unit
  val get_age : t -> int
  val set_age : int -> t -> unit
  val get_norm_wrt_rule : t -> int
  val set_norm_wrt_rule : int -> t -> unit
  val reset_interest : t -> unit
  val print_interest : i1 -> i2 -> i3 -> unit
  val print_equation : t -> unit
  val print_list_of_equations : t list -> unit
  val is_a_trivial_pair_of_terms : 
    int * regle_etendue dnet -> term * term -> bool
end

(*
  EQUATION_SET is the type of the modules of sets of equations 
  used by the completion.
*)

module type EQUATION_SET =
sig
  type equation
  type eq_set
  exception No_more_equation
  val make_empty_equation_set : unit -> eq_set
  val next_equation : eq_set -> equation
  val insert_equation : equation -> eq_set -> unit
  val carefully_insert_equation : equation -> eq_set -> unit
  val is_empty : eq_set -> bool
  val card : eq_set -> int
  val min_interest1 : eq_set -> i1
  val min_interest2 : eq_set -> i2
  val min_interest3 : eq_set -> i3
  val set_min_interest1 : i1 -> eq_set -> unit
  val set_min_interest2 : i2 -> eq_set -> unit
  val set_min_interest3 : i3 -> eq_set -> unit
  val print_min_interest : eq_set -> unit
  val print_equation_set : eq_set -> unit
  val fold_left : ('a -> equation -> 'a) -> 'a -> eq_set -> 'a
  val fold_right : (equation -> 'a -> 'a) -> eq_set -> 'a -> 'a
  val iter : (equation -> unit) -> eq_set -> unit
  val find : (equation -> bool) -> eq_set -> equation
  val set_to_list : eq_set -> equation list
end
  
(*
  Equation_set is a functor which builds a module of set of equations from
  a module of equations.
*)

module Equation_set = functor (Eqt : EQUATION) -> 
struct
  
open Eqt
    
type equation = t
type eq_set = 
    { 
      mutable e : t list;
      mutable i1 : i1;
      mutable i2 : i2;
      mutable i3 : i3
    }

exception No_more_equation
  
(* 
   When a set is empty, the minimal interests of its equations are
   maximal.
*)
  
let make_empty_equation_set () = 
  { 
    e = []; 
    i1 = Max; 
    i2 = Max; 
    i3 = Max
  }
  
  
let next_equation eq_set =
  try 
    let eq = List.hd eq_set.e in
      eq_set.e <- List.tl eq_set.e;
      begin
  	try
	  let eq' = List.hd eq_set.e in
	    eq_set.i1 <- interest1_of eq';
	    eq_set.i2 <- interest2_of eq';
	    eq_set.i3 <- interest3_of eq'
      	with 
	    Failure "hd" ->
	      begin
		eq_set.i1 <- Max;
		eq_set.i2 <- Max;
		eq_set.i3 <- Max;
	      end
      end;
      eq
  with 
      Failure "hd" -> raise No_more_equation
	  
	  
	  
let insert_equation eq eq_set =
  let u = first_term eq
  and v = second_term eq in
    
  let rec insert_rec = function
      [] -> [eq]
    | eq'::l as l' ->
       	begin
	  match compare_equation eq eq' with
	      Less_than -> eq::l'
	    | _ ->
		let u' = first_term eq'
		and v' = second_term eq' in
		  if ((egaux_ac u u') & (egaux_ac v v')) or
		    ((egaux_ac u v') & (egaux_ac v u'))
		  then l'
		  else eq'::(insert_rec l)
       	end
  in
    eq_set.e <- insert_rec eq_set.e;
    eq_set.i1 <- interest1_of (List.hd eq_set.e);
    eq_set.i2 <- interest2_of (List.hd eq_set.e);
    eq_set.i3 <- interest3_of (List.hd eq_set.e)


let carefully_insert_equation eq eq_set =
  let u = first_term eq
  and v = second_term eq in
    
  let rec enlever_entourage = 
    let eq_as_rule = make_basic_rule (u,v) in
    let dnet_for_eq = 
      add_rule_to_dnet RIGHT eq_as_rule (create_dnet LEFT eq_as_rule) in 
      function
	  [] -> []
      	| eq'::l ->
      	    if is_a_trivial_pair_of_terms dnet_for_eq 
	      (first_term eq',second_term eq')
      	    then (enlever_entourage l)
      	    else eq'::(enlever_entourage l)
	      
	      
  and insert_rec = function
      [] -> [eq]
    | eq'::l as l' ->
       	begin
	  match compare_equation eq eq' with
	      Less_than -> 
		let l'' = enlever_entourage l' in
		  eq::l''
	    | _ ->
		let u' = first_term eq'
		and v' = second_term eq' in
		  if est_un_entourage_de (u',v') (u,v)
		  then l'
		  else 
		    begin
(*
  if ((size_eq u v) <= (size_eq u' v')) 
  & (est_un_entourage_de (u,v) (u',v'))
  then insert_rec l
  else 
*) 
		      eq'::(insert_rec l)
		    end
       	end
  in
    eq_set.e <- insert_rec eq_set.e;
    eq_set.i1 <- interest1_of (List.hd eq_set.e);
    eq_set.i2 <- interest2_of (List.hd eq_set.e);
    eq_set.i3 <- interest3_of (List.hd eq_set.e)
      
      
let is_empty eq_set = eq_set.e = []
				   
let card eq_set = List.length eq_set.e
		    
let min_interest1 eq_set = eq_set.i1
			     
let min_interest2 eq_set = eq_set.i2
			     
let min_interest3 eq_set = eq_set.i3
			     
let set_min_interest1 i1 eq_set =
  eq_set.i1 <- i1
    
let set_min_interest2 i2 eq_set =
  eq_set.i2 <- i2
    
let set_min_interest3 i3 eq_set =
  eq_set.i3 <- i3
    
let print_min_interest eq_set =
  print_interest eq_set.i1 eq_set.i2 eq_set.i3
    
let print_equation_set eq_set = print_list_of_equations eq_set.e
				  
let fold_left f a eq_set = List.fold_left f a eq_set.e
			     
let fold_right f eq_set a = List.fold_right f eq_set.e a
			      
let iter f eq_set = List.iter f eq_set.e
		      
let find p eq_set = 
  let rec find_list = function
      h::t -> if (p h) then h else (find_list t)
    | [] -> raise Not_found
  in
    find_list eq_set.e
      
      
let set_to_list eq_set = eq_set.e
			   
end
  
  
class pair_of_terms (t1 : term) (t2 : term) =
  object
  val mutable term1 = t1;
  val mutable term2 = t2;	     
  method first_term = term1;
  method second_term = term2;
  method set_first_term s = term1 <- s;
  method set_second_term s = term2 <- s;
  method print = 
    begin
      open_hovbox 0;
      print_term term1;
      print_string " =";
      print_break 1 0;
      print_term term2;
      close_box ()
    end
end
  
  
class equation_to_process (t1 : term) (t2 : term) = 
  object
  inherit pair_of_terms t1 t2
  val mutable norm_wrt_rule = -1;
  val mutable age = -1;
  val mutable interest = -1;
  method set_norm_wrt_rule n = norm_wrt_rule <- n;
  method get_norm_wrt_rule = norm_wrt_rule;
  method set_age a = age <- a;
  method get_age = age;
  method get_interest = interest;
  method set_interest i = interest <- i;  
  method reset_interest =
    begin
      interest <- -1;
    end;
  method reinit =
    begin
      norm_wrt_rule <- -1;
      age <- -1;
      interest <- -1;
    end
end 
  
class equation_as_obj (t1 : term) (t2 : term) = 
  object
  inherit pair_of_terms t1 t2
  val mutable norm_wrt_rule = -1;
  val mutable age = -1;
  val mutable orientation = false;
  val mutable interest1 = (Uncomputed : i1);
  val mutable interest2 = (Uncomputed : i2);
  method set_norm_wrt_rule n = norm_wrt_rule <- n;
  method get_norm_wrt_rule = norm_wrt_rule;
  method set_age a = age <- a;
  method get_age = age;
  method set_orientation o = orientation <- o;
  method is_oriented = orientation;
  method get_interest1 = interest1;
  method set_interest1 i = interest1 <- i;  
  method get_interest2 = interest2;
  method set_interest2 i = interest2 <- i;  
  method reset_interest =
    begin
      match interest1 with
	  Some _ -> interest1 <- Uncomputed
	| _ -> ();
      match interest2 with
	  Some _ -> interest2 <- Uncomputed
	| _ -> ()
    end;
  method reset_orientation = orientation <- false;
  method reinit =
    begin
      norm_wrt_rule <- -1;
      age <- -1;
      orientation <- false;
      match interest1 with
	  Some _ -> interest1 <- Uncomputed
	| _ -> ();
      match interest2 with
	  Some _ -> interest2 <- Uncomputed
	| _ -> ()
    end
end 
  
  
class equation_g (t1 : term) (t2 : term) =
  object
  inherit equation_as_obj t1 t2
  val mutable interest3 = (Uncomputed : i3)
  method get_interest3 = interest3;
  method set_interest3 i = interest3 <- i;
  method reset_interest =
      begin
	match interest1 with
	    Some _ -> interest1 <- Uncomputed
	  | _ -> ();
	match interest2 with
	    Some _ -> interest2 <- Uncomputed
	  | _ -> ();
        match interest3 with
	    Some _ -> interest3 <- Uncomputed
	  | _ -> ()
      end;
  method reinit =
    begin
      norm_wrt_rule <- -1;
      age <- -1;
      orientation <- false;
      match interest1 with
	  Some _ -> interest1 <- Uncomputed
	| _ -> ();
      match interest2 with
	  Some _ -> interest2 <- Uncomputed
	| _ -> ();
      match interest3 with
	  Some _ -> interest3 <- Uncomputed
	| _ -> ()
    end
end


class conj (t1 : term) (t2 : term) =
  object
  inherit pair_of_terms t1 t2
  val mutable norm_wrt_rule = -1;
  val mutable subst = [build_subst []];
  method get_norm_wrt_rule = norm_wrt_rule;
  method set_norm_wrt_rule n = norm_wrt_rule <- n
end


let make_equation_as_obj t1 t2 =
  new equation_as_obj t1 t2
    
    
let make_aged_equation_as_obj a t1 t2 =
  let e = new equation_as_obj t1 t2
  in
    e#set_age a;
    e
      
      
let print_list_of_pairs_of_terms = function
    [] -> print_string "{}"
  | e::l ->
      print_string "{";
      open_hovbox 0;
      e#print;
      let nb_eq = 
	List.fold_left 
	  (fun n e' -> 
	     print_string ",";
	     print_break 0 0;
	     e'#print;
	     succ n)
	  1 l 
      in
	print_string "}";
	print_break 1 0;
	print_string "(";
	print_int nb_eq;
	print_string " equations)"



let est_instance_d_une_regle_etendue_a_la_position p (u,v) u_p re =
  match re.o with
      LEFT ->
	begin
	  let g = re.regle.l
	  and d = re.regle.r in
	    try
	      let theta = filtrage_ac u_p g in
		egaux_ac v (replace u (substitute_assoc theta d) p)
	    with No_match -> 
	      let x = re.regle.x in
		if is_a_var_id x 
		then 
		  begin
		    let f = top_symbol_id g
		    and lg = liste_des_sous_termes g in
		      try
			let theta = filtrage_ac u_p 
				      (TERM(f,(VAR x)::lg)) in
			  egaux_ac v
			    (replace u (substitute_assoc theta 
					  (add_left_var f x d)) p)
		      with No_match -> false
		  end
		else false
	end
    | RIGHT ->
	begin
	  let g = re.regle.r
	  and d = re.regle.l in
	    try
	      let theta = filtrage_ac u_p g in
		egaux_ac v (replace u (substitute_assoc theta d) p)
	    with No_match -> 
	      let x = re.regle.y in
		if is_a_var_id x 
		then 
		  begin
		    let f = top_symbol_id g
		    and lg = liste_des_sous_termes g in
		      try
			let theta = filtrage_ac u_p 
				      (TERM(f,(VAR x)::lg)) in
			  egaux_ac v
			    (replace u (substitute_assoc theta 
					  (add_left_var f x d)) p)
		      with No_match -> false
		  end
		else false
	end


let rec est_instance_d_un_systeme_de_regles_etendues_a_la_position p u_v u_p = 
  function
      [] -> false
    | r::rule_set ->
      if r.regle.orientation
      then 
	est_instance_d_un_systeme_de_regles_etendues_a_la_position 
	  p u_v u_p rule_set
      else 
	if est_instance_d_une_regle_etendue_a_la_position p u_v u_p r
	then true
	else 
	  est_instance_d_un_systeme_de_regles_etendues_a_la_position 
	    p u_v u_p rule_set


let is_a_trivial_pair_of_terms_pos p (u,v as u_v) dnet =
  let u' = sous_terme_a_la_position u p in
  let liste_des_regles_possibles = find_rules u' dnet in
    est_instance_d_un_systeme_de_regles_etendues_a_la_position 
      p u_v u' liste_des_regles_possibles

  
module Equation1 =
  struct
    type t = equation_as_obj
    let interest1_of (e : t) = e#get_interest1
    let interest2_of (e : t) = e#get_interest2
    let interest3_of _ = No_value
    let set_interest1 i (e : t) = e#set_interest1 i
    let set_interest2 i (e : t) = e#set_interest2 i
    let set_interest3 _ _ = ()
    let compare_equation =
      extension_lexico_ordres
	[(*(function (e,e') -> 
	    if (e#get_age + 70) < e'#get_age
	    then Less_than
	    else 
	      if (e'#get_age + 70) < e#get_age
	      then Greater
	      else Equivalent);*)
	 (fun e e' -> compare_i2 (interest2_of e) (interest2_of e'));
	 (fun e e' -> compare_i1 (interest1_of e) (interest1_of e'))]
    let first_term (e : t) = e#first_term
    let second_term (e : t) = e#second_term
    let set_first_term s (e :t) = e#set_first_term s
    let set_second_term s (e : t) = e#set_second_term s
    let is_oriented (e : t) = e#is_oriented
    let set_orientation o (e : t) = e#set_orientation o
    let reset_orientation (e : t) = e#reset_orientation
    let get_age (e : t) = e#get_age
    let set_age a (e : t) = e#set_age a
    let reset_interest (e : t) = e#reset_interest
    let get_norm_wrt_rule (e : t) = e#get_norm_wrt_rule
    let set_norm_wrt_rule n (e : t) = e#set_norm_wrt_rule n
    let make_equation t1 t2 = ((new equation_as_obj t1 t2) : t)
    let make_aged_equation a t1 t2 = 
      let (e : t) = new equation_as_obj t1 t2 in
	e#set_age a;
	e
    let print_interest i1 i2 _ =
      let print_aux = function
	  Max -> print_string "Max"
	| No_value -> print_string "_"
	| Uncomputed -> print_string "Uncomputed"
	| Some i -> print_int i 
      in
      	print_aux i1;
	print_string ",";
	print_aux i2
    let print_equation (e : t) = e#print;
      print_string " (age = ";
      print_int e#get_age;
      print_string ", interest = ";
	print_interest (interest1_of e) (interest2_of e) No_value;
      print_string ")"
    let print_list_of_equations = function
    	[] -> print_string "{}"
      | e::l ->
	  print_string "{";
	  open_hovbox 0;
	  print_equation e;
	  let nb_eq = 
	    List.fold_left 
	      (fun n e' -> 
		 print_string ",";
		 print_break 0 0;
		 print_equation e';
		 succ n)
	      1 l 
	  in
	    print_string "}";
	    print_break 1 0;
	    print_string "(";
	    print_int nb_eq;
	    print_string " equations)"
    let is_a_trivial_pair_of_terms d (u,v as e) =
      let rec is_a_trivial_pair pos =
	if is_a_trivial_pair_of_terms_pos pos e d
	then true
	else 
	  try
	    let pos' = next_pos_p u pos in
	      is_a_trivial_pair_of_terms_pos pos' e d
	  with No_more_pos -> false 
      in
	if options_courantes.ordered_completion
	then (egaux_ac u v) or (is_a_trivial_pair Top)
	else egaux_ac u v
  end

module Equation_set1 = Equation_set (Equation1)

module Equation_with_goal =
  struct
    type t = equation_g
    let interest1_of (e : t) = e#get_interest1
    let interest2_of (e : t) = e#get_interest2
    let interest3_of (e : t) = e#get_interest3
    let set_interest1 i (e : t) = e#set_interest1 i
    let set_interest2 i (e : t) = e#set_interest2 i
    let set_interest3 i (e : t) = e#set_interest3 i
    let compare_equation =
      extension_lexico_ordres
	[(fun e e' -> compare_i3 (interest3_of e) (interest3_of e'));
	 (fun e e' -> compare_i2 (interest2_of e) (interest2_of e'));
	 (fun e e' -> compare_i1 (interest1_of e) (interest1_of e'))]
    let first_term (e : t) = e#first_term
    let second_term (e : t) = e#second_term
    let set_first_term s (e :t) = e#set_first_term s
    let set_second_term s (e : t) = e#set_second_term s
    let is_oriented (e : t) = e#is_oriented
    let set_orientation o (e : t) = e#set_orientation o
    let reset_orientation (e : t) = e#reset_orientation
    let get_age (e : t) = e#get_age
    let set_age a (e : t) = e#set_age a
    let reset_interest (e : t) = e#reset_interest
    let get_norm_wrt_rule (e : t)= e#get_norm_wrt_rule
    let set_norm_wrt_rule n (e : t) = e#set_norm_wrt_rule n
    let make_equation t1 t2 = ((new equation_g t1 t2) : t)
    let make_aged_equation a t1 t2 = 
      let (e : t) = new equation_g t1 t2 in
	e#set_age a;
	e
    let print_interest i1 i2 i3 =
      let print_aux print = function
	  Max -> print_string "Max"
	| No_value -> print_string "_"
	| Uncomputed -> print_string "_"
	| Some i -> print i 
      in
      	print_aux print_int i1;
	print_string ",";
	print_aux print_int i2;
	print_string ",";
	print_aux 
	  (function vi ->
	     let l = Array.length vi in
	     if l = 0
	     then print_string "()"
	     else 
	       begin
		 print_string "(";
		 print_int vi.(0);
		 for j=1 to (pred l) do
		   print_string ",";
		   print_int vi.(j)
		 done;
		 print_string ")"
	       end) 
	     i3

    let print_equation (e : t) = 
      e#print;
      print_string " (age = ";
      print_int e#get_age;
      print_string ", interest = ";
      print_interest (interest1_of e) (interest2_of e) (interest3_of e);
      print_string ")"
    let print_list_of_equations = function
    	[] -> print_string "{}"
      | e::l ->
	  print_string "{";
	  open_hovbox 0;
	  print_equation e;
	  let nb_eq = 
	    List.fold_left 
	      (fun n e' -> 
		 print_string ",";
		 print_break 0 0;
		 print_equation e';
		 succ n)
	      1 l 
	  in
	    print_string "}";
	    print_break 1 0;
	    print_string "(";
	    print_int nb_eq;
	    print_string " equations)"
    let is_a_trivial_pair_of_terms d (u,v as e) =
      let rec is_a_trivial_pair pos =
	if is_a_trivial_pair_of_terms_pos pos e d
	then true
	else 
	  try
	    let pos' = next_pos_p u pos in
	      is_a_trivial_pair_of_terms_pos pos' e d
	  with No_more_pos -> false 
      in
	if options_courantes.ordered_completion
	then (egaux_ac u v) or (is_a_trivial_pair Top)
	else egaux_ac u v
  end


module Equation_with_goal_set = Equation_set (Equation_with_goal)

module Conjectures =
  struct
    type t = conj
    let interest1_of _ = No_value
    let interest2_of _ = No_value
    let interest3_of _ = No_value
    let set_interest1 _ _ = ()
    let set_interest2 _ _ = ()
    let set_interest3 _ _ = ()
    let compare_equation _ _ = Equivalent
    let first_term (e : t) = e#first_term
    let second_term (e : t) = e#second_term
    let set_first_term s (e : t) = e#set_first_term s
    let set_second_term s (e : t) = e#set_second_term s
    let make_equation s1 s2 = new conj s1 s2
    let make_aged_equation _ s1 s2 = new conj s1 s2
    let is_oriented _ = false
    let set_orientation _ _ = ()
    let reset_orientation _ = ()
    let get_age _ = failwith "A conjecture does not possess an age"
    let set_age _ _ = ()
    let get_norm_wrt_rule (e : t) = e#get_norm_wrt_rule
    let set_norm_wrt_rule n (e : t) = e#set_norm_wrt_rule n
    let reset_interest _ = ()
    let print_interest _ _ _ = ()
    let print_equation (e : t) = e#print
    let print_list_of_equations (l : t list) =
      print_list_of_pairs_of_terms l
    let is_a_trivial_pair_of_terms _ (u,v) = egaux_ac u v
  end


module Conjectures_set = Equation_set (Conjectures)

module Rules_set =
  struct
    type equation = regle
    type eq_set =
	{
	  mutable e : regle list;
	  mutable i1 : i1;
	  mutable i2 : i2;
	  mutable i3 : i3}
	
    exception No_more_equation

    let make_empty_equation_set () =
      {
	e = [];
	i1 = Max;
	i2 = Max;
	i3 = Max}

    let next_equation rule_set =
      failwith "next_equation should not be called on a rule set"

    let insert_equation rule rule_set =
      rule_set.e <- rule_set.e @ [rule]
    
    let carefully_insert_equation rule rule_set =
      rule_set.e <- rule_set.e @ [rule]
    
    let is_empty rule_set =
      rule_set.e = []

    let card rule_set = List.length rule_set.e
    
    let min_interest1 rule_set = rule_set.i1    
    let min_interest2 rule_set = rule_set.i2
    let min_interest3 rule_set = rule_set.i3

    let set_min_interest1 i1 rule_set =
      rule_set.i1 <- i1

    let set_min_interest2 i2 rule_set =
      rule_set.i2 <- i2

    let set_min_interest3 i3 rule_set =
      rule_set.i3 <- i3

    let print_min_interest rule_set =
      let print_interest i1 i2 i3 =
      	let print_aux print = function
	    Max -> print_string "Max"
	  | No_value -> print_string "_"
	  | Uncomputed -> print_string "_"
	  | Some i -> print i 
      	in
      	  print_aux print_int i1;
	  print_string ",";
	  print_aux print_int i2;
	  print_string ",";
	  print_aux 
	    (function vi ->
	       let l = Array.length vi in
		 if l = 0
		 then print_string "()"
		 else 
		   begin
		     print_string "(";
		     print_int vi.(0);
		     for j=1 to (pred l) do
		       print_string ",";
		       print_int vi.(j)
		     done;
		     print_string ")"
		   end)
	    i3 in
	print_interest rule_set.i1 rule_set.i2 rule_set.i3
    
    let print_equation_set rule_set = print_liste_de_regles rule_set.e
    
    let fold_left f a rule_set = List.fold_left f a rule_set.e
    
    let fold_right f rule_set a = List.fold_right f rule_set.e a

    let iter f rule_set = List.iter f rule_set.e

    let find p rule_set = 
      let rec find_list = function
	  h::t -> if (p h) then h else (find_list t)
    	| [] -> raise Not_found
      in
    	find_list rule_set.e

    let set_to_list rule_set = rule_set.e
  
end
