(***************************************************************************

(This sentence has been added automatically, it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: llist.ml,v 1.1 2000/05/16 17:36:43 monate Exp $

***************************************************************************)


type 'a frozen = 
   Freeze of (unit -> 'a)
 | Val of 'a;;

type 'a llist =
   Nil
 | Cons of 'a cell

and 'a cell = 
    {         tete : 'a;
      mutable queue : 'a llist frozen
    }
;;

let hdl = function
  Nil -> failwith "hdl"
| Cons {tete = x; queue = _ } -> x
;;

let tll = function
  Nil -> failwith "tll"
| Cons {tete = _ ; queue = Val l} -> l
| Cons ({tete = _ ; queue = Freeze th} as cell) ->
      let tail = th () in 
      cell.queue <- Val tail;
      tail
;;
     

let rec lmem x = function
    Nil -> false
  | Cons {tete = h; queue = Val t} -> 
      if x=h
      then true
      else lmem x t
  | Cons ({tete = h; queue = Freeze ft} as cell) ->
      if x=h
      then true
      else 
	begin
	  let t = ft () in
	  cell.queue <- Val t;
	  lmem x t
	end
;;

let rec mapl f = function
  Nil -> Nil
| Cons {tete = x; queue = Val l} -> 
      Cons {tete = f x; queue = Freeze (function () -> mapl f l)}
| Cons ({tete = x; queue = Freeze th} as cell) ->
      let thunk () =
      let tail = th () in 
      cell.queue <- Val tail;
      mapl f tail in
      Cons {tete = f x; queue = Freeze thunk}
;;

let rec do_llist f n = function
  Nil -> ()
| Cons {tete = x; queue = Val l} ->
      if (n>0) 
      then begin 
           f x;
           do_llist f (n-1) l
           end
| Cons ({tete = x; queue = Freeze th} as cell) -> 
      if (n>0) 
      then begin
           f x;
           let tail = th () in
           cell.queue <- Val tail;
           do_llist f (n-1) tail 
           end
;;

let rec do_all_llist f = function
  Nil -> ()
| Cons {tete = x; queue = Val l} ->
           f x;
           do_all_llist f l
| Cons ({tete = x; queue = Freeze th} as cell) -> 
           f x;
           let tail = th () in
           cell.queue <- Val tail;
           do_all_llist f tail 
;;

let rec appendl x y = match (x,y) with
  Nil, (Val l) -> l
| Nil, (Freeze th) -> th ()
| (Cons {tete = x; queue = Val l}),  ll ->
        Cons {tete = x; queue = Freeze (function () -> appendl l ll)}
| (Cons ({tete = x; queue = Freeze th} as cell)), ll -> 
      let thunk () =
      let tail = th () in 
      cell.queue <- Val tail;
      appendl tail ll in
      Cons {tete = x; queue = Freeze thunk}
;;

let rec flat_mapl f = function
  Nil -> Nil
| Cons {tete = x; queue = Val l} -> 
      appendl (f x) (Freeze (function () -> flat_mapl f l))
| Cons ({tete = x; queue = Freeze th} as cell) ->
      let thunk () =
      let tail = th () in 
      cell.queue <- Val tail;
      flat_mapl f tail in
      appendl (f x) (Freeze thunk)
;;

let rec flat_map2 f = function
  [] -> Nil
| x::l -> appendl (f x) (Freeze (function () -> flat_map2 f l))
;;

let rec list_to_llist = function
    [] -> Nil
  | h::t -> Cons {tete = h; queue = Val (list_to_llist t)}
;; 

let rec llist_to_list = function
    Nil -> []
  | Cons {tete = h; queue = Val t} -> h::(llist_to_list t)
  | Cons ({tete = h; queue = Freeze ft} as cell) ->
      let t = ft () in
      cell.queue <- Val t;
      h::(llist_to_list t)
;; 

let rec lazy_exists p = function
    Nil -> false
  | Cons {tete = h; queue = Val t} -> 
      if (p h) 
      then true
      else lazy_exists p t
  | Cons ({tete = h; queue = Freeze ft} as cell) ->
      if (p h)
      then true
      else 
	begin
	  let t = ft () in
	  cell.queue <- Val t;
	  lazy_exists p t
	end
;;

let rec lazy_assoc x = function
    Nil -> raise Not_found
  | Cons {tete = (y,z); queue = Val t} ->
      if (x=y)
      then z
      else lazy_assoc x t
  | Cons ({tete = (y,z); queue = Freeze ft} as cell) ->
      if (x=y)
      then z
      else 
	begin 
        let t = ft () in
      	cell.queue <- Val t;
	lazy_assoc x t
	end
;;


let rec llist_it f = function
    Nil -> (function b -> b)
  | Cons {tete = h; queue = Val t} ->
      ( function b ->
      	let b' = llist_it f t b in
      	f h b')
  | (Cons ({tete = h; queue = Freeze ft} as cell)) ->
      ( function b -> 
      	let t = ft () in
      	cell.queue <- Val t;
      	let b' = llist_it f t b in
      	f h b')
;;

let rec llist_length = function
    Nil -> 0
  | Cons {tete = h; queue = Val t} ->
      succ (llist_length t)
  | Cons ({tete = h; queue = Freeze ft} as cell) ->
      let t = ft () in
      cell.queue <- Val t;
      succ (llist_length t)
;;
      
let rec llist_copy l = l;;
