(***************************************************************************

  Main Program
  Created on February 7, 93

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: main.ml,v 1.2 2001/04/20 13:42:21 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: main.ml,v 1.2 2001/04/20 13:42:21 marche Exp $


***************************************************************************)

open Specif;;
open Tree_automata;;
open Types;;

open Format;;
open Abstract_syntax;;
open Norm_theory;;
open Orderings;;
open Term_orderings;;
open Total_AC_ordering;;
open Completion_state;;
open Strategy;;
open Goal_oriented_strategy;;
open Symbols;;
open Terms;;
open Equations;;
open Equations_as_obj;;
open Normalised_rewriting;;
open Messages;;
open Args;;
open Matching;;
open Unification;;
open Theorie;;
open Errors;;
open Dependency_pairs;;
open Induction;;
open Substitution;;
open Dnet;;

module Counter = Tree_automata.Make(struct type symbol = symbol_id type term = Terms.term  type state = Types.state type sorte = Symbols.sorte list end)

(*==========

Reading of specification file

===========*)

let show_error s =
  print_string 
    (sprintf "%s error line %d columns %d-%d "
       s
       etat_analyse.numero_ligne
       etat_analyse.numero_colonne_avant
       etat_analyse.numero_colonne_apres);
  if (etat_analyse.dernier_token_lu <> "")
  then 
    print_string 
      (sprintf " (near token '%s')\n" etat_analyse.dernier_token_lu) 
  else force_newline ()
;;

let read_specifications () =
  let in_file =
    try open_in options_courantes.nom_fichier
    with Sys_error s ->
      try open_in (options_courantes.nom_fichier ^ ".cim")
      with Sys_error s ->
        try open_in (options_courantes.nom_fichier ^ ".nc")
        with Sys_error s -> 
          raise (Arg.Bad (sprintf "Could not open file %s: %s" options_courantes.nom_fichier s))
  in
  let lexbuf = Lexing.from_channel in_file in
    try
      let rec collect_data = function
      	  [] -> ([],[],[],ORDRE_INTERACTIF,[],[],[])
      	| s::l -> 
      	    let (th,uth,ax,o,pbs,conjs,induc) = collect_data l in
      	      match s with
		  Abstract_sortes _ -> (th,uth,ax,o,pbs,conjs,induc)
	      	| Abstract_subsortes l -> 
		    store_subsort_table l;
		    (th,uth,ax,o,pbs,conjs,induc)
	      	| Abstract_operators op -> (th,uth,ax,o,pbs,conjs,induc)
	      	| Abstract_theory tail -> 
		    (th @ (List.map theory_of_abstract_theory tail),
		     uth,ax,o,pbs,conjs,induc)  
	      	| Abstract_unification_theory tail -> 
		    (th,uth @ (List.map theory_of_abstract_theory tail),
		     ax,o,pbs,conjs,induc)  
	      	| Abstract_axioms al -> (th,uth,ax @ al,o,pbs,conjs,induc)  
(*
                | Abstract_rules rul ->(th,uth,ax,o,pbs,conjs,induc)  
*)
	      	| Abstract_ordering ord -> (th,uth,ax,ord,pbs,conjs,induc)  
	      	| Abstract_problems pl -> (th,uth,ax,o,pbs @ pl,conjs,induc)  
	      	| Abstract_conjectures pl -> 
		    (th,uth,ax,o,pbs,conjs @ pl,induc)  
	      	| Abstract_unification_type t ->
                    begin
		      match t with
		    	  Unif_type_AC_only -> 
			    select_ac_only_unification() 
      	       	      	| Unif_type_AC_complete -> 
			    select_ac_complete_unification()
      	       	      	| Unif_type_plain -> 
			    select_plain_unification()
      	       	      	| _ -> ()
		    end;  
		    (th,uth,ax,o,pbs,conjs,induc)
	      	| Abstract_include file_name -> (th,uth,ax,o,pbs,conjs,induc)
	      	| Abstract_inductive pl ->(th,uth,ax,o,pbs,conjs,induc @ pl)
 		      
		      
      in collect_data (Parser.donnees Lexer.token lexbuf) 
	   
    with
      	Erreur_de_syntaxe s ->
          show_error "Syntax";
          print_string s;
          force_newline ();
          abort()
      | Semantical_error s ->
          show_error "Semantical";
          print_string s;
          force_newline ();
          abort()
      | Parsing.Parse_error ->
          show_error "Syntax";
          abort()
      | Parsing.YYexit _ ->
          show_error "Syntax";
          print_string "Exception  'yyexit' raised. Please report.";
	  force_newline ();
          abort()
;;


(*============
  
Run completion

============*)

let run_completion order axioms conjectures =
  begin
    if options_courantes.niveau_de_verbosite >= 1
    then 
      begin
      	message "------------------------------------------";
	print_string "Normalised completion modulo the theory:";
        print_break 1 2;
	print_theorie theorie_canonique_courante.theories;
        force_newline();
	print_string "defined by the canonical set of rules:";
        print_break 1 2;
        print_liste_de_regles theorie_canonique_courante.regles;
        force_newline();   
	print_string "of the set of equations:";
        print_break 1 2;
        print_list_of_pairs_of_terms axioms;
        force_newline ();
	message "------------------------------------------";
	print_flush ()
      end;
    if options_courantes.orientation_seule 
    then 
      let regles = orientation_seule (fun _ _ -> Greater) axioms in
      	regles, 
      	List.fold_right
	  (fun r d -> add_rule_to_dnet LEFT r d)
	  regles (max_int, empty_dnet())
    else  
      let result=
	if options_courantes.goal_oriented <> 0
        then goal_oriented_completion order axioms conjectures 
        else completion order axioms conjectures in
      	open_vbox 0;
      	force_newline();
      	print_string "Result:";
      	force_newline();
      	print_liste_de_regles (fst result);
      	force_newline ();        
      	close_box();
      	Print_statistiques.print_statistiques();
      	print_flush();
      	result
  end
;;


(*======================

solves a problem

========================*)

let solve_problem order regles dnet = function
    EQUATION_A_PROUVER(u,v) ->
      let u1 = 
	normalisation_normalisee_par_un_systeme_de_regles (-1) 
	  (if options_courantes.ordered_completion
	   then order
	   else (fun _ _ -> Greater)) 
	   u dnet
      and v1 = 
	normalisation_normalisee_par_un_systeme_de_regles (-1) 
	  (if options_courantes.ordered_completion
	   then order
	   else (fun _ _ -> Greater)) 
	  v dnet in 
	open_hovbox 0;
      	force_newline();
      	print_string "The equation";
      	print_break 1 2;
      	open_hovbox 0;
      	print_term u;
      	print_string " =";
      	print_break 1 0;
      	print_term v;
      	close_box ();
      	print_break 1 0;
      	if egaux_ac u1 v1
      	then 
	  begin
            print_string "is valid, common normal form:";
            print_break 1 2;
            print_term u1
      	  end
      	else 
	  begin
            print_string "is not valid, normal forms are";
            print_break 1 2;
            print_term u1;
            print_break 1 0;
            print_string "and";
            print_break 1 2;
            print_term v1
      	  end ;
      	force_newline ();
      	close_box();
      	print_flush()
	  
  | TERMES_A_COMPARER(u,v) ->
      begin
	open_hovbox 0;
      	print_string "Comparison of";
        print_break 1 0;
        print_term u;
        print_break 1 0;
        print_string "and";
        print_break 1 0;
        print_term v;
        print_string ": ";
	print_string (string_of_comparison_result (order u v)) ;
        force_newline();
	close_box();
	print_flush ()
      end
(*
  | POLYS_A_COMPARER(u,v) ->
  begin
  open_hovbox 0;
  print_string "Comparison of";
  (fun(x,y)->print_break x y) (1,0);   
  print_string (Poly.ecrit_polynome u);
  (fun(x,y)->print_break x y) (1,0);
  print_string "and";
  (fun(x,y)->print_break x y) (1,0);
  print_string (Poly.ecrit_polynome v);
  print_string ": ";
  print_string (string_of_comparison_result
  (Ord_poly.compare_poly u v)) ;
  force_newline();
  close_box();
  print_flush ()
  end
  *)
  | EQUATION_A_MATCHER (u,v) -> 
      begin
      	open_hovbox 0;
      	print_string "Matching of";
      	print_break 1 0;
      	print_term u;
      	print_break 1 0;
      	print_string "and";
      	print_break 1 0;
      	print_term v;
      	print_string ":";
      	force_newline();
      	close_box();
      	print_flush ()
      end;
      
      begin
        try 
	  let sol = filtrage_ac u v in
            print_string " solution:";
            force_newline ();
       	    open_hovbox 0;
            List.iter 
	      (function (x,t) -> 
		 print_term (VAR x);
		 print_string "=";
		 print_term t;
		 force_newline())
              sol;
            close_box();
	    print_flush ()
        with No_match -> 
	  begin
            print_string "No solution.";
            force_newline ()
          end;
	  force_newline();
	  print_flush()
      end

  | EQUATION_A_RESOUDRE(typ,u,v) ->
      begin
      	open_hovbox 0;
      	print_string "Unification of";
      	print_break 1 0;
      	print_term u;
      	print_break 1 0;
      	print_string "and";
      	print_break 1 0;
      	print_term v;
      	print_string ":";
      	force_newline();
      	close_box();
      	print_flush ()
      end;

      begin
        match typ with
      	    Unif_type_AC_only -> select_ac_only_unification()
      	  | Unif_type_AC_complete -> select_ac_complete_unification()
      	  | Unif_type_plain -> select_plain_unification()
      	  | Unif_type_default -> ()
      end;
      
      let sols = set_of_unifiers u v in
      	if (List.length sols = 0)
      	then 
	  begin
            print_string "No solution.";
            force_newline ()
	  end
      	else 
	  begin
            print_int (List.length sols);
            print_string " solution(s):";
            force_newline ();
            List.fold_left 
	      (fun n theta -> 
		 begin
		   open_hovbox 0;
		   print_string "Solution ";
		   print_int n;
		   print_string ":";
		   print_break 1 2;
		   print_substitution theta;
		   force_newline ();
		   close_box();
		   n+1
      		 end
              ) 1 sols;
	    ()
	  end;
      	force_newline();
      	print_flush()

  | TERME_A_REDUIRE(u) ->
      let u1 = 
	if options_courantes.ordered_completion
      	then (normalisation_normalisee_par_un_systeme_de_regles (-1) 
                order u dnet)
      	else (normalisation_normalisee_par_un_systeme_de_regles 
	        (-1) (fun _ _ -> Greater) u dnet) in 
	begin
          print_string "The normal form of ";
          print_break 1 0;
          print_term u;
          print_break 1 0;
          print_string "is";
          print_break 1 0;
          print_term u1;
          force_newline ();
	  print_string 
	    (sprintf "(by %d reductions)" 
	       (last_number_of_normalized_rewrite_steps()));
          force_newline ();
	  print_flush()
      	end

;;


(*********************************)
(* preuve de theoremes inductifs *)
(*********************************)

let run_inductive_completion order inducthm =
  force_newline ();
  
  if options_courantes.load_automaton
  then 
    begin
      message "Please Wait : Loading automaton...";
      try 
	let auto_in_channel = open_in  options_courantes.automaton_input in
	  induction.automate <- (Marshal.from_channel auto_in_channel : gauto);
	  close_in auto_in_channel
      with 
	  Failure(eee)  -> 
	    begin 
	      message "\n File is corrupted !";
	      message eee;
	      message"\n I prefer to compute the automaton again.";
	      message "Please Wait : building normal forms automaton...";
    	      induction.automate <- One(clean_automata (build_nf_automata induction.sys_regles))
	    end
	| _ -> 
	    begin 
	      message "\n File is corrupted !";
	      message "Unexpected error. Please report.";
	      message"\n I prefer to compute the automaton again.";
	      message "Please Wait : building normal forms automaton...";
    	      induction.automate <- 
	      One(clean_automata (build_nf_automata induction.sys_regles))
	    end
    end
  else
    begin
      message "Please Wait : building normal forms automaton...";
      induction.automate <- 
      One (clean_automata (build_nf_automata induction.sys_regles));
      if options_courantes.save_automaton 
      then
	begin
	  begin
	    message "Please Wait : saving automaton...";
	    try 
	      let auto_out_channel = open_out_bin 
				       options_courantes.automaton_output in
		Marshal.to_channel auto_out_channel (induction.automate) [Marshal.Closures];
		close_out auto_out_channel;
	    with
	      	eee -> 
		  begin 
		    message "WARNING ! Could not properly save automaton.\n I will try to continue without saving."
	      	  end;
	  end;
	end;
    end;
  print_clean_automata  (gauto2auto induction.automate); 
  force_newline ();
  message "Computing number of path...";
  force_newline ();
  let n = Counter.count_paths (fst (gauto2auto induction.automate )) in
    match n with Counter.Infinite -> message "This automaton accepts an infinite tree language.\n"
      |Counter.Card(n) -> message ("This automaton accepts " ^ (string_of_int n) ^ " distinct terms.\n");

  message "Proving inductive theorems:";
  induction.induction_on <- true;
  try
    let regles_complement = 
      if options_courantes.ordered_completion
      then failwith "Well, you cannot prove an inductive theorem with ordered completion. Sorry."
      else (completion 
	      order
              inducthm
              []) 
    in
      begin
      	message "\n the following equations are inductive theorems";
      	print_list_of_pairs_of_terms inducthm;
      	message "\n the resulting rewrite system is :";
      	print_liste_de_regles (fst regles_complement)
      end
  with Disproof -> begin
    message "\n not all of the following equations are inductive theorems";
    print_list_of_pairs_of_terms inducthm
  end
;;


(*============

Programme principal

============*)

let main () =

try

Encompassment.definir_symbole_egal();

lire_ligne_de_commande ();

if options_courantes.nom_fichier = ""
   then raise (Arg.Bad "no file specified");

if options_courantes.niveau_de_verbosite >= 1
   then begin
       	let r = Gc.get () in
	r.Gc.verbose <- options_courantes.niveau_de_verbosite;
	Gc.set r;
      	message (sprintf "Reading file \"%s\"...\n" options_courantes.nom_fichier)
        end;

let (theorie,unification_theory,axiomes,ordre,problemes,conjectures,inducthm)=read_specifications() 
in

construit_theorie_canonique(theorie);

compile_theorie_courante();

declare_theorie_unification unification_theory (listof_all_functions ());

let automatic_ordering = options_courantes.automatic_ordering in
let ordre_externe = 
  if automatic_ordering != 0
  then total_AC_ordering automatic_ordering axiomes
  else ordre in

let ordre_interne = ordre_of_ordre_disponible ordre_externe in

if options_courantes.niveau_de_verbosite >= 2
   then  begin
         message "----------------------";
         open_hovbox 0;
      	 print_string "Theory:";
         print_break 1 2;
	 print_theorie theorie;
         force_newline ();
	 close_box ();
	 print_flush();
         message "----------------------";
         open_hovbox 0;
         print_string "Unification theory:";
         print_break 1 2;
	 print_theorie unification_theory;
	 force_newline ();
	 close_box ();
	 print_flush();
         message "----------------------";
         open_hovbox 0;
	 print_string "Axioms:";
         print_break 1 2;
	 print_list_of_pairs_of_terms axiomes;
	 force_newline ();
	 close_box ();
	 print_flush();
	 message "----------------------";
         open_hovbox 0;
	 print_string "Ordering:";
         print_break 1 2;
         print_ordre ordre_externe;
	 force_newline ();
	 close_box ();
	 print_flush();
	 (*
	  printf "----------------------\n";
	  printf "Problems to solve:\n";
	  print_liste_d_equations problemes;
	  *)
	 message "----------------------";
	 print_string "Conjectures:";
         print_break 1 2;
	 print_list_of_pairs_of_terms conjectures;
	 force_newline ();
         message "----------------------";
         print_string "Inductive queries:";
         print_break 1 2;
         print_list_of_pairs_of_terms inducthm;
         force_newline ();
	 print_flush()
      	 end;





let regles, dnet =
  if axiomes=[]
  then [],(max_int, empty_dnet())
  else run_completion ordre_interne axiomes conjectures
    
in

if not options_courantes.orientation_seule 
then 
  begin
    open_vbox 0;
    force_newline();
    print_string "Result:";
    force_newline();
    print_liste_de_regles regles;
    force_newline ();        
    close_box();
    Print_statistiques.print_statistiques();
    print_flush();
    
  end;
induction.sys_regles <- regles @ theorie_canonique_courante.regles;
print_liste_de_regles induction.sys_regles;

(*=================

Preuves des problemes

=================*)

if problemes <> [] then
  begin
    force_newline();
    message "Solving problems:";
    List.iter (solve_problem ordre_interne regles dnet) problemes;
    
    if options_courantes.niveau_de_verbosite >= 1
    then Print_statistiques.print_statistiques()
	
  end;

(*********************************)
(* preuve de theoremes inductifs *)
(*********************************)

if inducthm <> [] then
  run_inductive_completion ordre_interne inducthm



(*********************************************************)

with
  Sys_error s            -> 
    message (sprintf "System error: %s" s)
| Arg.Bad s           -> 
    begin
    message ("Bad command line arguments: " ^ s);
    message "Usage: cime [options] <source file name>";
    message "options are:";
    message "";
    message "display options:";
    message "";
    message "  -pp           : pretty print, e.g. a+a+a is printed 3a";
    message "  -echo         : echo standard input";
    message "  -v            : set verbose level to 1";
    message "  -verbose n    : set verbose level to n";
    message "";
    message "completion strategy options:";
    message "";
    message "  -states       : show states of completion";
    message "  -strat n      : use strategy n";
    message "  -nocompl      : do not perform completion";
    message "  -confl        : check confluence";
    message "  -ordered      : perform ordered completion";
    message "";
    message "rewrite strategy options:";
    message "";
    message "  -inner        : innermost strategy";
    message "  -outer        : outermost strategy";
    message "";
    message "ordering options:";
    message "";
    message "  -auto-ord n   : automatic ordering ";
    message "            n=1 : RPO with lexicographic status ";
    message "            n=2 : RPO with multiset status ";
    message "";
    message "other options:";
    message "";
    message "  -rew-bound n  : use n as rewrite bound";
    message "  -undo         : allows 'undo' in interactive ordering"
    end
| Erreur_de_syntaxe s    -> 
    message (sprintf "Syntactic analysis failed (%s)." s)
| Semantical_error s -> 
    message (sprintf "Semantical error: %s." s)
| Standard_completion.Completion_failed -> 
    message "\nResult: completion failed."
| G_completion.Completion_failed -> 
    message "\nResult: completion failed."
| Standard_completion.Completion_aborted -> 
    message "\nResult: completion aborted."
| G_completion.Completion_aborted -> 
    message "\nResult: completion aborted."
| Abort              -> message "Execution aborted."
| Invalid_argument s -> message (sprintf "Invalid_argument: %s." s);
      	       	       	message "Please report to Claude.Marche@lri.fr."
| Failure s          -> message (sprintf "Failure: %s." s);
      	       	       	message "Please report to Claude.Marche@lri.fr."
| Out_of_memory -> message "\nOut of memory! Aborted."
(*
| End_of_file -> message "\nEnd of file met! Aborted."
*)
| ex -> message "Uncaught exception! Please report to Claude.Marche@lri.fr.";
        raise ex
;;


let _ = 
  begin
  Printexc.catch main ();
  if options_courantes.print_dioph >=1
    then 
      begin
        Printf.fprintf options_courantes.dioph_output "0\n0";
        close_out options_courantes.dioph_output
      end;
  force_newline();
  print_flush()
(**);
Time_profiler.print_profile()
(**)
  end;;













