/***************************************************************************

C functions calling Eker algorithm for AC matching: body

Project:       Cime
Author(s):     Claude March\'e

Created on January 27, 93

Last modified by $Author: monate $

File name: $Source: /users/demons/demons/master-sources-repository//cime/ocaml_src/oldcimroot/matching_eker.c,v $
$Revision: 1.1 $ 

Last modified on $Date: 2000/05/16 17:36:45 $

***************************************************************************/

/*
#define DEBUG_FILTRAGE 
*/

#define FREE

void failwith(char *s);

#include "camllist.h" 
#include "camlterm.h"
#include "camlpair.h"

#include "ac_matching_eker.h"

#if defined (OCAML)
#include <caml/alloc.h> 
#include <caml/memory.h>
#else
#include <alloc.h> 
#include <memory.h>
#endif

#include <stdio.h>
#include <stdlib.h>

/*=================================

Variables globales de ce module

=================================*/

static int nombre_d_appels_au_matching = 0; 
static int nombre_d_appels_au_matching_reussis = 0; 
static int plus_grand_numero_non_utilise = 0;
static int plus_grand_numero_de_variable_constante_non_utilise = 0;


/*=====================================

Fonction d'initialisation

======================================*/

void initialise_ac_matching(void) 
{ 
  plus_grand_numero_non_utilise = 0; 
  plus_grand_numero_de_variable_constante_non_utilise = -1;
}





/*===============================

Statistiques sur les appels au matching 

ac_matching_stats() renvoie un couple d'entiers (a,b,c,d) o\`u

a = nombre d'appels au filtrage 
b = nombre d'appels reussis au filtrage 


================================*/



value statistiques_filtrage_ac_eker(value u) 
{ 
  value r;

  r = alloc_tuple(2); 
  Field(r,0) = Val_int(nombre_d_appels_au_matching);
  Field(r,1) = Val_int(nombre_d_appels_au_matching_reussis); 
  return r; 
}





/*===============================================

Encodage des termes de CAML dans la structure d'Eker

Pour \'eviter d'avoir \`a effectuer des renommages, on donne en
param\^etre un entier qui sera utilis\'e comme d\'ecalage dans les
num\'eros des variables. Ainsi, en utilisant comme d\'ecalage un
nombre \'egal au plus grand num\'ero de variable non encore utilis\'e,
on est s\^ur de cr\'eer un terme n'ayant aucune variable commune avec
tous les termes cr\'ees pr\'ec\'edemment. Ce m\'ecanisme sera
utilis\'e pour le calcul des paires critiques.

Enfin, on doit donner \'egalement un bool\'een~: lorsqu'il vaut FALSE,
les variables du terme cr\'e\'e ne sont pas consid\'er\'ees comme des
variables mais comme des nouvelles constantes. Ce m\'ecanisme sera
utilis\'e pour le filtrage.


================================================*/


#define MALLOC(t) ((t *) malloc(sizeof(t)))


/***********************************

Functions for building terms and term lists

************************************/

TERM *make_term(TERM_TYPE sym_type,
		int sym_num, 
		TERM_LIST *args)
{
  TERM *t;
  TERM_LIST *p;
  int len;

  t = MALLOC(TERM);

  t->type = sym_type;
  t->sym = sym_num;
  if (sym_type==VARIABLE)
    {
      t->rest.v.var_nr = sym_num;
    }

  if (args !=NULL)
    {
      len = 1; 
      for (p = args;
	   (p->next_arg) != NULL ; 
	   p = p->next_arg)
	{
	  len++;
	}
      t->rest.f.list_len = len;
      t->rest.f.arg_list = args;
      t->rest.f.arg_tail = p;
    }
    
  return(t);
}


TERM_LIST *make_term_list(TERM *head,
			  TERM_LIST *tail)
{
  TERM_LIST *l;

  l = MALLOC(TERM_LIST);
  l->arg = head;
  l->next_arg = tail;
  return(l);
}





TERM *fabrique_terme_eker(terme t, BOOL b);


TERM_LIST *fabrique_liste_de_termes_eker(liste l,
				    BOOL b) 
{ 
  TERM_LIST *result;
  Push_roots(r,1);
  
  r[0] = l;
  if (N_EST_PAS_VIDE(r[0]))
    { 
      result = make_term_list(fabrique_terme_eker(TETE_DE_LISTE(r[0]), 
					     b),
			      fabrique_liste_de_termes_eker(QUEUE_DE_LISTE(r[0]),
						       b)); 
    } 
  else 
    { 
      result = NULL;
    }
  Pop_roots();
  return result;
}


TERM *fabrique_terme_eker(terme t, BOOL b) 
{
  TERM_TYPE f; 
  int numero_variable;
  TERM *result;
  Push_roots(r,1);
  
  r[0] = t;
  
  /*
    printf("type de t : %d\n",type_de_l_operateur(t));
    printf("type de r[0] : %d\n",type_de_l_operateur(r[0]));
    */

  switch (type_de_l_operateur(r[0])) 
    { 
    case TYPE_CAML_FREE: 
      {
	f = (N_EST_PAS_VIDE(liste_des_sous_termes(r[0]))) ? FUNCTION : CONSTANT;
        result = make_term(f,
			   numero_de_l_operateur(r[0]),
			   fabrique_liste_de_termes_eker(liste_des_sous_termes(r[0]),
						    b));
	break;
      } 
    case TYPE_CAML_AC:
      {
	result = make_term(AC_NORMAL,
			   numero_de_l_operateur(r[0]),
			   fabrique_liste_de_termes_eker(liste_des_sous_termes(r[0]),
						    b));
	break;
      }
    case TYPE_CAML_C:
      {
	result = make_term(COMMUTATIVE,
			   numero_de_l_operateur(r[0]),
			   fabrique_liste_de_termes_eker(liste_des_sous_termes(r[0]),
						    b));
	break;
      }
    case TYPE_CAML_VARIABLE:
      {
	if (b)
	  {
	    /*
	      si b=TRUE, on fabrique une vraie variable 
	      */
	    numero_variable = numero_de_l_operateur(r[0]);
	    if (numero_variable >= plus_grand_numero_non_utilise) 	 
	      { 	
		plus_grand_numero_non_utilise = numero_variable+1; 	
	      } 	
	    result = make_term(VARIABLE,numero_variable,NULL);
	  }
	else
	  { 	
	    /*
	      si b=FALSE, on fabrique une nouvelle constante 
	      */
	    numero_variable = ~numero_de_l_operateur(r[0]);
	    if (numero_variable <= plus_grand_numero_de_variable_constante_non_utilise) 	 
	      { 	
		plus_grand_numero_de_variable_constante_non_utilise = numero_variable-1; 	
	      } 	
	    result = make_term(CONSTANT,numero_variable,NULL); 
	  }
	break;
      }
    default:
      {
	failwith("invalid argument in fabrique_terme_eker()");
	result = make_term(CONSTANT,-1,NULL); 
      }
    }
  Pop_roots();
  return result;
}





/*============================================

Decodage des termes d'Eker en termes CAML

============================================*/

/*
terme recupere_terme_instancie(TERM *t, TERM *assign[]);

liste recupere_liste_termes_instancies(TERM_LIST *l, TERM *assign[]) 
{
  value result;
  Push_roots(r,2);

  if (l == NULL) 
    {
      result = liste_vide();
    }
  else
    { 
      r[0] = recupere_terme_instancie(l->arg,assign);
      r[1] = recupere_liste_termes_instancies(l->next_arg,assign); 
      result = construit_liste(r[0],r[1]);
    }
  Pop_roots();
  return result;
}

liste recupere_liste_compressee(AC_LIST *l, TERM *assign[]) 
{
  int i;
  value result;
  Push_roots(r,2);

  if (l == NULL) 
    {
      result = liste_vide();
    }
  else
    { 
      r[0] = recupere_terme_instancie(l->arg,assign);
      r[1] = recupere_liste_compressee(l->next_ac,assign); 
      for (i=1; i<= l->mult; i++)
	{
	  r[1] = construit_liste(r[0],r[1]);
	}
      result = r[1];
    }
  Pop_roots();
  return result;
}

terme recupere_terme_instancie(TERM *t, TERM *assign[]) 
{
  type_operateur typ;
  int num;
  value result;
  Push_roots(r,1);

  switch (t->type)
    {
      case VARIABLE : 
	{ 
	  num = t->sym; 
	  if (assign[num] != NULL)
	    {
	      result = recupere_terme_instancie(assign[num],assign);
	    }
	  else
	    {
	      typ = TYPE_CAML_VARIABLE; 
	      r[0] = liste_vide();
	      result = construit_terme(typ,num,r[0]);
	    }
	  break;
	}
      case CONSTANT : 
	{ 
	  if (t->sym <0) 
	    {
	      typ = TYPE_CAML_VARIABLE; 
	      num = ~(t->sym); 
	    }
	  else
	    {
	      typ = TYPE_CAML_FREE; 
	      num = t->sym; 
	    }
          r[0] = liste_vide();
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      case AC_NORMAL : 
	{ 
	  typ = TYPE_CAML_AC; 
	  num = t->sym; 
          r[0] = recupere_liste_termes_instancies(t->rest.f.arg_list,assign);
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      case AC_COMPRESSED : 
	{ 
	  typ = TYPE_CAML_AC; 
	  num = t->sym; 
          r[0] = recupere_liste_compressee(t->rest.a.ac_list,assign);
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      case FUNCTION : 
	{ 
	  typ = TYPE_CAML_FREE; 
	  num = t->sym; 
          r[0] = recupere_liste_termes_instancies(t->rest.f.arg_list,assign);
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      default :
	{
	  failwith("Invalid argument in recupere_terme_instancie()\n");
	  typ = TYPE_CAML_FREE;
	  num = 0;
	  r[0] = liste_vide();
	  result = construit_terme(typ,num,r[0]);
	}
    }

  Pop_roots();
  return result;
}
*/

terme recupere_terme_eker(TERM *t);

liste recupere_liste_termes_ekers(TERM_LIST *l) 
{
  value result;
  Push_roots(r,2);

  if (l == NULL) 
    {
      result = liste_vide();
    }
  else
    { 
      r[0] = recupere_terme_eker(l->arg);
      r[1] = recupere_liste_termes_ekers(l->next_arg); 
      result = construit_liste(r[0],r[1]);
    }
  Pop_roots();
  return result;
}

liste recupere_liste_compressee(AC_LIST *l) 
{
  int i;
  value result;
  Push_roots(r,2);

  if (l == NULL) 
    {
      result = liste_vide();
    }
  else
    { 
      r[0] = recupere_terme_eker(l->arg);
      r[1] = recupere_liste_compressee(l->next_ac); 
      for (i=1; i<= l->mult; i++)
	{
	  r[1] = construit_liste(r[0],r[1]);
	}
      result = r[1];
    }
  Pop_roots();
  return result;
}

terme recupere_terme_eker(TERM *t) 
{
  type_operateur typ;
  int num;
  value result;
  Push_roots(r,1);

  switch (t->type)
    {
      case VARIABLE : 
	{ 
	  num = t->sym; 
	  typ = TYPE_CAML_VARIABLE; 
	  r[0] = liste_vide();
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      case CONSTANT : 
	{ 
	  if (t->sym <0) 
	    {
	      typ = TYPE_CAML_VARIABLE; 
	      num = ~(t->sym); 
	    }
	  else
	    {
	      typ = TYPE_CAML_FREE; 
	      num = t->sym; 
	    }
          r[0] = liste_vide();
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      case AC_NORMAL : 
	{ 
	  typ = TYPE_CAML_AC; 
	  num = t->sym; 
          r[0] = recupere_liste_termes_ekers(t->rest.f.arg_list);
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      case COMMUTATIVE : 
	{ 
	  typ = TYPE_CAML_C; 
	  num = t->sym; 
          r[0] = recupere_liste_termes_ekers(t->rest.f.arg_list);
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      case AC_COMPRESSED : 
	{ 
	  typ = TYPE_CAML_AC; 
	  num = t->sym; 
          r[0] = recupere_liste_compressee(t->rest.a.ac_list);
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      case FUNCTION : 
	{ 
	  typ = TYPE_CAML_FREE; 
	  num = t->sym; 
          r[0] = recupere_liste_termes_ekers(t->rest.f.arg_list);
	  result = construit_terme(typ,num,r[0]);
	  break;
	}
      default :
	{
	  failwith("Invalid argument in recupere_terme_eker()\n");
	  typ = TYPE_CAML_FREE;
	  num = 0;
	  r[0] = liste_vide();
	  result = construit_terme(typ,num,r[0]);
	}
    }

  Pop_roots();
  return result;
}














/*=========================================

Fonction de filtrage:

(filtrage_ac s l r) renvoie un couple $(b,t)$ tel que~:

- si b=true alors $s$ est une instance AC de l, et t est l'instance de
$r$ correspondante;

- si b=false alors $s$ n'est pas instance de l, et t=s.

============================================*/


value filtrage_ac_eker(value t, value mb_gauche) 
{
  void  *match_buf;
  TERM  **assignment;
  TERM  *s, *left;
  int   j;
  value resultat;
  BOOL  no_match;

  Push_roots(r,4);


  r[0] = t;
  r[1] = mb_gauche;
  r[2] = liste_vide();

#if defined(DEBUG_FILTRAGE)
      printf("Matching AC...\n");
#endif      

  no_match = TRUE;

  if (   (type_de_l_operateur(r[0]) 
	  != type_de_l_operateur(mb_gauche)) 
      || (numero_de_l_operateur(r[0]) 
	  != numero_de_l_operateur(mb_gauche)))
    {
#if defined(DEBUG_FILTRAGE)
      printf(" <- Fail (not the same top symbol)\n");
#endif
    }
  else
    {
      
      nombre_d_appels_au_matching++;

      initialise_ac_matching();
      /*
	printf("fabrication de s...\n");
	*/
      s = fabrique_terme_eker(r[0],FALSE); 
#if defined(DEBUG_FILTRAGE)
      printf("s = ");
      print_term_eker(s);
      printf("\n");
#endif
      /*
	printf("fabrication de left...\n");
	*/
      left = fabrique_terme_eker(r[1],TRUE); 
#if defined(DEBUG_FILTRAGE)
      printf("left = ");
      print_term_eker(left);
      printf("\n");
#endif

#if defined(DEBUG_FILTRAGE)
      printf("plus_grand_numero_non_utilise = %d\n", plus_grand_numero_non_utilise);
#endif

      match_buf = build_match(left, s, plus_grand_numero_non_utilise);
#if defined(DEBUG_FILTRAGE)
      printf("build_match() called\n");
#endif

      assignment = calloc(plus_grand_numero_non_utilise+1,sizeof(TERM *));
#if defined(DEBUG_FILTRAGE)
      printf("calloc() called\n");
#endif
      for (j = 0; j < plus_grand_numero_non_utilise; j++)
	{
	  assignment[j] = NULL;
	}

      if (extract_match(match_buf, assignment)) 
	{
	  nombre_d_appels_au_matching_reussis++; 

#if defined(DEBUG_FILTRAGE)
	  printf(" <- Success\n");
	  printf("#############################################\n");
	  for (j = 0; j < plus_grand_numero_non_utilise; j++)
	    {
	      printf("x%d = ",j);
	      print_term_eker (assignment[j]);
	      printf("\n");
	    }
	  printf("#############################################\n");
	  fflush(stdout);
#endif 

	  for (j = 0; j < plus_grand_numero_non_utilise; j++)
	    {
	      if (assignment[j] != NULL)
		{
		  r[3] = make_pair(Val_int(j),recupere_terme_eker(assignment[j]));
		  r[2] = construit_liste(r[3],r[2]);
		}
	    }
      
	  /*
	    destructions des termes construits
	    */
	  
	  for (j = 0; j < plus_grand_numero_non_utilise; j++)
	    {
	      if (assignment[j] != NULL)
		{
#if defined(FREE)
#if defined(DEBUG_FILTRAGE)
		  printf("s = ");
		  print_term_eker(s);
		  printf("\n");
		  printf("destroying assignment[%d]\n",j);
#endif
	          destroy_term(assignment[j]);
#endif
		}
	    }

	  no_match = FALSE;
	}
      else
	{
#if defined(DEBUG_FILTRAGE)
	  printf(" <- Fail\n");
#endif
	}

#if defined(FREE)
      free(assignment);
#if defined(DEBUG_FILTRAGE)
      printf("s = ");
      print_term_eker(s);
      printf("\n");
      printf("destroying match_buf\n");
#endif
      destroy_match(match_buf);
#if defined(DEBUG_FILTRAGE)
      printf("s = ");
      print_term_eker(s);
      printf("\n");
      printf("destroying left\n");
#endif
      destroy_term(left);
#if defined(DEBUG_FILTRAGE)
      printf("s = ");
      print_term_eker(s);
      printf("\n");
      printf("destroying s\n");
#endif
      destroy_term(s);
#endif


#if defined(DEBUG_FILTRAGE)
      printf("Done.\n");
#endif

    }
#if defined(DEBUG_FILTRAGE)
  fflush(stdout);
#endif

  resultat = r[2];  
  Pop_roots();
  if (no_match)
    {
      failwith("no match");
    }
  return resultat; 
}





value filtrage_ac_partage_eker(value n,value t, value g, terme d) 
{
  value resultat;

  failwith("filtrage_ac_partage_eker() non implante !\n");
  resultat = alloc_tuple(2);
  Field(resultat,0) = Val_false; 
  Field(resultat,1) = Val_false;
  return resultat; 
}







