(***************************************************************************

 (Basic) Narrowing on conjectures : body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: narrowing.ml,v 1.2 2001/04/20 13:42:21 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: narrowing.ml,v 1.2 2001/04/20 13:42:21 marche Exp $

***************************************************************************)

open Messages
open Args
open Errors
open Symbols
open Positions
open Terms
open Orderings
open Equations
open Equations_as_obj
open Dnet
open Ac_rewriting
open Norm_theory
open A
open Unification
open Unif_comb

   
module type NARROWING =
sig
  type conjecture
  val narrow : conjecture -> conjecture
end

module Narrowing =
  functor (Conj : EQUATION) ->
    
struct 
  
open Conj

exception Not_narrowable

let rec cartesian_product f l1 = function
    [] -> []
  | h2::t2 ->
      let l1_x_h2 =
	List.map (function h -> f h h2) l1
      and l1_x_t2 = cartesian_product f l1 t2 in
	l1_x_h2 @ l1_x_t2

    
let head_narrowing rule (t, list_subst as c) = 
  match rule.orientation with
      true ->
	begin
	  let new_list_subst = resoud_unif_pb [(t,rule.l)] in
	    match new_list_subst with
		[] -> raise Not_narrowable
	      | _ ->
		  let list_subst' = 
		    cartesian_product 
		      (fun subst1 subst2 -> 
			 resoud_unif_pb (subst1 @ subst2))  
		      list_subst new_list_subst in
		    (rule.r,list_subst')
	end
    | false -> raise Not_narrowable


let increase_nar_number bound = 
  bound := succ (!bound);
  if !bound>=options_courantes.normalization_steps_bound
  then 
    begin
      message 
	("There have been more than "
	 ^(string_of_int !bound)
	 ^" consecutive rewrite steps.");
      message "Please check that the rewrite system is terminating,";
      message "or use -rew-bound option.";
      abort()
    end


let rec outermost_narrow_pos bound nar (t,sigma as t_sigma) p q =
  match t with 
      VAR _ -> t_sigma
    | _  -> 
      	begin
	  try 
	    let s,sigma' = nar ((sous_terme_a_la_position t q),sigma) in
              increase_nar_number bound;
              let t' = replace t s q in
              	outermost_narrow_path bound nar (t',sigma') q Top
          with Not_narrowable -> 
	    begin
	      try let q' = next_pos_l t q in
              	outermost_narrow_pos bound nar t_sigma p q'
	      with No_more_pos -> t_sigma
	    end
      	end
	
and outermost_narrow_path bound nar (t,sigma as t_sigma) p q = 
  match t with
      VAR _ -> t_sigma
    | _  -> 
    	begin
	  try 
	    let s,sigma' = nar ((sous_terme_a_la_position t q),sigma) in
	      increase_nar_number bound;
	      let t' = replace t s q in
             	outermost_narrow_path bound nar (t',sigma') q Top
	  with Not_narrowable -> 
	    begin
	      try let q' = allonge p q in
              	outermost_narrow_path bound nar t_sigma p q'
              with No_more_pos -> 
		try let q''= next_pos_l t q in
		  outermost_narrow_pos bound nar t_sigma p q''
            	with No_more_pos -> t_sigma
	    end
	end

	
let outermost_narrow nar t_sigma = 
  let bound = ref 0 in
    outermost_narrow_pos bound nar t_sigma Top Top


end

module type USEFUL_FOR_NARROWING =
sig
  type conjectures_set
  val is_a_useful_term_for_nar : term -> conjectures_set -> int
  val is_a_useful_eq_for_nar : term -> term -> conjectures_set -> int
end

module Useful_for_narrowing = 
  functor (Conj : EQUATION) ->
    functor (ConjSet : EQUATION_SET with type equation = Conj.t) ->
      
struct

open Conj
open ConjSet

type conjecture = Conj.t
type conjectures_set = ConjSet.eq_set

let rec reducible_by_a_set_of_rules_at_any_pos o t dnet =
  match t with
      VAR _ -> false
    | TERM (f,l) -> 
	(List.exists 
	   (function t' -> reducible_by_a_set_of_rules_at_any_pos o t' dnet)
	   l) or
	reducible_by_a_set_of_rule o t dnet


let is_a_useful_dnet_for_nar dnet conj = 
  if (reducible_by_a_set_of_rules_at_any_pos 
	(fun _ _ -> Greater) (first_term conj) dnet) 
    or
	(reducible_by_a_set_of_rules_at_any_pos 
	   (fun _ _ -> Greater) (second_term conj) dnet)
  then 0
  else 1


let is_a f = 
  match norm_theory_of f with
      A _ | AU _ -> true
    | _ -> false


let list_of_rules u v = 
  match u with
      VAR _ -> []
    | TERM (f,_) ->
      	let u_v_rule = make_basic_rule (u,v) in
	  if is_a f
	  then 
	    begin
	      let x = new_var [] in
	      let u' = ajoute_variable_a_droite f x u
	      and v' = ajoute_variable_a_droite f x v in
	      let u_v_rule' = make_basic_rule (u',v') in
		[u_v_rule;u_v_rule']
	    end
	  else [u_v_rule]


let is_a_useful_term_for_nar u conjs =
  let rules = list_of_rules u u in
  let eq_dnet = 
    List.fold_left
      (fun res r -> add_rule_to_dnet LEFT r res)
      (max_int, empty_dnet())
      rules in
  let res = ConjSet.fold_left 
	      (fun res conj ->
		 (is_a_useful_dnet_for_nar eq_dnet conj) + res)
	      0 conjs in
(*
  Format.open_hvbox 0;
  Format.force_newline();
  Format.print_string (string_of_term u);
  Format.print_string "=";
  Format.print_string (string_of_term v);
  if res = 0
  then Format.print_string " est utile"
  else Format.print_string " est inutile";
  Format.force_newline();
  Format.print_flush();
*)
    res


let is_a_useful_eq_for_nar u v conjs =
  (is_a_useful_term_for_nar u conjs) + (is_a_useful_term_for_nar v conjs)
  

end

  module Standard_useful_for_narrowing = 
  Useful_for_narrowing (Conjectures) (Conjectures_set)

