(***************************************************************************

  Normalised rewriting

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: normalised_rewriting.ml,v 1.2 2001/04/20 13:42:22 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: normalised_rewriting.ml,v 1.2 2001/04/20 13:42:22 marche Exp $

***************************************************************************)


open Positions;;
open Norm_theory;;
open Ac_rewriting;;
open Generic_normalization;;
open Args;;
open Messages;;
open Errors;;
open Orderings;;

open Symbols;;
open Terms;;
open Equations;;
open Substitution;;
open Dnet;;

(*==============================

teste si une substitution est reductible par la
theorie courante

===============================*)

exception Reducible;;

let s_reducible theta =
  let ordre = fun _ _ -> Uncomparable in
    try
      VarMap.iter
	(fun x t -> 
      	   if (reducible_by_a_set_of_rule 
		 ordre t theorie_canonique_courante.regles_compilees)
      	   then raise Reducible)
	theta;
      false
    with Reducible -> true
;;

let rs_reducible ordre rule_dnet theta =
  if s_reducible theta
  then true
  else 
    try
      VarMap.iter
	(fun x t -> 
      	   if (reducible_by_a_set_of_rule ordre t rule_dnet)
      	   then raise Reducible)
	theta;
      false
    with Reducible -> true
;;


(*==============================

Normalisation d'un terme par la theorie courante

================================*)

let normalisation_par_la_theorie_courante s =
  (normalisation_ac_par_un_systeme_de_regles 
     (fun _ _ -> Greater) s
     theorie_canonique_courante.regles_compilees);;


let normalisation_forcee_par_la_theorie_courante s =
  (normalisation_ac_forcee_par_un_systeme_de_regles 
     (fun _ _ -> Greater) s
     theorie_canonique_courante.regles_compilees);;


let reecriture_normalisee_par_un_systeme_de_regles max ordre s rule_dnet =
  if max<0 
  then (reecriture_ac_par_un_systeme_de_regles 0 ordre
          (normalisation_par_la_theorie_courante s) rule_dnet)
  else (reecriture_ac_par_un_systeme_de_regles max ordre s rule_dnet)
;;

      
let reecriture_normalisee_en_tete_par_un_systeme_de_regles 
    max ordre s rule_dnet =
  if max<0 
  then (reecriture_ac_en_tete_par_un_systeme_de_regles 0 ordre
          (normalisation_par_la_theorie_courante s) rule_dnet)
  else (reecriture_ac_en_tete_par_un_systeme_de_regles max ordre s rule_dnet)
;;
        


(*=====================================

normalisation_normalisee_par_un_systeme_de_regles it\`ere la fonction
pr\'ec\'edente tant que c'est possible.

(normalisation_normalisee_par_un_systeme_de_regles s rule_set) renvoie un
terme $t$ tel que $s$ se r\'e\'ecrit de facon normalis\'ee en $t$ par
$rule_set$ et $t$ est $rule_set$-irr\'eductible.

======================================*)


let last_number_of_normalized_rewrite_steps = ref 0;;

let normalisation_normalisee_par_un_systeme_de_regles_with_test bound
  max ordre s rule_dnet =
  if options_courantes.outermost_reduction_strategy
  then
    let s',max' = 
      if max < 0 
      then normalisation_par_la_theorie_courante s,0 
      else s,max 
    in
    let m = ref max' 
    in
    let red_S x = normalisation_forcee_par_la_theorie_courante x
    and red_R x = reecriture_normalisee_en_tete_par_un_systeme_de_regles
                    (if !m < 0 then 0 else !m) ordre x rule_dnet 
    in 
    let red x = 
      if !m < 0 (* x n'est peut-etre pas en S-forme normale *)
      then 
	begin
      	  try let x' = red_S x in
	    try let x'' = red_R x' in
              m:= -1;
              x''
            with Irreducible -> x'
          with Irreducible -> let x'' = red_R x in m:= -1; x''
      	end
      else 
	let x'' = red_R x in m:= -1; x''
    in
      outermost_normalize_pos bound red s' Top Top

  else 
    let red t =
      try
	reecriture_ac_en_tete_par_un_systeme_de_regles_sans_subst
	  (-1) (fun _ _ -> Greater) t
	  theorie_canonique_courante.regles_compilees
      with Irreducible ->
	reecriture_ac_en_tete_par_un_systeme_de_regles_sans_subst
	  0 ordre t rule_dnet
    in
      efficient_innermost_normalize_avec_test bound red VarMap.empty s 
;;



let normalisation_normalisee_par_un_systeme_de_regles max s rule_dnet =
    last_number_of_normalized_rewrite_steps:= 0;
    normalisation_normalisee_par_un_systeme_de_regles_with_test 
      last_number_of_normalized_rewrite_steps max s rule_dnet
;; 

let last_number_of_normalized_rewrite_steps () =
 !last_number_of_normalized_rewrite_steps
;;


(*
  normalisation_forcee_par_un_systeme_de_regles normalise un terme,
  et retourne Irreducible s'il est deja en forme normale.
*)

let normalisation_forcee_par_un_systeme_de_regles max ordre s rule_dnet =
  if max >= 0 (* on sait que s est en FN pour la theorie courante S *)
  then 
    begin 
      let n = ref 0 in
      let s' = 
	normalisation_normalisee_par_un_systeme_de_regles_with_test 
          n max ordre s rule_dnet in
      if !n = 0
      then raise Irreducible
      else s'
    end
  else 
    let m = ref 0
    and n = ref 0 in
    let s' = 
      normalisation_ac_par_un_systeme_de_regles_avec_test
        m (fun _ _ -> Greater) s 
	theorie_canonique_courante.regles_compilees in
    let s'' = 
      normalisation_normalisee_par_un_systeme_de_regles_with_test 
        n 0 ordre s' rule_dnet in
    if (!m = 0) & (!n = 0)
    then raise Irreducible
    else s''
;;



let normalise_pair max ordre u v rule_dnet =
  try 
    let u'=
      normalisation_forcee_par_un_systeme_de_regles max ordre u rule_dnet in
    (u',
     normalisation_normalisee_par_un_systeme_de_regles max ordre v rule_dnet)
  with Irreducible ->
    (u,normalisation_forcee_par_un_systeme_de_regles max ordre v rule_dnet)
;;
   
