(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: orient.ml,v 1.2 2001/04/20 13:42:22 marche Exp $

***************************************************************************)

open Args;;
open Orderings;;
open Terms;;
open Norm_theory;;
open A;;
open Au;;
open Acu;;
open Aci;;
open Acui;;
open Acun;;
open G;;
open Ag;;
open Cr;;
open Br;;
open Fp;;
open Int_theory;;
open Rat;;
open User;;
open Equations_as_obj;;
open Completion_state;;
open Simplifications;;
open Format;;

module type ORIENT = 
  sig
    val orient : term ordering -> unit
  end

module Orient =
  functor (Eqt : EQUATION) ->
  functor (EqtSet : EQUATION_SET with type equation = Eqt.t) ->
  functor (Compl : COMPLETION_STATE with type equation = Eqt.t
				    and type equation_set = EqtSet.eq_set) ->
  functor (Simpl : SIMPLIFICATIONS with type equation = Eqt.t) ->
    struct

open Eqt;;
open EqtSet;;
open Compl;;
open Simpl;;

(*====================================

Calcul $R,E$ ensembles des r\`egles normalis\'ees et des \'equations critiques
de $(u,v)$ sachant que $u\succ v$

=======================================*)

let couple_normalisant (* ordre *) u v = function
    No_theory      -> failwith "Strategy.couple_normalisant"
  | ACU (n,m)      -> couple_normalisant_acu n m u v
  | AU (n,m)       -> couple_normalisant_au n m u v
  | A (n)          -> couple_normalisant_a n u v
  | ACI (n)        -> couple_normalisant_aci n u v
  | ACUI (n,m)     -> couple_normalisant_acui n m u v
  | ACUN (n,m,k)     -> couple_normalisant_acun n m k u v
  | G (n,m,k)      -> couple_normalisant_g (* ordre *) n m k u v
  | AG (n,m,k)     -> couple_normalisant_ag (* ordre *) n m k u v
  | CR (n,m,k,j,i) -> couple_normalisant_cr (* ordre *) n m k j i u v
  | BR (n,m,k,j) -> couple_normalisant_br (* ordre *) n m k j u v
  | FP (p,n,m,k,j,i) -> couple_normalisant_fp (* ordre *) p n m k j i u v
  | Int (plus_z,coef,plus,zero,moins,mult,un) -> 
      couple_normalisant_int (* ordre *) plus_z coef plus zero moins 
	mult un u v
  | Rat (un_z,coef,irred,plus,zero,moins,mult,un) -> 
      couple_normalisant_rat (* ordre *) un_z coef irred plus zero moins 
	mult un u v
  | USER r_list -> couple_normalisant_user r_list u v
;;

(*====================================

Calcul $R,E$ ensembles des r\`egles normalis\'ees et des \'equations critiques
de $(u,v)$ sachant que $u$ et $v$ sont incomparables.

=======================================*)

let couple_normalisant_sym u v = function
    No_theory        -> failwith "Ordered_strategy.couple_normalisant"
  | ACU (n,m)        -> couple_normalisant_sym_acu n m u v
  | AU (n,m)         -> couple_normalisant_sym_au n m u v
  | A (n)            -> couple_normalisant_sym_a n u v
  | ACI (n)          -> couple_normalisant_sym_aci n u v
  | ACUI (n,m)       -> couple_normalisant_sym_acui n m u v
  | ACUN (n,m,k)     -> couple_normalisant_sym_acun n m k u v
  | G (n,m,k)        -> couple_normalisant_sym_g n m k u v
  | AG (n,m,k)       -> couple_normalisant_sym_ag n m k u v
  | CR (n,m,k,j,i)   -> couple_normalisant_sym_cr n m k j i u v
  | BR (n,m,k,j)     -> couple_normalisant_sym_br n m k j u v
  | FP (p,n,m,k,j,i) -> couple_normalisant_sym_fp p n m k j i u v
  | Rat (un_z,coef,irred,plus,zero,moins,mult,un) -> 
         couple_normalisant_sym_rat un_z coef irred plus zero moins mult un u v
  | USER r_list      -> couple_normalisant_sym_user r_list u v
;;
  
let rec flat_map_split f = function
    [] -> ([],[])
  | t::l -> 
      let (e1,r1) = f t
      and (e2,r2) = flat_map_split f l in
      	(e1 @ e2, r1 @ r2)
;;  

let couple_normalisant_general (* ordre *) eq = 
  let u =first_term eq
  and v =second_term eq 
  and a = etat.date in
  let (ec,rc) = flat_map_split (couple_normalisant (* ordre *) u v) 
                  theorie_canonique_courante.theories in
  let ec' = List.map (fun (u,v) -> (make_aged_equation a u v)) ec
  and rc' = List.map (fun (u,v) -> (make_aged_equation a u v)) rc in
    if rc' = []
    then ([eq],ec')
    else (rc',ec')
;;

let couple_normalisant_sym_general (* ordre *) eq = 
  let u =first_term eq
  and v =second_term eq 
  and a = etat.date in
  let (ec,rc) = flat_map_split (couple_normalisant_sym (* ordre *) u v) 
                  theorie_canonique_courante.theories in
  let ec' = List.map (fun (u,v) -> (make_aged_equation a u v)) ec
  and rc' = List.map (fun (u,v) -> (make_aged_equation a u v)) rc in
    if rc' = []
    then ([eq],ec')
    else (rc',ec')
;;

let orient order =
  let eq = next_equation etat.preoriented_eqs in
  let psi,theta =     
    if is_oriented eq
    then (couple_normalisant_general eq)
    else (couple_normalisant_sym_general eq) in
    if (options_courantes.niveau_de_verbosite >= 1)
    then 
      begin
      	print_string "Normalising pair of the equation";
      	print_break 1 0;
      	print_equation eq;
      	print_string " is";
      	force_newline ();
      	print_string  "Psi =";
      	print_list_of_equations psi;
      	print_string ";";
      	print_break 1 0;
      	print_string "Theta =";
      	print_list_of_equations theta;
      	force_newline ();
      	print_flush ()
      end;
    if (options_courantes.niveau_de_verbosite >= 2)
    then 
      begin
    	print_string " ---- SIMPLIFYING...";
    	force_newline ()
      end;  
    let final_psi,psi_collapsed = 
      simplify_and_left_split 
	(if options_courantes.ordered_completion
	 then order
	 else (fun _ _ -> Greater))
 	psi etat.disc_net
    and final_theta = 
      simplify 
	(if options_courantes.ordered_completion
	 then order
	 else (fun _ _ -> Greater))
	theta etat.disc_net in
      if (options_courantes.niveau_de_verbosite >= 1)
      then 
    	begin
	  print_string "Theta Simplified =";
	  print_list_of_equations final_theta;
	  force_newline ();
	  print_flush ()
    	end;
      add_new_eqs final_theta;
      add_new_eqs psi_collapsed;
      List.iter (add_new_pre_rule (get_age eq)) final_psi;;

    end;;

module Standard_orient =
  Orient (Equation1) (Equation_set1) (Standard_completion) 
    (Standard_simplifications);;

module G_orient = 
  Orient (Equation_with_goal) (Equation_with_goal_set) (G_completion) 
    (G_simplifications);;
