(***************************************************************************

(This sentence has been added automatically, it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: print_statistiques.ml,v 1.1 2000/05/16 17:36:47 monate Exp $

***************************************************************************)

open Unix;;
open Args;;
open Format;;
open Messages;;
open C_matching;;
open Match_comb;;
open Matching;;
open Unification;;
open Clash;;
open Rpo;;
open Poly_interp;;
(*
open Strategy;;
*)
open Critical_pair_criteria;;


let compteur_nombre_de_regles_produites = ref 0;;
let compteur_nombre_de_regles_retenues = ref 0;;
let compteur_echec_matching_par_taille = ref 0;;

let incr_nombre_de_regles_produites () = 
  compteur_nombre_de_regles_produites :=
    !compteur_nombre_de_regles_produites+1;;

let incr_nombre_de_regles_retenues n = 
  compteur_nombre_de_regles_retenues :=
    !compteur_nombre_de_regles_retenues+n;;


let reset_stat_counters () =
  compteur_nombre_de_regles_produites := 0;
  compteur_nombre_de_regles_retenues := 0;
  compteur_echec_matching_par_taille := 0
;;

(* impression des statisques *)

let print_statistiques () =
open_vbox 0;
force_newline();
(*
let (a,b,c) = lire_statistiques_filtrage() in
  

printf "Nombre d'appels au filtrage etendu  : %d\n" a;
printf "Nombre d'appels au filtrage general : %d" b;
if a <> 0 
   then printf " (%d%%)\n" (100 * b / a) 
   else printf "\n";
printf "Nombre d'appels reussis             : %d" c;
if a <> 0 
   then printf " (%d%%)\n" (100 * c / a) 
   else printf "\n";
*)

let (a,b) = if options_courantes.c_matching
            then statistiques_filtrage_ac() 
	    else filtrage_stats() 
(*
and (c,d) = statistiques_unification_ac() in
*)  
and (c,d) = unification_stats() 
and e = nombre_de_paires_critiques_essentielles() 

and f = !compteur_nombre_de_regles_produites
and g = !compteur_nombre_de_regles_retenues
and h = matching_failures_due_to_size()

(*
and (ma,mb,mc,mr,m) = matching_cache_statistics ()
*)
in

(*
print_string (sprintf "Number of calls to AC matching      : %d/%d" a mb);
*)
print_string (sprintf "Number of calls to AC matching      : %d" a);
force_newline();
print_string (sprintf "Number of failures due to the size  : %d" h);
force_newline();
print_string (sprintf "Number of failures due to clash  : %d" (!Clash.nb_clash));
force_newline();
print_string (sprintf "Number of successful calls          : %d" b);
if a <> 0 
   then print_string (sprintf " (%d%%)" (100*b/a));
force_newline();
print_string (sprintf "Number of calls to unification      : %d" c);
force_newline();
print_string (sprintf "Number of unifiers generated        : %d" d);
if c <> 0 
   then print_string (sprintf " (%.2f average)" ((float d)/.(float c)));
force_newline();
force_newline();
print_string (sprintf "Number of critical pairs considered : %d" e);
if c <> 0 
   then print_string (sprintf " (%d%%)" (100*e/d));
force_newline();
print_string (sprintf "Number of rules produced            : %d" f);
force_newline();
print_string (sprintf "Number of rules retained            : %d" g);
force_newline();
force_newline();

if options_courantes.niveau_de_verbosite >= 1
  then
  begin
  let (a,b,c) = rpo_statistics ()
  in
  if b>0
    then
    begin
    print_string (sprintf "Number of addition to rpo cache     : %d" a);
    force_newline();	
    print_string (sprintf "Number of accesses to rpo cache     : %d" b);
    force_newline();
    print_string (sprintf "Number of successful accesses       : %d (%d%%)" (b-c) (100*(b-c)/b));
    force_newline();
    force_newline()
    end
  end;

  let (a,b,c,rate,len) = matching_cache_statistics ()
  in
  if b>0
    then
    begin
    print_string (sprintf "Number of addition to matching cache : %d" a);
    force_newline();	
    print_string (sprintf "Number of accesses to matching cache : %d" b);
    force_newline();
    print_string (sprintf "Number of successful accesses        : %d (%d%%)" (b-c) (100*(b-c)/b));
    force_newline();
    print_string (sprintf "collision rate                       : %f" rate);
    force_newline();
    print_string (sprintf "max hash length                      : %d" len);
    force_newline();
    force_newline()
    end;

print_string ("      times       |   user   |  system  |  total ");
force_newline();

print_string ("------------------+----------+----------+----------");
force_newline();

let r=times() in
let total_time = r.tms_utime +. r.tms_stime in
print_string (sprintf "%-17s | %8.3f | %8.3f | %8.3f " "Total times" r.tms_utime r.tms_stime total_time);
force_newline (); 

if total_time >= 60.0
  then 
  begin
  let um = Pervasives.truncate(r.tms_utime /. 60.0) 
  and sm = Pervasives.truncate(r.tms_stime /. 60.0) 
  and tm = Pervasives.truncate(total_time /. 60.0) 
  in
  let us = Pervasives.truncate(r.tms_utime -. float(um) *. 60.0) 
  and ss = Pervasives.truncate(r.tms_stime -. float(sm) *. 60.0) 
  and ts = Pervasives.truncate(total_time -. float(tm) *. 60.0) 
  in
  print_string (sprintf "%-17s |(%2dmn %02ds)|(%2dmn %02ds)|(%2dmn %02ds)" "" um us sm ss tm ts);
  force_newline ()
end;

let ut,st = unification_times() in
let total_time = ut +. st in
print_string (sprintf "%-17s | %8.3f | %8.3f | %8.3f " "Unification times" ut st total_time);
force_newline ();
if total_time >= 60.0
   then 
   begin
   let um = Pervasives.truncate(ut /. 60.0) 
   and sm = Pervasives.truncate(st /. 60.0) 
   and tm = Pervasives.truncate(total_time /. 60.0) 
   in
   let us = Pervasives.truncate(ut -. float(um) *. 60.0) 
   and ss = Pervasives.truncate(st -. float(sm) *. 60.0) 
   and ts = Pervasives.truncate(total_time -. float(tm) *. 60.0) 
   in
     print_string (sprintf "%-17s |(%2dmn %02ds)|(%2dmn %02ds)|(%2dmn %02ds)" "" um us sm ss tm ts);
     force_newline ()
   end;


let ut,st = matching_times() in
let total_time = ut +. st in
print_string (sprintf "%-17s | %8.3f | %8.3f | %8.3f " "Matching times" ut st total_time);
force_newline ();

if total_time >= 60.0
  then 
  begin
  let um = Pervasives.truncate(ut /. 60.0) 
  and sm = Pervasives.truncate(st /. 60.0) 
  and tm = Pervasives.truncate(total_time /. 60.0) 
  in
  let us = Pervasives.truncate(ut -. float(um) *. 60.0) 
  and ss = Pervasives.truncate(st -. float(sm) *. 60.0) 
  and ts = Pervasives.truncate(total_time -. float(tm) *. 60.0) 
  in
  print_string (sprintf "%-17s |(%2dmn %02ds)|(%2dmn %02ds)|(%2dmn %02ds)" "" um us sm ss tm ts);
   force_newline ()
   end;

let ut,st = poly_times() in
let total_time = ut +. st in
print_string (sprintf "%-17s | %8.3f | %8.3f | %8.3f " "Poly times" ut st total_time);
force_newline ();

if total_time >= 60.0
  then 
  begin
  let um = Pervasives.truncate(ut /. 60.0) 
  and sm = Pervasives.truncate(st /. 60.0) 
  and tm = Pervasives.truncate(total_time /. 60.0) 
  in
  let us = Pervasives.truncate(ut -. float(um) *. 60.0) 
  and ss = Pervasives.truncate(st -. float(sm) *. 60.0) 
  and ts = Pervasives.truncate(total_time -. float(tm) *. 60.0) 
  in
  print_string (sprintf "%-17s |(%2dmn %02ds)|(%2dmn %02ds)|(%2dmn %02ds)" "" um us sm ss tm ts);
   force_newline ()
   end;

close_box();
print_flush()
;;

