(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: select_new_rule.ml,v 1.2 2001/04/20 13:42:22 marche Exp $

***************************************************************************)


open Args;;
open Orderings;;
open Content;;
open Terms;;
open Equations;;
open Equations_as_obj;;
open Completion_state;;
open Simplifications;;
open Induction;;
open Dnet;;
open Format;;

module type SELECT_NEW_RULE =
  sig
    val select_new_rule : term ordering -> unit
  end

module Select_new_rule =
  functor (Eqt : EQUATION) ->
  functor (EqtSet : EQUATION_SET with type equation = Eqt.t) ->
  functor (RlSet : EQUATION_SET with type equation = regle) ->
  functor (ConjSet : EQUATION_SET) ->
  functor (Compl : COMPLETION_STATE with type equation = Eqt.t
				    and type equation_set = EqtSet.eq_set
				    and type rule_set = RlSet.eq_set
				    and type conjectures_set = ConjSet.eq_set) ->
  functor (EqtSimpl : SIMPLIFICATIONS with type equation = Eqt.t
				      and type equation_set = EqtSet.eq_set
				      and type rule_set = RlSet.eq_set) ->
  functor (ConjSimpl : SIMPLIFICATIONS with type equation = ConjSet.equation
				       and type equation_set = ConjSet.eq_set) ->
    struct

open Eqt;;
open EqtSet;;
open Compl;;
open Messages;;

let interest_of_first_eq_list = function
  [] -> Max
| (eq::_) -> interest1_of eq
;;

let interest_of_first_preoriented_eq = function
  [] -> Max
| (eq::_) -> interest2_of eq
;;


let is_already_present_as_a_rule (u,v) =
  try
    RlSet.find 
      (function rule -> 
	 (not rule.orientation) & 
	 (((rule.l = u) & (rule.r = v)) or
	  ((rule.l = v) & (rule.r = u))))
      etat.rules;
    true
  with Not_found -> false;;

let add_new_rule ordre eq =
  let add_rule eq =
    let u = first_term eq
    and v = second_term eq in
      if options_courantes.ordered_completion &
      	is_already_present_as_a_rule (u,v)
      then ()
      else
	begin
          let rule = make_rule ordre (first_term eq,second_term eq) in
	  let num = rule.num
	  and l = rule.l
	  and r = rule.r in

	    rule.rule_interest <- 
	      begin
		match interest1_of eq with 
		    Some i as i1 -> i1 
		  | _ -> failwith "add_new_rule"
	      end;
(*
              Some (interest_of_date (size_eq l r) etat.date);
*)
	    rule.rule_lhs_interest <- 
	      begin
		match interest2_of eq with
		    Some i as i2 -> i2
		  | _ -> failwith "add_new_rule"
	      end;
(*
	      Some (interest_of_date 
		      (if rule.orientation 
		       then (size_rule l r) 
		       else (min (size_rule l r) (size_rule r l)))
		      etat.date);
*)
	    rule.rule_interest_wrt_goal <- 
	      interest3_of eq;
(*
	    if options_courantes.goal_oriented <> 0
	    then 
	      rule.rule_interest_wrt_goal <-
	      	Some (compute_interest_wrt_goal 
			(match rule.rule_interest with
			     Some i1 -> i1
			   | _ -> failwith "add_new_rule")
			(match rule.rule_lhs_interest with
			     Some i2 -> i2
			   | _ -> failwith "add_new_rule")
			l r);
*)
	    open_hovbox 0;
            if rule.orientation
            then print_string "New rule produced :"
            else print_string "New equation produced :";
	    print_break 2 1;
	    print_regle rule;
	    force_newline ();
	    close_box();
	    print_flush();

    (* Collapse *)

	    let rules,collapsed_rules,collapsed_numbers = 
	      EqtSimpl.collapse ordre etat.rules rule in
	      RlSet.insert_equation rule rules;
	      etat.rules <- rules;
	      etat.disc_net <- List.fold_right 
	 	remove_rule_from_dnet collapsed_numbers etat.disc_net; 
	      etat.disc_net <- add_rule_to_dnet LEFT rule etat.disc_net;
	      if not rule.orientation
	      then
    	    	etat.disc_net <- 
	    	add_rule_to_dnet RIGHT rule etat.disc_net;

	      if (options_courantes.niveau_de_verbosite >= 1)
	      then 
		begin
		  print_string "Collapsed rules became:";
		  print_list_of_equations collapsed_rules;
		  force_newline ();
		  print_flush ()
		end;

	      Print_statistiques.incr_nombre_de_regles_retenues 
		(-(List.length collapsed_rules));

	      let simplified_collapsed_rules = 
		EqtSimpl.simplify ordre collapsed_rules etat.disc_net in

		if (options_courantes.niveau_de_verbosite >= 1)
		then 
		  begin
		    print_string "Simplified collapsed rules:";
		    print_list_of_equations simplified_collapsed_rules;
		    force_newline ();
		    print_flush ()
		  end;
		add_new_eqs simplified_collapsed_rules;

		try (get_rule_of_index etat.major_deduce_index); ()
		with Not_found -> increment_major_deduce_index ();
		  if etat.major_deduce_index = max_int
		  then 
		    begin
		      etat.major_deduce_index <- num;
		      etat.minor_deduce_index <- 0; 
		      increment_minor_deduce_index ();
		      RlSet.set_min_interest1
			(Some (size_eq rule.l rule.r))
			etat.rules;
		      RlSet.set_min_interest2
		 	(Some (if rule.orientation 
		       	       then size_rule rule.l rule.r 
		       	       else  
				 max (size_rule rule.l rule.r) 
				   (size_rule rule.r rule.l)))
			etat.rules;
		      RlSet.set_min_interest3 
			rule.rule_interest_wrt_goal etat.rules;
		    end 
	end
    in
      
      if induction.induction_on
      then 
    	if ground_reducible (first_term eq)
    	then add_rule eq
    	else raise Disproof
      else add_rule eq
;;

let select_new_rule order =
  let eq = next_equation etat.pre_rules 
  and actual_order =
    if options_courantes.ordered_completion 
    then order 
    else (fun _ _ -> Greater) in
    Print_statistiques.incr_nombre_de_regles_produites ();
    Print_statistiques.incr_nombre_de_regles_retenues 1;
     (* Collapse is made in add_new_rule *)
    add_new_rule actual_order eq; 
     (* Compose *)
    let rules = EqtSimpl.compose actual_order etat.rules etat.disc_net in
      etat.rules <- rules;
     (* Let's try solving conjectures *)
      if etat.prove_conjectures
      then 
	begin
	  let conjs,simpl_conjs = 
	    ConjSimpl.simplify_and_split actual_order 
	      etat.conjectures etat.disc_net in
	  let changement = ((ConjSet.card conjs) < 
			    (ConjSet.card etat.conjectures)) in 
	  let new_conjs =
	    List.iter 
	      (function c -> ConjSet.insert_equation c conjs)
	      simpl_conjs;
	    conjs in
	    if ConjSet.is_empty new_conjs
	    then 
      	      begin
		message "All conjectures have been proved !";
		Print_statistiques.print_statistiques();
		force_newline();
		match ask "Should I (s)top or (c)ontinue ? "
		  ["s";"c"]
		  "Please answer 's' or 'c'"
		with
                    "s" -> raise Completion_aborted
		  | "c" -> etat.prove_conjectures <- false
		  | _ -> failwith "Bug in Strategy.completion"
	      end
	    else
              if changement 
              then
                begin
                  print_string "-----------------------------";
	          force_newline();
       	          print_string "Conjectures are now :";
	          force_newline();
       	          ConjSet.print_equation_set new_conjs;
	          force_newline();
	          print_string "-----------------------------";
	          force_newline();
       	          print_flush()
	        end;
	    etat.conjectures <- new_conjs;
	    if (options_courantes.goal_oriented !=0) & changement
	    then update_state_wrt_conj ()
	end;
      
     (* Simplify *)
      let eq,eq_simpl = 
	EqtSimpl.simplify_and_split actual_order
	  etat.eqs_to_process etat.disc_net in
	etat.eqs_to_process <- eq;
	add_new_eqs eq_simpl;
	let eq,eq_simpl =
	  EqtSimpl.simplify_and_split_list
	    actual_order etat.postponed_eqs etat.disc_net in
	  etat.postponed_eqs <- eq;
	  add_new_eqs eq_simpl;
	  let eq,eq_simpl = 
	    EqtSimpl.simplify_and_split 
	      actual_order etat.preoriented_eqs etat.disc_net in
	    etat.preoriented_eqs <- eq;
	    add_new_eqs eq_simpl;
	    let eq,eq_simpl = 
	      EqtSimpl.simplify_and_split 
		actual_order etat.pre_rules etat.disc_net in
	      etat.pre_rules <- eq;
	      add_new_eqs eq_simpl
		
;;

    end;;



module Standard_select_new_rule = 
  Select_new_rule (Equation1) (Equation_set1) 
    (Rules_set) (Conjectures_set) 
    (Standard_completion) (Standard_simplifications) (Conj_simplifications);;

module G_select_new_rule = 
  Select_new_rule (Equation_with_goal) (Equation_with_goal_set) 
    (Rules_set) (Conjectures_set) 
    (G_completion) (G_simplifications) (Conj_simplifications);;
