(***************************************************************************

  Simplifications rules: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: simplifications.ml,v 1.2 2001/04/20 13:42:23 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: simplifications.ml,v 1.2 2001/04/20 13:42:23 marche Exp $

***************************************************************************)

open Format;;
open Messages;;
open Terms;;
open Equations;;
open Equations_as_obj;;
open Generic_normalization;;
open Dnet;;
open Normalised_rewriting;;
open Orderings;;


module type SIMPLIFICATIONS =
    sig
      type equation
      type equation_set
      type rule_set
      exception Trivial_equation
      val normalize : equation list -> equation list
      val simplify : 
	term ordering -> equation list -> int * regle_etendue dnet 
	  -> equation list
      val simplify_and_split :
  	term ordering -> equation_set -> int * regle_etendue dnet 
	  -> equation_set * equation list
      val simplify_and_split_list :
  	term ordering -> equation list -> int * regle_etendue dnet 
	  -> equation list * equation list
      val simplify_and_left_split :
  	term ordering -> equation list -> int * regle_etendue dnet 
	  -> equation list * equation list
      val forced_left_simplify :
  	term ordering -> equation -> int * regle_etendue dnet 
	  -> equation
      val collapse :
	term ordering -> rule_set -> regle 
	  -> rule_set * (equation list) * (int list)
      val compose :
  	term ordering -> rule_set -> int * regle_etendue dnet -> rule_set
    end;;

module Simplifications =
  functor (Eqt : EQUATION) ->
  functor (EqtSet : EQUATION_SET with type equation = Eqt.t) ->
  functor (RlSet : EQUATION_SET with type equation = regle) ->

struct

open Eqt;;
open EqtSet;;

type equation = t;;
type equation_set = EqtSet.eq_set;;
type rule_set = RlSet.eq_set;;

exception Trivial_equation;;

(*
(normalize se) $S$-normalise les \'equations de $se$, et les efface
si elles deviennent triviales, et retourne l'ensemble des \'equations
obtenues.
*)

let normalize se = 
  let normalize_and_check_if_trivial eq = 
    let u = first_term eq
    and v = second_term eq in
    if get_norm_wrt_rule eq < 0
    then 
      let u1 = normalisation_par_la_theorie_courante u
      and v1 = normalisation_par_la_theorie_courante v in 
	if egaux_ac u1 v1
        then failwith "trivial_for_normalize"
        else 
	  begin
	      reset_interest eq;
	      reset_orientation eq;
	      set_first_term u1 eq;
	      set_second_term v1 eq;
	      set_norm_wrt_rule 0 eq;
	      eq
	  end
    else 
      if egaux_ac u v
      then failwith "trivial_for_normalize"
      else eq
  in 
      List.fold_left 
	(fun res eq -> 
	   try 
	     let eq' = normalize_and_check_if_trivial eq
	     in
	       eq'::res
	   with Failure "trivial_for_normalize" -> res)
	[] se
;;


let max_num_rules (_,rule_dnet) =
  let rec max_num_rules2 max = function 
      [] -> max
    | re::sre -> 
      	let num_rule = re.regle.num 
	in
      	  if num_rule > max
      	  then max_num_rules2 num_rule sre
      	  else max_num_rules2 max sre

  in
    max_num_rules2 0 (liste_des_indices_du_dnet rule_dnet)
;;

(*
(simplify se sr dn) normalise les \'equations de $se$ par $sr/S$, et les efface
si elles deviennent triviales, et retourne l'ensemble des \'equations
obtenues. dn est le reseau de discrimination associe a dn
*)

let rec simplify ordre se rdn = 
  let max = max_num_rules rdn
  in 
  let simplify_and_check_if_trivial eq =
    let u = first_term eq
    and v = second_term eq
    and n = get_norm_wrt_rule eq 
    in
      if is_a_trivial_pair_of_terms rdn (u,v)
      then failwith "trivial_for_simplify"
      else 
      	begin
      	  let u1 = 
	    normalisation_normalisee_par_un_systeme_de_regles 
	      n ordre u rdn 
       	  and v1 = 
	    normalisation_normalisee_par_un_systeme_de_regles 
	      n ordre v rdn in
	    if is_a_trivial_pair_of_terms rdn (u1,v1)
	    then failwith "trivial_for_simplify"
	    else 
	      begin
		  set_first_term u1 eq;
		  set_second_term v1 eq;
               	  set_norm_wrt_rule max eq;
		  reset_interest eq;
		  reset_orientation eq;
               	  eq
              end
      	end
    in 
      List.fold_left 
	(fun res eq -> 
	   try 
	     let eq' = simplify_and_check_if_trivial eq
	     in
	       eq'::res
	   with Failure "trivial_for_simplify" -> res)
	[] se
;;

(*
let simplify = 
  Time_profiler.profile2
    "Simplifications.simplify"
    simplify
;;
*)

(*
(simplify_and_split se sr dn) normalise les \'equations de $se$ par $sr$, et
les efface si elles deviennent triviales, et retourne l'ensemble des
\'equations obtenues en deux parties~: la premi\`ere est l'ensemble
des \'equations qui n'ont pas \'et\'e simplifi\'ees, et la deuxi\`eme
les autres. $dn$ est le reseau de discrimination associe a $sr$.
*)

let simplify_and_split ordre se rdn = 
  let max = max_num_rules rdn in
    fold_left 
      (fun (eq_set,eq_list as res) e ->
	 let u = first_term e
	 and v = second_term e
	 and n = get_norm_wrt_rule e in
	   try 
	     let (u',v') = (normalise_pair n ordre u v rdn) in 
	       if is_a_trivial_pair_of_terms rdn (u',v')
	       then res
	       else 
		 begin
		   reset_interest e;
		   reset_orientation e;
		   set_first_term u' e;
		   set_second_term v' e;
		   set_norm_wrt_rule max e;
		   (eq_set,e::eq_list)
		 end
	   with Irreducible -> 
	     begin
	       set_norm_wrt_rule max e;
	       insert_equation e eq_set;
	       res
	     end)
      (make_empty_equation_set (), []) se
;;

let simplify_and_split_list ordre se rdn = 
  let max = max_num_rules rdn in
    List.fold_right 
      (fun e (eqs,simpl_eqs as res) ->
	 let u = first_term e
	 and v = second_term e
	 and n = get_norm_wrt_rule e in
	   try 
	     let (u',v') = (normalise_pair n ordre u v rdn) in 
	       if is_a_trivial_pair_of_terms rdn (u',v')
	       then res
	       else 
		 begin
		   reset_interest e;
		   reset_orientation e;
		   set_first_term u' e;
		   set_second_term v' e;
		   set_norm_wrt_rule max e;
		   (eqs,e::simpl_eqs)
		 end
	   with Irreducible -> 
	     begin
	       set_norm_wrt_rule max e;
	       (e::eqs,simpl_eqs)
	     end)
      se ([],[])
;;

let simplify_and_left_split ordre se rdn = 
  let max = max_num_rules rdn in
  let rec simplify_and_left_split_partage = function
      [] -> ([],[])
    | (eq::se) -> 
 	let (se1,se2) as se1_se2 = simplify_and_left_split_partage se 
	and u = first_term eq
	and v = second_term eq
	and n = get_norm_wrt_rule eq in 
 	  try 
	    let u' = normalisation_forcee_par_un_systeme_de_regles 
	      	       n ordre u rdn 
	    and v' = normalisation_normalisee_par_un_systeme_de_regles 
	      	n ordre v rdn in
	      if is_a_trivial_pair_of_terms rdn (u',v')
	      then se1_se2
	      else 
	    	begin
		  set_first_term u' eq;
		  set_second_term v' eq;
		  reset_interest eq;
		  reset_orientation eq;
		  set_norm_wrt_rule max eq;
		  (se1,eq::se2)
	    	end
 	  with Irreducible -> 
	    try 
	      let v' = normalisation_forcee_par_un_systeme_de_regles 
			 n ordre v rdn in
	    	if is_a_trivial_pair_of_terms rdn (u,v')
	    	then se1_se2
	    	else
		  begin
		    set_second_term v' eq;
		    reset_interest eq;
(*
		    eq#reset_orientation;
*)
            	    set_norm_wrt_rule max eq;
            	    ((eq::se1),se2)
		  end
	    with Irreducible -> 
	      begin 
              	set_norm_wrt_rule max eq;
              	(eq::se1,se2)
              end
  	in 
	  simplify_and_left_split_partage se
;;


let forced_left_simplify ordre eq rule_dnet = 
  let max = max_num_rules rule_dnet
  and u = first_term eq
  and v = second_term eq
  and n = get_norm_wrt_rule eq in
    try 
      let u' = normalisation_forcee_par_un_systeme_de_regles 
		 n ordre u rule_dnet
      and v' = normalisation_normalisee_par_un_systeme_de_regles 
		 n ordre v rule_dnet in
    	if is_a_trivial_pair_of_terms rule_dnet (u',v')
    	then raise Trivial_equation
    	else 
	  begin
	    set_first_term u' eq;
	    set_second_term v' eq;
	    reset_orientation eq;
	    reset_interest eq;
            set_norm_wrt_rule max eq;
            eq
	  end
	  
    with Irreducible -> 
      let v' = normalisation_forcee_par_un_systeme_de_regles 
	  n ordre v rule_dnet in
    	if is_a_trivial_pair_of_terms rule_dnet (u,v')
    	then raise Trivial_equation
    	else 
	  begin
	    set_second_term v' eq;
	    reset_orientation eq;
	    reset_interest eq;
            set_norm_wrt_rule max eq;
            eq
	  end
;;

(*===================

(collapse sr r) collapse les r\`egles de $sr$ par la r\`egle $r$, et
retourne un couple $(sr',se)$ o\`u $sr'$ est l'ensemble des r\`egles non
modifi\'ees, et $se$ l'ensemble des \'equations obtenues par
effondrement.

===================*)

let collapse ordre set_of_rules r2 = 
  let max = r2.num in
  let dnet_for_r2 = 
    match r2.orientation with
      	true -> create_dnet LEFT r2 
      | false -> add_rule_to_dnet RIGHT r2 (create_dnet LEFT r2) in
  RlSet.fold_left
    (fun (sr1,se1,sn1 as res) rule ->
       match rule.orientation with
	   true -> 
	     begin
	       try 
		 let u' = normalisation_forcee_par_un_systeme_de_regles
			    0 ordre rule.l dnet_for_r2 in
		   begin
      	             open_hovbox 0;
      	             print_string "Rule";
                     print_break 1 0;
                     print_regle rule;
                     print_break 1 0;
      	             print_string "collapsed.";
                     force_newline ();
                     close_box();
                     print_flush();
		     let col_rule = make_equation u' rule.r in
      	               sr1, col_rule::se1,rule.num::sn1 
	           end
	       with Irreducible -> 
		 RlSet.insert_equation rule sr1;
		 res
	     end
	     
	 | false -> 
	     begin
	       try 
		 let u' = normalisation_forcee_par_un_systeme_de_regles
			    0 ordre rule.l dnet_for_r2 in
		   begin
      	             open_hovbox 0;
      	             print_string "Rule";
                     print_break 1 0;
                     print_regle rule;
                     print_break 1 0;
      	             print_string "collapsed.";
                     force_newline ();
                     close_box();
                     print_flush();
		     let col_rule = make_equation u' rule.r in
      	               sr1, col_rule::se1, rule.num::sn1 
		   end
	       with Irreducible ->               
		 try 
		   let v' = normalisation_forcee_par_un_systeme_de_regles
			      0 ordre rule.r dnet_for_r2 in 
                     begin
      		       open_hovbox 0;
      		       print_string "Rule";
		       print_break 1 0;
		       print_regle rule;
		       print_break 1 0;
      		       print_string "collapsed.";
		       force_newline ();
		       close_box();
		       print_flush();
		       let col_rule = make_equation rule.l v' in
      			 sr1, col_rule::se1, rule.num::sn1 
	             end
		 with Irreducible -> 
		   RlSet.insert_equation rule sr1;
		   res
	     end)
    (let empty_set = RlSet.make_empty_equation_set () in
       RlSet.set_min_interest1 (RlSet.min_interest1 set_of_rules) empty_set;
       RlSet.set_min_interest2 (RlSet.min_interest2 set_of_rules) empty_set;
       RlSet.set_min_interest3 (RlSet.min_interest3 set_of_rules) empty_set;
       (empty_set,[],[]))
    set_of_rules
;;
		
(*===================

(compose sr1 dn2) compose les r\`egles de $sr_1$ par $dn_2$, et
retourne l'ensemble des r\`egles obtenues. 

===================*)

let compose ordre sr1 dn2 =
  RlSet.fold_left
    (fun res rule ->
       let rule' =
	 match rule.orientation with
	     false -> rule
	   | true ->
	       try
	    	 let v1 = normalisation_forcee_par_un_systeme_de_regles 
		  	    (-1) ordre rule.r dn2 in
		   {
		     num = rule.num;
		     orientation=rule.orientation;
      		     x=rule.x;
      		     y=rule.y;
      		     l=rule.l;
      		     r=v1;
       		     rule_interest = rule.rule_interest;
       		     rule_interest_wrt_goal = rule.rule_interest_wrt_goal;
       		     rule_lhs_interest= rule.rule_lhs_interest 
		   }
	       with Irreducible -> rule in
	 RlSet.insert_equation rule' res;
	 res)
    (let empty_set = RlSet.make_empty_equation_set () in
       RlSet.set_min_interest1 (RlSet.min_interest1 sr1) empty_set;
       RlSet.set_min_interest2 (RlSet.min_interest2 sr1) empty_set;
       RlSet.set_min_interest3 (RlSet.min_interest3 sr1) empty_set;
       empty_set) 
    sr1
;;

end;;

module Standard_simplifications = 
  Simplifications (Equation1) (Equation_set1) (Rules_set);;

module G_simplifications = 
  Simplifications (Equation_with_goal) (Equation_with_goal_set) (Rules_set);;

module Conj_simplifications =
  Simplifications (Conjectures) (Conjectures_set) (Rules_set);;



