(***************************************************************************

(This sentence has been added automatically, it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: simplifications.mli,v 1.1 2000/05/16 17:36:48 monate Exp $

***************************************************************************)

(* 

Simplifications rules: specifications

Last modified by: $Author: monate $
File name:        $Source: /users/demons/demons/master-sources-repository//cime/ocaml_src/oldcimroot/simplifications.mli,v $
Revision:         $Revision: 1.1 $ 
Last modified on: $Date: 2000/05/16 17:36:48 $

*)


open Terms;;
open Equations;;
open Equations_as_obj;;
open Dnet;;
open Orderings;;

module type SIMPLIFICATIONS =
    sig
      
type equation
type equation_set
type rule_set

exception Trivial_equation
(*
  (normalize E) $S$-normalise les \'equations de $E$, et les efface
  si elles deviennent triviales, et retourne l'ensemble des \'equations
  obtenues.
  NB : les \'equations argument sont modifi\e'es en place.
*)

val normalize : equation list -> equation list

(*
  (simplify E R D) normalise les \'equations de $E$ par $R$, et les efface
  si elles deviennent triviales, et retourne l'ensemble des \'equations
  obtenues. $D$ doit \^etre le r\'eseau de discrimination associ\'e \`a $R$.
  NB : les \'equations argument sont modifi\e'es en place.
*)

val simplify : 
  term ordering -> equation list -> int * regle_etendue dnet 
    -> equation list
(*
  (simplify_and_split E R DN) normalise les \'equations de $E$ par $R$, et
  les efface si elles deviennent triviales, et retourne l'ensemble des
  \'equations obtenues en deux parties~: la premi\`ere est l'ensemble
  des \'equations qui n'ont pas \'et\'e simplifi\'ees, et la deuxi\`eme
  les autres. $DN$ doit etre le reseau de discrimination associe a $R$.
  NB : les \'equations argument sont modifi\e'es en place.
*)

val simplify_and_split :
  term ordering -> equation_set -> int * regle_etendue dnet 
    -> equation_set * equation list

val simplify_and_split_list :
  term ordering -> equation list -> int * regle_etendue dnet 
    -> equation list * equation list


(*
  (simplify_and_left_split E R) normalise les \'equations de $E$ par $R$, et
  les efface si elles deviennent triviales, et retourne l'ensemble des
  \'equations obtenues en deux parties~: la premi\`ere est l'ensemble
  des \'equations qui n'ont pas \'et\'e simplifi\'ees a gauche, et la 
  deuxi\`eme les autres.
  NB : les \'equations argument sont modifi\e'es en place.
*)

val simplify_and_left_split :
  term ordering -> equation list -> int * regle_etendue dnet 
    -> equation list * equation list

(*
  (forced_left_simplify eq R DN) retourne l'equation obtenue en simplifiant
  eq par R (vide si eq devient triviale), leve l'exception Irreducible
  si eq n'est pas reductible. $DN$ doit \^etre le r\'eseau de dicrimination 
  associe \`a $R$.
*)

val forced_left_simplify :
  term ordering -> equation -> int * regle_etendue dnet 
    -> equation

(*
  (collapse R r) collapse les r\`egles de $R$ par la r\`egle $r$, et
  retourne un couple $(R',E,N)$ o\`u $R'$ est l'ensemble des r\`egles non
  modifi\'ees, et $E$ l'ensemble des \'equations obtenues par
  effondrement et $L$ la liste de leurs numeros.
*)

val collapse : 
  term ordering -> rule_set -> regle ->
    rule_set * (equation list) * (int list);;

(*
(compose R1 DN2) compose les r\`egles de $R_1$ par $DN_2$, et
retourne l'ensemble des r\`egles obtenues. 
*)

val compose :
  term ordering -> rule_set -> int * regle_etendue dnet -> rule_set;;

    end;;

module Standard_simplifications : 
  (SIMPLIFICATIONS with type equation = Equation1.t
		   and type equation_set = Equation_set1.eq_set
		   and type rule_set = Rules_set.eq_set);;

module G_simplifications : 
  (SIMPLIFICATIONS with type equation = Equation_with_goal.t
		   and type equation_set = Equation_with_goal_set.eq_set
		   and type rule_set = Rules_set.eq_set);;

module Conj_simplifications :
  (SIMPLIFICATIONS with type equation = Conjectures.t
		   and type equation_set = Conjectures_set.eq_set
		   and type rule_set = Rules_set.eq_set);;









