(***************************************************************************

  Completion strategy: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: strategy.ml,v 1.2 2001/04/20 13:42:23 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  Last modified by: $Author: marche $
  Revision:         $Revision: 1.2 $ 
  Last modified on: $Date: 2001/04/20 13:42:23 $

***************************************************************************)


open Format
open Messages
open Args
open Content
open Terms
open Equations
open Equations_as_obj
open Equation1
open Equation_set1
open Orderings
open Symmetrization
open Standard_symmetrization
open Ord_interactive
open Stack
open Normalised_rewriting
open Simplifications
open Standard_simplifications
open Completion_state
open Standard_completion
open Symmetrise
open Standard_symmetrise
open Orient
open Standard_orient
open Select_new_rule
open Standard_select_new_rule
open Deduce
open Standard_deduce


type transition_rule =
    SYMMETRISE
  | ORIENT
  | SELECT_NEW_RULE
  | DEDUCE



let indice_du_min ordre l =
  let rec indice_du_min_aux ind_min min ind_cour = function
      [] -> ind_min
    | cour::t -> 
     	if ordre cour min = Less_than
     	then indice_du_min_aux ind_cour cour (ind_cour+1) t
     	else indice_du_min_aux ind_min min (ind_cour+1) t 
  in
    indice_du_min_aux (-1) Max 0 l


let what_to_do () = 
  if options_courantes.orientation_prioritaire
  then 
    if not (is_empty etat.eqs_to_process)
    then SYMMETRISE
    else 
      if  not (is_empty etat.preoriented_eqs)
      then ORIENT
      else 
	if not (is_empty etat.pre_rules)
        then SELECT_NEW_RULE
	else DEDUCE
	  
  else 
    match indice_du_min compare_i1 [min_interest1 etat.eqs_to_process;
				    min_interest1 etat.preoriented_eqs;
				    min_interest1 etat.pre_rules;
				    Rules_set.min_interest1 etat.rules]
    with
      	0 -> SYMMETRISE
      | _ -> 
	  begin
            match indice_du_min compare_i2 [min_interest2 etat.preoriented_eqs;
					    min_interest2 etat.pre_rules;
					    Rules_set.min_interest2 etat.rules]
	    with
              	0 -> ORIENT
	      | 1 -> SELECT_NEW_RULE
	      | _ -> DEDUCE
          end
	  
	


let steps_to_proceed = ref 0

let orientation ordre eq = 
  let u=eq#first_term 
  and v=eq#second_term in
    match ordre u v with
    	Greater -> u,v
      | Less_than -> v,u
      | _ -> raise Non_orientable

  

let orientation_seule ordre l =
  List.map
    (function eq -> 
       let u_v = (orientation ordre eq) in
	 (make_rule ordre u_v)) l



(*=====================================

Boucle principale de la compl\'etion

=======================================*)

let completion order eq_list conj_list = 

  config_completion_state ();
  
  etat.conjectures <- 
  List.fold_left 
    (fun res conj -> 
       let u = conj#first_term
       and v = conj#second_term in
       let u1 = normalisation_par_la_theorie_courante u
       and v1 = normalisation_par_la_theorie_courante v in 
	 if egaux_ac u1 v1
	 then res
	 else
	   let conj' = Conjectures.make_equation u1 v1 in
	     Conjectures_set.insert_equation conj' res;
	     res)
    (Conjectures_set.make_empty_equation_set ()) conj_list;
  etat.prove_conjectures <- not (Conjectures_set.is_empty etat.conjectures);
  
  add_new_eqs 
    (normalize 
       (List.map 
	  (function eq -> 
	     let eq' = make_aged_equation etat.date 
			 eq#first_term eq#second_term in
	       set_interest2 No_value eq';
	       eq') 
	  eq_list));
  
  try 
    
    while (not (is_empty etat.eqs_to_process)) or 
          (not (is_empty etat.preoriented_eqs)) or
          (not (is_empty etat.pre_rules)) or
          (etat.major_deduce_index < max_int)
    do
      begin
      	etat.date <- succ etat.date;
      	
      	if (options_courantes.afficher_etats) 
      	then 
    	  begin
	    if (!steps_to_proceed <= 1) then afficher_etat ();
	    if !steps_to_proceed <= 1
	    then 
	      begin
              	message 
		  "n<Return> to proceed n steps (default 1), q to interrupt";
      	      	let n= read_line() in 
		  if n="q" 
      		  then raise Completion_aborted
		  else steps_to_proceed := 
		    try int_of_string(n)
		    with Failure _ -> 1
	      end
	    else steps_to_proceed := !steps_to_proceed - 1;
	    ()
    	  end;
    
      	match what_to_do () with
    	    SYMMETRISE -> symmetrise order
	  | ORIENT -> orient order
	  | SELECT_NEW_RULE -> select_new_rule order
	  | DEDUCE -> deduce order
	    
      end
    done;

    if etat.postponed_eqs <> []
    then 
      begin
      	print_string "Only unorientable equations left:";
      	force_newline ();
      	print_list_of_equations etat.postponed_eqs;
      	force_newline ();
      	print_flush ();
      	raise Completion_failed
      end;
    
    if etat.prove_conjectures
    then message "Warning : not all conjectures were proved";
    
    (Rules_set.set_to_list etat.rules), etat.disc_net
      
  with ex -> 
    begin
(*
      afficher_etat ();
*)
      raise ex    
    end


(*
let completion =
  Time_profiler.profile3
    "Strategy.completion"
    completion
*)




