(***************************************************************************

Substitutions: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: substitution.ml,v 1.1 2000/05/16 17:36:49 monate Exp $

***************************************************************************)

open Symbols;;
open Terms;;
open Positions;;

(*

a substitution is a map from a finite set of ids of variables into terms

*)

type substitution = (var_id,term) VarMap.t ;;

(*

[build_subst [(x1,t1);..;(xn,tn)]] builds the substitution
x1->t1,..xn->tn

*)

let build_subst l = 
  List.fold_left 
    (fun theta (x,t) -> 
       try 
	 let _ = VarMap.find x theta
	 in raise (Invalid_argument "build_subst")
       with Not_found -> VarMap.add x t theta)
    VarMap.empty
    l
;;


let subst x t = VarMap.add x t VarMap.empty;;


let substitute_assoc theta s  = 
let rec substitute_rec = function
   ((VAR n) as v) -> 
      (try (List.assoc n theta) with Not_found -> v)
 | TERM(f,l) -> 
      if (is_ac f)
      	 then let rec substitute_and_flatten = function
                  [] -> []
                | t::m -> begin
      	       	       	  match (substitute_rec t) with
                      	    VAR _ as v -> v::(substitute_and_flatten m)
                      	  | TERM(g,l) as u ->
                              if (f=g)
      	       	       	       	 then l@(substitute_and_flatten m)
       	       	                 else u::(substitute_and_flatten m) 
			  end
              in TERM(f,substitute_and_flatten l)
      	 else TERM(f,List.map substitute_rec l)
in  substitute_rec s
;;

(*
let substitute_assoc =
  Time_profiler.profile2
    "Substitution.substitute_assoc"
    substitute_assoc
;;
*)

let substitute theta s  = 
(*
  printf "Je substitue par %s dans le terme %s\n" (string_of_substitution theta) (string_of_term s); 
  flush stdout;
*)
let rec substitute_rec = function
   (VAR n) as v -> 
     begin
       try 
	 VarMap.find n theta
       with Not_found -> v
     end
 | TERM(f,l) -> 
     if (is_ac f)
     then let rec substitute_and_flatten = function
         [] -> []
       | t::m -> begin
      	   match (substitute_rec t) with
               VAR _ as v -> v::(substitute_and_flatten m)
             | TERM(g,l) as u ->
                 if (f=g)
      	       	 then l@(substitute_and_flatten m)
       	       	 else u::(substitute_and_flatten m) 
	 end
     in TERM(f,substitute_and_flatten l)
     else TERM(f,List.map substitute_rec l)
       
in let t = substitute_rec s in
(*
  printf "Resultat : %s\n" (string_of_term t);
  flush stdout;
*)
  t

;;


let replace_and_substitute theta context p t =
(substitute theta (replace context t p))
;;

(*
let replace_and_substitute = 
  Time_profiler.profile4 
    "Substitution.replace_and_substitute" 
    replace_and_substitute;;
*)


let instancie = substitute;;

let instancie_equation theta (s,t) = 
      ((instancie theta s),(instancie theta t));;

let instancie_liste_equations theta p = 
    List.map (instancie_equation theta) p;;






let string_of_substitution theta =
  let s = ref "{" in 
   VarMap.iter 
     (fun x t -> s := !s^","^(string_of_term (VAR x))^
      	"->"^(string_of_term t)) theta;
    !s^"}"
;;

open Format;;

let print_substitution theta =
   begin
   open_hvbox 0;
   let est_vide = ref true in 
   VarMap.iter 
     (fun x t -> 
	begin
          est_vide:=false;
          print_term (VAR x);
          print_string "->";
          print_break 0 0;
      	  open_hovbox 0;
          print_term t;
          close_box ();
          print_break 2 0
        end) 
     theta;
     if !est_vide then print_string "id";
     close_box ()
   end;;
