(***************************************************************************

Substitutions: specifications

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: substitution.mli,v 1.1 2000/05/16 17:36:49 monate Exp $

***************************************************************************)

open Symbols;;
open Positions;;
open Terms;;

(*

a substitution is a map from a finite set of ids of variables into terms

*)

type substitution = (var_id,term) VarMap.t ;;

(*

[build_subst [(x1,t1);..;(xn,tn)]] builds the substitution
x1->t1,..xn->tn

*)

val build_subst : (var_id * term) list -> substitution;;

(*

[subst x t] builds the elementary substitution x->t

*)

val subst : var_id -> term -> substitution;;

(*

[substitute theta t] applies the substitution [theta] to the term [t] 

IMPORTANT: this function takes care about flattening w.r.t. to AC
symbols, that is if [theta] and [t] are flat the result is also
flat. For example, [substitute {x->a+b} x+c] returns [a+b+c] and not
[(a+b)+c]

*)

val substitute : 
  substitution -> term 
  -> term
;;

val substitute_assoc : 
  (var_id * term) list -> term 
  -> term
;;

(*

[replace_and_substitute theta C p t] builds $\theta(C[t]_p)$ 

IMPORTANT: this function also takes care about flattening w.r.t. to AC
symbols. For example, [replace_and_substitute {x->a+b} x+f(a+b)
[2.1.1] c+d] returns [a+b+f(c+d+b)]

*)

val replace_and_substitute : 
  substitution -> term -> position -> term 
  -> term
;;

val instancie : substitution -> term -> term;;

val instancie_equation : substitution -> (term * term) -> (term * term);;

val instancie_liste_equations : 
      substitution -> (term * term) list -> (term * term) list;;



(*

  conversions vers chaines

*)

val string_of_substitution :
  substitution 
  -> string
;;

(*

  affichages dans des boites

*)

val print_substitution :
  substitution 
  -> unit
;;

