(***************************************************************************

  Basic definitions about symbols: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: symbols.ml,v 1.3 2001/04/20 13:42:23 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  Last modified by: $Author: marche $
  Revision:         $Revision: 1.3 $ 
  Last modified on: $Date: 2001/04/20 13:42:23 $

***************************************************************************)

open C_declare_operator 
open Printf
open Arg
open Errors
open Format


(*===========================

abstract types for symbols

============================*)

type symbol_id = int
type var_id = int
		
type fix_type = 
    PREFIX | INFIX | POSTFIX | DEFAULT
      

open Orderings_generalities
      
type precedence = symbol_id ordering

type table_de_precedences = comparison_result array array
			      
type precedence_description =
    One_symbol of symbol_id
  | Equal_to of symbol_id * precedence_description
  | Larger_than of symbol_id * precedence_description
  | Lower_than of symbol_id * precedence_description

(*==============================

Pr\'ec\'edence totale engendr\'ee par une cha\^{\i}ne d'op\'erateurs.

(prec_totale [f1;...;fn]) est la pr\'ec\'edence $f_1>\cdots>f_n$.

==============================*)

(*&&&*)
type sorte_de_base  = string 
type sorte = (sorte_de_base list*sorte_de_base)
(*&&&&*)


let not_a_symbol_id = -1
let not_a_var_id = -1
		     
let is_a_var_id x = x>=0
		      
let leftify_var x = x+x

let rightify_var x = x+x+1

(**)
let unleftify_var x = x/2
			
let unrightify_var x = x/2
			 
let is_a_left_variable x = (x mod 2) = 0

let is_a_right_variable x = (x mod 2) = 1
(**)

(*
let fresh_var_ids n = fresh_var_ids_rec 0 n
where rec fresh_var_ids_rec i n =
  if n=0 then []
         else i::(fresh_var_ids_rec (i+1) (n-1))

*)


let fresh_new_var () = 0


let new_var e = 1+(List.fold_right max e (-1))


let next_new_var x = x+1


let int_of_symbol_type = function
    VARIABLE -> 0
  | AC -> 1
  | C -> 2
  | FREE -> 3




let compare_symbol_ids f g = f - g

module Symbol_id_ord =
struct 
  type t = symbol_id 
  let compare = compare_symbol_ids 
end

module SymbolSet = Ordered_sets.Make(Symbol_id_ord)
		     
module SymbolMap = Ordered_maps.Make(Symbol_id_ord)

let compare_var_ids x y = x - y

module VarOrd =
struct 
  type t = var_id 
  let compare = compare_var_ids 
end

module VarSet = Ordered_sets.Make(VarOrd)
		  
module VarMap = Ordered_maps.Make(VarOrd)


let compare_typed_symbols (t1,f1) (t2,f2) = 
  let x = (int_of_symbol_type t1) - (int_of_symbol_type t2) in
  if x = 0 
  then compare_symbol_ids f1 f2
  else x


let leq_symbol_ids f g = f <= g



type type_table_de_symboles = 
    {
      mutable symboles_de_fonction : 
	(string * symbol_type * int * fix_type * (sorte list)) array;
      mutable variables : (string*(sorte list)) array
    }

let table_des_symboles =
  {
    symboles_de_fonction = [||];
    variables       = [||]
  }


let free_symbols_table () =
  table_des_symboles.symboles_de_fonction <- [||];
  table_des_symboles.variables <- [||]


let is_free f = 
  match table_des_symboles.symboles_de_fonction.(f) with 
      (_,FREE,_,_,_) -> true
    | _ -> false

let is_ac f = 
  match table_des_symboles.symboles_de_fonction.(f) with 
      (_,AC,_,_,_) -> true
    | _ -> false
	  
let is_c f = 
  match table_des_symboles.symboles_de_fonction.(f) with 
      (_,C,_,_,_) -> true
    | _ -> false
	  

let string_of_symbol_id f = 
  if f>=0 & f < (Array.length table_des_symboles.symboles_de_fonction)
  then 
    match table_des_symboles.symboles_de_fonction.(f) with 
	(s,_,_,_,_) -> s
  else if f=(-1) then "omega" else failwith "Bug in Symbol.string_of_symbol_id"


let print_symbol_id f =
  print_string (string_of_symbol_id f)


let print_symbol_id_debug f =
  print_string (string_of_int f)


let print_var_id_debug f =
  print_string (string_of_int f)


let string_of_var_id x =
  if x>=0 & x < (Array.length table_des_symboles.variables)
  then fst (table_des_symboles.variables.(x))
  else "V_"^string_of_int(x)


let string_of_typed_symbol = function
    (VARIABLE,x) -> string_of_var_id x
  | (_,f)   -> string_of_symbol_id f
	


exception Trouve of symbol_type * symbol_id
  
let recherche_symbole s =
  try
    for i=0 to (Array.length table_des_symboles.symboles_de_fonction)-1 do
      match table_des_symboles.symboles_de_fonction.(i) with
          (x,t,_,_,_) -> 
	    if s=x
	    then raise (Trouve (t,i))
    done;
    for i=0 to (Array.length table_des_symboles.variables)-1 do
      if (fst table_des_symboles.variables.(i))=s
      then raise (Trouve (VARIABLE,i))
    done;
    raise Not_found
      
  with
      Trouve (t,f) -> t,f
	

(*
let typed_symbol_of_string s =
   try recherche_symbole s
   with Not_found ->
   semantical_error ("symbole '" ^ s ^ "' non declare")
*)

let symbol_id_of_string s =
(*
   try 
*)
  match recherche_symbole s with 
      (VARIABLE,_) -> raise Not_found
    | (_,f) -> f
(*
   with Not_found ->
   semantical_error ("symbole '" ^ s ^ "' non declare")
*)


let var_id_of_string s =
  match recherche_symbole s with 
      (VARIABLE,v) -> v
    | _ -> raise Not_found


(*
let definir_operateur t n fix srt s = 
  try 
    recherche_symbole s;
    semantical_error ("redeclared symbol '" ^ s ^ "'")
  with Not_found -> match t with
      VARIABLE   -> 
	
                  (*printf "'%s' declare comme variable\n" s;*)
        
        table_des_symboles.variables <-
        Array.append table_des_symboles.variables [|(s,srt)|]
    | t     -> (*
		 printf "'%s' declare comme operateur " s;
		 if t=AC
                 then printf "AC\n"
                 else printf "libre d'arite %d\n" n;
               *)
	let fix' = 
          if fix = DEFAULT
          then match t with
	      C | AC -> INFIX
            | _ -> PREFIX
          else fix
        in
(* declare operator to C *)
(**)
	  c_declare_operator t (Array.length 
				  table_des_symboles.symboles_de_fonction);
(**)
          table_des_symboles.symboles_de_fonction <-
          Array.append table_des_symboles.symboles_de_fonction 
            [|(s,t,n,fix',srt)|]

*)


let arity f = 
  match table_des_symboles.symboles_de_fonction.(f)
  with (_,_,a,_,_) -> a


let is_constant f = (arity f = 0)
		      
let is_unary f = (arity f = 1)

(*
let arity_of_typed_symbol = function
   (VARIABLE,_) -> 0
 | (_,f) -> arity f

*)

let is_infix f = 
  match table_des_symboles.symboles_de_fonction.(f)
  with (_,_,_,INFIX,_) -> true
    | _ -> false


let is_postfix f = 
  match table_des_symboles.symboles_de_fonction.(f)
  with (_,_,_,POSTFIX,_) -> true
    | _ -> false


let plus_grand_numero_de_symbole () =
  (Array.length table_des_symboles.symboles_de_fonction) - 1


let get_symbol_id s =
  try symbol_id_of_string s
  with Not_found ->
    try var_id_of_string s;
      semantical_error ("variable symbol '" ^ s ^ "' cannot be used as a function symbol")
    with Not_found ->
      semantical_error ("undeclared symbol '" ^ s ^ "'")



(*=======================================

Symboles TOP : chaque symbole possede un symbole TOP associe, pour
faire les paires de dependances (on appelle cela la `top_copy' d'un symbole) 

=========================================*)


let top_copy_assoc_list = ref ([] : (symbol_id * symbol_id) list)

let definir_operateur t n fix sorte s = 
  try 
    recherche_symbole s;
    semantical_error ("redeclared symbol '" ^ s ^ "'")
  with 
      Not_found -> 
	match t with
	    VARIABLE   -> (*
			    printf "'%s' declare comme variable\n" s;
			  *)
              table_des_symboles.variables <-
              Array.append table_des_symboles.variables [|s,sorte|]
	  | t     -> (*
		       printf "'%s' declare comme operateur " s;
		       if t=AC
                       then printf "AC\n"
                       else printf "libre d'arite %d\n" n;
             	     *)
	      let fix' = 
		if fix = DEFAULT
		then match t with
		    C | AC -> INFIX
		  | _ -> PREFIX
		else fix
	      and top_copy_name = "TOP("^s^")"
	      and s_id = (Array.length table_des_symboles.symboles_de_fonction)
              in
(* declare operator to C *)
(**)
		c_declare_operator t s_id;
		c_declare_operator t (s_id+1);
(**)
		table_des_symboles.symboles_de_fonction <-
		Array.append table_des_symboles.symboles_de_fonction 
		  [|(s,t,n,fix',sorte);(top_copy_name,t,n,fix',sorte)|];
		top_copy_assoc_list := (s_id,s_id+1)::!top_copy_assoc_list



let top_copy s_id =
  try
    List.assoc s_id !top_copy_assoc_list
  with Not_found -> failwith "bug dans top_copy"


(* bidouille infaaaame *)
let is_a_top_copy x = x mod 2 = 1







(*=======================================

Ordres sur les symboles

=========================================*)


(*==============================

Pr\'ec\'edence totale engendr\'ee par une cha\^{\i}ne d'op\'erateurs.

(prec_totale [f1;...;fn]) est la pr\'ec\'edence $f_1>\cdots>f_n$.

==============================*)

let rec prec_totale prec_list f1 f2 =
  if f1=f2
  then Equivalent
  else prec_tot_aux f1 f2 prec_list

and prec_tot_aux f1 f2 = function
    [] -> Uncomparable
  | f::fl ->
      if f=f1
      then
	if List.mem f2 fl
	then Greater_than
	else Uncomparable
      else 
	if f=f2
	then 
	  if List.mem f1 fl
	  then Less_than
	  else Uncomparable
	else prec_tot_aux f1 f2 fl


(*==============================

Pr\'ec\'edence partielle engendr\'ee par une table

==============================*)

let precedence_par_table table f g = 
(*
    ((VARIABLE,f),(VARIABLE,g)) -> if f=g 
			    then EQUIVALENT
			    else INCOMPARABLE
  | ((VARIABLE,_),_)       -> INCOMPARABLE
  | (_,(VARIABLE,_))       -> INCOMPARABLE
  | ((_,f),(_,g))     -> table.(f).(g)
*)
    table.(f).(g)

let print_table_de_precedences table =
    for i=0 to (Array.length table)-1 do
      printf "%10s" (string_of_symbol_id i);
      for j=0 to (Array.length table)-1 do
	 printf "|%s" (match table.(i).(j) with
	       Greater_than -> ">"
	     | Less_than -> "<"
	     | Uncomparable          -> "#"
	     | Equivalent            -> "="
             | _ -> failwith "bug in Symbols.print_table_de_precedences")
      done;
      printf "\n";
      printf "----------";
      for j=0 to (Array.length table)-1 do
	 printf "|-"
      done;
      printf "\n"
    done

(*==============================

Calcul de la cloture transitive

le nombre donne en parametre indique le plus grand indice
de la table

==============================*)

let matrice_initiale n =

let remplir_diagonale table =
  for i=0 to (Array.length table)-1 do
    table.(i).(i) <- Equivalent
  done;
  table 

in 
  remplir_diagonale (Array.create_matrix n n Uncomparable)


let superieur_ou_egal = function
    Equivalent -> true
  | Greater_than -> true
  | Greater_or_equivalent -> true
  | _ -> false

let inferieur_ou_egal = function
    Equivalent -> true
  | Less_than -> true
  | Less_or_equivalent -> true
  | _ -> false
    
let ajoute_paire (f,g) table =
(*
    printf "ajoute_paire %s %s\n" (string_of_symbol_id f) (string_of_symbol_id g);
*)
  for h1=0 to (Array.length table)-1 do
    for h2=0 to (Array.length table)-1 do
      if superieur_ou_egal table.(h1).(f) &
	 superieur_ou_egal table.(g).(h2) 
	then 
      	  if inferieur_ou_egal table.(h1).(h2) 
	    then 
      	      begin
(*
	      printf "probleme : on a deja %s <= %s\n" (string_of_symbol_id h1) (string_of_symbol_id h2) ;
*)
	      semantical_error 
	        ("Incompatibilities in the precedence: impossible to add " ^ (string_of_symbol_id f) ^ ">" ^ (string_of_symbol_id g))
	      end
	    else 
      	      begin
	      table.(h1).(h2) <- Greater_than;
	      table.(h2).(h1) <- Less_than
	      end
    done
  done;
(*
    print_table_de_precedences table;
*)
  table


let ajoute_equiv (f,g) table =
(*
    printf "ajoute_equiv %s %s\n" (string_of_symbol_id f) (string_of_symbol_id g);
*)
    for h1=0 to (Array.length table)-1 do
      for h2=0 to (Array.length table)-1 do
	if table.(h1).(f)=Equivalent & table.(g).(h2)=Equivalent 
	   then begin
		match table.(h1).(h2) with
		  Equivalent | Uncomparable ->
		     begin
		     table.(h1).(h2) <- Equivalent;
		     table.(h2).(h1) <- Equivalent
		     end
		  | _ -> begin
(*
		       printf "probleme : on a deja %s et %s comparables\n" (string_of_symbol_id h1) (string_of_symbol_id h2) ;
*)
		       semantical_error
		         ("Incompatibilities in the precedence: impossible to add "^(string_of_symbol_id f)^"="^(string_of_symbol_id g))
		       end
		end
	   else 
	if superieur_ou_egal table.(h1).(f) & superieur_ou_egal table.(g).(h2)
	   then begin
		match table.(h1).(h2) with
		  Greater_than | Uncomparable ->
		     begin
		     table.(h1).(h2) <- Greater_than;
		     table.(h2).(h1) <- Less_than
		     end
		  | _ -> begin
(*
		       printf "probleme : on a deja %s <= %s\n" (string_of_symbol_id h1) (string_of_symbol_id h2) ;
*)
		       semantical_error
		         ("Incompatibilities in the precedence: impossible to add " ^ (string_of_symbol_id f) ^ "=" ^ (string_of_symbol_id g))
		       end
		end
	else
	if inferieur_ou_egal table.(h1).(f) & inferieur_ou_egal table.(g).(h2)
	   then begin
		match table.(h1).(h2) with
		  Less_than | Uncomparable ->
		     begin
		     table.(h1).(h2) <- Less_than;
		     table.(h2).(h1) <- Greater_than
		     end
		  | _ -> begin
(*
		       printf "probleme : on a deja %s => %s\n" (string_of_symbol_id h1) (string_of_symbol_id h2) ;
*)
		       semantical_error
		         ("Incompatibilities in the precedence: impossible to add " ^ (string_of_symbol_id f) ^ " = " ^ (string_of_symbol_id g))
		       end
		end
      done
    done;
(*
    print_table_de_precedences table;
*)
    table


let first_symbol = function
    One_symbol(f)     -> f
  | Equal_to(f,_)     -> f
  | Larger_than(f,_) -> f
  | Lower_than(f,p)   -> f


let rec add_precedence table = function
    One_symbol(_)     -> table
  | Equal_to(f,p)     -> ajoute_equiv (f,first_symbol p) 
					  (add_precedence table p)
  | Larger_than(f,p) -> ajoute_paire (f,first_symbol p)
					  (add_precedence table p)
  | Lower_than(f,p)   -> ajoute_paire (first_symbol p,f)
					  (add_precedence table p)

let table_de_cloture_transitive max preclist =
  let table=List.fold_left add_precedence (matrice_initiale (max+1)) preclist
  in 
(*  
  print_table_de_precedences table;
*)
  table



(* ======================================================================

functions that extract lists of functions symbols from the symbol table.

======================================================================= *)

let rec listof_p_fun p n =
	if (n < 0)
		then  []
      		else if (p n)
			then n::(listof_p_fun p (n-1))
			else (listof_p_fun p (n-1))


let listof_p_functions p = listof_p_fun p (plus_grand_numero_de_symbole ())

let listof_ac_functions () = listof_p_functions is_ac

let listof_unary_functions () = listof_p_functions is_unary

let listof_constant_functions () = 
    listof_p_functions (function x -> (arity x) = 0)

let listof_c_functions () = listof_p_functions is_c

let listof_other_functions () = 
    listof_p_functions (function x -> (is_free x) & ((arity x) > 1))


let listof_all_functions () =
  listof_p_functions (function x -> true)

let listof_all_non_top_functions () =
  listof_p_functions (function x -> not (is_a_top_copy x))

let print_listof_functions l = 
	List.iter (function x -> printf "%s " (string_of_symbol_id x)) l;
	printf "\n"

	
(* =======================================================================

check mapo condition. The condition is: For any AC function symbol f,
the set of function symbols smaller or equivalent to f must be of the form

	f > u_1 > ... > u_n [ > g [ > a ] ]

where the u_i are unary, g is AC, and a is a constant. The parts in
brackets [ ] are optional, and n may be 0.

======================================================================== *)

(* -----------------------------------------------------------------------
	We use a table of pred_tables_entries to store wether a function
	symbol f 
	- is minimal (PREDNONE);
	- has more than one immediate predecessor, or a predecessor which
  	  is not unary, constant or AC (PREDERROR);
	- has exactly one predecessor g, and g is unary, constant or AC 
	  (PRED g)
------------------------------------------------------------------------ *)

type prec_table_entry =
	PREDNONE | PREDERROR | PRED of symbol_id


(* initialize predtable *)
let predtable_init size = Array.create size PREDNONE 

(* print a predtable *)
let print_predtable table =
    begin
    for i=0 to (Array.length table)-1 do
      printf "%10s | " (string_of_symbol_id i);
      match table.(i) with
      	PREDERROR	-> printf "ERROR\n"
      |	PREDNONE	-> printf "NONE\n"
      |	PRED g		-> printf "%10s\n" (string_of_symbol_id g)
    done;
    printf "\n"
    end



(* -----------------------------------------------------------------------
	The following functions are used to build a predtable from the
	precedence list.
------------------------------------------------------------------------ *)
(* test whether function symbol f is constant, unary or AC *)
let is_acuc f = is_constant f || is_unary f || is_ac f 

(* add an equivalence f=g to the predtable t *)
let add_pred_equiv p t =
	t.(fst p) <- PREDERROR;
	t.(snd p) <- PREDERROR;
	t


(* add a relation f>g to the predtable t *)
let add_pred_pair p t =
  let f= fst p and g=snd p in
	begin
	match t.(f) with
		PREDNONE   -> if is_acuc g
				then t.(f) <- PRED g
				else t.(f) <- PREDERROR
	      | PREDERROR  -> ()
	      |	PRED h	   -> if h=g
				then ()
				else t.(f) <- PREDERROR
	end
	; t

	
(* add a precedence list to a predtable *)
let rec add_prec_to_predtable predtable = function
    One_symbol(_)     -> predtable
  | Equal_to(f,p)     -> add_pred_equiv (f,first_symbol p)
                                          (add_prec_to_predtable predtable p)
  | Larger_than(f,p) -> add_pred_pair (f,first_symbol p)
                                          (add_prec_to_predtable predtable p)
  | Lower_than(f,p)   -> add_pred_pair (first_symbol p,f)
                                          (add_prec_to_predtable predtable p)

(* 

   The following functions are used to check, for a given predtable,
   whether the mapo condition holds.  This could be optimized by using
   a path compression technique as in Tarjan's union-find algorithm.

*)

(* create an exception *)

let mapo_excep i =
  semantical_error ("Mapo precedence clash when checking the predecessor of '"
		    ^ (string_of_symbol_id i)
		    ^ "'")


(* check whether h is either minimal, or wether it has a predecessor
which is a minimal constant. This function is called for AC symbols h
found while traversing a chain.
*)

let check_for_minac h table =
 match table.(h) with
   PREDNONE -> ()
 | PREDERROR -> raise (mapo_excep h)
 | PRED c -> if (not (is_constant c))
 		then raise (mapo_excep h)
		else match table.(c) with
		  PREDNONE -> ()
		| _ ->  raise (mapo_excep c)


(* check the entry act in the predtable table by traversing a chain *)

let rec check_mapo_single h table =
  match table.(h) with
      PREDNONE   -> ()
    | PREDERROR  -> raise (mapo_excep h)
    | PRED k	   -> 
	(* a constant must not have a predecessor *)
	if (is_constant h) 
	then raise (mapo_excep h)
	else 
	  if (is_ac k) 
	  then (check_for_minac k table)
	  else (* is-unary h *) (check_mapo_single k table)
	  

(* --------------------------------------------------------------------------
	All together now.
--------------------------------------------------------------------------- *)

let check_mapo_condition max preclist =
  let table = 
    List.fold_left add_prec_to_predtable (predtable_init (max+1)) preclist
  in
  (* print_predtable table; *)
  for i=0 to (Array.length table)-1 do
    if is_ac(i) then check_mapo_single i table else ()
  done
  
  
let check_extended_mapo_condition max table = ()


(*---------------------------------------------------------------------------
        Les fonctions de typage 
----------------------------------------------------------------------------*)


let get_sorts_of_symbol f = 
 if f>=0 & f < (Array.length table_des_symboles.symboles_de_fonction)
    then match table_des_symboles.symboles_de_fonction.(f)
            with (_,_,_,_,s) -> s
    else failwith "Bug in Symbol.get_sort_of_symbol"



let get_sorts_of_variable x =
  if x>=0 & x < (Array.length table_des_symboles.variables)
    then match table_des_symboles.variables.(x)
            with (_,s) -> s
    else failwith "Bug in Symbol.get_sort_of_variable"





let  print_sorte ss = 
    
  let rec sortetostring = function
      ([],s) -> (s)
    | ((a::l),s) -> (a ^" -> " ^ (sortetostring (l,s)))
  in
  print_string (sortetostring ss) 
 

let print_base_sorte ss =  print_string (ss)
 
let base_sorte2string = fun s -> s 

type type_liste_sortes_de_bases = 
    {
    mutable aa: sorte_de_base list;
    mutable comparison: (sorte_de_base*sorte_de_base ) list
  } 

let sortes_de_base = {aa=[];comparison=[]}

let list_of_base_sorts = fun () -> sortes_de_base.aa  

let make_new_sort = fun x -> sortes_de_base.aa <- ((fun l1 l2->List.fold_right (fun e l->if List.mem e l2 then l else e::l)l1 l2) [x] sortes_de_base.aa );list_of_base_sorts ()


let see_as_functional_sort ll = 
  if List.exists (fun x-> not (List.mem x sortes_de_base.aa)) ll
  then failwith "Unknown Sort.list used in declaration"
  else
let u = List.rev ll in 
match u with [] -> failwith "Fatal Error in Symbols.see_as_functional_sort."
| (a::l) -> (List.rev l,a)

let rec nieme l n = match l,n with
  (a::l),0 -> a 
| (a::l),n -> nieme l (n-1)
| [],_ -> failwith "Condition Invalide : this error is not supposed to happen." 


let see_as_base_sort str = 
nieme  sortes_de_base.aa 
(try (fun a->let rec f n=function b::tl->if b=a then n else f (n+1) tl|_->raise Not_found in f 0) str sortes_de_base.aa 
with Not_found -> failwith "Unknown Sort.list used in subsort declaration")

let store_subsort_table l = sortes_de_base.comparison <- l 


let get_subsort_table () = sortes_de_base.comparison 

let sort_body = fun (x,y) -> x
let sort_head = fun (x,y) -> y

let sub_sort = fun x y -> List.mem (x,y) sortes_de_base.comparison

let base_2_sorte = fun x -> ([],x)


let make_profile = fun (x,y) -> x,y 

(*
let max_used_var_s = ref(1)

let register_sorted_variable = fun 
 x s -> definir_operateur VARIABLE 0 DEFAULT [s] x  


let fresh_new_var_s sorte = 
let rec recnewvar n = 
if (n=(!(max_used_var_s)+1)) 
then  
  begin
    register_sorted_variable (string_of_var_id (!max_used_var_s + 1)) sorte;
    n
  end
else  
  if (List.hd (get_sorts_of_variable n))=sorte then n else recnewvar (n+1)
in 
recnewvar 0 


let rec next_new_var_s = fun x sorte ->
  if (x < (!max_used_var_s))
  then  
    if ((List.hd(get_sorts_of_variable (x+1)))=sorte) then (x+1) 
    else next_new_var_s (x+1) sorte
  else 
    begin
      register_sorted_variable (string_of_var_id (x+1)) sorte;
      (x+1)
    end



let new_var_s e sorte = 
let minnewvar=(List.fold_right max e (!max_used_var_s))
in
next_new_var_s minnewvar sorte 

*)
