(***************************************************************************

  This module defines basic things about symbols and signature

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: symbols.mli,v 1.3 2001/04/20 13:42:23 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI
  
  $Id: symbols.mli,v 1.3 2001/04/20 13:42:23 marche Exp $

***************************************************************************)

open C_declare_operator;;

(*

abstract types for symbols

*)

type symbol_id ;;
type var_id;;

type sorte;;
type sorte_de_base;;

type fix_type = 
    PREFIX | INFIX | POSTFIX | DEFAULT
;;

val free_symbols_table : unit -> unit;;

val not_a_symbol_id : symbol_id;;

val not_a_var_id : var_id;;

val is_a_var_id : var_id -> bool;;

(* built-in laws for function symbols *)
val is_ac	: symbol_id -> bool;;
val is_c	: symbol_id -> bool;;
val is_free	: symbol_id -> bool;;

(* extract the list of ac, resp. unary, function symbols. The argument
is the largest possible function symbol *)
val listof_ac_functions	: unit -> symbol_id list;;
val listof_unary_functions	: unit -> symbol_id list;;
val listof_c_functions        : unit -> symbol_id list;;
val listof_constant_functions : unit -> symbol_id list;;
val listof_other_functions    : unit -> symbol_id list;;
val listof_all_functions      : unit -> symbol_id list;;
val listof_all_non_top_functions : unit -> symbol_id list;;

val print_listof_functions	: symbol_id list -> unit;;

val leftify_var : var_id -> var_id;;

val rightify_var : var_id -> var_id;;

val unleftify_var : var_id -> var_id;;

val unrightify_var : var_id -> var_id;;

val is_a_left_variable : var_id -> bool;;

val is_a_right_variable : var_id -> bool;;


(*

  abstract methods for generating "new" variables

  [fresh_new_var ()] simply returns a variable
    BUT MAY ALREADY BE IN USE

  [new_var e] returns a variable not present in [e]

  [next_new_var x] returns a create variable "after" x

  i.e. instructions 

    [x_1] = [(new_var e)];
    [x_2] = [(next_new_var x_1)];
    [x_3] = [(next_new_var x_2)];
    etc...

  have the same semantics as

    [x_1] = [(new_var e)];
    [x_2] = [(new_var (Set.add x_1 e))];
    [x_3] = [(new_var (set_add x_2 (set_add x1 e)))];
    etc..

  but is MUCH MORE efficient!

*)


val fresh_new_var : unit -> var_id;;

(* [val fresh_new_var_s : sorte -> var_id ;;] *)

val new_var : var_id list -> var_id;;

(* [val new_var_s : var_id list -> sorte -> var_id;;] *)

val next_new_var : var_id -> var_id;;

(* [val next_new_var_s : var_id -> sorte -> var_id;;] *)


(*

comparaison functions

0 if equal, <0 if less then, >0 otherwise 

*)

val compare_symbol_ids : symbol_id -> symbol_id -> int;;

module SymbolSet : Ordered_sets.OrderedSet with type 'a elt = symbol_id;;

module SymbolMap : Ordered_maps.OrderedMap with type 'a key = symbol_id;;

val compare_var_ids : var_id -> var_id -> int;;

module VarOrd : Ordered_types.OrderedType with type t=var_id;;

module VarSet : Ordered_sets.OrderedSet with type 'a elt = var_id;;

module VarMap : Ordered_maps.OrderedMap with type 'a key = var_id;;

(*
[val compare_typed_symbols : typed_symbol -> typed_symbol -> int;;]
*)

val leq_symbol_ids : symbol_id -> symbol_id -> bool;;

(*

Lexicography of function symbols

*)

(*
[val typed_symbol_of_string : string -> typed_symbol;;]
*)

(*

[(symbol_id_of_string s)] retourne le [symbol_id] correspondant a la
chaine [s], leve l'exception [Not_found] s'il n'existe pas

*)

val symbol_id_of_string : 
	string 
	-> symbol_id;;

(*

[(var_id_of_string s)] retourne le [var_id] correspondant a la
chaine [s], leve l'exception [Not_found] s'il n'existe pas

*)

val var_id_of_string : string -> var_id;;

(*
[val string_of_typed_symbol : typed_symbol -> string;;]
*)

val string_of_symbol_id : symbol_id -> string;;

val print_symbol_id : symbol_id -> unit;;

val print_symbol_id_debug : symbol_id -> unit;;

val print_var_id_debug : var_id -> unit;;

val string_of_var_id : var_id -> string;;

val arity : symbol_id -> int;;

(*
[val arity_of_typed_symbol : typed_symbol  -> int;;]
*)

val is_infix : symbol_id -> bool;;

val is_postfix : symbol_id -> bool;;

val definir_operateur : 
  symbol_type -> int -> fix_type -> sorte list -> string  -> unit;;

(*

  [(top_copy f)] retourne le [symbol_id] de la version "TOP" de [f], pour
  les paires de dependances

*)

val top_copy : symbol_id -> symbol_id ;;
       
val plus_grand_numero_de_symbole : unit -> int;;

(*

[(get_symbol_id s)] retourne le [symbol_id] correspondant a la chaine [s].

si [s] non declare : erreur

si [s] declare mais correspond a une variable : erreur specifique

*)

val get_symbol_id : string -> symbol_id;;

(*

Orderings on symbols

*)

open Orderings_generalities;;

type precedence = symbol_id ordering;;

type table_de_precedences = comparison_result array array;;

type precedence_description =
    One_symbol of symbol_id
  | Equal_to of symbol_id * precedence_description
  | Larger_than of symbol_id * precedence_description
  | Lower_than of symbol_id * precedence_description;;

(*

Pr\'ec\'edence totale engendr\'ee par une cha\^{\i}ne d'op\'erateurs.

[(prec_totale [f1;...;fn])] est la pr\'ec\'edence $f_1>\cdots>f_n$.

*)

val prec_totale : symbol_id list -> precedence;;

(*

Pr\'ec\'edence partielle engendr\'ee par une table

*)

val precedence_par_table : 
  table_de_precedences 
  -> precedence
;;

(*

Calcul de la cloture transitive

le nombre donne en parametre indique le plus grand indice
de la table

*)


val table_de_cloture_transitive : 
  int -> precedence_description list 
  -> table_de_precedences 
;;

val print_table_de_precedences :
  table_de_precedences 
  -> unit;;

(*

Check the mapo condition, see [symbols.ml] for the exact definition.
The function [check_mapo_condition] raises the exception [Failure]
when the condition is violated.

*)

val check_mapo_condition: int -> precedence_description list -> unit;;

val check_extended_mapo_condition: int ->  table_de_precedences -> unit;;



(*

Prise en charge des sortes 

*)


val list_of_base_sorts : unit -> sorte_de_base list ;;

val get_sorts_of_symbol : symbol_id -> sorte list ;;

val get_sorts_of_variable : var_id -> sorte list ;;
(*
[val register_sorted_variable : string -> sorte -> unit ;;]
 *)

val print_sorte : sorte -> unit ;;
val print_base_sorte : sorte_de_base -> unit ;;
val base_sorte2string : sorte_de_base -> string ;;

(*

cree une nouvelle sorte a partir d'une chaine et renvoie la liste des
sortes existantes

*)

val make_new_sort : string -> (sorte_de_base list) ;;

val see_as_functional_sort : string list -> sorte ;;

val see_as_base_sort : string -> sorte_de_base ;;

val store_subsort_table : (sorte_de_base*sorte_de_base) list -> unit ;;

val get_subsort_table : unit -> (sorte_de_base*sorte_de_base) list;;


val sort_body : sorte -> sorte_de_base list;;
val sort_head : sorte -> sorte_de_base ;;

val sub_sort : sorte_de_base -> sorte_de_base -> bool ;;

val base_2_sorte : sorte_de_base -> sorte ;;

val make_profile : ( sorte_de_base list * sorte_de_base ) -> sorte ;;
 
