(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: symmetrise.ml,v 1.2 2001/04/20 13:42:24 marche Exp $

***************************************************************************)


open Args;;
open Orderings;;
open Ord_interactive;;
open Terms;;
open Equations_as_obj;;
open Completion_state;;
open Stack;;
open Generic_normalization;;
open Simplifications;;
open Symmetrization;;
open Format;;
open Messages;;

module type SYMMETRISE =
  sig
    val symmetrise : term ordering -> unit
  end

module Symmetrise =
  functor (Eqt : EQUATION) ->
  functor (EqtSet : EQUATION_SET with type equation = Eqt.t) ->
  functor (Compl : COMPLETION_STATE with type equation = Eqt.t
				    and type equation_set = EqtSet.eq_set) ->
  functor (Simpl : SIMPLIFICATIONS with type equation = Eqt.t) ->
  functor (Sym : SYMMETRIZATION with type equation = Eqt.t) -> 
    struct

open Eqt;;
open EqtSet;;
open Compl;;
open Simpl;;
open Sym;;

(* Symmetrise *)
let symmetrise order =

(* push the state onto the stack if the Undo_possible option is on *)
(* the state is pushed even if the answer to the next ordering question *)
(* is undo ! *)
  if options_courantes.undo_possible 
  then 
    begin
      let new_state = copy_state () in
      	push new_state choice_stack
    end;
(* end of pushing the state *)

  let eq = next_equation etat.eqs_to_process in
    
    try 
      let sym_eq = (symmetrize order eq) in
    	if (options_courantes.niveau_de_verbosite >= 1)
    	then 
	  begin
      	    print_string "Orienting and symmetrizing the equation ";
      	    print_equation eq;
      	    print_break 1 0;
      	    print_string "into";
      	    print_break 1 40;
      	    print_equation sym_eq;
      	    force_newline ();
      	    print_flush ()
	  end;
    	try 
	  let sym_eq = forced_left_simplify 
			 (if options_courantes.ordered_completion
			  then order 
			  else (fun _ _ -> Greater)) 
			 sym_eq etat.disc_net in
	    add_new_eq sym_eq
    	with 
	    Irreducible -> add_new_preoriented_eq sym_eq
	  | Trivial_equation -> ()
		
    with Non_orientable -> 
      if options_courantes.ordered_completion
      then
      	begin
          let weak_sym_eq = (weak_symmetrize order eq) in
            if (options_courantes.niveau_de_verbosite >= 1)
            then 
	      begin
	      	print_string "Orienting and symmetrizing the equation ";
              	print_equation eq;
              	print_break 1 0;
       	      	print_string "into";
              	print_break 1 40;
              	print_equation weak_sym_eq;
       	      	force_newline ();
              	print_flush ()
	      end;
            try 
	      let simpl_sym_eq = forced_left_simplify 
				   order weak_sym_eq etat.disc_net in 
	      	add_new_eq simpl_sym_eq
            with 
	      	Irreducible -> add_new_preoriented_eq weak_sym_eq
              | Trivial_equation -> ()
		    
      	end
      else 
      	begin
      	  open_hovbox 0;
      	  print_string "Postponing the equation";
      	  print_break 1 2;
      	  print_equation eq;
      	  force_newline ();
      	  close_box ();
      	  print_flush ();
      	  add_postponed_eq eq
      	end

  | Undo 1 -> 
      try 
        if options_courantes.undo_possible
        then 
	  begin
            pop (choice_stack); (* pop twice ! *)
      	    let old_state = pop(choice_stack) in 
	      restore_state old_state
          end
        else raise Empty
      with Empty -> begin
        message "No more undo information";
	add_new_eq eq
      end
;;

    end;;

module Standard_symmetrise = 
  Symmetrise (Equation1) (Equation_set1) (Standard_completion) 
    (Standard_simplifications) (Standard_symmetrization);;

module G_symmetrise = 
  Symmetrise (Equation_with_goal) (Equation_with_goal_set) (G_completion) 
    (G_simplifications) (G_symmetrization);;
