(***************************************************************************

  Basic definitions about terms: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: terms.ml,v 1.2 2001/04/20 13:42:24 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: terms.ml,v 1.2 2001/04/20 13:42:24 marche Exp $

***************************************************************************)

open Symbols
open Positions


(*===========================

Le type terme, ses constructeurs
et observateurs

===========================*)

type term = 
    VAR of var_id 
  | TERM of symbol_id * (term list)
  

(*----------------

(top_symbol_id t) returns the top symbol_if of the term t,
(liste_des_sous_terms t) returns the list of subterms of t,
both raise exception Invalid_argument if t is a variable.

-----------------*)

let top_symbol_id = function
    TERM (f,_) -> f
  | VAR _ -> raise (Invalid_argument "Terms.top_symbol_id");;

let liste_des_sous_termes = function
    TERM(_,l) -> l
  | VAR _ -> raise (Invalid_argument "Terms.liste_des_sous_termes");;




(*

  [(set_of_symbols t)] returns the set of all symbols occurring in [t]

*)

let rec set_of_symbols = function
    VAR _ -> SymbolSet.empty
  | TERM(f,l) ->
      SymbolSet.add f
      	(List.fold_right
	   (fun t set -> SymbolSet.union (set_of_symbols t) set) l
	   SymbolSet.empty)


let rec leftify_term = function
    TERM(f,l) -> TERM(f,List.map leftify_term l)
  | VAR x -> VAR (leftify_var x)


let rec rightify_term = function
    TERM(f,l) -> TERM(f,List.map rightify_term l)
  | VAR x -> VAR (rightify_var x)


let rec est_clos = function
    VAR _ -> false
  | TERM(_,l) -> List.for_all (function x -> est_clos x) l


let rec variable_est_dans_terme x = function
    VAR y -> x=y
  | TERM(_,l) -> (List.exists (variable_est_dans_terme x) l)

let variable_est_dans_equation x (s,t) = 
  (variable_est_dans_terme x s) or (variable_est_dans_terme x t)

let variable_est_dans_liste x p = 
  List.exists (variable_est_dans_equation x) p
 

let is_a_variable = function
    VAR _ -> true
  | _ -> false


let is_a_constant = function
    TERM (f,[]) -> true
  | _ -> false



let rec flatten_term_list f = function
    [] -> []
  | (TERM(g,l) as t)::m -> 
      if g=f
      then l @ (flatten_term_list f m)
      else t::(flatten_term_list f m)
  | v::m -> v::(flatten_term_list f m)


let head_flatten_term = function
    TERM(f,l) as t -> 
      if is_ac f
      then TERM(f,(flatten_term_list f l))
      else t
  | v -> v


let rec flatten_term = function
    TERM(f,l) -> head_flatten_term (TERM(f,(List.map flatten_term l)))
  | v -> v

let add_list x l =
  if List.mem x l then l else x::l
 

let rec add_variables_de_liste accu = function
    [] -> accu
  | t::l -> 
      let accu' = add_variables_de_terme accu t
      in add_variables_de_liste accu' l

and add_variables_de_terme accu = function
    VAR x -> add_list x accu
  | TERM(_,l)   -> add_variables_de_liste accu l


let variables_de_liste_de_termes l = add_variables_de_liste [] l;;

let variables_du_terme t = add_variables_de_terme [] t;;

(*
let variables_du_terme =
  Time_profiler.profile1 
    "Terms.variables_du_terme"
    variables_du_terme
;;
*)

let rec add_variables_lineaires_de_liste accu accu_nlin = function
    [] -> (accu,accu_nlin)
  | t::l -> 
      let (accu',accu_nlin') = add_variables_lineaires_de_terme accu accu_nlin t
      in add_variables_lineaires_de_liste accu' accu_nlin' l

and add_variables_lineaires_de_terme accu accu_nlin = function
    VAR x -> 
      if List.mem x accu
      then (accu, add_list x accu_nlin)
      else ((x::accu), accu_nlin)
  | TERM(_,l) -> add_variables_lineaires_de_liste accu accu_nlin l


let variables_lineaires_du_terme t = 
  let (accu,accu_nlin) = add_variables_lineaires_de_terme [] [] t 
  in Listutils.subtract accu accu_nlin


let variables_equation (s,t) = 
  let accu = add_variables_de_terme [] s 
  in add_variables_de_terme accu t


let rec variables_de_liste_equations = function
    [] -> []
  | (s,t)::l -> 
      let accu = add_variables_de_terme (variables_de_liste_equations l) s 
      in add_variables_de_terme accu t

  
(*=========================================

Fonction taille

(taille_terme t) retourne la taille de $t$, c'est-\`a-dire le
nombre de symboles dans $t$, y compris les variables.


=========================================*)

let rec symb_measure_op op f = function
    VAR _ -> 0
  | TERM (g,l) -> 
      let m = 
	List.fold_left 
	  (fun res t -> op res (symb_measure_op op f t)) 0 l
      in
	if f=g
	then 
	  begin
	    if is_ac f
	    then (List.length l) + m
	    else 1 + m
	  end
	else m


let symb_measure_sum f t = 
  symb_measure_op (+) f t


let symb_measure_max f t = 
  symb_measure_op max f t


let nesting f = function
    VAR _ -> 0
  | TERM (_,[]) -> 0
  | TERM (g,l) as t ->
      let rec hnesting = function
	  (VAR _), current, absolute -> max current absolute
	| (TERM (_,[])), current, absolute -> max current absolute
	| (TERM (h,k)), current, absolute ->
	    if h=f
	    then 
	      begin
		let succ_current = 
		  if is_ac f 
		  then current + (pred (List.length k))
		  else succ current
		in
		  List.fold_left
		    (fun res ti -> max res (hnesting (ti, succ_current, absolute)))
		    0 k
	      end
	    else
	      let max_current_absolute = max current absolute
	      in
	      	List.fold_left
		  (fun res ti -> max res (hnesting (ti, 0, (max_current_absolute))))
		  0 k
      in
	hnesting (t,0,0)


let rec taille_terme_var_size n = function
    VAR _ -> n
  | TERM (f,l) -> 
      let debut = if is_ac f then pred (List.length l) else 1
      in List.fold_left (fun res t -> res + (taille_terme_var_size n t)) 
	   debut l


let rec taille_terme = function
    VAR _ -> 1
  | TERM (f,l) -> 
      let debut = if is_ac f then pred (List.length l) else 1
      in
      	List.fold_left (fun res t -> res + (taille_terme t)) 
	  debut l


let rec taille_terme_sans_variables = function
    VAR _ -> 0
  | TERM (f,l) ->
      let debut = if is_ac f then pred (List.length l) else 1
      in
	List.fold_left (fun res t -> res + (taille_terme_sans_variables t)) 
	  debut l


let rec taille_terme_variables_legeres = function
    VAR _ -> 1
  | TERM (f,l) ->
      let debut = if is_ac f then 2 * (pred (List.length l)) else 2
      in
	List.fold_left (fun res t -> res + (taille_terme_variables_legeres t))
	  debut l


let plus_petit_terme s t = 
  if (taille_terme s) < (taille_terme t) then s else t

let plus_grand_terme s t = 
  if (taille_terme s) < (taille_terme t) then t else s

(*=================================

In order to have efficient functions for equality modulo AC and
others, we define a total ordering > on the set of terms with variables. 
Of course this ordering does not have any morphic properties: it is
non monotonic, non stable by substitution, etc.

The only interessing properties we want are

 - it is total
 - s =_AC t iff s' = t' syntactically, were s' and t' are sorted
   versions of s and t, defined by

if s=f(s_1,...,s_n), s' = f (s'_i_1,.., s'_i_n)

were s'_1 = sorted(s_i) and s'_i_1 < ... s'_i_n

===========================*) 



(*======================

Compare flattened/sorted terms 

returns <0 if t1 < t2, 0 if t1 = t2, and >0 otherwise

==========================*)

let rec term_comp t1 t2 =   
  match (t1,t2) with
      TERM (f1,l1), TERM (f2,l2) ->
        let x = compare_symbol_ids f1 f2 in
	  if x=0 
	  then 
            let rec comp_list = function
		([],[]) -> 0
              | ([],_) -> -1
              | (_,[]) -> 1
              | (s1::l1,s2::l2) -> 
		  let c=(term_comp s1 s2) in
		    if c<>0 then c else comp_list (l1,l2)
	    in comp_list (l1,l2) 
	  else x
    | VAR _,TERM _ -> -1
    | TERM _ ,VAR _ -> 1
    | (VAR x1,VAR x2) -> compare_var_ids x1 x2


let leq_term t1 t2 = (term_comp t1 t2) <=0 


let rec sort_term = function
    VAR _ as v -> v
  | TERM (f,l) -> 
      if is_free f
      then TERM (f,List.map sort_term l)
      else TERM (f,Sort.list leq_term (List.map sort_term l))



(*=================================

\'Egalit\'e syntaxique et
\'Egalit\'e modulo AC sur les termes aplatis

==================================*)

let rec egaux_synt t1 t2 =
  match t1 with 
      VAR x ->
	begin
	  match t2 with 
	      VAR y -> x=y
	    | _ -> false
	end
    | TERM (f,l) ->
	match t2 with
	    TERM (g,m) ->
	      if f=g 
	      then try List.for_all2 egaux_synt l m with _ -> false 
	      else false
	  | _ -> false



(*
let egaux_ac (s,t) = 
  if (symbole_de_tete s)=(symbole_de_tete t)
     then egaux_synt ((sort_term s),(sort_term t))
     else false
;;
*)

let rec egaux_ac t1 t2 = 
  match t1 with
      VAR x ->
	begin
	  match t2 with
	      VAR y -> x=y
	    | _ -> false
	end
    | TERM (f,l) ->
	match t2 with
	    TERM (g,m) ->
	      if f=g
	      then
		if is_free f
		then 
		  try List.for_all2 egaux_ac l m
		  with _ -> failwith "bug in Terms.egaux_ac"
		else listes_egales_ac l m
	      else false
	  | _ -> false

and listes_egales_ac l m =
  let rec l_congr = function
      [], [], [] -> true
    | (s::l), (t::m as t_m), n -> 
	if egaux_ac s t
	then l_congr (l@n, m, [])
	else l_congr (l, t_m, (s::n))
    | _, _, _ -> false
  in l_congr (l,m,[])

(*
let egaux_ac =
  Time_profiler.profile1
    "Terms.egaux_ac"
    egaux_ac
;;
*)

let rec sous_termes_partages = function
    []   -> []
  | s::l -> 
      let rec ajouter_s = function
	  [] -> [(s,1)]
	| (t,k)::l -> 
	    if egaux_synt (* egaux_ac *) s t
            then (s,k+1)::l
            else (t,k)::(ajouter_s l)
      in ajouter_s (sous_termes_partages l)

let rec duplique n s = 
  if n=1 
  then [s]
  else s::(duplique (n-1) s)


let reconstruit_combinaison_lineaire f = function
    [] -> failwith "liste vide dans reconstruit_combinaison_lineaire"
  | [(s,1)] -> s
  | l -> TERM(f,Listutils.flat_map (function (s,k) -> duplique k s) l);;


(*===========================================
Conversion d'un terme en chaine de caracteres
===========================================*)

let rec string_of_term = function
    TERM (f,[]) -> string_of_symbol_id f
  | VAR x -> string_of_var_id x
  | TERM (f,(s::l as s_l)) -> 
      if is_infix f
      then (string_of_term2 s) ^ (string_of_infix_term f l)
      else 
	if is_postfix f
        then (string_of_lterm s_l) ^ (string_of_symbol_id f)
        else (string_of_symbol_id f) ^ (string_of_lterm s_l)


and string_of_term2 = function
   TERM (f,[]) -> string_of_symbol_id f
 | VAR x -> string_of_var_id x
 | TERM (f,(s::l as s_l)) -> 
     if is_infix f
     then "("^(string_of_term2 s)^(string_of_infix_term f l)^")"
     else 
       if is_postfix f
       then (string_of_lterm s_l) ^ (string_of_symbol_id f)
       else (string_of_symbol_id f) ^ (string_of_lterm s_l)

and string_of_infix_term f = function
    []	-> ""
  | s::l -> 
      (string_of_symbol_id_with_spaces f) ^
      (string_of_term2 s) ^ 
      (string_of_infix_term f l)

and string_of_symbol_id_with_spaces f =
   let t = string_of_symbol_id f in
     match t with
	 "" (* "+" | "." | "*" *) -> t
       | _ -> " "^t^" "
 
and string_of_lterm = function
    []   -> "()"
  | t::l -> 
      let rec str_of_lterm_aux = function
	  []   -> ")"
	| t::l -> ","^(string_of_term t)^(str_of_lterm_aux l)
      in  "("^(string_of_term t)^(str_of_lterm_aux l);;


(*
let string_of_term_list = function
   []   -> "()"
 | t::l -> "(" ^ (string_of_term t) 
               ^ (string_of_end_term_list l)
where rec string_of_end_term_list = function
     []   -> ")"
   | t::l -> ",\n" ^ (string_of_term t) 
                   ^ (string_of_end_term_list l);;
*)

let rec string_of_raw_term = function
    TERM (f,[])   -> "TERM("^(string_of_symbol_id f)^",[])"
  | VAR x -> "VAR("^(string_of_var_id x)^")"
  | TERM(f,s::l) -> 
      (List.fold_left
       	 (fun s t ->
	    s^","^(string_of_raw_term t))
       	 ("TERM("^(string_of_symbol_id f)^",["^(string_of_raw_term s))
       	 l)^"])"
;;



(*==================================================
Conversion d'un terme aplati en chaine de caracteres
mais format special pour symboles AC
===================================================*)

(* 
let rec string_of_term = function
   TERM(f,[])   -> string_of_symbol_id f
 | VAR x          -> string_of_var_id x
 | TERM(f,(s::l)) -> if (is_ac f) 
                     then (string_of_acterm f (s::l))
                     else if is_infix f
                          then (string_of_term2 s) ^ (string_of_infix_term f l)
                          else (string_of_symbol_id f) ^ (string_of_lterm (s::l))
 
and string_of_acterm f = function
      [] -> ""
    | [s]    -> (string_of_term2 s)
    | (s::l) -> if (is_a_constant s) or (is_a_variable s)
                then (string_of_acterm2 f s l)
                else (string_of_term2 s) ^ (string_of_symbol_id_with_spaces f) ^ (string_of_acterm f l)

and string_of_acterm2 f x = function
     []     -> (string_of_term2 x)
   | (s::l) -> if (x=s)
               then (string_of_aclist f x 2 l)
               else (string_of_term2 x) ^ (string_of_symbol_id_with_spaces f) ^ (string_of_acterm f (s::l))

               
and string_of_aclist f x nbocc = fun
       []     -> ""
     | [s]    -> if (x=s)
                 then "(" ^ (string_of_int (nbocc+1)) ^ "_" ^ (string_of_symbol_id f) ^ " " ^ (string_of_term x) ^ ")" 
                 else "(" ^ (string_of_int nbocc) ^ "_" ^ (string_of_symbol_id f) ^ " " ^ (string_of_term x) ^ ")" 
                          ^  (string_of_symbol_id_with_spaces f)  ^ (string_of_term2 s)
     | (s::l) -> if (x=s)
                 then (string_of_aclist f x (nbocc+1) l)
                 else "(" ^ (string_of_int nbocc) ^ "_" ^ (string_of_symbol_id f) ^ " " ^ (string_of_term x) ^ ")"
                                     ^ (string_of_symbol_id_with_spaces f)  ^ (string_of_acterm f (s::l))


and string_of_term2 t = 
   match t with
   TERM(f,[])	  -> string_of_symbol_id f
 | VAR x          -> string_of_var_id x
 | TERM(f,s::l) -> if (is_ac f) 
                   then (string_of_acterm f (s::l))
                   else if is_infix f
                        then "(" ^ (string_of_term2 s) ^ (string_of_infix_term f l)^")"
                        else (string_of_symbol_id f) ^ (string_of_lterm (s::l))


and string_of_infix_term f = function
   []	-> ""
 | s::l	-> (string_of_symbol_id_with_spaces f) ^
           (string_of_term2 s) ^ 
           (string_of_infix_term f l)

and string_of_symbol_id_with_spaces f =
   let t = string_of_symbol_id f in
     match t with
     "+" | "." | "*" -> t
    | _ -> " "^t^" "
 
and string_of_lterm = function
  []   -> "()"
| t::l -> "("^(string_of_term t)^(str_of_lterm_aux l)
  where rec str_of_lterm_aux = function
     []   -> ")"
   | t::l -> ","^(string_of_term t)^(str_of_lterm_aux l);;


let string_of_term_list = function
   []   -> "()"
 | t::l -> "(" ^ (string_of_term t) 
               ^ (string_of_end_term_list l)
where rec string_of_end_term_list = function
     []   -> ")"
   | t::l -> ",\n" ^ (string_of_term t) 
                   ^ (string_of_end_term_list l);;

*)


(*=====================================================
Operations classiques
=====================================================*)



let replace s t p = 
(*
  printf "je mets %s a la position %s dans %s\n" 
      	 (string_of_term t)
      	 (string_of_position p)
      	 (string_of_term s);
  flush stdout;
*)
  let error s t p = 
    raise (Invalid_argument ("(replace "^(string_of_term s)^
			     " "^(string_of_term t)^
			     " "^(string_of_position p)^")"))
  in
  let rec do_replace pos s' = match pos with
      Top -> t
    | Pos (i,Top) ->
	begin
	  match s' with
	      TERM(f,l) -> 
		if is_ac f
      		then 
		  let rec replace_ac n = function
		      s::l -> 
			if n=1 
			then 
			  try 
			    if f = top_symbol_id t
      			    then (liste_des_sous_termes t) @ l
			    else t::l
			  with Invalid_argument _ -> t::l
			else s::(replace_ac (pred n) l)
		    | [] -> error s t p
		  in TERM (f,replace_ac i l)
		else 
		  let rec replace_list n = function
		      s::l -> 
			if n=1 
			then t::l
			else s::(replace_list (pred n) l)
		    | [] -> error s t p
		  in TERM (f,replace_list i l)
	    | _ -> error s t p
	end
    | Pos (i,q) -> 
	begin
	  match s' with
	      TERM (f,l) -> 
		let rec replace_list n = function
		    [] -> error s t p
		  | s::l -> 
		      if n=1 
		      then (do_replace q s)::l
		      else s::(replace_list (pred n) l)
		in TERM(f,replace_list i l)
	    | _ -> error s t p
	end
  in do_replace p s


(*
let replace = 
  Time_profiler.profile3
    "Terms.replace"
    replace
;;
*)

let add_left_var f x t =
  try 
    if f = top_symbol_id t
    then TERM (f,VAR x::(liste_des_sous_termes t))
    else TERM (f,[VAR x;t])
  with Invalid_argument _ -> TERM (f,[VAR x;t])




(*

renommages 

*)


let rec rename_term_in_env theta v = function
    VAR x -> 
      begin
      	try VAR(List.assoc x theta),theta,v
      	with Not_found -> VAR v,(x,v)::theta,(next_new_var v )
      end
  | TERM (f,l) -> 
      let l',theta',v' = rename_list theta v l 
      in TERM(f,l'),theta',v'

and rename_list theta v = function
    [] -> [],theta,v
  | t::l -> 
      let t',theta',v' = rename_term_in_env theta v t
      in let l',theta'',v'' = rename_list theta' v' l
      in t'::l',theta'',v''


let rename_term t = 
  let u,_,_ = rename_term_in_env [] (fresh_new_var ()) t
  in u


let rename_pair (u,v) =
  let u',theta,x = rename_term_in_env [] (fresh_new_var ()) u
  in let v',_,_ = rename_term_in_env theta x v
  in u',v'



(*
  Enumeration des positions d'un terme en profondeur d'abord, 
  et en largeur d'abord : ce sera utilise dans la strategie de
  reduction outermost.
*)

let rec sous_terme_a_la_position t = function
    Top -> t
  | Pos(i,p) -> 
      match t with
          TERM(_,l) -> 
	    begin
	      try 
		let t' = List.nth l (pred i)
		in sous_terme_a_la_position t' p
	      with Failure "nth" -> 
		failwith "sous_terme_a_la_position"
	    end
        | _ -> failwith "sous_terme_a_la_position"
;;


let rec next_pos_p t = function
    Top -> 
      begin
	match t with
            VAR _ -> raise No_more_pos
	  | TERM (_,[]) -> raise No_more_pos
	  | _ -> Pos(1,Top)
      end
  | Pos(i,p) -> 
      begin
	match t with
            TERM(_,l) -> 
	      begin
		try 
		  let ti = List.nth l (pred i)
		  in let p' = next_pos_p ti p 
		  in Pos(i,p')
              	with 
		    Failure "nth" -> 
                      failwith "next_pos_p"
                  | No_more_pos -> 
		      if (List.length l) > i 
                      then Pos(i+1,Top)
		      else raise No_more_pos
	      end
          | _ -> failwith "next_pos_p"
      end



let rec depth = function
    Top -> 0
  | Pos(_,p) -> (depth p) + 1


let has_no_son t p = 
  try 
    let t' = sous_terme_a_la_position t p in
      match t' with
          VAR _ -> true
	| TERM(_,[]) -> true
	| _ -> false
  with Failure "sous_terme_a_la_position" -> failwith "has_no_son"
;;

let rec pos_smaller p q = match p,q with
    Top, Top -> false
  | Top, _  -> true
  | (Pos _), Top -> false
  | Pos(i,p'), Pos(j,q') -> 
      let dp = (depth p') 
      and dq = (depth q') in
        (dp < dq) or 
        ((dp = dq) & (i < j)) or 
        ((i=j) & (pos_smaller p' q'))
;;

let rec next_pos_l t = function
  Top -> (match t with
                VAR _ -> raise No_more_pos
	      | TERM (_,[]) -> raise No_more_pos
	      | _ -> Pos(1,Top))
| q -> let (p,i) = decompose_pos q in 
       try let t' = sous_terme_a_la_position t p in
           (match t' with
                 TERM(_,l) -> 
                     if (List.length l) > i
                     then build_pos p (i+1)
		     else (try let p' = ref (next_pos_l t p) in
		               while (has_no_son t !p') & (pos_smaller !p' q) do
                                      p':= next_pos_l t !p'
		               done;
                               if has_no_son t !p'
                               then failwith "plus de pos"
                               else build_pos !p' 1
                           with No_more_pos -> first_at_depth t (2+(depth p))
                              | Failure "plus de pos" -> raise No_more_pos)
              | _ -> failwith "next_pos_l")
       with Failure "sous_terme_a_la_position" -> failwith "next_pos_l"

and first_at_depth t = function
  0 -> Top
| n -> if (n<0) then 
       failwith "first_at_depth"
       else let n' = n-1 in
            try let p = ref (first_at_depth t n) in
	        while (has_no_son t !p) do
	              p:= next_pos_l t !p
		done;
		build_pos !p 1
	    with No_more_pos -> raise No_more_pos
;;        





open Format;;
open Args;;

(*==================================================
Affichage d'un terme aplati dans une box.
Format special pour symboles AC.
===================================================*)

let rec print_term1 = function 
    TERM(f,[])     -> print_string (string_of_symbol_id f)
  |  VAR x          -> print_string (string_of_var_id x)
  | TERM(f,(s::l as s_l)) -> 
      if is_ac f 
      then print_acterm f s_l
      else 
	if is_infix f
	then 
	  begin
            print_term2 s;
            print_break 0 0;
            print_infix_term f l
          end
	else 
	  if is_postfix f
          then 
	    if (arity f) = 1
            then print_unary_postfix_term f s_l
            else 
	      begin
		print_lterm s_l;
		print_string (string_of_symbol_id f)
              end
          else 
	    if (arity f) = 1
            then print_unary_term f s_l
          else 
	    begin
              print_string (string_of_symbol_id f);
              print_lterm s_l
            end

and print_term2 = function        (* idem mais entoure par de ()'s          *)
                                  (* pour mettre dans une expression infixe *)
    TERM (f,[])	  ->  print_string (string_of_symbol_id f)  
  | VAR x          -> print_string (string_of_var_id x)
  | TERM (f,(s::l as s_l)) -> 
      if is_ac f 
      then print_actermb f s_l
      else 
	if is_infix f
	then 
	  begin
            print_string "(";
            print_term2 s;
            print_break 0 0;
            print_infix_term f l;
            print_string ")"
          end
      else 
	if is_postfix f
        then 
	  if (arity f) = 1
          then print_unary_postfix_term f s_l
          else 
	    begin
              print_lterm s_l;
              print_string (string_of_symbol_id f) 
            end
        else 
	  if (arity f) = 1
          then print_unary_term f s_l
          else 
	    begin
              print_string (string_of_symbol_id f); 
              print_lterm s_l
            end

and print_infix_term f = function
   []	-> ()
  | s::l -> 
      begin
        print_symbol_id_with_spaces f;
        print_break 0 0;
        print_term2 s;
        print_infix_term f l
      end

and print_acterm f = function  (* on ne compte que les var. et constantes  *)  
    [] -> print_break 1 0
  | [s]    -> print_term2 s    
  | s::l -> 
      if (is_a_constant s) or (is_a_variable s)
      then print_acterm2 f s l
      else 
	begin
          print_term2 s;
          print_break 0 0;
          print_symbol_id_with_spaces f;
          print_break 0 0;
          print_acterm f l
        end

and print_actermb f = function   (* idem mais entoure par de ()'s          *)
    [] -> print_break 1 0        (* pour mettre dans une expression infixe *) 
  | [s]    -> print_term2 s    
  | s::l -> 
      if (is_a_constant s) or (is_a_variable s)
      then print_acterm2b f s l
      else 
	begin
          print_string "(";
          print_term2 s;
          print_break 0 0;
          print_symbol_id_with_spaces f;
          print_break 0 0;
          print_actermb f l;
          print_string ")"
        end

and print_acterm2 f x = function  (* on commence a compter seulement si on a *)
    []     -> print_term1 x       (* au moins 2 fois la meme var/const       *)
  | s::l as s_l -> 
      if x=s
      then print_aclist f x 2 l
      else 
	begin
          print_term2 x;
          print_break 0 0;
          print_symbol_id_with_spaces f;
          print_break 0 0;
          print_actermb f s_l
        end

and print_acterm2b f x = function (* idem mais entoure par de ()'s          *) 
     [] -> print_term1 x          (* pour mettre dans une expression infixe *) 
   | s::l as s_l -> 
       if x=s
       then print_aclist f x 2 l
       else 
	 begin
           print_string "(";
           print_term2 x;
           print_break 0 0;
           print_symbol_id_with_spaces f;
	   print_break 0 0;
           print_acterm f s_l;
           print_string ")"
         end
               
and print_aclist f x nbocc = function
    [] -> 
      begin
        print_string "(";
        print_int nbocc; 
        print_string "_"; 
        print_string (string_of_symbol_id f);
        print_string " ";
        print_term1 x;
        print_string ")"
      end
  | [s] -> 
      if x=s
      then 
	begin
          print_string "(";
          print_int (nbocc + 1); 
          print_string "_"; 
          print_string (string_of_symbol_id f);
          print_string " ";
          print_term1 x;
	  print_string ")"
        end 
      else 
	begin
          print_string "(";
          print_int nbocc;
          print_string "_"; 
          print_string (string_of_symbol_id f);
          print_string " ";
          print_term1 x;
          print_string ")";
          print_symbol_id_with_spaces f;
          print_break 0 0;
          print_term2 s
        end
     | s::l as s_l -> 
	 if x=s
         then print_aclist f x (nbocc + 1) l
         else 
	   begin
             print_string "(";
             print_int nbocc;
             print_string "_"; 
             print_string (string_of_symbol_id f);
             print_string " ";
             print_term1 x;
             print_string ")";
             print_symbol_id_with_spaces f;
             print_break 0 0;
             print_acterm f s_l
           end

and print_unary_term  f = function
    [s] ->  
      if is_a_variable s
      then 
	begin
          print_string (string_of_symbol_id f); 
          print_lterm [s]
        end
      else 
	if (top_symbol_id s) = f
        then print_unary_term2 f (liste_des_sous_termes s)
        else
	  begin
            (print_string (string_of_symbol_id f)); 
            (print_lterm [s])
          end
  | _  -> failwith "Wrong arity in a unary function symbol"

and print_unary_term2 f = function
      [s] -> if (is_a_variable s)
             then begin
                  (print_string (string_of_symbol_id f)); 
                  (print_string "(");
                  (print_string (string_of_symbol_id f)); 
                  (print_lterm [s]);
                  (print_string ")")
                  end
             else if ((top_symbol_id s) = f)
                  then print_unary_term3 f 3 (liste_des_sous_termes s)
                  else begin
                       (print_string (string_of_symbol_id f)); 
                       (print_string "(");
                       (print_string (string_of_symbol_id f)); 
                       (print_lterm [s]);
                       (print_string ")")
                       end
     | _  -> (print_string "")      (* cas exclu *) 

and print_unary_term3 f n = function
      [s] -> if ((top_symbol_id s) = f)
             then print_unary_term3 f (n+1) (liste_des_sous_termes s)
             else begin
                  (print_string (string_of_symbol_id f)); 
                  (print_string "^");
                  (print_int n);
                  (print_lterm [s])
                  end
     | _  -> (print_string "")      (* cas exclu *) 

and print_unary_postfix_term  f = function
       [s] ->  if (is_a_variable s)
               then begin
                    (print_lterm [s]);
                    (print_string (string_of_symbol_id f)) 
                    end
               else if ((top_symbol_id s) = f)
                    then print_unary_postfix_term2 f (liste_des_sous_termes s)
                    else begin
                         (print_lterm [s]);
                         (print_string (string_of_symbol_id f)) 
                         end
     | _  -> (print_string "")      (* cas exclu *) 

and print_unary_postfix_term2 f = function
      [s] -> if (is_a_variable s)
             then begin
                  print_string "(";
                  print_lterm [s];
                  print_string (string_of_symbol_id f); 
                  print_string ")";
                  print_string (string_of_symbol_id f)
                  end
             else if ((top_symbol_id s) = f)
                  then print_unary_postfix_term3 f 3 (liste_des_sous_termes s)
                  else begin
                       print_string "(";
                       print_lterm [s];
                       print_string (string_of_symbol_id f); 
                       print_string ")";
                       print_string (string_of_symbol_id f)
                       end
     | _  -> print_string ""      (* cas exclu *) 

and print_unary_postfix_term3 f n = function
      [s] -> if ((top_symbol_id s) = f)
             then print_unary_term3 f (n+1) (liste_des_sous_termes s)
             else begin
                  print_lterm [s];
                  print_string (string_of_symbol_id f); 
                  print_string "^";
                  print_int n
                  end
     | _  -> print_string ""      (* cas exclu *) 

and print_symbol_id_with_spaces f =
   let t = string_of_symbol_id f in
     match t with
     "+" | "." | "*" -> (print_string t)
    | _ -> begin
           (print_string " ");
           (print_string t);
           ((fun(x,y)->print_break x y) (1,0))
           end


and print_lterm = function
  []   -> (print_string "()")
| t::l -> begin
          (print_string "(");
          (print_term1 t);
          (print_lterm2 l)
          end

and print_lterm2 = function
   []   -> (print_string ")")
 | t::l -> begin 
           (print_string ",");
           ((fun(x,y)->print_break x y) (0,0));
           (print_term1 t);
           (print_lterm2 l)
           end;;


(*==================================================
Affichage d'un terme aplati dans une box.
Format pour parsing (sous reserves que le parser
                     ignore \n et \t            ).
===================================================*)


let rec pprint_term1 = function
   TERM(f,[])   -> print_string (string_of_symbol_id f)
 | VAR x        -> print_string (string_of_var_id x)
 | TERM(f,s::l) -> if is_infix f
                   then begin
                        pprint_term2 s;
                        (fun(x,y)->print_break x y) (0,0);
                        pprint_infix_term f l
                        end
                   else if is_postfix f 
                        then begin
                             print_string "(";
                             pprint_term1 s;
                             pprint_lterm l;
                             print_string (string_of_symbol_id f)
                             end
                        else begin
                             print_string (string_of_symbol_id f);
                             print_string "(";
                             pprint_term1 s;
                             pprint_lterm l
                             end

and pprint_term2 = function
   TERM(f,[])	  -> print_string (string_of_symbol_id f)
 | VAR x          -> print_string (string_of_var_id x)
 | TERM(f,s::l) -> if is_infix f
                   then begin
                        print_string "(";
                        pprint_term2 s;
                        (fun(x,y)->print_break x y) (0,0);
                        pprint_infix_term f l;
                        print_string ")"
                        end
                   else if is_postfix f 
                        then begin
                             print_string "(";
                             pprint_term1 s;
                             pprint_lterm l;
                             print_string (string_of_symbol_id f)
                             end
                        else begin
                             print_string (string_of_symbol_id f);
                             print_string "(";
                             pprint_term1 s;
                             pprint_lterm l
                             end

and pprint_infix_term f = function
   []	-> ()
 | s::l	-> begin
           print_symbol_id_with_spaces f;
           (fun(x,y)->print_break x y) (0,0);
           pprint_term2 s;
           pprint_infix_term f l
           end


and print_symbol_id_with_spaces f =
   let t = string_of_symbol_id f in
     match t with
     "+" | "." | "*" -> print_string t
    | _ -> begin
           print_string " ";
           print_string t;
           (fun(x,y)->print_break x y) (1,0)
           end

and pprint_lterm = function
   []   -> print_string ")"
 | t::l -> begin 
           print_string ",";
           (fun(x,y)->print_break x y) (0,0);
           pprint_term1 t;
           pprint_lterm l
           end;;


(*===================================================*)

let print_term t = 
  begin
  open_hovbox 0;
  if options_courantes.print_to_parse
    then pprint_term1 t
    else print_term1 t;
  close_box ()
  end;;

