(***************************************************************************

Basic definitions about terms: specifications

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: terms.mli,v 1.1 2000/05/16 17:36:51 monate Exp $

***************************************************************************)


open Symbols;;
open Positions;;

(*===========================

Le type terme, ses constructeurs
et observateurs

===========================*)

type term = 
    VAR of var_id 
  | TERM of symbol_id * (term list)
;;


(*----------------

[(top_symbol_id t)] returns the top symbol id of the term [t],

[(liste_des_sous_termes t)] returns the list of subterms of [t],

both raise exception [Invalid_argument] if [t] is a variable.

-----------------*)

val top_symbol_id : term -> symbol_id;;

val liste_des_sous_termes  : term -> term list;;


(*

  [(set_of_symbols t)] returns the set of all symbols occurring in [t]

*)

val set_of_symbols : term -> symbol_id SymbolSet.t;;

(*----------------

----------------*)

val leftify_term : term -> term;;

val rightify_term : term -> term;;

val variable_est_dans_terme : var_id -> term -> bool;;

val variable_est_dans_equation : var_id -> (term * term) -> bool;;

val variable_est_dans_liste : var_id -> (term * term) list -> bool;;

val is_a_variable : term -> bool;;

val is_a_constant : term -> bool;;

val est_clos : term -> bool;;

val term_comp : term -> term -> int;;

val leq_term : term -> term -> bool;;

val sort_term :  term  -> term;;

(*=========================================

Fonctions taille

(taille_terme t) retourne la taille de $t$, c'est-\`a-dire le
nombre de symboles dans $t$, y compris les variables.

(taille_terme_sans_variables t) ne compte pas les variables

(taille_terme_variables_legeres t) compte les variables pour moitie

(plus_petit_terme s t) retourne le plus petit terme de s et t
vis-a-vis de l'ordre taille_terme

(plus_grand_terme s t) retourne le plus grand terme de s et t
vis-a-vis de l'ordre taille_terme

=========================================*)

val nesting : symbol_id -> term -> int;;

val symb_measure_sum : symbol_id -> term -> int;;

val symb_measure_max : symbol_id -> term -> int;;

val taille_terme_var_size : int -> term -> int;;

val taille_terme : term -> int;;

val taille_terme_sans_variables : term -> int;;

val taille_terme_variables_legeres : term -> int;;

val plus_petit_terme : term -> term -> term;;

val plus_grand_terme : term -> term -> term;;

(*=========================================

Fonctions variables_du_terme et
variables_de_liste_de_termes.

Symbols.new_var doit etre utilisee sur le resultat de variables_du_terme ou
variables_de_liste_de_termes pour engendrer un numero de variable
absent d'un terme ou d'une liste de termes donnes

==========================================*)

val variables_du_terme : 
      term -> var_id list;;

val variables_lineaires_du_terme : 
      term -> var_id list;;

val variables_de_liste_de_termes : 
      term list -> var_id list;;

val variables_equation : (term * term) -> var_id list;;

val variables_de_liste_equations : (term * term)list -> var_id list;;

(*========================================

Fonctions d'aplatissement.

aplatir aplati enti\`erement un terme, et aplatir_en_tete n'effectue
l'aplatissement qu'en t\^ete.

========================================*)

(*
val aplatir_en_tete : term -> term;;
val aplatir : term -> term;;
val aplatir_liste_de_termes : symbol_id -> term list -> term list;;
*)

val flatten_term : term -> term;;
val head_flatten_term : term -> term;;
val flatten_term_list : symbol_id -> term list -> term list;;

(*=================================

\'Egalit\'e syntaxique et modulo AC sur les termes aplatis

==================================*)

val egaux_synt : term -> term -> bool;;
val egaux_ac : term -> term -> bool;;


(*==================

Remplacement

[replace s t p] returns $s[t]_p$

IMPORTANT: This functions takes care of flattening w.r.t. AC symbols, that is if [s] and [t] are flat, then the result is also flat. For example:

[replace a+b+c d+e 1] returns [a+d+e+b]

===================*)

val replace : term -> term -> position -> term;;


(*====================

[add_left_var f x t] fabrique f(x,t) en aplatissant eventuellement

ATTENTION: on suppose ici f AC

======================*)

val add_left_var :
  symbol_id -> var_id -> term
  -> term
;;

(*====================

Fonctions de partage

(sous_termes_partage l) renvoie une liste [(l1,n1);...;(lk,nk)]
telle que l =_{\equiv} l1,..,l1,....,lk,...lk
                       \__  __/      \__  __/
                          \/            \/
                         n1 fois       nk fois


(reconstruit_combinaison_lineaire n [l1,n1;...,lk,nk]) =
  
  n1l1 + ... + nklk  ou +=(AC,n)

====================*)

val sous_termes_partages :
      term list -> (term * int) list;;

val duplique : int -> term -> term list;;

val reconstruit_combinaison_lineaire :
      symbol_id -> (term * int) list -> term;;



(*===========================================
Conversion d'un terme en chaine de caracteres
===========================================*)

val string_of_term : term -> string;;
val string_of_raw_term : term -> string;;

(*===========================================

These functions rename variables in terms

[rename_term t] returns a term equal to t modulo renamming, with
canonical variable ids

[rename_term (t,u)] does the same for a pair, with the same renaming
substitution

=============================================*)

val rename_term:
  term
  -> term
;;

val rename_pair:
  term * term
  -> term * term
;;


(*
  Enumeration des positions d'un terme en profondeur d'abord, 
  et en largeur d'abord : ce sera utilise dans la strategie de
  reduction outermost.
*)

val sous_terme_a_la_position : term -> position -> term;;

val next_pos_p : term -> position -> position;;

val next_pos_l : term -> position -> position;;


(*===========================================

Affichage d'un terme, d'une \'equation 
ou d'une r\`egle dans des box imbriquees.
Format special pour les symboles AC.

===========================================*)

val print_term : term -> unit;;
val pprint_term1 : term -> unit;;
val print_term1 : term -> unit;;

