(***************************************************************************

X interface Main Program

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: xmain.ml,v 1.1 2000/05/16 17:36:51 monate Exp $

***************************************************************************)


open Format
open Date_and_architecture  
open Abstract_syntax
open Norm_theory
open Pretty_printing
open Orderings
open Term_orderings
open Total_AC_ordering
open Xstrategy
open Ordered_strategy
open Symbols
open Terms
open Equations
open Normalised_rewriting
open Ordered_normalised_rewriting
open Messages
open Args
open Matching
open Unification
open Unif_theorie
open Errors


open Tk
open Xconfig

let title_string = "XCiME - demo version"

(*==========

Reading of specification file

===========*)

let show_error s =
  print_string (sprintf "%s error line %d columns %d-%d "
       	       	       	s
      	       	        etat_analyse.numero_ligne
      	       	       	etat_analyse.numero_colonne_avant
      	       	       	etat_analyse.numero_colonne_apres);
  if (etat_analyse.dernier_token_lu <> "")
    then print_string (sprintf " (near token '%s')\n" etat_analyse.dernier_token_lu) 
    else force_newline ()

let read_specifications () =
  let in_file =
    try open_in xconfig.file_name
    with Sys_error s ->
        try open_in (xconfig.file_name ^ ".cim")
        with Sys_error s ->
           try open_in (xconfig.file_name ^ ".nc")
           with Sys_error s -> 
               raise (Arg.Bad (sprintf "Could not open file %s: %s" xconfig.file_name s))
  in
  let lexbuf = Lexing.from_channel in_file
  in
    try
      let rec collect_data = function
      	[] -> ([],[],[],[],ORDRE_INTERACTIF,[],[])
      | s::l -> 
      	  let (op,th,uth,ax,o,pbs,conjs)=collect_data l
      	  in match s with
	      Abstract_operators (oper) -> (oper @ op,th,uth,ax,o,pbs,conjs)
	    | Abstract_theory tail ->(op,th @ (List.map theory_of_abstract_theory tail),uth,ax,o,pbs,conjs)  
	    | Abstract_unification_theory tail ->(op,th,uth @ (List.map theory_of_abstract_theory tail),ax,o,pbs,conjs)  
	    | Abstract_axioms al ->(op,th,uth,ax @ al,o,pbs,conjs)  
	    | Abstract_ordering ord ->(op,th,uth,ax,ord,pbs,conjs)  
	    | Abstract_problems pl ->(op,th,uth,ax,o,pbs @ pl,conjs)  
	    | Abstract_conjectures pl ->(op,th,uth,ax,o,pbs,conjs @ pl)  
	    | Abstract_unification_type t ->
                begin
		match t with
		    Unif_type_AC_only -> select_ac_only_unification() 
      	       	  | Unif_type_AC_complete -> select_ac_complete_unification()
      	       	  | Unif_type_plain -> select_plain_unification()
      	       	  | _ -> ()
		end;  
		(op,th,uth,ax,o,pbs,conjs)
	    | Abstract_include file_name -> (op,th,uth,ax,o,pbs,conjs)
 	

      in collect_data (Parser.donnees Lexer.token lexbuf) 

    with
      Erreur_de_syntaxe s ->
        show_error "Syntax";
        print_string s;
        force_newline ();
        abort()
    | Semantical_error s ->
        show_error "Semantical";
        print_string s;
        force_newline ();
        abort()
    | Parsing.Parse_error ->
        show_error "Syntax";
        abort()
    | Parsing.YYexit _ ->
        show_error "Syntax";
        print_string "Exception  'yyexit' raised. Please report.";
	force_newline ();
        abort()




let bye = function () -> closeTk () ;  exit (0) 
    
let top = openTk () 




let logo_frame = Frame.create top [BorderWidth (Pixels 4)] 

let caml_inside_label = Label.create logo_frame 
    [ Bitmap (BitmapFile "images/caml-inside.xbm"); 
      BorderWidth (Pixels 4);
      Relief Raised;      	       	       	    
      Foreground Black ;
      Background White] 


let logo_lri_image = Imagephoto.create 
    [ File "images/logo-lri.gif" ] 

let logo_lri_label = Label.create logo_frame 
    [ ImagePhoto logo_lri_image; 
      BorderWidth (Pixels 4);
      Relief Raised
    ] 


let select_menu_font () = ()


let menu_frame = Frame.create top [BorderWidth (Pixels 2); Relief Raised] 

let file_menu = Menubutton.create menu_frame [Text "File"; Font xconfig.menu_font] 

let file_menu_items = Menu.create file_menu [] 

let examples_menu = Menubutton.create menu_frame [Text "Examples"; Font xconfig.menu_font] 

let examples_menu_items = Menu.create examples_menu [] 

let menu_club_prive_ecossais = Menu.create examples_menu_items [TearOff false] 

let menu_groupes_abeliens = Menu.create examples_menu_items [TearOff false] 

let menu_anneaux_commutatifs = Menu.create examples_menu_items [TearOff false] 

let menu_morphisme_anneaux_commutatifs = Menu.create examples_menu_items [TearOff false] 

let menu_grobner_sur_z = Menu.create examples_menu_items [TearOff false] 

let menu_grobner_sur_f5 = Menu.create examples_menu_items [TearOff false] 

let menu_grobner_sur_q = Menu.create examples_menu_items [TearOff false] 

let menu_circuit_neg_3 = Menu.create examples_menu_items [TearOff false] 

let menu_entiers_peano = Menu.create examples_menu_items [TearOff false] 

let menu_entiers_binaires = Menu.create examples_menu_items [TearOff false] 

let menu_insert_sort = Menu.create examples_menu_items [TearOff false] 

let options_menu = Menubutton.create menu_frame [Text "Options"; Font xconfig.menu_font] 

let options_menu_items = Menu.create options_menu [] 

let help_menu = Menubutton.create menu_frame [Text "Help"; Font xconfig.menu_font] 

let help_menu_items = Menu.create help_menu [] 

(*
let source_frame = Frame.create top [Relief Sunken; BorderWidth (Pixels 2)]
let source_text = Text.create source_frame [Wrap WrapNone; TextHeight 40; TextWidth 80; Font xconfig.text_font]
let source_scroll = Scrollbar.create source_frame []
*)

(*
let log_frame = Frame.create top [Relief Sunken; BorderWidth (Pixels 2)]
let log_text = Text.create log_frame [Wrap WrapNone; TextHeight 20; TextWidth 80; Font xconfig.text_font]
let log_scroll = Scrollbar.create log_frame []
*)

(*
let info_line = Label.create top [BorderWidth (Pixels 2); Relief Sunken]
*)








let load_spec () = 

try

Symbols.free_symbols_table();
C_declare_operator.clear_c_operator_table();
Xstrategy.clear_completion_state();
Norm_theory.clear_current_theory();
Equations.reset_rule_counter();
Print_statistiques.reset_stat_counters();
Critical_pair_criteria.reset_stat_counters();
Encompassment.definir_symbole_egal();

select_ac_complete_unification();


let (signature,
     theorie,unification_theory,axiomes,ordre,problemes,conjectures) = 
  read_specifications() 
in

display_signature signature;

(*
Symbols.show_symbols_table();
*)

let automatic_ordering = options_courantes.automatic_ordering in
let ordre_externe = if (fun x y->x<>y) automatic_ordering 0
                    then total_AC_ordering automatic_ordering axiomes
                    else ordre in

let ordre_interne = ordre_of_ordre_disponible ordre_externe in

if options_courantes.niveau_de_verbosite >= 2
   then  begin
         message "----------------------";
         open_hovbox 0;
      	 print_string "Theory:";
         (fun(x,y)->print_break x y) (1,2);
	 print_theorie theorie;
         force_newline ();
	 close_box ();
	 print_flush();
         message "----------------------";
         print_string "Unification theory:";
         (fun(x,y)->print_break x y) (1,2);
	 print_theorie unification_theory;
	 force_newline ();
         message "----------------------";
	 print_string "Axioms:";
         (fun(x,y)->print_break x y) (1,2);
	 print_liste_d_equations axiomes;
	 force_newline ();
	 message "----------------------";
	 print_string "Order:";
         (fun(x,y)->print_break x y) (1,2);
         print_ordre ordre_externe;
	 force_newline ();
	 (*
	  printf "----------------------\n";
	  printf "Problems to solve:\n";
	  print_liste_d_equations problemes;
	  *)
	 message "----------------------";
	 print_string "Conjectures:";
         (fun(x,y)->print_break x y) (1,2);
	 print_liste_d_equations conjectures;
	 force_newline ();
	 print_flush()
      	 end;

construit_theorie_canonique(theorie);

compile_theorie_courante();

declare_theorie_unification(unification_theory);


if options_courantes.niveau_de_verbosite >= 1
   then begin
      	message "------------------------------------------";
	print_string "Normalised completion modulo the theory:";
        (fun(x,y)->print_break x y) (1,2);
	print_theorie theorie_canonique_courante.theories;
        force_newline();
	print_string "defined by the canonical set of rules:";
        (fun(x,y)->print_break x y) (1,2);
        print_liste_de_regles theorie_canonique_courante.regles;
        force_newline();   
	print_string "of the set of equations:";
        (fun(x,y)->print_break x y) (1,2);
        print_liste_d_equations axiomes;
        force_newline ();
	message "------------------------------------------"
	end;
print_flush ();

(*
let regles=
  if options_courantes.orientation_seule 
     then orientation_seule 
          (function (s,t) -> Greater)
          axiomes
     else  if options_courantes.ordered_completion
           then (ordered_completion 
                            ordre_interne
                            axiomes
                            conjectures)
           else (completion ordre_interne
                            axiomes
                            conjectures)

in
*)

initialise_completion 
  ordre_externe
  axiomes
  conjectures

with
  Erreur_de_syntaxe s    -> 
    message (sprintf "Syntactic analysis failed (%s)." s)
| Semantical_error s -> 
    message (sprintf "Semantical error: %s." s)
;;


(*
if not options_courantes.orientation_seule 
  then 
  begin
  open_vbox 0;
  force_newline();
  print_string "Result:";
  force_newline();
  print_liste_de_regles regles;
  force_newline ();        
  close_box();
  Print_statistiques.print_statistiques();
  print_flush()
  end
*)





let select_file_name name = 
  begin 
  xconfig.file_name <- "../examples/"^name;
  Wm.title_set top (title_string^" ("^name^")");    
  load_spec ()
  end

let select_text_font () =
(*
  let dialog_window = toplevelw.create (Support.new_toplevel_widget "dialog") [] in
  let message = Message.create dialog_window [Text "coucou"] in
  let entree = Entry.create dialog_window [] in
  Wm.title_set dialog_window "Font selection";
  pack [message] []
*)
()



let create_menus () =
  begin
    Menu.add_command file_menu_items 
      [Label "Load Spec"; State Disabled; Command (fun () -> select_file_name "")];
    Menu.add_command file_menu_items 
      [Label "Save Spec"; State Disabled];
    Menu.add_separator file_menu_items;
    Menu.add_command file_menu_items [Label "Quit"; Accelerator "(C-q)"; Command bye];
    Menubutton.configure file_menu [Menu file_menu_items];

    Menu.add_command examples_menu_items 
      [Label "Socrate est mortel ?"; 
	Command (fun () -> select_file_name "others/socrate.cim")];

    Menu.add_command examples_menu_items 
      [Label "Problme des camlons"; 
	Command (fun () -> select_file_name "others/probleme-des-cameleons.cim")];

    Menu.add_cascade examples_menu_items 
      [ Label "Problme du club priv cossais"; 
	Menu menu_club_prive_ecossais];

    Menu.add_command menu_club_prive_ecossais
      [ Label "modulo AC";
	Command (fun () -> 
	  select_file_name 
	    "scottish-private-club-problem/club-prive-AC.cim")];

    Menu.add_command menu_club_prive_ecossais
      [ Label "modulo BR";
 	Command (fun () -> 
	  select_file_name 
	    "scottish-private-club-problem/club-prive-BR.cim")];

    Menu.add_command examples_menu_items
      [ Label "Entiers relatifs";
 	Command (fun () -> 
	  select_file_name 
	    "arithmetic/peano/int.cim")];

    Menu.add_command examples_menu_items
      [ Label "Entiers avec inf";
 	Command (fun () -> 
	  select_file_name 
	    "arithmetic/peano/int-inf.cim")];

    
    Menu.add_cascade examples_menu_items 
      [ Label "Groupes abliens";
 	Menu menu_groupes_abeliens];

    Menu.add_command menu_groupes_abeliens
      [ Label "modulo AC";
	Command (fun () -> 
	  select_file_name 
	    "Abelian-groups-theory/groupes-abeliens-AC.cim")];

    Menu.add_command menu_groupes_abeliens
      [ Label "modulo ACU";
 	Command (fun () -> 
	  select_file_name 
	    "Abelian-groups-theory/groupes-abeliens-ACU.cim")];
    
    Menu.add_cascade examples_menu_items 
      [ Label "Anneaux commutatifs";
 	Menu menu_anneaux_commutatifs];

    Menu.add_command menu_anneaux_commutatifs
      [Label "modulo AC";
 	Command (fun () -> select_file_name "commutative-rings-theory/anneaux-commutatifs-AC.cim")];

    Menu.add_command menu_anneaux_commutatifs
      [Label "modulo AG";
 	Command (fun () -> select_file_name "commutative-rings-theory/anneaux-commutatifs-AG.cim")];

    Menu.add_cascade examples_menu_items 
      [ Label "Morphisme d'anneaux commutatifs";
 	Menu menu_morphisme_anneaux_commutatifs];

    Menu.add_command menu_morphisme_anneaux_commutatifs
      [Label "modulo AC";
 	Command (fun () -> 
	  select_file_name 
	    "commutative-rings-theory/morphisme-anneaux-commutatifs-AC.cim")];

    Menu.add_command menu_morphisme_anneaux_commutatifs
      [Label "modulo CR";
 	Command (fun () -> 
	  select_file_name 
	    "commutative-rings-theory/morphisme-anneaux-commutatifs-CR.cim")];

    Menu.add_cascade examples_menu_items 
      [ Label "Base de Grbner sur Z";
 	Menu menu_grobner_sur_z];

    Menu.add_command menu_grobner_sur_z
      [ Label "modulo AC";
 	State Disabled;
 	Command (fun () -> select_file_name "grobner-bases/simple-over-Z-AC.nc")];

    Menu.add_command menu_grobner_sur_z
      [Label "modulo CR";
 	Command (fun () -> select_file_name "grobner-bases/simple-over-Z.nc")];


    Menu.add_cascade examples_menu_items 
      [ Label "Base de Grbner sur Z mod 5";
 	Menu menu_grobner_sur_f5];

    Menu.add_command menu_grobner_sur_f5
      [ Label "modulo AC";
	State Disabled;
 	Command (fun () -> select_file_name "grobner-bases/over-f5-AC.nc")];

    Menu.add_command menu_grobner_sur_f5
      [Label "modulo F(5)";
 	Command (fun () -> select_file_name "grobner-bases/over-f5.nc")];


    Menu.add_cascade examples_menu_items 
      [ Label "Circuit  neg-3 ";
 	Menu menu_circuit_neg_3];

    Menu.add_command menu_circuit_neg_3
      [Label "modulo AC";
 	Command (fun () -> select_file_name "circuits/neg3-AC.cim")];

    Menu.add_command menu_circuit_neg_3
      [Label "modulo BR";
 	Command (fun () -> select_file_name "circuits/neg3.cim")];


    Menu.add_cascade examples_menu_items
      [ Label "Tri par insertion";
 	Menu menu_insert_sort];

    Menu.add_command menu_insert_sort
      [Label "modulo AC";
 	Command (fun () -> select_file_name "insert_sort/sort.cim")];

    Menu.add_command menu_insert_sort
      [Label "modulo BR";
 	Command (fun () -> select_file_name "insert_sort/sort_with_bool.cim")];

    Menu.add_command menu_insert_sort
      [Label "modulo BR et ACU";
 	Command (fun () -> select_file_name "insert_sort/sort_with_bool_and_bags.cim")];


    Menu.add_command examples_menu_items 
      [Label "Fichier modifiable"; 
	Command (fun () -> select_file_name "demo/demo.cim")];

    Menubutton.configure examples_menu [Menu examples_menu_items];

    Menu.add_command options_menu_items [Label "Text font"; State Disabled; Command select_text_font];
    Menu.add_command options_menu_items [Label "Menu font"; State Disabled; Command select_menu_font];
    Menubutton.configure options_menu [Menu options_menu_items];

    Menu.add_command help_menu_items [Label "No help available!"];
    Menubutton.configure help_menu [Menu help_menu_items]
  end




(*============

Programme principal

============*)

let main () =

message (sprintf "Welcome to XCiME v1.0 - %s\n\n" date_and_architecture);

try

lire_ligne_de_commande ();

if options_courantes.niveau_de_verbosite >= 1
   then begin
       	let r = Gc.get () in
	r.Gc.verbose <- true;
	Gc.set r;
        end;

Wm.title_set top title_string;
Wm.minsize_set top 1 1;

(*
Toplevel.configure top [Background (NamedColor "black")];
*)

(*
Toplevelw.configure top [Cursor (CursorFileFg("images/cam-green-happy.xpm",Black))] ;
*)

create_menus ();

Xstrategy.initialiser_fenetres top logo_frame;


pack [logo_frame] [Side Side_Left; Fill Fill_Y];
pack [logo_lri_label;caml_inside_label ] [Side Side_Top; PadY (Pixels 10)]; 

pack [menu_frame] [Side Side_Top; Fill Fill_X];
pack [file_menu; examples_menu; options_menu; help_menu] [Side Side_Left];

Xstrategy.pack_fenetres();

(*
bind top [[Control],XKey("q")] (BindSet([], function (EvInfo) -> bye ())) ;
*)

if options_courantes.nom_fichier <> ""
then 
  begin
    xconfig.file_name <- options_courantes.nom_fichier;
    load_spec ()
  end;

mainLoop () 


with
  Sys_error s            -> 
    message (sprintf "System error: %s" s)
| Arg.Bad s           -> 
    begin
    message ("Bad command line arguments: " ^ s);
    message "Usage: cime [options] <source file name>";
    message "options are:";
    message "";
    message "display options:";
    message "";
    message "  -pp           : pretty print, e.g. a+a+a is printed 3a";
    message "  -echo         : echo standard input";
    message "  -v            : set verbose level to 1";
    message "  -verbose n    : set verbose level to n";
    message "";
    message "completion strategy options:";
    message "";
    message "  -states       : show states of completion";
    message "  -strat n      : use strategy n";
    message "  -nocompl      : do not perform completion";
    message "  -confl        : check confluence";
    message "  -ordered      : perform ordered completion";
    message "";
    message "rewrite strategy options:";
    message "";
    message "  -inner        : innermost strategy";
    message "  -outer        : outermost strategy";
    message "";
    message "ordering options:";
    message "";
    message "  -auto-ord n   : automatic ordering ";
    message "            n=1 : RPO with lexicographic status ";
    message "            n=2 : RPO with multiset status ";
    message "";
    message "other options:";
    message "";
    message "  -rew-bound n  : use n as rewrite bound";
    message "  -undo         : allows 'undo' in interactive ordering"
    end
(*
| Completion_failed -> message "\nResult: completion failed."
| Completion_aborted -> message "\nResult: completion aborted."
*)
| Abort              -> message "Execution aborted."
| Invalid_argument s -> message (sprintf "Invalid_argument: %s." s);
      	       	       	message "Please report to Claude.Marche@lri.fr."
| Failure s          -> message (sprintf "Failure: %s." s);
      	       	       	message "Please report to Claude.Marche@lri.fr."
| ex -> message "Uncaught exception! Please report to Claude.Marche@lri.fr.";
        raise ex




let _ = 
  try
    begin
      Printexc.catch main ();
      force_newline();
      print_flush()
    end
  with e -> closeTk (); raise e



(*
(*=================

Preuves des problemes

=================*)

if problemes <> [] then
   begin
   force_newline();
   message "Solving problems:";
   List.iter (function
            EQUATION_A_PROUVER(u,v) ->
            let u1= (normalisation_normalisee_par_un_systeme_de_regles (-1) u regles)
            and v1= (normalisation_normalisee_par_un_systeme_de_regles (-1) v regles)
            in open_hovbox 0;
	       force_newline();
      	       print_string "The equation";
               (fun(x,y)->print_break x y)(1,2);
               print_equation (make_equation u v);
               (fun(x,y)->print_break x y)(1,0);
      	       if (egaux_ac (u1,v1))
                 then begin
                      print_string "is valid, common normal form:";
                      (fun(x,y)->print_break x y)(1,2);
                      print_term u1
                      end
                 else begin
                      print_string "is not valid, normal forms are";
                      (fun(x,y)->print_break x y)(1,2);
                      print_term u1;
                      (fun(x,y)->print_break x y)(1,0);
                      print_string "and";
                      (fun(x,y)->print_break x y) (1,2);
                      print_term v1
                      end ;
               force_newline ();
	       close_box();
	       print_flush()
        | TERMES_A_COMPARER(u,v) ->
		begin
	          open_hovbox 0;
      	       	  print_string "Comparison of";
                  (fun(x,y)->print_break x y) (1,0);   
                  print_term u;
                  (fun(x,y)->print_break x y) (1,0);
                  print_string "and";
                  (fun(x,y)->print_break x y) (1,0);
                  print_term v;
                  print_string ": ";
		  print_string (string_of_comparison_result
				  ((ordre_interne) (u,v))) ;
                  force_newline();
		  close_box();
		  print_flush ()
                end
(*
        | POLYS_A_COMPARER(u,v) ->
		begin
	          open_hovbox 0;
      	       	  print_string "Comparison of";
                  (fun(x,y)->print_break x y) (1,0);   
                  print_string (Poly.ecrit_polynome u);
                  (fun(x,y)->print_break x y) (1,0);
                  print_string "and";
                  (fun(x,y)->print_break x y) (1,0);
                  print_string (Poly.ecrit_polynome v);
                  print_string ": ";
		  print_string (string_of_comparison_result
				  (Ord_poly.compare_poly u v)) ;
                  force_newline();
		  close_box();
		  print_flush ()
                end
*)
        | EQUATION_A_MATCHER (u,v) -> begin
	                               open_hovbox 0;
      	       	       	       	       print_string "Matching of";
                                       (fun(x,y)->print_break x y) (1,0);   
                                       print_term u;
                                       (fun(x,y)->print_break x y) (1,0);
                                       print_string "and";
                                       (fun(x,y)->print_break x y) (1,0);
                                       print_term v;
                                       print_string ":";
                                       force_newline();
				       close_box();
				       print_flush ()
                                       end;

            begin
            try let sol = filtrage_ac u v in
                    print_string " solution:";
                    force_newline ();
       	            open_hovbox 0;
                    List.iter (function (x,t) -> print_term (VAR x);
                                               print_string "=";
                                               print_term t;
                                               force_newline())
                            sol;
                    close_box();
		    print_flush ()
            with No_match -> begin
                             print_string "No solution.";
                             force_newline ()
                             end;
	      force_newline();
	      print_flush()
            end
        | EQUATION_A_RESOUDRE(typ,u,v) -> begin
	                               open_hovbox 0;
      	       	       	       	       print_string "Unification of";
                                       (fun(x,y)->print_break x y) (1,0);   
                                       print_term u;
                                       (fun(x,y)->print_break x y) (1,0);
                                       print_string "and";
                                       (fun(x,y)->print_break x y) (1,0);
                                       print_term v;
                                       print_string ":";
                                       force_newline();
				       close_box();
				       print_flush ()
                                       end;

            begin
            match typ with
      	       Unif_type_AC_only -> select_ac_only_unification()
      	     | Unif_type_AC_complete -> select_ac_complete_unification()
      	     | Unif_type_plain -> select_plain_unification()
      	     | Unif_type_default -> ()
      	    end;

            let sols = set_of_unifiers u v in
	    if (List.length sols = 0)
               then begin
                    print_string "No solution.";
                    force_newline ()
                    end
               else begin
                    print_int (List.length sols);
                    print_string " solution(s):";
                    force_newline ();
                    List.fold_left (fun n theta -> 
                               begin
			       open_hovbox 0;
		               print_string "Solution ";
			       print_int n;
			       print_string ":";
                               (fun(x,y)->print_break x y)(1,2);
                               print_substitution theta;
                               force_newline ();
			       close_box();
			       n+1
      	       	       	       end
                            ) 1 sols;
		    ()
		    end;
	      force_newline();
	      print_flush()
          | TERME_A_REDUIRE(u) ->
            let u1=if options_courantes.ordered_completion
                   then (o_normalisation_normalisee_par_un_systeme_de_regles (-1) 
                          ordre_interne u regles)
                   else (normalisation_normalisee_par_un_systeme_de_regles (-1) u regles)
            in begin
               print_string "The normal form of ";
               (fun(x,y)->print_break x y) (1,0);
               print_term u;
               (fun(x,y)->print_break x y) (1,0);
               print_string "is";
               (fun(x,y)->print_break x y) (1,0);
               print_term u1;
               force_newline ();
	       print_string (sprintf "(by %d reductions)" (last_number_of_normalized_rewrite_steps()));
               force_newline ();
	       print_flush()
               end
          ) problemes;

   Print_statistiques.print_statistiques()

   end



*)
