(***************************************************************************

This module allows to define orderings on finite sets.

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: finite_orderings.ml,v 1.6 2001/03/15 14:49:08 marche Exp $

***************************************************************************)


open Orderings_generalities;;

exception Incompatible;;

(*i

module type Finite_ordering_type =
sig

  type 'a elt

  type 'a finite_ordering

  val identity_ordering : 'a finite_ordering;;
  
  val ordering : 'a finite_ordering -> 'a Orderings_generalities.ordering;;

  val add_equiv : 'a finite_ordering -> 'a -> 'a -> 'a finite_ordering;;

  val add_greater : 'a finite_ordering -> 'a -> 'a -> 'a finite_ordering;;

end
;;

module Make(M : Ordered_types.OrderedPolyType) : 
  Finite_ordering_type with type 'a elt = 'a M.t
;;
		       
		       
		       
i*)

			 
class type ['a] finite_ordering_class = 
  object
    method compare : 'a Orderings_generalities.ordering
    method add_equiv : 'a -> 'a -> unit
    method add_greater : 'a -> 'a -> unit
  end
;;

let get comp t x y =
  if comp x y = 0
  then Equivalent
  else
    try
      let m = Classical_maps.find x t in
      Classical_maps.find y m
    with Not_found -> Uncomparable
;;

let set comp t x y v =
  let m = 
    try
      Classical_maps.find x t 
    with Not_found -> Classical_maps.empty comp
  in 
  let m = Classical_maps.add y v m in
  Classical_maps.add x m t
;;

class ['a] finite_ordering compare =
  object

    val mutable table = 
      (Classical_maps.empty compare : 
       ('a,('a,comparison_result) Classical_maps.t) Classical_maps.t)

    val comp = compare

    method compare x y = get comp table x y

    method add_equiv x y =
      match get comp table x y with
	  Uncomparable ->
	    let t = set comp table x y Equivalent in
	    let t = set comp t y x Equivalent in
	    let t =
	      Classical_maps.fold
		(fun x1 m t ->
		   Classical_maps.fold
		     (fun y1 v t -> 
			let c1 = get comp t x1 x
			and c2 = get comp t y y1
			in
			if c1 = Equivalent & c2 = Equivalent
			then
			  match get comp t x1 y1 with
			      Uncomparable ->
				let t = set comp t x1 y1 Equivalent in
				set comp t y1 x1 Equivalent
			    | Equivalent -> t
			    | _ -> raise Incompatible
			else 
			  if is_greater_or_equal c1 & is_greater_or_equal c2
			  then 
			    if is_less_or_equal (get comp t x1 y1)
			    then raise Incompatible
			    else
			      let t = set comp t x1 y1 Greater_than in
			      set comp t y1 x1 Less_than
			  else 
			    if is_less_or_equal c1 & is_less_or_equal c2
			    then 
			      if is_greater_or_equal (get comp t x1 y1)
			      then raise Incompatible
			      else
				let t = set comp t x1 y1 Less_than in
				set comp t y1 x1 Greater_than
			    else t)
		     t
		     t)
		t
		t
	    in
	    table <- t
	| Equivalent -> ()
	| _ -> raise Incompatible

    method add_greater x y =
      if is_less_or_equal (get comp table x y)
      then
	raise Incompatible
      else
	let t = set comp table x y Greater_than in
	let t = set comp t y x Less_than in
	let t =
	  Classical_maps.fold
	    (fun x1 m t ->
	       Classical_maps.fold
		 (fun y1 v t -> 
		    if is_greater_or_equal (get comp t x1 x) 
		      & is_greater_or_equal (get comp t y y1) 
		    then
		      if is_less_or_equal (get comp t x1 y1)
		      then raise Incompatible
		      else
			let t = set comp t x1 y1 Greater_than in
			set comp t y1 x1 Less_than
		    else t)
		 t
		 t)
	    t
	    t
      in
      table <- t
     
  end
;;

let identity_ordering compare = new finite_ordering compare;;





type 'a precedence =
    One of 'a
  | Gt of 'a * 'a precedence
  | Lt of 'a * 'a precedence
  | Eq of 'a * 'a precedence
;;

let first p = 
  match p with
      One(x) -> x
    | Gt(x,_) -> x
    | Lt(x,_) -> x
    | Eq(x,_) -> x
;;

let rec add_list order prec = 
  match prec with
      Gt(x,y) -> 
	order#add_greater x (first y);
	add_list order y
    | Lt(x,y) -> 
	order#add_greater (first y) x;
	add_list order y
    | Eq(x,y) -> 
	order#add_equiv x (first y);
	add_list order y
    | _ -> ()
;;


let rec map_prec f p =
  match p with
    | One(x) -> One(f x)
    | Gt(x,l) -> Gt(f x,map_prec f l)
    | Lt(x,l) -> Lt(f x,map_prec f l)
    | Eq(x,l) -> Eq(f x,map_prec f l)
;;
