(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: parallel.ml4,v 1.2 2001/04/20 13:42:24 marche Exp $

***************************************************************************)

open Format

(* This handle the case when pvm is not detected. *) 
ifndef PVM then 
module M = 
  struct
let smp = ref false

let is_god () = true

let max_process = ref 0
(* [compute f next] returns the list of all [f (next_element())]. 
   The computation is done in parallel with a pvm client.*)

let init_server () = ()

let compute (f: 'a->'b) (st: 'a Stream.t) (acc: 'b -> 'c -> 'c) (v0: 'c) : 'c =
  let rec compute myacc =
    parser 
	[< >] -> myacc
      | [< 'x ; r >] -> compute (acc x myacc)
  in
  compute v0 st

let computer_client () = assert false
  end

else (* Here is the pvm case.*)

module M = 
struct
let smp = ref true

let is_god () =
  try  ignore (Pvm.parent());false with Failure "PvmNoParent" -> true

let mesg_tag_init = Pvm.msgtag_of_int 8

let mesg_tag_result = Pvm.msgtag_of_int 9

let mesg_tag_f = Pvm.msgtag_of_int 10

let mesg_tag_x = Pvm.msgtag_of_int 11

let reset_mesg_tag = Pvm.msgtag_of_int 15

(* Maximum number of computation processes.  Does not count the
  toplevel itself. Can be changed with [#smp i] in the toplevel.*)
let max_process = ref 2

let cim2exec= 
  let myself = Sys.argv.(0) in
  let current_dir = Sys.getcwd () in
  let mybase = Filename.dirname myself in
  Sys.chdir mybase;
  let my_absolute_path = Sys.getcwd () in
  Sys.chdir current_dir;
  Filename.concat my_absolute_path (Filename.basename myself)

(* [compute f next] returns the list of all [f (next_element())]. 
   The computation is done in parallel with a pvm client.*)

let mytid = 
  try 
    Pvm.mytid () 
  with 
      Failure "PvmSysErr" -> 
	printf "Launching pvmd...\n";print_flush();
	try 
	  ignore (Sys.command "echo quit | pvm");
	  Pvm.mytid ()
	with _ -> printf "Hum. I cannot launch pvm. Please do it yourself.";
	  exit 1
	    
let process_array = Array.create (!max_process+1) mytid 

let clean_up_server () =
    for i = 1 to !max_process do 
      Pvm.kill process_array.(i)
    done;
  Pvm.exit ()

let init_server () = 
  Pvm.routedirect ();
  for i = 1 to !max_process do
    process_array.(i) <- Pvm.simple_spawn cim2exec "";
  done;
  let bufid = Pvm.initsend Pvm.DataRaw in
  Pvm.pkobj mytid;
  Pvm.mcast process_array mesg_tag_init;
  for i = 1 to !max_process do
    ignore (Pvm.recv Pvm.anyTid Pvm.anyMsgTag);
  done;
  at_exit clean_up_server
      

let compute (f: 'a->'b) (st: 'a Stream.t) (acc: 'b -> 'c -> 'c) (v0: 'c) =
  ignore (Pvm.initsend Pvm.DataRaw);
  Pvm.pkobj f;
  Pvm.mcast process_array mesg_tag_x;
  let result = ref v0 in  
  let running = ref 0 in
  let finished = ref false in
  let free_tid = ref (List.tl (Array.to_list process_array)) in
  while not (!finished && (!running == 0)) do
    while !running < !max_process && not !finished do
      try 
	let x = Stream.next st in
	let where = List.hd !free_tid in
	free_tid := List.tl !free_tid;
	ignore (Pvm.initsend Pvm.DataRaw);    
	Pvm.pkobj x;
	Pvm.send where mesg_tag_x;
	incr running
      with Stream.Failure ->
	finished := true
    done;
    if not (!finished && (!running == 0)) then
      let buffid = Pvm.recv Pvm.anyTid Pvm.anyMsgTag in
      let x,tag,ti = Pvm.bufinfo buffid in
      free_tid := ti ::!free_tid;
      decr running;
      result := acc (Pvm.upkobj ()) !result
  done;
  ignore (Pvm.initsend Pvm.DataRaw);
  Pvm.mcast process_array reset_mesg_tag;
  !result
      
let computer_client () = 
    Pvm.routedirect ();
    ignore (Pvm.recv Pvm.anyTid Pvm.anyMsgTag);
    let ancestor_pid = (Pvm.upkobj () :Pvm.tid) in
    printf "Nb %x got ancestor (%x)\n" (Pvm.int_of_tid mytid) 
      (Pvm.int_of_tid ancestor_pid); 
    print_flush ();
    ignore (Pvm.initsend Pvm.DataRaw);
    Pvm.pkobj "OK";
    Pvm.send ancestor_pid mesg_tag_init;
    let rec loop () =
      ignore (Pvm.recv Pvm.anyTid Pvm.anyMsgTag);
      let f = (Pvm.upkobj ():'a->'b) in
      printf "Nb %x got f\n" (Pvm.int_of_tid mytid) ;print_flush ();
      let end_of_computation = ref false in
      while not !end_of_computation  do
	let buffid = Pvm.recv ancestor_pid Pvm.anyMsgTag in
	let _,tag,ti = Pvm.bufinfo buffid in
	if tag <> reset_mesg_tag then
	  let x = (Pvm.upkobj ():'a) in
	  try 
	    let r = f x in
	    ignore (Pvm.initsend Pvm.DataRaw);
	    Pvm.pkobj r;
	    Pvm.send ancestor_pid mesg_tag_result;
	  with _ -> assert false
	else
	  end_of_computation := true
      done;
      loop ()
    in
    loop ()
  end
;;

(*i

Would be useful if we had polymorphic methods.

class type virtual ['a,'b,'c] compute =
      object
	method virtual is_god : unit -> bool
	method virtual max_process : int ref
	method virtual init_server : unit -> unit
	method virtual computer_client : unit -> unit
	method virtual compute : ('a -> 'b) -> 'a Stream.t -> 
	  ('b -> 'c -> 'c ) -> 'c -> 'c
      end

class virtual ['a,'b,'c] computer:['a,'b,'c] compute = 
  object
    method virtual is_god : unit -> bool
    method virtual max_process : int ref
    method virtual init_server : unit -> unit
    method virtual computer_client : unit -> unit
    method virtual compute : ('a -> 'b) -> 'a Stream.t -> 
      ('b -> 'c -> 'c ) -> 'c -> 'c
  end


class ['a,'b,'c] up =
  object
  inherit ['a,'b,'c] computer
  method is_god () = (failwith "PvmNoParent" : bool)
  method max_process = ref 0
  method init_server () = ()
  method compute (f: 'a->'b) (st: 'a Stream.t) 
    (acc: 'b -> 'c -> 'c) (v0: 'c) : 'c =
    let rec compute myacc =
      parser 
	[< >] -> myacc
	| [< 'x ; r >] -> compute (acc x myacc)
    in
    compute v0 st
  method computer_client () = assert false
end
i*)
