


(*

\subsection{The regexp datatype and its constructors}

The type of regular expressions is abstract. Regular expressions may
be built from the following constructors :

\begin{itemize}

\item [empty] is the regexp that denotes no word at all.

\item [epsilon] is the regexp that denotes the empty word.

\item [char c] returns a regexp that denotes only the single-character
word [c].

\item [chars s] returns a regexp that denotes any single-character
word belonging to set of chars [s].

\item [string str] denotes the string [str] itself.  

\item [star e] where [e] is a regexp, denotes the Kleene iteration of
[e], that is all the words made of concatenation of zero, one or more
words of [e].

\item [alt e1 e2] returns a regexp for the union of languages of [e1]
and [e2].

\item [seq e1 e2] returns a regexp for the concatenation of languages
of [e1] and [e2].

\item [opt e] returns a regexp for the set of words of [e] and the
empty word.

\item [some e] denotes all the words made of concatenation of one or
more words of [e].

\end{itemize}

*)

(* type 'a regexp = 'a Regular_expr.regexp ;;

module CharSet : Regular_expr.CharSet
*)
(* Inttagset.IntTagSetModule with type 'a elt = 'a
*)
open Regular_expr

val empty : 'a regexp;;
val epsilon : 'a regexp;;
val char : 'a  -> 'a regexp;;
val star : 'a regexp -> 'a regexp;;
val alt : 'a regexp -> 'a regexp -> 'a regexp;;
val seq : 'a regexp -> 'a regexp -> 'a regexp;;
val opt : 'a regexp -> 'a regexp;;
val some : 'a regexp -> 'a regexp;;


(*

  \subsection{Compilation of regular expressions}

*)
 

type 'a compiled_regexp;;

val compile : 'a regexp -> 'a compiled_regexp;;


