(***************************************************************************

Discrimination nets in a naive way: non linearity is not handled,
subterms whose top is a non-free symbol are considered.

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: naive_dnet.mli,v 1.3 2002/07/16 16:03:47 contejea Exp $

***************************************************************************)



type 'symbol dnet ;;



(*

  [compile sigma r] compiles the TRS [r] on signature [sigma] into a
  discriminate net, to be used with the function [discriminate]
  in order to perform many to one matching.

*)

val compile : 
  'symbol #Signatures.signature ->
    'symbol Rewrite_rules.t list -> 'symbol dnet

(*

  [compile_for_unification sigma r] compiles the TRS [r] on signature 
  [sigma] into a discriminate net, to be used with the function 
  [discriminate] in order to perform many to one unification.

*)

val compile_for_unification : 
  'symbol #Signatures.signature ->
    'symbol Rewrite_rules.t list -> ('symbol Rewrite_rules.t list * 'symbol dnet)

(*

  [discriminate d t] returns the list of rules of [d] which may match
  term [t]. One as to call a complete matching algorithm to each of
  the resulting rules, to take into account non-linearities and C or
  AC symbols.

  Examples:

  \begin{itemize}

  \item for any term headed by $f$, a non-linear rule of the form
  $f(x,x) \rightarrow r$ will be in the returned list.

  \item for any term headed by AC symbol $+$, a rule of the form
  $l_1+\cdots+l_n \rightarrow r$ will be in the returned list.

  \end{itemize}

*)


val discriminate :
  'symbol dnet -> 'symbol Gen_terms.term -> 'symbol Rewrite_rules.t list

val print : 'symbol #Signatures.signature -> 'symbol dnet -> unit


