(***************************************************************************

Standard unification

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: standard_unification.ml,v 1.3 2002/07/11 13:25:19 pguillon Exp $

***************************************************************************)



open Gen_terms;;



(* [(unification term1 term2)] returns the most general unifier of
  [term1] and [term2]. Raises No_unifier if no unifier is found.

   This is standard unification : all symbols are assumed to be free.

*)

exception No_unifier;;

open Format;;

let rec unification_no_occur_check term1 term2 = 

(*
  print_string "unif of ";
  print_term 
    (new Signatures.dummy_sig)
    (new Variables.from_string "")
    term1;

  print_string " and ";
  print_term 
    (new Signatures.dummy_sig)
    (new Variables.from_string "")
    term2;
  print_string " : ";
*)

  let s = 
    match (term1.Hcons.node,term2.Hcons.node) with
      (Var x,_) ->  
	Variables.VarMap.add x term2 Variables.VarMap.empty
    | (_,Var y) ->  
	Variables.VarMap.add y term1 Variables.VarMap.empty
    | (Term(f,l),Term(g,m)) ->
	 if f=g 
	 then
	   try
	     List.fold_right2 
	       (fun t1 t2 subst ->
		  Substitution.merge_substs 
		    (unification_no_occur_check t1 t2) subst)
	       l m Variables.VarMap.empty
	   with 
	       Invalid_argument _ -> raise No_unifier
	     | Substitution.Conflict -> raise No_unifier
	 else raise No_unifier


  in
(*
      Substitution.print 
	(new Signatures.dummy_sig)
	(new Variables.from_string "")
	s;
*)
      s

;;


let rec pre_unifiable sigma t1 t2 = 

  match (t1.Hcons.node,t2.Hcons.node) with
      (Var x,_) -> true
    | (_,Var y) -> true
    | (Term(f,l),Term(g,m)) ->
	 if f=g 
	 then 
	   if sigma#is_free f
	   then
	     try
	       List.for_all2 (pre_unifiable sigma) l m
	     with 
		 Invalid_argument _ -> false
	   else true
	 else false


