(***************************************************************************

Innermost strategies

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: innermost.mli,v 1.5 2002/07/18 14:48:34 contejea Exp $

***************************************************************************)

open Gen_terms;;
open Substitution;;

(*

  [(innermost_normalize find memo sign red t)] returns the innermost normal
form of [t] w.r.t to the reduction relation [red].

  [red] is a function which, given a term [t], returns a pair
  [(u,sigma)] such that [t] rewrites to [u sigma], or raises
  [Irreducible] if [t] is irreducible.

  The substitution will be applied during the normalization process.
  [find] and [memo] are memoization functions.
*)


val innermost_normalize : 
  ('symbol term -> 'symbol term) ->  ('symbol term -> 'symbol term -> unit) ->
    'symbol #Signatures.signature -> 'symbol HashTerm.t ->
      ('symbol term -> 'symbol term * 'symbol substitution) -> 
	'symbol term -> 'symbol term ;;

(*

  [(force_innermost_normalize find memo red t)] does the same as
  [(innermost_normalize find memo red t)], but raises [Irreducible] if no
  reduction at all is possible.
  [find] and [memo] are memoization functions.

*)

exception Irreducible;;

val force_innermost_normalize : 
  ('symbol term -> 'symbol term) ->  ('symbol term -> 'symbol term -> unit) ->
    'symbol #Signatures.signature -> 'symbol HashTerm.t ->
      ('symbol term -> 'symbol term * 'symbol substitution) -> 
	'symbol term -> 'symbol term ;;

(* [safe_innermost_normalize find memo sign n red t] does the same but where [n]
is a bound for the number of rewrite steps to apply. If this bound is
reached, then this function raises the exception [Unnormalized u]
where [u] is the reduct of [t] obtained so far *)


exception Unnormalized;;

val safe_innermost_normalize : 
  ('symbol term -> 'symbol term) ->  ('symbol term -> 'symbol term -> unit) ->
    'symbol #Signatures.signature -> 'symbol HashTerm.t ->
      int -> 
	('symbol term -> 'symbol term * 'symbol substitution) -> 
	  'symbol term -> 'symbol term ;;

val safe_force_innermost_normalize : 
  ('symbol term -> 'symbol term) ->  ('symbol term -> 'symbol term -> unit) ->
    'symbol #Signatures.signature -> 'symbol HashTerm.t ->
      int -> 
	('symbol term -> 'symbol term * 'symbol substitution) -> 
	  'symbol term -> 'symbol term ;;


