(***************************************************************************

Abstract syntax: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: abstract_syntax.ml,v 1.3 1998/04/28 13:36:02 marche Exp $

***************************************************************************)

(*

Obsolete header:

Last modified by: $Author: marche $
File name:        $Source: /users/demons/demons/master-sources-repository//cime/ocaml_src/syntax/abstract_syntax.ml,v $
Revision:         $Revision: 1.3 $ 
Last modified on: $Date: 1998/04/28 13:36:02 $

***************************************************************************)

open Errors;;
open Symbols;;
open Norm_theory;;
open Equations;;
open Equations_as_obj;;
open Term_orderings;;

exception Erreur_de_syntaxe of string;;

type lexer_state_type =
  Standard_lexer | Poly_lexer | Ident_lexer | Term_lexer
;;

type type_etat_analyse =
  {
    mutable numero_ligne : int;
    mutable numero_colonne_avant : int;
    mutable numero_colonne_apres : int;
    mutable dernier_token_lu : string;
    mutable lexer_state : lexer_state_type;
    mutable top_allowed : bool 
  };;

type predefined_or_user_theory =
  PRDF of string list
| USR of regle list
;;

type abstract_syntax =
  Abstract_include of string
| Abstract_operators of (string list * Symbols.fix_type * (C_declare_operator.symbol_type * int)) list
| Abstract_theory of predefined_or_user_theory list
| Abstract_unification_theory of predefined_or_user_theory list
| Abstract_unification_type of unif_type
| Abstract_axioms of pair_of_terms list
| Abstract_ordering of ordre_disponible
| Abstract_problems of probleme list
| Abstract_conjectures of pair_of_terms list
| Abstract_inductive of pair_of_terms list
| Abstract_sortes of sorte_de_base list 
| Abstract_subsortes of (sorte_de_base*sorte_de_base) list
;;

let etat_analyse =
     { numero_ligne = 1;
       numero_colonne_apres = 0;
       numero_colonne_avant = 0;
       dernier_token_lu = "";
       lexer_state = Standard_lexer;
       top_allowed = false
     };;

let predefined_theory_of_abstract_theory id_list = 
  let theory_id = List.hd id_list
  and params = List.tl id_list 
  in match theory_id with 
    "ACU" -> 
      begin
      match params with
        [i1;i2] -> 
      	  let f1=(get_symbol_id i1)
      	  and f2=(get_symbol_id i2) in
	  if not(is_ac f1)
            then semantical_error
                        "First symbol must be AC in ACU theory"
            else if (not(is_free f2)) or (arity(f2)<>0)
                   then semantical_error
                          "Second symbol must be a constant in ACU theory"
           else ACU(f1,f2) 
      | _ -> semantical_error
                        "ACU theory needs exactly two parameters"
      end

  | "AU" ->
      begin
      match params with
        [i1;i2] -> 
      	  let f1=(get_symbol_id i1)
          and f2=(get_symbol_id i2) in
	  if (not(is_free f1)) or (arity(f1)<>2)
          then semantical_error
                        "First symbol must be binary in AU theory"
          else if (not(is_free f2)) or (arity(f2)<>0)
            then semantical_error
                        "Second symbol must be a constant in AU theory"
            else AU(f1,f2) 
      | _ -> semantical_error
                        "AU theory needs exactly two parameters"
      end

  | "A" ->
      begin
      match params with
        [i1] -> 
      	  let f1=(get_symbol_id i1) in   
      	if (not(is_free f1)) or (arity(f1)<>2)
          then semantical_error
                           "symbol must be binary in theory A"
        else A(f1) 
      | _ -> semantical_error
                        "A theory needs exactly one parameter"
      end

  | "ACI" ->
      begin
      match params with
        [i1] -> 
      	  let f1=(get_symbol_id i1) in
	if not(is_ac f1)
          then semantical_error
                           "symbol must be AC in theory ACU"
        else ACI(f1)
      | _ -> semantical_error
                        "ACI theory needs exactly one parameter"
      end
 
  | "ACUI" ->
      begin
      match params with
        [i1;i2] -> 
          let f1=(get_symbol_id i1)
          and f2=(get_symbol_id i2) in
	if not(is_ac f1)
          then semantical_error
                        "First symbol must be AC in theory ACUI"
        else if (not(is_free f2)) or (arity(f2)<>0)
          then semantical_error
                        "Second symbol must be a constant in theory ACUI"
        else ACUI(f1,f2)
      | _ -> semantical_error
                        "ACUI theory needs exactly two parameters"
      end

  | "ACUN" ->
      begin
      match params with
        [i1;i2;i3] -> 
      let f1=(get_symbol_id i1)
      and f2=(get_symbol_id i2) 
      and n=(try (int_of_string i3) 
             with Failure "int_of_string" -> semantical_error
                          "Third parameter must an integer in theory ACUN") in

	if not(is_ac f1)
          then semantical_error
                        "First symbol must be AC in theory ACUN"
        else if (not(is_free f2)) or (arity(f2)<>0)
          then semantical_error
                        "Second symbol must be a constant in theory ACUN"
        else ACUN(f1,f2,n)
      | _ -> semantical_error
                        "ACUN theory needs exactly three parameters"
      end

  | "G" -> 
      begin
      match params with
        [i1;i2;i3] -> 
      let f1=(get_symbol_id i1)
      and f2=(get_symbol_id i2)
      and f3=(get_symbol_id i3) in
	if (not(is_free f1)) or (arity(f1)<>2)
          then semantical_error
                        "First symbol must be binary in theory G"
        else if (not(is_free f2)) or (arity(f2)<>0)
          then semantical_error
                        "Second symbol must be a constant in theory G"
        else if (not(is_free f3)) or (arity(f3)<>1)
          then semantical_error
                        "Third symbol must be unary in theory G"
        else G(f1,f2,f3) 
      | _ -> semantical_error
                        "G theory needs exactly three parameters"
      end

  | "AG" ->
      begin
      match params with
        [i1;i2;i3] -> 
      let f1=(get_symbol_id i1)
      and f2=(get_symbol_id i2)
      and f3=(get_symbol_id i3) in
	if not(is_ac f1)
          then semantical_error
                        "First symbol must be AC in theory AG"
        else if (not(is_free f2)) or (arity(f2)<>0)
          then semantical_error
                        "Second symbol must be a constant in theory AG"
        else if (not(is_free f3)) or (arity(f3)<>1)
          then semantical_error
                        "Third symbol must be unary in theory AG"
        else AG(f1,f2,f3) 
      | _ -> semantical_error
                        "AG theory needs exactly three parameters"
      end
   
| "CR" -> 
      begin
      match params with
        [i1;i2;i3;i4;i5] -> 
      let f1=(get_symbol_id i1)
      and f2=(get_symbol_id i2)
      and f3=(get_symbol_id i3)
      and f4=(get_symbol_id i4)
      and f5=(get_symbol_id i5) in
	if not(is_ac f1)
          then semantical_error
                        "First symbol must be AC in theory CR"
        else if (not(is_free f2)) or (arity(f2)<>0)
          then semantical_error
                        "Second symbol must be a constant in theory CR"
        else if (not(is_free f3)) or (arity(f3)<>1)
          then semantical_error
                        "Third symbol must be unary in theory CR"
        else if not(is_ac f4)
          then semantical_error
                        "Fourth symbol must be AC in theory CR"
        else if (not(is_free f5)) or (arity(f5)<>0)
          then semantical_error
                        "Fifth symbol must be a constant in theory CR"
        else CR(f1,f2,f3,f4,f5)
       | _ -> semantical_error
                       "CR theory needs exactly five parameters"
      end

    
| "BR" ->
     begin
      match params with
        [i1;i2;i3;i4] -> 
       let f1=(get_symbol_id i1)
      and f2=(get_symbol_id i2)
      and f3=(get_symbol_id i3)
      and f4=(get_symbol_id i4) in
	if not(is_ac f1)
          then semantical_error
                        "First symbol must be AC in theory BR"
        else if (not(is_free f2)) or (arity(f2)<>0)
          then semantical_error
                        "Second symbol must be a constant in theory BR"
        else if not(is_ac f3)
          then semantical_error
                        "Third symbol must be AC in theory BR"
        else if (not(is_free f4)) or (arity(f4)<>0)
          then semantical_error
                        "Fourth symbol must be a constant in theory BR"
        else BR(f1,f2,f3,f4)
       | _ -> semantical_error
                       "BR theory needs exactly four parameters"
      end

    
| "FP" ->
      begin
      match params with
        [i1;i2;i3;i4;i5;i6] -> 
      let p = try (int_of_string i1)
	      with Failure "int_of_string" ->
		     semantical_error
                              "First parameter must an integer in theory FP" 
      and f1=(get_symbol_id i2)
      and f2=(get_symbol_id i3)
      and f3=(get_symbol_id i4)
      and f4=(get_symbol_id i5)
      and f5=(get_symbol_id i6) in
        if not(is_ac f1)
          then semantical_error
                        "First symbol must be AC in theory FP"
        else if (not(is_free f2)) or (arity(f2)<>0)
          then semantical_error
                        "Second symbol must be a constant in theory FP"
        else if (not(is_free f3)) or (arity(f3)<>1)
          then semantical_error
                        "Third symbol must be unary in theory FP"
        else if not(is_ac f4)
          then semantical_error
                        "Fourth symbol must be AC in theory FP"
        else if (not (is_free f5)) or (arity(f5)<>0)
          then semantical_error
                        "Fifth symbol must be a constant in theory FP"
        else FP(p,f1,f2,f3,f4,f5) 
       | _ -> semantical_error
                       "FP theory needs exactly six parameters, the first one being an integer"
      end
| "Int" ->
      begin
      match params with
        [i1;i2;i4;i5;i6;i7;i8] -> 
      let plus_z=(get_symbol_id i1)
      and coef=(get_symbol_id i2)
      and plus=(get_symbol_id i4)
      and zero=(get_symbol_id i5)
      and moins=(get_symbol_id i6)
      and mult=(get_symbol_id i7)
      and un=(get_symbol_id i8) in
        if not(is_ac plus_z)
          then semantical_error
                        "First symbol must be AC in theory Int"
        else if (not(is_free coef)) or (arity(coef)<>1)
          then semantical_error
                        "Second symbol must be unary in theory Int"
        else if not(is_ac plus)
          then semantical_error
                        "Third symbol must be AC in theory Int"
        else if (not(is_free zero)) or (arity(zero)<>0)
          then semantical_error
                        "Fourth symbol must be a constant in theory Int"
        else if (not(is_free moins)) or (arity(moins)<>1)
          then semantical_error
                        "Fifth symbol must be unary in theory Int"
        else if not(is_ac mult)
          then semantical_error
                        "Sixth symbol must be AC in theory Int"
        else if (not (is_free un)) or (arity(un)<>0)
          then semantical_error
                        "Seventh symbol must be a constant in theory Int"
        else Int(plus_z,coef,plus,zero,moins,mult,un) 
       | _ -> semantical_error
                       "Int theory needs exactly seven parameters"
      end
| "Rat" ->
      begin
      match params with
        [i1;i2;i3;i4;i5;i6;i7;i8] -> 
      let un_z=(get_symbol_id i1)
      and coef=(get_symbol_id i2)
      and irred=(get_symbol_id i3)
      and plus=(get_symbol_id i4)
      and zero=(get_symbol_id i5)
      and moins=(get_symbol_id i6)
      and mult=(get_symbol_id i7)
      and un=(get_symbol_id i8) in
        if (not(is_free un_z)) or (arity(un_z)<>0)
          then semantical_error
                        "First symbol must be a constant in theory Rat"
        else if (not(is_free coef)) or (arity(coef)<>1)
          then semantical_error
                        "Second symbol must be unary in theory Rat"
        else if (not(is_free irred)) or (arity(irred)<>2)
          then semantical_error
                        "Third symbol must be binary in theory Rat"
        else if not(is_ac plus)
          then semantical_error
                        "Fourth symbol must be AC in theory Rat"
        else if (not(is_free zero)) or (arity(zero)<>0)
          then semantical_error
                        "Fifth symbol must be a constant in theory Rat"
        else if (not(is_free moins)) or (arity(moins)<>1)
          then semantical_error
                        "Sixth symbol must be unary in theory Rat"
        else if not(is_ac mult)
          then semantical_error
                        "Seventh symbol must be AC in theory Rat"
        else if (not (is_free un)) or (arity(un)<>0)
          then semantical_error
                        "Eighth symbol must be a constant in theory Rat"
        else Rat(un_z,coef,irred,plus,zero,moins,mult,un) 
       | _ -> semantical_error
                       "Rat theory needs exactly eight parameters"
      end
| _ -> semantical_error ("I don't know what is theory "^theory_id)
;;

let theory_of_abstract_theory = function
  PRDF id_list -> predefined_theory_of_abstract_theory id_list
| USR r_list -> USER r_list
;;






