(***************************************************************************

Lexicographie

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: lexer.mll,v 1.2 2001/04/20 13:42:27 marche Exp $

***************************************************************************)

(*

Obsolete header:

Project:                Cime
Author(s):             Claude March\'e

File name:        lexique.mll
Created on:              26 fev 93
Last modified on: 26 fev 93

***************************************************************************)


{

open Abstract_syntax
open Parser
open Symbols

open Lexing

(*
open Printf
*)

let incr_num_col n =
  etat_analyse.numero_colonne_avant <- etat_analyse.numero_colonne_apres;
  etat_analyse.numero_colonne_apres <- etat_analyse.numero_colonne_apres+n


let get_last_token lexbuf =
(*
    printf "Token: %s\n" (Lexing.lexeme lexbuf);
    flush stdout;
*)
  let lexeme = (Lexing.lexeme lexbuf)
  in 
  incr_num_col (String.length lexeme);
  etat_analyse.dernier_token_lu <- lexeme

let get_string lexbuf =
  let lexeme =  (Lexing.lexeme lexbuf)
  in STRING(String.sub lexeme 1 (String.length lexeme - 2))

(* 
  hash table of keywords
  any number is appropriate in place of 17, a prime number is better 
*)

let keywords_table = let kt = ((Hashtbl.create 17):(string,token) Hashtbl.t)
in List.iter  
    (function (s,k) -> Hashtbl.add kt s k)
    [
    ("sort",          SORTES);
    ("subsort",        SUBSORTES);
    ("as",              AS);
(*    ("end",		 FIN); *)
    ("end",		 KW_END);
    ("operators",	 OPERATEURS);
    ("constant",	 CONSTANTE);
    ("unary",		 UNAIRE);
    ("binary",		 BINAIRE);
    ("infix",		 INFIXE);
    ("prefix",		 PREFIXE);
    ("postfix",		 POSTFIXE);
    ("variable",	 TOKEN_VARIABLE);
    ("theory",		 THEORIE);
    ("unification",	 UNIFICATION);
    ("matching",	 MATCHING);
    ("commutative",	 TOKEN_C);
    ("AC",		 TOKEN_AC);
    ("USER",		 TOKEN_USER);
    ("reduce",		 REDUIRE);
    ("match",		 MATCHER);
    ("unify",		 UNIFIER);
    ("compare",		 COMPARER);
    ("with",		 WITH);
    ("unif_libre",	 LIBRE_UNIFIER);
    ("order",		 ORDRE);
    ("rpo",		 TOKEN_RPO);
    ("rps",		 TOKEN_RPS);
    ("mapo",		 TOKEN_MAPO);
    ("poly",             TOKEN_POLY);
    ("semi_interactive", TOKEN_SEMI_INTERACTIF);
    ("interactive",	 TOKEN_INTERACTIF);
    ("mul",		 MUL);
    ("lexico",		 TOKEN_LEXICO);
    ("lrlex",		 LRLEX);
    ("rllex",		 RLLEX);
    ("axioms",		 EQUATIONS);
    ("only",		 ONLY);
    ("plain",		 PLAIN);
    ("type",		 TYPE);
    ("complete",	 COMPLETE);
    ("problems",	 PROBLEMES);
    ("conjectures",	 CONJECTURES);
    ("inductive",       INDUCTIVE);
    ("termination",     TERMINAISON);
    ("dp_termination",     DP_TERMINATION)

  ];
  kt


let ident_fix s = 
  match etat_analyse.lexer_state with
    Standard_lexer -> IDENT s
  | Poly_lexer -> IDENT s
  | Ident_lexer -> IDENT s
  | Term_lexer -> 
      try
       	let f = symbol_id_of_string s
       	in if is_postfix f
        then POSTFIX_IDENT s
        else if is_infix f
        then INFIX_IDENT s
        else PREFIX_IDENT s
      with Not_found -> PREFIX_IDENT s (* une variable... *)
	  

let keyword_or_ident s = 
  try (Hashtbl.find keywords_table s)
  with Not_found -> ident_fix s

let poly_var_or_ident s =
  match etat_analyse.lexer_state with
    Poly_lexer -> POLY_VAR s
  | _ -> ident_fix s

let oper_or_ident s =
  match etat_analyse.lexer_state with
    Poly_lexer -> 
      begin
      match s with
         "+" -> PLUS
       | "-" -> MINUS
       | "^" -> EXP
       | "." -> MULT
       | _ -> failwith "Bug in lexer.oper_or_ident"
      end
  | _ -> ident_fix s

let int_or_ident s =
  match etat_analyse.lexer_state with
    Poly_lexer 
  | Standard_lexer -> INT (int_of_string s)
  | Ident_lexer 
  | Term_lexer -> ident_fix s

let top_ident s =
  if etat_analyse.top_allowed
  then ident_fix s
  else Errors.semantical_error "TOP symbols are not allowed here"


}

rule token = parse
    [' ''\t']  		{ incr_num_col 1;
                          token lexbuf }
  | ('%'[^'\n']*)?'\n'	{ etat_analyse.numero_ligne <- etat_analyse.numero_ligne+1;
                          etat_analyse.numero_colonne_avant <- 0;
                          etat_analyse.numero_colonne_apres <- 0;
                          token lexbuf }
  | ['a'-'z']['0'-'9']* { get_last_token(lexbuf);
                          poly_var_or_ident(Lexing.lexeme lexbuf) }
  | ['+''-''.''^']      { get_last_token(lexbuf);
                          oper_or_ident(Lexing.lexeme lexbuf) }
  | ['0'-'9']+          { get_last_token(lexbuf);
                          int_or_ident(Lexing.lexeme lexbuf) }
  | (['a'-'z''A'-'Z''0'-'9''_''\'']+|['^''+''.''&''*''-''/''!''?''@''~''#'])('_'(['a'-'z''A'-'Z''0'-'9''_''\'']+|['^''+''.''&''*''-''/''!''?''@''~''#']))* 
			{ get_last_token(lexbuf);
			  keyword_or_ident(Lexing.lexeme lexbuf) }
  | "TOP("['a'-'z']['0'-'9']*")" { get_last_token(lexbuf);
                                   top_ident(Lexing.lexeme lexbuf) }
  | "TOP("['+''-''.''^']")"      { get_last_token(lexbuf);
                                   top_ident(Lexing.lexeme lexbuf) }
  | "TOP("['0'-'9']+")"          { get_last_token(lexbuf);
                                   top_ident(Lexing.lexeme lexbuf) }
  | "TOP("(['a'-'z''A'-'Z''0'-'9''_''\'']+|['^''+''.''&''*''-''/''!''?''@''~''#'])('_'(['a'-'z''A'-'Z''0'-'9''_''\'']+|['^''+''.''&''*''-''/''!''?''@''~''#']))*")" 
			{ get_last_token(lexbuf);
			  top_ident(Lexing.lexeme lexbuf) }
  | ','            	{ get_last_token(lexbuf); VIRGULE }
  | ':'            	{ get_last_token(lexbuf); DEUX_POINTS }
  | ';'            	{ get_last_token(lexbuf); POINT_VIRGULE }
  | '('            	{ get_last_token(lexbuf); PARGAUCHE }
  | ')'            	{ get_last_token(lexbuf); PARDROITE }
  | '['            	{ get_last_token(lexbuf); CROGAUCHE }
  | ']'            	{ get_last_token(lexbuf); CRODROIT }
  | '='            	{ get_last_token(lexbuf); EGAL }
  | '-''>'            	{ get_last_token(lexbuf); FLECHE }
  | '>'            	{ get_last_token(lexbuf); SUPERIEUR }
  | '<'            	{ get_last_token(lexbuf); INFERIEUR }
  | '"'[^'"']*'"'      	{ get_last_token(lexbuf); get_string(lexbuf) }
  | eof			{ etat_analyse.dernier_token_lu <- "";
                          EOF }

