(***************************************************************************

Fonctions de conversions de termes en chaine de caracteres : corps

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: pretty_printing.ml,v 1.2 1998/04/28 13:36:08 marche Exp $

***************************************************************************)

(*

Obsolete header:

Project:                Cime
Author(s):             Claude March\'e

File name:        termstr.mli
Created on:              03 fev 93
Last modified on: 18 oct 95 (pretty printing)

***************************************************************************)

open Format
open Symbols
open Terms
open Equations
open Args



open Substitution

(*===========================================

Affichage d'un terme, d'une \'equation 
ou d'une r\`egle dans des box imbriquees.
Format special pour les symboles AC.

===========================================*)

(*===========================================

Affichage d'un terme, d'une \'equation 
ou d'une r\`egle dans des box imbriquees.

===========================================*)

(*==================================================
Affichage d'un terme aplati dans une box.
Format special pour symboles AC.
===================================================*)

let rec print_term1 = function 
  TERM(f,[])     -> (print_string (string_of_symbol_id f))
|  VAR x          -> (print_string (string_of_var_id x))
| TERM(f,(s::l)) -> if (is_ac f) 
                    then (print_acterm f (s::l))
                    else if is_infix f
                         then begin
                              (print_term2 s);
                              ((fun(x,y)->print_break x y) (0,0));
                              (print_infix_term f l)
                              end
                         else if is_postfix f
                              then if ((arity f) = 1)
                                   then (print_unary_postfix_term f (s::l))
                                   else begin
                                        (print_lterm (s::l));
                                        (print_string (string_of_symbol_id f))
                                        end
                              else if ((arity f) = 1)
                                   then (print_unary_term f (s::l))
                                   else begin
                                        (print_string (string_of_symbol_id f));
                                        (print_lterm (s::l))
                                        end

and print_term2 = function        (* idem mais entoure par de ()'s          *)
   TERM(f,[])	  ->              (* pour mettre dans une expression infixe *)
                     (print_string (string_of_symbol_id f))  
 | VAR x          -> (print_string (string_of_var_id x))
 | TERM(f,s::l) -> if (is_ac f) 
                   then (print_actermb f (s::l))
                   else if (is_infix f)
                        then begin
                             (print_string "(");
                             (print_term2 s);
                             ((fun(x,y)->print_break x y) (0,0));
                             (print_infix_term f l);
                             (print_string ")")
                             end
                        else if is_postfix f
                             then if ((arity f) = 1)
                                  then (print_unary_postfix_term f (s::l))
                                  else begin
                                       (print_lterm (s::l));
                                       (print_string (string_of_symbol_id f)) 
                                       end
                             else if ((arity f) = 1)
                                  then (print_unary_term f (s::l))
                                  else begin
                                       (print_string (string_of_symbol_id f)); 
                                       (print_lterm (s::l))
                                       end

and print_infix_term f = function
   []	-> (print_string "")
 | s::l	-> begin
           (print_symbol_id_with_spaces f);
           ((fun(x,y)->print_break x y) (0,0));
           (print_term2 s);
           (print_infix_term f l)
           end

and print_acterm f = function  (* on ne compte que les var. et constantes  *)  
      [] -> ((fun(x,y)->print_break x y) (1,0))
    | [s]    -> (print_term2 s)    
    | (s::l) -> if (is_a_constant s) or (is_a_variable s)
                then (print_acterm2 f s l)
                else begin
                     (print_term2 s);
                     ((fun(x,y)->print_break x y) (0,0));
                     (print_symbol_id_with_spaces f);
                     ((fun(x,y)->print_break x y) (0,0));
                     (print_acterm f l)
                     end

and print_actermb f = function   (* idem mais entoure par de ()'s          *)
      [] -> ((fun(x,y)->print_break x y) (1,0))  (* pour mettre dans une expression infixe *) 
    | [s]    -> (print_term2 s)    
    | (s::l) -> if (is_a_constant s) or (is_a_variable s)
                then (print_acterm2b f s l)
                else begin
                     (print_string "(");
                     (print_term2 s);
                     ((fun(x,y)->print_break x y) (0,0));
                     (print_symbol_id_with_spaces f);
                     ((fun(x,y)->print_break x y) (0,0));
                     (print_actermb f l);
                     (print_string ")")
                     end

and print_acterm2 f x = function  (* on commence a compter seulement si on a *)
     []     -> (print_term1 x)    (* au moins 2 fois la meme var/const       *)
   | (s::l) -> if (x=s)
               then (print_aclist f x 2 l)
               else begin
                    (print_term2 x);
                    ((fun(x,y)->print_break x y) (0,0));
                    (print_symbol_id_with_spaces f);
                    ((fun(x,y)->print_break x y) (0,0));
                    (print_actermb f (s::l))
                    end

and print_acterm2b f x = function (* idem mais entoure par de ()'s          *) 
     []     -> (print_term1 x)    (* pour mettre dans une expression infixe *) 
   | (s::l) -> if (x=s)
               then (print_aclist f x 2 l)
               else begin
                    (print_string "(");
                    (print_term2 x);
                    ((fun(x,y)->print_break x y) (0,0));
                    (print_symbol_id_with_spaces f);
                    ((fun(x,y)->print_break x y) (0,0));
                    (print_acterm f (s::l));
                    (print_string ")")
                    end
               
and print_aclist f x nbocc = function
       []     -> begin
                 (print_string "(");
                 (print_int nbocc); 
                 (print_string "_"); 
                 (print_string (string_of_symbol_id f));
                 (print_string " ");
                 (print_term1 x);
                 (print_string ")")
                 end
     | [s]    -> if (x=s)
                 then begin
                      (print_string "(");
                      (print_int (nbocc+1)); 
                      (print_string "_"); 
                      (print_string (string_of_symbol_id f));
                      (print_string " ");
                      (print_term1 x);
                      (print_string ")")
                      end 
                 else begin
                      (print_string "(");
                      (print_int nbocc);
                      (print_string "_"); 
                      (print_string (string_of_symbol_id f));
                      (print_string " ");
                      (print_term1 x);
                      (print_string ")");
                      (print_symbol_id_with_spaces f);
                      ((fun(x,y)->print_break x y) (0,0));
                      (print_term2 s)
                      end
     | (s::l) -> if (x=s)
                 then (print_aclist f x (nbocc+1) l)
                 else begin
                      (print_string "(");
                      (print_int nbocc);
                      (print_string "_"); 
                      (print_string (string_of_symbol_id f));
                      (print_string " ");
                      (print_term1 x);
                      (print_string ")");
                      (print_symbol_id_with_spaces f);
                      ((fun(x,y)->print_break x y) (0,0));
                      (print_acterm f (s::l))
                      end

and print_unary_term  f = function
       [s] ->  if (is_a_variable s)
               then begin
                    (print_string (string_of_symbol_id f)); 
                    (print_lterm [s])
                    end
               else if ((top_symbol_id s) = f)
                    then print_unary_term2 f (liste_des_sous_termes s)
                    else begin
                         (print_string (string_of_symbol_id f)); 
                         (print_lterm [s])
                         end
     | _  -> (print_string "")      (* cas exclu *) 

and print_unary_term2 f = function
      [s] -> if (is_a_variable s)
             then begin
                  (print_string (string_of_symbol_id f)); 
                  (print_string "(");
                  (print_string (string_of_symbol_id f)); 
                  (print_lterm [s]);
                  (print_string ")")
                  end
             else if ((top_symbol_id s) = f)
                  then print_unary_term3 f 3 (liste_des_sous_termes s)
                  else begin
                       (print_string (string_of_symbol_id f)); 
                       (print_string "(");
                       (print_string (string_of_symbol_id f)); 
                       (print_lterm [s]);
                       (print_string ")")
                       end
     | _  -> (print_string "")      (* cas exclu *) 

and print_unary_term3 f n = function
      [s] -> if ((top_symbol_id s) = f)
             then print_unary_term3 f (n+1) (liste_des_sous_termes s)
             else begin
                  (print_string (string_of_symbol_id f)); 
                  (print_string "^");
                  (print_int n);
                  (print_lterm [s])
                  end
     | _  -> (print_string "")      (* cas exclu *) 

and print_unary_postfix_term  f = function
       [s] ->  if (is_a_variable s)
               then begin
                    (print_lterm [s]);
                    (print_string (string_of_symbol_id f)) 
                    end
               else if ((top_symbol_id s) = f)
                    then print_unary_postfix_term2 f (liste_des_sous_termes s)
                    else begin
                         (print_lterm [s]);
                         (print_string (string_of_symbol_id f)) 
                         end
     | _  -> (print_string "")      (* cas exclu *) 

and print_unary_postfix_term2 f = function
      [s] -> if (is_a_variable s)
             then begin
                  print_string "(";
                  print_lterm [s];
                  print_string (string_of_symbol_id f); 
                  print_string ")";
                  print_string (string_of_symbol_id f)
                  end
             else if ((top_symbol_id s) = f)
                  then print_unary_postfix_term3 f 3 (liste_des_sous_termes s)
                  else begin
                       print_string "(";
                       print_lterm [s];
                       print_string (string_of_symbol_id f); 
                       print_string ")";
                       print_string (string_of_symbol_id f)
                       end
     | _  -> print_string ""      (* cas exclu *) 

and print_unary_postfix_term3 f n = function
      [s] -> if ((top_symbol_id s) = f)
             then print_unary_term3 f (n+1) (liste_des_sous_termes s)
             else begin
                  print_lterm [s];
                  print_string (string_of_symbol_id f); 
                  print_string "^";
                  print_int n
                  end
     | _  -> print_string ""      (* cas exclu *) 

and print_symbol_id_with_spaces f =
   let t = string_of_symbol_id f in
     match t with
     "+" | "." | "*" -> (print_string t)
    | _ -> begin
           (print_string " ");
           (print_string t);
           ((fun(x,y)->print_break x y) (1,0))
           end


and print_lterm = function
  []   -> (print_string "()")
| t::l -> begin
          (print_string "(");
          (print_term1 t);
          (print_lterm2 l)
          end

and print_lterm2 = function
   []   -> (print_string ")")
 | t::l -> begin 
           (print_string ",");
           ((fun(x,y)->print_break x y) (0,0));
           (print_term1 t);
           (print_lterm2 l)
           end


(*==================================================
Affichage d'un terme aplati dans une box.
Format pour parsing (sous reserves que le parser
                     ignore \n et \t            ).
===================================================*)


let rec pprint_term1 = function
   TERM(f,[])   -> print_string (string_of_symbol_id f)
 | VAR x        -> print_string (string_of_var_id x)
 | TERM(f,s::l) -> if is_infix f
                   then begin
                        pprint_term2 s;
                        (fun(x,y)->print_break x y) (0,0);
                        pprint_infix_term f l
                        end
                   else if is_postfix f 
                        then begin
                             print_string "(";
                             pprint_term1 s;
                             pprint_lterm l;
                             print_string (string_of_symbol_id f)
                             end
                        else begin
                             print_string (string_of_symbol_id f);
                             print_string "(";
                             pprint_term1 s;
                             pprint_lterm l
                             end

and pprint_term2 = function
   TERM(f,[])	  -> print_string (string_of_symbol_id f)
 | VAR x          -> print_string (string_of_var_id x)
 | TERM(f,s::l) -> if is_infix f
                   then begin
                        print_string "(";
                        pprint_term2 s;
                        (fun(x,y)->print_break x y) (0,0);
                        pprint_infix_term f l;
                        print_string ")"
                        end
                   else if is_postfix f 
                        then begin
                             print_string "(";
                             pprint_term1 s;
                             pprint_lterm l;
                             print_string (string_of_symbol_id f)
                             end
                        else begin
                             print_string (string_of_symbol_id f);
                             print_string "(";
                             pprint_term1 s;
                             pprint_lterm l
                             end

and pprint_infix_term f = function
   []	-> ()
 | s::l	-> begin
           print_symbol_id_with_spaces f;
           (fun(x,y)->print_break x y) (0,0);
           pprint_term2 s;
           pprint_infix_term f l
           end


and print_symbol_id_with_spaces f =
   let t = string_of_symbol_id f in
     match t with
     "+" | "." | "*" -> print_string t
    | _ -> begin
           print_string " ";
           print_string t;
           (fun(x,y)->print_break x y) (1,0)
           end

and pprint_lterm = function
   []   -> print_string ")"
 | t::l -> begin 
           print_string ",";
           (fun(x,y)->print_break x y) (0,0);
           pprint_term1 t;
           pprint_lterm l
           end



(*=====================================

  string_of_term , no line feed

  -> dans terms.ml

======================================*)


(*
let rec string_of_term1 = function
   TERM(f,[])   -> (string_of_symbol_id f)
 | VAR x        -> (string_of_var_id x)
 | TERM(f,s::l) -> if is_infix f
                   then begin
                        (string_of_term2 s) ^ 
                        (string_of_infix_term f l)
                        end
                   else if is_postfix f 
                        then begin
                             "(" ^ 
                             (string_of_term1 s)^
                             (string_of_lterm l)^
                             (string_of_symbol_id f)
                             end
                        else begin
                             (string_of_symbol_id f)^
                             "("^
                             (string_of_term1 s)^
                             (string_of_lterm l)
                             end

and string_of_term2 = function
   TERM(f,[])	  -> (string_of_symbol_id f)
 | VAR x          -> (string_of_var_id x)
 | TERM(f,s::l) -> if is_infix f
                   then begin
                        "("^
                        (string_of_term2 s)^
                        (string_of_infix_term f l)^
                        ")"
                        end
                   else if is_postfix f 
                        then begin
                             "("^
                             (string_of_term1 s)^
                             (string_of_lterm l)^
                             (string_of_symbol_id f)
                             end
                        else begin
                             (string_of_symbol_id f)^
                             "("^
                             (string_of_term1 s)^
                             (string_of_lterm l)
                             end

and string_of_infix_term f = function
   []	-> ()
 | s::l	-> begin
           (string_of_symbol_id_with_spaces f)
           (string_of_term2 s)
           (string_of_infix_term f l)
           end


and string_of_symbol_id_with_spaces f =
   let t = string_of_symbol_id f in
     match t with
     "+" | "." | "*" -> t
    | _ -> begin
           " "^t^" "
           end

and string_of_lterm = function
   []   -> ")"
 | t::l -> begin 
           ","^
           (string_of_term1 t)^
           (string_of_lterm l)
           end

*)

(*===================================================*)

let print_term t = 
  begin
  open_hovbox 0;
  if options_courantes.print_to_parse
    then pprint_term1 t
    else print_term1 t;
  close_box ()
  end

let print_equation e = 
  begin
  open_hovbox 0;
(*
  print_string "[max=";
  print_int e.n;
  print_string "] ";
*)
  print_term e.u;
  print_string " =";
  (fun(x,y)->print_break x y) (1,0);
  print_term e.v;
(*
  (fun(x,y)->print_break x y) (1,0);
  print_string "[";
  print_int e.interest;
  print_string ",";
  print_int e.lhs_interest;
  print_string "]";
*)
  close_box ()
  end


(*
let pprint_equation e = 
  begin
  (open_hovbox 0);
  (pprint_term e.u);
  (print_string " =");
  ((fun(x,y)->print_break x y) (1,0));
  (pprint_term e.v);
(*
  (fun(x,y)->print_break x y) (1,0);
  print_string "[";
  print_int e.interest;
  print_string ",";
  print_int e.lhs_interest;
  print_string "]";
*)
  (close_box ())
  end
*)

let print_liste_d_equations =  function
   []   -> (print_string "{}")
 | e::l -> 
  let rec print_fin_de_liste_d_equations x y= match (x,y) with
     []    , n -> begin
                 (print_string "}");
                 ((fun(x,y)->print_break x y) (1,0));
                 (print_string "(");
                 (print_int n);
                 (print_string " equations)")
                 end
   | (e::l), n -> begin
                 (print_string  ",");
                 ((fun(x,y)->print_break x y) (0,0));
                 (print_equation e);
                 print_fin_de_liste_d_equations l (n+1)
                 end 
        in begin 
           (print_string "{");
           (open_hovbox 0);
           (print_equation e);
           (print_fin_de_liste_d_equations l 1);
           (close_box ())
           end

(*
let pprint_liste_d_equations =  function
   []   -> (print_string "{}")
 | e::l ->
    let rec pprint_fin_de_liste_d_equations x y = match (x,y) with
     []    , n -> begin
                 (print_string "}");
                 ((fun(x,y)->print_break x y) (1,0));
                 (print_string "(");
                 (print_int n);
                 (print_string " equations)")
                 end
   | (e::l), n -> begin
                 (print_string  ",");
                 ((fun(x,y)->print_break x y) (0,0));
                 (pprint_equation e);
                 (pprint_fin_de_liste_d_equations l (n+1))
                 end
        in begin 
           (print_string "{");
           (open_hovbox 0);
           (pprint_equation e);
           (pprint_fin_de_liste_d_equations l 1);
           (close_box ())
           end
*)

let print_regle rule =
  print_string "["; 
  print_int rule.num;
  print_string "] ";
  open_hovbox 0;
  print_term rule.l;
  if rule.orientation
  then print_string " ->"
  else print_string " <->";
  (fun(x,y)->print_break x y) (1,0);
  print_term rule.r;
(*
  (fun(x,y)->print_break x y) (1,0);
  print_string "[";
  print_int rule.rule_interest;
  print_string ",";
  print_int rule.rule_lhs_interest;
  print_string "]";
*)
  close_box ()


(*
let pprint_regle rule =
  print_string "["; 
  print_int rule.num;
  print_string "] ";
  open_hovbox 0;
  pprint_term rule.l;
  print_string " ->";
  (fun(x,y)->print_break x y) (1,0);
  pprint_term rule.r;
(*
  (fun(x,y)->print_break x y) (1,0);
  print_string "[";
  print_int rule.rule_interest;
  print_string ",";
  print_int rule.rule_lhs_interest;
  print_string "]";
*)
  close_box ()
*)


let print_liste_de_regles = function
   []   -> (print_string "{}")
 | r::l ->
   let rec print_fin_de_liste_de_regles x y= match (x,y) with 
     []   ,  n -> begin
                 (print_string " }");
                 ((fun(x,y)->print_break x y) (1,0));
                 (print_string "(");
                 (print_int n);
                 (print_string " rules)")                 
                 end
   | (r::l), n -> begin
                 (print_string  ",");
                 (force_newline ());
                 (print_regle r);
                 (print_fin_de_liste_de_regles l (n+1))
                 end
        in begin
           (print_string "{ ");
           (open_hovbox 0);
           (print_regle r);
           (print_fin_de_liste_de_regles l 1);
           (close_box ())
           end

(* 
let print_liste_de_regles = function
   []   -> print_string "{}"
 | r::l -> print_string ("{" ^ (string_of_regle r));
           print_fin_de_liste_de_regles l 1
where rec print_fin_de_liste_de_regles = fun
     []     n -> print_string ("} (" ^ (string_of_int n) ^ " rules)")
   | (r::l) n -> print_string (",\n" ^ (string_of_regle r));
                 print_fin_de_liste_de_regles l (n+1)
*)

(*
let pprint_liste_de_regles = function
   []   -> (print_string "{}")
 | r::l ->
  let rec pprint_fin_de_liste_de_regles x y= match (x,y) with 
     []  ,   n -> begin
                 (print_string " }");
                 ((fun(x,y)->print_break x y) (1,0));
                 (print_string "(");
                 (print_int n);
                 (print_string " rules)")                 
                 end
   | (r::l), n -> begin
                 (print_string  ",");
                 (force_newline ());
                 (pprint_regle r);
                 (pprint_fin_de_liste_de_regles l (n+1))
                 end
        in begin
           (print_string "{ ");
           (open_hovbox 0);
           (pprint_regle r);
           (pprint_fin_de_liste_de_regles l 1);
           (close_box ())
           end
*)

(* A supprimer *)
(*
let string_of_regle (REGLE(num_rule,x,u,v)) =
"[" ^ (string_of_int num_rule) ^ "] " ^
(*
"("^(string_of_int x)^") "^
*)
(* 
(string_of_term u) ^ " -> " ^ (string_of_term v)

let string_of_equation = function
  EQUATION(_,_,_,u,v) -> (string_of_term u) ^ " = " ^ (string_of_term v)

let string_of_liste_d_equations = function
   []   -> "()"
 | e::l -> "(" ^ (string_of_equation e) 
               ^ (string_of_fin_de_liste_d_equations l)
where rec string_of_fin_de_liste_d_equations = function
     []   -> ")"
   | e::l -> ",\n" ^ (string_of_equation e) 
                   ^ (string_of_fin_de_liste_d_equations l)
*)
(*
let string_of_regle_partagee regle =
  (string_of_regle (regle_of_regle_partagee regle))
*)
(*
let string_of_liste_de_regles = function
   []   -> "{}"
 | r::l -> "" ^ (string_of_regle r) 
               ^ (string_of_fin_de_liste_de_regles l)
where rec string_of_fin_de_liste_de_regles = function
     []   -> ")"
   | r::l -> ",\n" ^ (string_of_regle r) 
                   ^ (string_of_fin_de_liste_de_regles l)
*)
(*
let print_liste_de_regles_partagees R = 
   print_liste_de_regles (List.map regle_of_regle_partagee R)
*)



*)


(*===========================================

Conversion d'une chaine de caracteres en terme

Effectu\'ee par la grammaire dans parser.mly 

============================================*)



let term_of_string s =
  let lexbuf = Lexing.from_string (s ^ ";") in
    Parser.term_line Lexer.token lexbuf



(*===========================================

Lecture d'un ensemble d'\'equations au clavier

===========================================*)


(*
let lire_liste_d_equations () =
  let lexbuf = Lexing.from_channel stdin in
    try Termgram.EqList Termlex.Token lexbuf with
      Failure s -> print_f "erreur de syntaxe";
                   []
      | e -> raise e
*)


(*===========================================

Lecture de la precedence

===========================================*)

(*
let lire_precedence () =
  let lexbuf = Lexing.from_channel stdin in
    try Termgram.OpList Termlex.Token lexbuf with
      Failure s -> print_f "erreur de syntaxe";
                   []
      | e -> raise e
*)




let string_of_substitution theta =
  let s = ref "{" in 
   Map.iter (fun x t -> s := !s^","^(string_of_term (VAR x))^
      	       	       	      "->"^(string_of_term t)) theta;
   !s^"}"

let print_substitution theta =
   begin
   open_hvbox 0;
   let est_vide = ref true in 
   Map.iter (fun x t -> begin
                         est_vide:=false;
                         print_term (VAR x);
                         print_string "->";
                         (fun(x,y)->print_break x y) (0,0);
      	       	       	 open_hovbox 0;
                         print_term t;
                         close_box ();
                         (fun(x,y)->print_break x y) (2,0)
                         end) 
      	     theta;
   if !est_vide then print_string "id";
   close_box ()
   end

