(***************************************************************************

  Term orderings: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: mapo.ml,v 1.8 2002/05/27 12:07:25 contejea Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: mapo.ml,v 1.8 2002/05/27 12:07:25 contejea Exp $

***************************************************************************)

open Orderings;;
open Symbols;;
open Terms;;
open Ac_rewriting;;
open Rpo;;
open Equations;;
open Dnet;;


(* =======================================================================

functions that create the single rewrite rules for mapo. The numbers
refer to the original article of Delor and Puel where the order is defined.

======================================================================== *)

(*-------	f(g(x,y),z) -> g(f(x,z),f(y,z))			-------- *)

let mapo_rule_1 (f,g) =
    let x = fresh_new_var () in
    let y = next_new_var x in
    let z = next_new_var y in
    (make_basic_rule
	(TERM(f,[TERM(g,[(VAR x);(VAR y)]);(VAR z)]),
         TERM(g,[TERM(f,[(VAR x);(VAR z)]);TERM(f,[(VAR y);(VAR z)])])))
;;


(*------- 	f(x,g(y,z)) -> g(f(x,y),f(x,z))			-------- *)
(* useless since f is an AC operator !*)
(*
let mapo_rule_2 (f,g) =
    let x = fresh_new_var () in
    let y = next_new_var x in
    let z = next_new_var y in
    (make_basic_rule
	(TERM(f,[(VAR x);TERM(g,[(VAR y);(VAR z)])]),
         TERM(g,[TERM(f,[(VAR x);(VAR y)]);TERM(f,[(VAR x);(VAR z)])])))
;;
*)

(*--------	f(u(x),y) -> u(f(x,y))				------- *)

let mapo_rule_3 (f,u) =
    let x = fresh_new_var () in
    let y = next_new_var x in
    (make_basic_rule
        (TERM(f,[TERM(u,[(VAR x)]);(VAR y)]),
         TERM(u,[TERM(f,[(VAR x);(VAR y)])])))
;;


(*--------	f(x,u(y)) -> u(f(x,y))				------- *)
(* useless since f is an AC operator !*)
(*
let mapo_rule_4 (f,u) =
    let x = fresh_new_var () in
    let y = next_new_var x in
    (make_basic_rule
	(TERM(f,[(VAR x);TERM(u,[(VAR y)])]),
	 TERM(u,[TERM(f,[(VAR x);(VAR y)])])))
;;
*)

(*--------	u(g(x,y)) -> g(u(x),u(y))			------- *)

let mapo_rule_5 (g,u) =
    let x = fresh_new_var () in
    let y = next_new_var x in
    (make_basic_rule
	(TERM(u,[TERM(g,[(VAR x);(VAR y)])]),
	 TERM(g,[TERM(u,[(VAR x)]);TERM(u,[(VAR y)])])))
;;


(*-------	u(v(x)) -> v(u(x))				------- *)

let mapo_rule_6 (u,v) =
    let x = fresh_new_var () in
    (make_basic_rule
	(TERM(u,[TERM(v,[(VAR x)])]),
	 TERM(v,[TERM(u,[(VAR x)])])))
;;
	


(* =======================================================================

The function that returns the complete set of rules.

======================================================================= *)

(* some basic list functions. Why are they not in caml's list library ? *)

(* allpairs [a1;...;an] [b1;...;bm] returns a list of n*m pairs:
	[ (a1,b1);(a1,b2);...;(an,b(m-1));(an,bm)]			*)

let rec allpairs l k = 
  match l with
      [] -> []
    | a::r -> (List.map (function x -> (a,x)) k) @ (allpairs r k)


(* 

   [(comb l k p f)] returns the list of all [f(a,b)] where [a]
   $\in$ [l], [b] $\in$ [k], and [(p a b)] holds.

*) 

let comb l k p f = List.map f (List.filter p (allpairs l k)) ;;


let mapo_rules prec =
  let u = listof_unary_functions ()
  and a = listof_ac_functions ()
  and smaller (x,y) = 
    match prec x y with
	Less_than -> true
      | _ -> false
  and greater (x,y) = 
    match prec x y with
	Greater -> true
      | _ -> false
  in
    (comb a a greater mapo_rule_1)
(*
    @ (comb a a greater mapo_rule_2)
*)
    @ (comb a u greater mapo_rule_3)
(*
    @ (comb a u greater mapo_rule_4)
*)
    @ (comb a u smaller mapo_rule_5)
    @ (comb u u greater mapo_rule_6)
;;

(* ======================================================================

The main function.

======================================================================= *)

let mapo (prec,status) =
  let mapo_sys = mapo_rules prec in
  let mapo_net = 
    List.fold_left
      (fun d r -> add_rule_to_dnet LEFT r d)
      (max_int,empty_dnet ()) mapo_sys
  and this_rpo = rpo (prec,status) in
    fun x y -> 
      this_rpo
	(normalisation_ac_par_un_systeme_de_regles 
	    (fun _ _ -> Greater) x mapo_net)
	(normalisation_ac_par_un_systeme_de_regles 
	   (fun _ _ -> Greater) y mapo_net)

