(***************************************************************************

ordering on Polynomials : body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: poly_ordering.ml,v 1.2 1998/04/28 13:36:14 marche Exp $

***************************************************************************)

(*

Obsolete header:


*********************************************************************************)

open Orderings
open Poly
open Poly_op



(*

  ordre d'evaluation sur les polynomes

  (compare_poly P Q) retourne

  Equivalent            si pour tt x1.. xn P(x1..xn)=Q(x1..xn)
  Greater               si pour tt x1.. xn P(x1..xn)>Q(x1..xn)
  Less_than             si pour tt x1.. xn P(x1..xn)<Q(x1..xn)
  Greater_or_equivalent si pour tt x1.. xn P(x1..xn)>=Q(x1..xn)
  Less_or_equivalent    si pour tt x1.. xn P(x1..xn)>=Q(x1..xn)
  Uncomparable          sinon

bien sur c'est imcomplet : en fait la specification est :

  si (compare_poly P Q) retourne

  Equivalent            alors pour tt x1.. xn P(x1..xn)=Q(x1..xn)
  Greater               alors pour tt x1.. xn P(x1..xn)>Q(x1..xn)
  Less_than             alors pour tt x1.. xn P(x1..xn)<Q(x1..xn)
  Greater_or_equivalent alors pour tt x1.. xn P(x1..xn)>=Q(x1..xn)
  Less_or_equivalent    alors pour tt x1.. xn P(x1..xn)>=Q(x1..xn)
  Uncomparable          alors on ne sait rien

*)

(*--------------------------------------------------------------*)
let sup_degre m1 m2 =                   (*test sur chaque degre des indet. dans l'ordre *)
  let rec sup_liste = function
  ([],[]) -> true
  |(a::l1 ,b::l2 ) -> (a>=b)&(sup_liste (l1,l2))
  |_ -> failwith "Ord_poly.sup_degre"
  in
  sup_liste ((vars_monome m1), (vars_monome m2))

let sup_degre_total m1 m2 =             (*test sur la somme des degres des indeterminees*)
  let v1 = vars_monome m1
  and v2 = (vars_monome m2) in
  let rec sum = function
    []  -> 0
  |a::l -> a+(sum l)
  in
  (sum v1)>=(sum v2)



(* 

  (separe_monomes p)  retourne un triplet (mp,m,mn)
  ou mp est la liste des monomes positifs de p
     m est le plus grand monome negatif de p
     mn est le reste des monomes negatifs de p

  leve l'exception Pas_de_monome_negatif si p ne possede aucun monome
  negatif

*)

exception Pas_de_monome_negatif of monome list

let rec separe_monomes p =       
   if (polynome_est_nul p) then raise (Pas_de_monome_negatif [])
   else let m = premier_monome p
        in
        try let (plus,mm,moins) = separe_monomes (reste_polynome p)
            in 
            if (coef_monome m)>0 then (m::plus, mm, moins)
            else if (sup_degre_total m mm) then (plus, m, mm::moins)
                 else (plus,mm,m::moins)
        with Pas_de_monome_negatif plus ->
            if (coef_monome m)>0 then raise (Pas_de_monome_negatif (m::plus))
            else (plus,m,[])

(* Xi -> (Xi + 1) *)

let translate liste =   
  let espace = List.length liste in
  let rec tsl n = function
    []  -> []
  | a::s        -> (polysum (polyvar (espace - n) espace) 
      	       	       	    (polycst (-1) espace))::(tsl (n-1) s)
  in
  tsl (espace-1) liste


(* calcule un minorant de poly+mp+mm    *)
let transforme (mp, mm, poly) =         
  let a = coef_monome mp
  and b = coef_monome mm
  in
  if (a+b)>=0
  then 
    let esp = (List.length (vars_monome mp)) in
    polysum poly
      (polysum (fabrique_polynome mp polynome_nul)
           (polysum (fabrique_polynome mm polynome_nul)
               (polyprod (substituer_polynomes 
                                  esp
                                  (translate (vars_monome mm))
                                  (fabrique_polynome mm polynome_nul)
                         )
                         (polysum (polycst (-1) esp)
                                  (fabrique_polynome 
                                     (fabrique_monome 
                                        1
                                        (List.map2 (fun x y -> x - y)
                                              (vars_monome mp)
                                              (vars_monome mm))
                                     )
                                     polynome_nul
                                  )
                         )
               )
           )
      ) 
  else
    let esp = (List.length (vars_monome mp)) in
    polysum poly 
            (polysum (fabrique_polynome mp polynome_nul)
                     (polysum (fabrique_polynome mm polynome_nul)
                              (polyprod (substituer_polynomes 
                                               esp
                                              (translate (vars_monome mm))
                                              (fabrique_polynome 
                                        (fabrique_monome (-a)(vars_monome mm))
                                                 polynome_nul)
                                        )
                                  (polysum (polycst (-1) esp)
                                                 (fabrique_polynome 
                                                       (fabrique_monome 
                                                             1
                                                   (List.map2 (fun x y -> x - y)
                                                           (vars_monome mp)
                                                            (vars_monome mm)
                                                              )
                                                           )
                                                           polynome_nul
                                                  )
                                         )
                               )
                      )
              ) 



(*

  sum_mono_list : monome list -> polynome         

  (sum_mono_list l) retourne la somme des monomes de la liste l

*)

let rec sum_mono_list = function         
  []    -> polynome_nul
| a::s  -> polysum (fabrique_polynome a polynome_nul) (sum_mono_list s)

(*--------------------------------------------------------------*)


(*    

  si (poly_positif_ou_nul p) retourne true alors on est sur
  que pour tt x1..xn positifs ou nuls, P(x1..xn) >=0 

*)

let rec poly_positif_ou_nul p =       
  try 
    let (plus,m,moins) = separe_monomes p 
    in
    let rec parcours = function
        ([],_,_,_) -> false
      | (mp::plus,plusdt,mm,moins) -> 
      	  if (sup_degre mp mm)
            then let prime = transforme (mp,mm,polysum (sum_mono_list (plusdt@plus))
                                                       (sum_mono_list  moins))
                 in
                 if poly_positif_ou_nul prime 
      	       	   then true
                   else parcours (plus,mp::plusdt,mm,moins)
            else parcours (plus,mp::plusdt,mm,moins)

      in parcours (plus, [], m, moins)

  with Pas_de_monome_negatif _ -> true

(*    

  si (poly_positif_strict p) retourne true alors on est sur
  que pour tt x1..xn positifs ou nuls, P(x1..xn) >0 

*)

let poly_positif_strict p = 
  if polynome_est_nul p
    then false
    else poly_positif_ou_nul (polysum p (polycst (-1) (espace_polynome p)))

(*---------------------------------------------------------------------*)
(* compare les polynomes p1 et p2       *)

let compare_poly p1 p2 =               
  let diff = polysum p1 (polyminus p2) in
    if (poly_positif_strict diff) then Greater
    else if (poly_positif_strict (polyminus diff)) then Less_than
         else let test4 = (poly_positif_ou_nul (polyminus diff)) in
                if (poly_positif_ou_nul diff)
                then if test4 then Equivalent
                     else Greater_or_equivalent
                else if test4 then Less_or_equivalent
                     else Uncomparable










