(***************************************************************************

Term orderings: specification

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: rpo.mli,v 1.5 2002/05/27 12:07:25 contejea Exp $

***************************************************************************)

open Orderings_generalities;;
open Gen_terms;;

type rpo_status =
     Multiset
   | Lr_lexico
   | Rl_lexico
;;

type 'symbol status_function = 'symbol -> rpo_status;;

(*

[(rpo prec stat)] is the recursive path ordering with respect to
precedence [prec] and status [stat]

\paragraph{Warning!} It is assumed that the status are valid with
respect to precedence, that is whenever $f$ and $g$ are equivalent
w.r.t. [prec] then they have the same status, and if it is a
lexicographic status, then $f$ and $g$ have the same arity.

Generally speaking, instead of using RPO with equivalent symbols in
the precedence, it is better to apply first a RPS to replace symbols
that are supposed to be equivalent by a representant of them. Moreover
this allows to replace a symbol by a symbol with smaller arity.

However, a partial check is made when this function is called by the
toplevel.

*)

val rpo: 
  'symbol ordering -> 'symbol status_function -> 'symbol term ordering;;

(*

  [check_rpo_conditions prec stat] checks whether conditions above are
  satisfied. Raises an exception if not. WARNING : NOT IMPLEMENTED.

[val check_rpo_conditions : ? -> ? -> unit;;]

*)



(*

[rpo_statistics ()] return a triple [(a,b,c)] where 

\begin{itemize}
\item [a] is the number of additions to rpo cache;
\item [b] is the number of accesses to rpo cache;
\item [c] is the number of failure accesses to rpo cache.
\end{itemize}

*)


val rpo_statistics : unit -> int * int * int;;
