(***************************************************************************

Recursive Program schemes

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: rps.ml,v 1.4 1999/02/17 12:42:41 contejea Exp $

***************************************************************************)

open Orderings;;
open Terms;;
open Equations;;
open Ac_rewriting;;
open Errors;;
open Symbols;;
open Format;;
open Dnet;;

(*

(rps R o) build an create term_ordering > from an old one o and a set of
rewrite rules R, defined by

  t1 > t2  iff  R-nf(t1) o R-nf(t2) 

this defines a reduction ordering only if R satisfies some properties.
(check_rps_conditions R) checks whether these conditions are
satisfied, raise a semantical_error if not.

*)

(*

(rps R o) build an create term_ordering > from an old one o and a set of
rewrite rules R, defined by

  t1 > t2  iff  R-nf(t1) o R-nf(t2) 

this defines a reduction ordering only if R satisfies some properties.
(check_rps_conditions R) checks whether these conditions are
satisfied, raise a semantical_error if not.

*)

let rps rule_net order t1 t2 = 
  let u1 = normalisation_ac_par_un_systeme_de_regles 
	     (fun _ _-> Greater) t1 rule_net
  and u2 = normalisation_ac_par_un_systeme_de_regles 
	     (fun _ _ -> Greater) t2 rule_net in 
  (*
  print_string "nf(t1) = ";
  print_string (string_of_term u1);
  force_newline();
  print_string "nf(t2) = ";
  print_string (string_of_term u2);
  force_newline();
  print_flush();
  *)
  let o = order u1 u2 in
  (*
  print_string (string_of_comparison_result o);
  force_newline();
  print_flush();
  *)
  o

(*

  RPS conditions

*)

let rec check_if_all_distinct_vars env = function
    [] -> ()
  | (TERM(_,_))::_ ->
      semantical_error 
	("Wrong RPS : subterms of left hand sides must be variables")
  | (VAR x)::l ->
      if List.mem x env
      then
	semantical_error 
	    ("Wrong RPS : the variable `"^(string_of_var_id x)^
	     "` occurs twice in a left hand side of a rule")
      else 
	check_if_all_distinct_vars (x::env) l
;;

let rec	check_if_symbols_occur env = function
    (VAR _) -> ()
  | TERM(f,l) ->
      if List.mem f env
      then
	semantical_error 
	    ("Wrong RPS : the rewritten symbol `"^(string_of_symbol_id f)^
	     "` occurs in a right hand side of a rule")
      else
	List.iter (check_if_symbols_occur env) l
;;

	  

let rec check_rules env = function 
    [] -> ()
  | rule::rule_set ->
      let lhs = rule.l
      in
      let f = top_symbol_id lhs
      and args = liste_des_sous_termes lhs
      in
      begin
	check_if_all_distinct_vars [] args;
      	if List.mem f env
	then
	  semantical_error 
	    ("Wrong RPS : the symbol `"^(string_of_symbol_id f)^
	     "` occurs twice in the left hand sides");
	check_if_symbols_occur (f::env) rule.r;
	check_rules (f::env) rule_set 
      end
;;
	
	    
let check_rps_conditions rule_set =
  check_rules [] rule_set
;;



