(***************************************************************************

  Term orderings: body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: term_orderings.ml,v 1.7 2002/05/27 12:07:26 contejea Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: term_orderings.ml,v 1.7 2002/05/27 12:07:26 contejea Exp $

***************************************************************************)

open Orderings;;
open Symbols;;
open Format;;
open Messages;;
open Errors;;

open Terms;;
open Rpo;;
open Equations;;
open Poly_interp;;


type ordre_disponible =
    ORDRE_INTERACTIF
  | RPO of (precedence_description list) * ((symbol_id * rpo_status) list)
  | MAPO of (precedence_description list)* ((symbol_id * rpo_status) list)
  | POLY of int * (symbol_id * IntPolynomials.poly) list
  | LEXICO of (ordre_disponible list)
  | SEMI_INTERACTIF of ordre_disponible
  | Rps of ordre_disponible * regle list 
;;

(*------------------------------------------------------
the orderings are defined in the following modules
--------------------------------------------------------*)

open Ord_interactive;;
open Semi_interactive;;
open Mapo;;
open Poly_interp;;
open Rps;;
open Dnet;;

(*

  string_of_order 

*)


let string_of_statut (f,s) = match s with
    MULTISET  -> (string_of_symbol_id f)^" mul"
  | RL_LEXICO -> (string_of_symbol_id f)^" rllex"
  | LR_LEXICO -> (string_of_symbol_id f)^" lrlex"
;;


let rec string_of_precedence_description = function
    One_symbol(f)     -> (string_of_symbol_id f)
  | Equal_to(f,p)     -> (string_of_symbol_id f)^" = "^
                         (string_of_precedence_description p)
  | Greater_than(f,p) -> (string_of_symbol_id f)^" > "^
                         (string_of_precedence_description p)
  | Lower_than(f,p)   -> (string_of_symbol_id f)^" < "^
                         (string_of_precedence_description p);;


let rec string_of_poly_var_list_end k n = 
  if n>0
    then 
      begin
      ",x"^(string_of_int k)^
      (string_of_poly_var_list_end (k+1) (n-1))
      end
    else ")"
;;

let string_of_poly_var_list n = 
  if n>0
  then 
    "(x1"^(string_of_poly_var_list_end 2 (n-1))
  else ""
;;


(*
let string_of_poly_interp (f,p) =
  "["^(string_of_symbol_id f)^"]"^
  (string_of_poly_var_list (arity f))^"="^
  (string_of_polynome p)
;;
 
let rec string_of_ordre = function
    ORDRE_INTERACTIF -> 
      "interactive"
  | RPO(prec,stats)  -> 
      "rpo("^
      (List.fold_left
	 (fun s p -> s^(string_of_precedence_description p))
	 "" prec)^";"^
      (List.fold_left
	 (fun s p -> s^(string_of_statut p))
	 "" stats)^")"
  | MAPO(prec,stats) -> 
      "mapo("^
      (List.fold_left
	 (fun s p -> s^","^(string_of_precedence_description p))
	 "" prec)^";"^
      (List.fold_left
	 (fun s p -> s^","^(string_of_statut p))
	 "" stats)^")"
  | LEXICO(ol)       ->	
      "lexico("^(string_of_ordre (List.hd ol))^
      (string_of_ordre_list (List.tl ol))
  | SEMI_INTERACTIF(o) -> 
      "semi_interactive("^(string_of_ordre o)^")"
  | POLY(mu,interp)       -> 
      "poly("^(string_of_int mu)^","^
      (List.fold_left 
	 (fun s i -> s^","^(string_of_poly_interp i))
	 "" interp)^")"
  | Rps(o,rule_set) ->
      "rps("^(string_of_ordre o)^";"^
      (List.fold_left 
	 (fun s r -> s^";"^(string_of_rule r))
	 (string_of_rule (List.hd rule_set))
	 (List.tl rule_set))^")"

and string_of_ordre_list = function
    [] 	     ->	")"
  | o::ol    -> ","^(string_of_ordre o)^(string_of_ordre_list ol)
;;
*)

(*=================================================

Printing functions. Only "print_ordre" is exported.

==================================================*)

let printf s x =
  print_string (sprintf s x)
;;



let print_statut (f,s) = match s with
    MULTISET  -> printf "%s multiset\n" (string_of_symbol_id f)
  | RL_LEXICO -> printf "%s right-left lexico\n" (string_of_symbol_id f)    
  | LR_LEXICO -> printf "%s left-right lexico\n" (string_of_symbol_id f);;


let rec print_precedence_description = function
    One_symbol(f)     -> printf "%s\n" (string_of_symbol_id f)
  | Equal_to(f,p)     -> printf "%s = " (string_of_symbol_id f);
                         print_precedence_description p
  | Greater_than(f,p) -> printf "%s > " (string_of_symbol_id f);
                         print_precedence_description p
  | Lower_than(f,p)   -> printf "%s < " (string_of_symbol_id f);
                         print_precedence_description p;;


let rec print_poly_var_list_end k n = 
  if n>0
    then 
      begin
      printf ",x%d" k;
      print_poly_var_list_end (k+1) (n-1)
      end
    else print_string ")"
;;

let print_poly_var_list n = 
  if n>0
    then 
      begin
      print_string "(x1";
      print_poly_var_list_end 2 (n-1)
      end
;;


let print_poly_interp (f,p) =
  printf "[%s]" (string_of_symbol_id f);
  print_poly_var_list (arity f);
  print_string "=";
  print_int_polynomial p;
  force_newline()
;;

 
let rec print_ordre = function
    ORDRE_INTERACTIF -> 
      begin
	print_string "Interactive ordering";
  	force_newline()
      end
  | RPO(prec,stats)  -> 
      begin
      	print_string "RPO";
      	force_newline();
        print_string " precedence :";
      	force_newline();
        List.iter print_precedence_description prec;
        print_string " status :";
      	force_newline();
        List.iter print_statut stats
      end
  | MAPO(prec,stats) -> 
      begin
      	print_string "MAPO";
      	force_newline();
        print_string " precedence :";
      	force_newline();
        List.iter print_precedence_description prec;
	print_string " status :";
      	force_newline();
        List.iter print_statut stats
      end
  | LEXICO(ol)       ->	
      begin
      	print_string "lexico(";
        print_ordre (List.hd ol);
        print_ordre_list (List.tl ol)
      end
  | SEMI_INTERACTIF(o) -> 
      begin
      	print_string "semi_interactive(";
        print_ordre o;
        print_string ")";
      	force_newline();
      end
  | POLY(mu,interp)       -> 
      begin
      	printf "polynomial ordering with mu=%d" mu;
        List.iter print_poly_interp interp
      end
  | Rps(o,rule_set) ->
      begin
	message "rps interpretation by the ordering";
	print_ordre o;
	message "with the rewrite system";
	print_liste_de_regles rule_set;
	force_newline()
      end

and print_ordre_list = function
    [] 	     ->	
      begin 
	print_string ")";
      	force_newline()
      end

  | o::ol    -> 
      begin
	print_string ",";
      	force_newline();
        print_ordre o;
        print_ordre_list ol
      end;;


(*===========================================================================

"ordre_of_ordre_disponible" calls the procedure for the selected ordering.
Term orderings are never called directly but always through this interface.

============================================================================*)

let statut_liste liste f =
    try List.assoc f liste
    with Not_found -> MULTISET;;

let rec ordre_of_ordre_disponible = function
    ORDRE_INTERACTIF ->
         ordre_interactif
  | RPO(prec,stats) ->
 	 let
		 table = (table_de_cloture_transitive 
                               	  (plus_grand_numero_de_symbole ()) prec)
         in
(**)
	   	(check_rpo_conditions (plus_grand_numero_de_symbole ()) prec);
(**)
		(* print_table_de_precedences table; *)
                rpo (precedence_par_table table, statut_liste stats) 
  | MAPO(prec,stats) ->
	 let
		 table = (table_de_cloture_transitive
                                  (plus_grand_numero_de_symbole ()) prec)
         in
(*
	   	(check_mapo_condition (plus_grand_numero_de_symbole ()) prec);
*)
(**)
	   	(check_extended_mapo_condition 
                                     (plus_grand_numero_de_symbole ()) table);
(**)
                mapo (precedence_par_table table,statut_liste stats)
  | POLY(mu,interp) ->
      let interp_map =
	if mu=0 then interp
	else 
	  List.map 
	    (function (f,p) -> (f,mu_translate mu p))
	    interp
      in
      List.iter 
	(function (f,p) -> 
	  try check_poly_interp f p
	      with Semantical_error s ->
		(*
		print_polynome p;
		force_newline();
		*)
		semantical_error ("while checking interpretation of "^(string_of_symbol_id f)^" : "^s))
	interp_map;
      poly_ord 
        (fun f -> 
      	  try (List.assoc f interp_map)
	  with Not_found -> semantical_error ("Symbol " ^ (string_of_symbol_id f) ^ " has no polynomial interpretation")
              ) 

  | LEXICO(ol) ->
  	 extension_lexico_ordres (List.map ordre_of_ordre_disponible ol)

  | SEMI_INTERACTIF(o) ->
  	 extension_semi_interactive (ordre_of_ordre_disponible o)

  | Rps(o,rule_set) ->
      begin
	check_rps_conditions rule_set;
	let rule_net = 
	  List.fold_left
	    (fun d r -> add_rule_to_dnet LEFT r d)
	    (max_int, empty_dnet()) rule_set
	in
	  rps rule_net (ordre_of_ordre_disponible o)
      end

;; 
