(***************************************************************************

(This sentence has been added automatically, it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: total_AC_ordering.ml,v 1.5 1998/04/28 13:36:21 marche Exp $

***************************************************************************)

open C_declare_operator;;
open Symbols;;
open Terms;;
open Equations;;
open Rps;;
open Rpo;;
open Poly_interp;;
open Term_orderings;;
open Format;;
open Norm_theory;;
open Dnet;;


let polysum = IntPolynomials.add;;
let polyprod = IntPolynomials.mult;;
let polycst x d = IntPolynomials.cte (Num.Int x);;
let polyvar i d = IntPolynomials.var (i-1);;


(*
  split_symbol_id_list (l1,l2,l3,l4,l5,l6) l repartit les symbol_ids de l
  dans les differentes listes li en distinguant les cas suivants
  - symbole libre d'arite au moins 2 (non associatif) -> l1
  - symbole libre unaire -> l3
  - symbole constant -> l5
  - symbole commutatif -> l2
  - symbole AC -> l4
  - symbole associatif -> l6
*)

let rec split_symbol_id_list = function
  [] -> ([],[],[],[],[],[])
| f::l -> let (l1,l2,l3,l4,l5,l6) = split_symbol_id_list l in
          if (is_free f)
          then (match (arity f) with
                      0 -> (l1,l2,l3,l4,f::l5,l6) 
                    | 1 -> (l1,l2,f::l3,l4,l5,l6) 
                    | 2 -> 
			begin
			  match norm_theory_of f with
			    A _ | AU _ | G _ -> (l1,l2,l3,l4,l5,f::l6)
			  | _ -> (f::l1,l2,l3,l4,l5,l6)
			end
		    | _ -> (f::l1,l2,l3,l4,l5,l6))
	  else (if (is_ac f)
	        then (l1,l2,l3,f::l4,l5,l6)
                else (l1,f::l2,l3,l4,l5,l6))
;;

let axiom_of_distributivity e = match e#first_term,e#second_term with
   TERM(f,[VAR x;TERM(g,[VAR y;VAR z])]),
   TERM(g',[TERM(f',[VAR x';VAR y']);TERM(f'',[VAR x'';VAR z'])]) ->
   if (f=f') & (f=f'') & (g=g') & (x=x') & (x=x'') & (y=y') & (z=z')
   then (f,g)
   else raise Not_found
|  TERM(f,[TERM(g,[VAR y;VAR z]);VAR x]),
   TERM(g',[TERM(f',[VAR y';VAR x']);TERM(f'',[VAR z';VAR x''])]) ->
   if (f=f') & (f=f'') & (g=g') & (x=x') & (x=x'') & (y=y') & (z=z')
   then (f,g)
   else raise Not_found
|  TERM(g',[TERM(f',[VAR x';VAR y']);TERM(f'',[VAR x'';VAR z'])]),
   TERM(f,[VAR x;TERM(g,[VAR y;VAR z])]) ->
   if (f=f') & (f=f'') & (g=g') & (x=x') & (x=x'') & (y=y') & (z=z')
   then (f,g)
   else raise Not_found
|  TERM(g',[TERM(f',[VAR y';VAR x']);TERM(f'',[VAR z';VAR x''])]),
   TERM(f,[TERM(g,[VAR y;VAR z]);VAR x]) ->
   if (f=f') & (f=f'') & (g=g') & (x=x') & (x=x'') & (y=y') & (z=z')
   then (f,g)
   else raise Not_found
| _ -> raise Not_found
;;

let rec couples_de_symboles_pour_D = function
  [] -> []
| e::l -> try let c = axiom_of_distributivity e in
              c::(couples_de_symboles_pour_D l)
	  with Not_found -> couples_de_symboles_pour_D l
;;

let rec type_vis_a_vis_de_D f = function
  [] -> 0
| (g,h)::l -> if (f=g) 
              then 1
	      else (if (f=h)
	            then 2
		    else type_vis_a_vis_de_D f l)
;;

let rec split_a_ac_symbols couples_D = function
  [] -> ([],[])
| f::l -> let (l1,l02) = split_a_ac_symbols couples_D l in
          (match (type_vis_a_vis_de_D f couples_D) with
                1 -> ((f::l1),l02)
               | _ -> (l1,(f::l02)))
;;

let premier_symbole = function
    One_symbol(f)     -> f
  | Equal_to(f,_)     -> f
  | Greater_than(f,_) -> f
  | Lower_than(f,_)   -> f
;;

let rec dernier_symbole = function
    One_symbol(f)     -> f
  | Equal_to(_,p)     -> dernier_symbole p
  | Greater_than(_,p) -> dernier_symbole p
  | Lower_than(_,p)   -> dernier_symbole p
;;



let rec recolle_precedences = function
  [] -> []
| p::[] -> []
| p1::(p2::l as ll) -> let res = recolle_precedences ll in
                       (Greater_than ((dernier_symbole p1),
                                     (One_symbol (premier_symbole p2))))::res
;;

let make_precedence_description (lfree,lcom,lu,lac1,lac02,lcst,la) =
  let prec_lcst = 
    (match lcst with
      h::t -> [List.fold_left (fun x y -> Equal_to (y,x))
                  (One_symbol h) t] 
    | [] -> [])

  and prec_lac02 = 
    (match lac02 with
      h::t -> [List.fold_left (fun x y -> Equal_to (y,x))
                  (One_symbol h) t] 
    | [] -> [])

  and prec_lu = 
    (match lu with
      h::t -> [List.fold_left (fun x y -> Greater_than (y,x))
                  (One_symbol h) t] 
    | [] -> [])

  and prec_lac1 = 
    (match lac1 with
      h::t -> [List.fold_left (fun x y -> Equal_to (y,x))
                  (One_symbol h) t] 
    | [] -> [])

  and prec_lcom = 
    (match lcom with
      h::t -> [List.fold_left (fun x y -> Greater_than (y,x))
                  (One_symbol h) t] 
    | [] -> [])

  and prec_lfree = 
    (match lfree with
      h::t -> [List.fold_left (fun x y -> Greater_than (y,x))
                  (One_symbol h) t] 
    | [] -> []) 
  and prec_la = 
    (match la with
      h::t -> [List.fold_left (fun x y -> Equal_to (y,x))
                  (One_symbol h) t] 
    | [] -> []) in

  let list_prec = 
    prec_lfree @ prec_lcom @ prec_lac1 @ prec_lu @ 
    prec_lac02 @ prec_lcst @ prec_la in

  (recolle_precedences list_prec) @ list_prec
;;


let make_rpo_status_lexico (lfree,lcom,lu,lac,lcst,la) =
  let lfree_st = List.map (function f -> (f,LR_LEXICO)) lfree
  and lcom_st = List.map (function f -> (f,MULTISET)) lcom
  and lac_st = List.map (function f -> (f,MULTISET)) lac 
  and la_st = List.map (function f -> (f,LR_LEXICO)) la in
  lfree_st @ lcom_st @ lac_st @ la_st
;;

let make_rpo_status_multiset (lfree,lcom,lu,lac,lcst,la) =
  let lfree_st = List.map (function f -> (f,MULTISET)) lfree
  and lcom_st = List.map (function f -> (f,MULTISET)) lcom
  and lac_st = List.map (function f -> (f,MULTISET)) lac 
  and la_st = List.map (function f -> (f,LR_LEXICO)) la in
  lfree_st @ lcom_st @ lac_st
;;

let rec make_list_n = function
  0 -> []
| m -> 0::(make_list_n (pred m))
;;

let make_list_i_n i n =
    (make_list_n (pred i)) @ (1::(make_list_n (n - i)))
;;



let mapi f l = 
  let rec mapi_aux i = function
         [] -> []
       | h::t -> (f i h)::(mapi_aux (succ i) t) in
  mapi_aux 1 l
;;

let make_poly_d (lfree,lcom,lu,lac1,lac02,lcst) =
  let lfree_poly = List.map (function f -> let n = (arity f) in
                        let poly_f = ref (polyvar 1 n) in
                        for i=2 to n do
                            poly_f := polysum
                                      (polyvar i n)
                                      !poly_f
			done;
                        (f,!poly_f))
                        lfree
  and lac1_poly = List.map (function f -> (f,polyprod (polyvar 1 2) 
                                                 (polyvar 2 2)))
                      lac1
  and lac02_poly = List.map (function f -> (f, polysum (polyvar 1 2)
                                         (polysum (polyvar 2 2) 
                                                  (polycst 2 2))))
                       lac02
  and lcom_poly = List.map (function f -> (f,polysum (polyvar 1 2) (polyvar 2 2)))
                      lcom
  and lu_poly = mapi (fun i f -> (f, polysum (polyprod (polyvar 1 1) 
                                                       (polycst (succ i) 1))
                                             (polycst 1 1)))
                     lu
  and lcst_poly = List.map (function f -> (f,(polycst 2 0))) lcst in
  
  lfree_poly @ lcom_poly @ lu_poly @ lac1_poly @ lac02_poly @ lcst_poly
;;


let make_poly_tot (lfree,lcom,lu,lac,lcst) = 
  let nb_free = List.length lfree
  and nb_com = List.length lcom
  and nb_u = List.length lu
  and nb_ac = List.length lac
  and nb_cst = List.length lcst in
  let nn = nb_free + nb_com + nb_u + nb_ac + nb_cst in
  let lac_poly = mapi (fun i f -> let ai = polycst (nn * ((nn * i) - 1)) 2
                                  and bi = polycst (nn * i) 2
				  and ci = polycst i 2 in
		        (f, (polysum (polyprod ai 
                                     (polyprod (polyvar 1 2)
                                               (polyvar 2 2)))
                            (polysum (polyprod bi (polyvar 1 2))
                            (polysum (polyprod bi (polyvar 2 2))
                                     ci)))))
                       lac
  and lfree_poly = mapi (fun i f -> let n = (arity f) in
                        let poly_nn = polycst nn n in
                        let poly_f = ref (polyprod poly_nn (polyvar 1 n)) in
                        for i=2 to n do
                            poly_f := polysum (polyprod poly_nn (polyvar i n))
                                              !poly_f
			done;
			poly_f := polysum (polycst (i+nb_ac) n) !poly_f;
                        (f,!poly_f))
                        lfree
  and lcom_poly = mapi (fun i f -> let poly_nn = polycst nn 2 in
                                  (f,(polysum (polyprod poly_nn (polyvar 1 2))
                                     (polysum (polyprod poly_nn (polyvar 2 2))
                                              (polycst (i+nb_ac+nb_free) 2)))))
                       lcom
  and lu_poly = mapi (fun i f -> (f,polysum (polyprod (polycst nn 1)
                                                      (polyvar 1 1))
                                            (polycst (i+nb_ac+nb_free+nb_com) 
                                                     1)))
                     lu
  and lcst_poly = mapi (fun i f -> (f,(polycst (i+nb_ac+nb_free+nb_com+nb_u) 
                                               0))) 
                       lcst in
  
  lfree_poly @ lcom_poly @ lu_poly @ lac_poly @ lcst_poly
;;


let total_AC_ordering type_ordre list_of_axioms =
  let (lfree,lcom,lu,lac,lcst,la) = 
    split_symbol_id_list (listof_all_functions ())
  and couples_D = couples_de_symboles_pour_D list_of_axioms in
  let rl = ref [] 
  and copy_list = ref [] in
  let la_copy = 
    List.map 
      (function f -> 
	let name_of_f_copy = "_"^(string_of_symbol_id f)^"_" in
	definir_operateur AC 2 INFIX [] name_of_f_copy;
 	let f_copy = get_symbol_id name_of_f_copy in
	let x = fresh_new_var () in
  	let y = next_new_var x in
 	let terme_f = TERM (f,[VAR x; VAR y])
	and terme_f_copy = TERM (f_copy,[VAR x; VAR y]) in
	let rule = make_basic_rule (terme_f, terme_f_copy) in
	rl := rule::(!rl);
	copy_list := (f,f_copy)::!copy_list;
 	f_copy) 
      la in
  let (lac1,lac02) = split_a_ac_symbols couples_D lac
  and (la1,la2) = split_a_ac_symbols couples_D la in
  let la1_copy = List.map (function f -> List.assoc f !copy_list) la1
  and la2_copy = List.map (function f -> List.assoc f !copy_list) la2 in
(*
  print_string "lac1 = ";
  print_listof_functions lac1;
  force_newline();
  print_string "lac02 = ";
  print_listof_functions lac02;
  force_newline();
  print_flush();
*)  
  let sslist_of_symbols = 
    (lfree,lcom,lu,lac1@la1_copy,lac02@la2_copy,lcst,la) 
  and slist_of_symbols =(lfree,lcom,lu,lac@la_copy,lcst,la) in

  let mapo_mult = MAPO ((make_precedence_description sslist_of_symbols),
                   (make_rpo_status_multiset slist_of_symbols))
  and mapo_lex = MAPO ((make_precedence_description sslist_of_symbols),
                   (make_rpo_status_lexico slist_of_symbols))
  and d = POLY (2,
		make_poly_d 
		  (lfree@la,lcom,lu,lac1@la1_copy,lac02@la2_copy,lcst))
  and tot = POLY (2,make_poly_tot (lfree@la,lcom,lu,lac@la_copy,lcst) )
  in match type_ordre with 
     1 -> 
       LEXICO [Rps (mapo_lex,!rl);
	       Rps (d,!rl);
	       Rps (tot,!rl);
	       mapo_lex]
   | 2 -> 
       (match lfree with 
	 [] -> 
	   LEXICO [Rps (mapo_lex,!rl);
		   Rps (d,!rl);
		   Rps (tot,!rl);
		   mapo_lex]
       |  _::_ -> 
	   LEXICO [Rps (mapo_mult,!rl);
		   Rps (mapo_lex,!rl);
		   Rps (d,!rl);
		   Rps (tot,!rl);
		   mapo_lex])
   | 3 -> SEMI_INTERACTIF(LEXICO [RPO ([],[])])
   | _ -> failwith "ordering type non available"
;;

