(***************************************************************************

Dependency pairs criteria, description and specification

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: dependency_pairs_criteria.mli,v 1.15 2002/07/01 12:01:36 contejea Exp $

***************************************************************************)


(*
\label{dependency_pair_criteria}

\subsection{Simple dependency pairs criterion}

  [(simple_dependency_pair_criterion R)] returns the set of ordering
  existence constraints given by the simple dependency pairs
  termination criterion on [R], which is the following :

  \begin{enumerate} 

  \item let $D$ be the set of \emph{defined} symbols of [R], that is
    the set of symbols occuring at root of a lefthand side of a rule
    of [R] ;

  \item a \emph{dependency pair} is any pair of terms $(s,t)$ such
    that $s$ is the lefthand side of a rule $s\rightarrow r$ in [R]
    and $t$ is any subterm of $r$ whose root symbol is defined.

  \item a condition for [R] terminating is that there is a weakly
    monotonic ordering $\geq$ such that $l\geq r$ for all rules of [R]
    and $s>t$ for all dependency pair of [R] ;

  \item $\geq$ must be AC-compatible.

  \end{enumerate} 

*)
val compute_defined_symbols :     
  ('symbol Rewrite_rules.t) list -> 
    'symbol Signatures.SymbolSet.t
;;

val all_rules_weakly_decreasing :
  ('symbol Rewrite_rules.t) list ->
    'symbol Termination_constraints.AtomicTerminationConstraintSet.t
;;

val all_dp_strictly_decreasing :
  ('symbol Gen_terms.term * 'symbol Gen_terms.term) list ->
    'symbol Termination_constraints.AtomicTerminationConstraintSet.t
;;

val compute_dependency_pairs :
  'symbol Signatures.SymbolSet.t ->
    ('symbol Rewrite_rules.t) list ->
      ('symbol Gen_terms.term * 'symbol Gen_terms.term) list
;;	

val simple_dependency_pair_criterion :
  'symbol #Signatures.signature ->
    'symbol Rewrite_rules.t list -> 
      'symbol Termination_constraints.conj_termination_constraints;;

