(***************************************************************************

Checking termination of a rewrite system by the basic criterion

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: generic_polynomials.mli,v 1.12 2002/06/25 13:18:41 urbain Exp $

***************************************************************************)

type polyinterptype = Linear | Simple | Simple_Mixed;;

val use_degree_two_interp : bool ref;;
val use_linear_interp : bool ref;;

(*

  module [GenericPolynomial] defines polynomials over the coefficients
  ring : polynomials of finite domain variables ; and variables
  indexed by integers.

*)

module Fd_polynomials_ring : Polynomials.RingType
  with
    type coef = Non_linear_solving.Fd_polynomials.poly
;;

module GenericPolynomials : Polynomials.PolynomialType
  with
    module Base_ring = Fd_polynomials_ring
  and
    type variable = int;;

(*

  [(print_generic_polynomial p)] outputs [p] to standard output

*)

val print_generic_polynomial : GenericPolynomials.poly -> unit;;

(*

  [(print_interp i)] outputs the polynomial interpretation [i] (that
  is a map which associates polynomials to symbols) to standard output

*)

val print_interp : 
  'a #Signatures.signature 
  -> ('a,GenericPolynomials.poly) Signatures.SymbolMap.t -> unit;;


(*

  [(polynomial_constraints_of_termination_constraints sigma i c)]
  returns a pair [(interp,c1,c2,c3)] where [interp] is a set of generic
  polynomial interpretations of symbols in [c] and [c1], [c2] and [c3]
  are lists of diophantine constraints over variables of [interp] such
  that :
  \begin{itemize}
  \item if [c1] then interp is AC-compatible
  \item if [c2] then interp is strictly monotonic
  \item if [c3] then [c] 
  \end{itemize}

  [sigma] is the signature on which constraints are defined, and [i]
  is a `predefined'' interpretation for some symbols, that is the
  function search only for an interpretation [interp] that extends
  [i]. Of course [i] may be empty.

*)

val polynomial_constraints_of_termination_constraints :
  'a #Signatures.signature 
  -> ('a, GenericPolynomials.poly) Signatures.SymbolMap.t 
    -> 'a Termination_constraints.conj_termination_constraints 
      -> 
      	('a, GenericPolynomials.poly) Signatures.SymbolMap.t 
      	* Non_linear_solving.non_linear_constraint list
      	* Non_linear_solving.non_linear_constraint list
      	* Non_linear_solving.non_linear_constraint list
;;
    



(*

  [(specialize_polynomial p s)] returns the polynomial obtained by
  instantiation of finite domain variables in coefficients of [p] by
  their value given by the map [s]

*)

val specialize_polynomial :
  GenericPolynomials.poly 
  -> (Finite_domains.fd_var_id, Numbers.t) Finite_domains.Fd_var_map.t 
    -> GenericPolynomials.poly


val build_new_interp :  
  ('symbol, GenericPolynomials.poly) Signatures.SymbolMap.t ->
    (Finite_domains.fd_var_id, Numbers.t) Finite_domains.Fd_var_map.t ->
      ('symbol, GenericPolynomials.poly) Signatures.SymbolMap.t;;


val poly_solve_termination_constraint :
  Numbers.t ->  'a #Signatures.signature 
    -> 'a Termination_constraints.conj_termination_constraints 
      -> unit
;;
    
