(***************************************************************************

Lexer for generic polynomials

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: genpoly_lexer.mll,v 1.3 2002/05/27 12:07:35 contejea Exp $

***************************************************************************)


{

  open Genpoly_syntax;;
  open Genpoly_parser;;
  open Marked_dp_criteria;;

  exception Syntax_error of string;;


  let interp s =
    try
      let is_marked = 
	String.get s 0 = '\'' & String.get s (pred (String.length s)) = '`'
      in
      let s' =
      	if is_marked 
      	then String.sub s 1 (pred (pred (String.length s)))
	else s
      in      	
      let f = !current_signature#symbol_of_string s
      in 
      let g =
	if is_marked
	then Marked f
	else Original f
      in INTERP(g)
    with 
	Not_found ->
	  raise (Syntax_error ("undefined identifier "^s))
	
}

rule token = parse
    [' ''\t''\n']+  	{ token lexbuf }
  | ['a'-'z''A'-'Z']['0'-'9']* 
                        { VAR(Lexing.lexeme lexbuf) }
  | '['                 { symbol lexbuf }
  | ';'                 { SEMICOLON }
  | '+'                 { PLUS }
  | '-'                 { MINUS }
  | '.'                 { MULT }
  | '^'                 { EXP }
  | ['0'-'9']+          { INT (Num.num_of_string (Lexing.lexeme lexbuf)) }
  | '('            	{ PARGAUCHE }
  | ')'            	{ PARDROITE }
  | eof			{ EOF }


and symbol = parse
    [' ''\t''\n']+      { symbol lexbuf }
  | [^']'' ''\t''\n']+  { let f = interp (Lexing.lexeme lexbuf) 
			  in skip_bracket lexbuf; f }

and skip_bracket = parse
    [' ''\t''\n']+      { skip_bracket lexbuf }
  | ']'                 { () }
