(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: graph.mli,v 1.3 2002/06/25 08:32:44 urbain Exp $

***************************************************************************)


(* A module of graphs with a function scc which returns a list of the strongly
   connected components (as lists of elements) of its argument.
   The selected algorithm is from Tarjan (1972) using deep-first search.
*)


module type G =
  sig
    type 'a elt         (* The type of the set elements. *)
    type 'a t           (* The abstract type of graphs. *)
    
    exception NodeNotInGraph
    exception NodeAlreadyInGraph

    val empty : 'a t (* The empty graph. *)
    val add_node : 'a t -> 'a elt -> 'a t
    val is_node : 'a t -> 'a elt -> bool

    val add_edge : 'a t -> 'a elt -> 'a elt -> 'a t
    val is_edge : 'a t -> 'a elt -> 'a elt -> bool

    val del_node : 'a t -> 'a elt -> 'a t
    val del_edge : 'a t -> 'a elt -> 'a elt -> 'a t

    val node_list : 'a t -> 'a elt list
    val edge_list : 'a t -> ('a elt * 'a elt) list
    val neighbours_list : 'a t -> 'a elt -> 'a elt list

    val iter_node : 'a t -> ('a elt -> unit) -> unit
    val iter_edge : 'a t -> ('a elt -> 'a elt -> unit) -> unit

    val fold : ('a elt -> 'b -> 'b) -> 'b -> 'a t -> 'b

    val transpose : 'a t ->  'a t

	(* [scc g] returns a list of list of elements in strongly 
	   connected components of graph [g]. *)
    val scc :  'a t -> 'a elt list list

(* [scc_clean g] returns a list of the strongly connected components
 of graph [g] which are not reduced to a single node with no edge, as
 lists of vertices. *)
    val scc_clean : 'a t -> 'a elt list list

(* [build_subgraph g l] returns the subgraph of [g] restricted to
   vertices in list [l]. *)    
    val build_subgraph : 'a t -> 'a elt list -> 'a t

  end

module Make(Ord: Ordered_types.OrderedPolyType): (G with type 'a elt = 'a Ord.t)



