(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: graph_criteria.ml,v 1.15 2002/07/11 13:26:15 pguillon Exp $

***************************************************************************)

open Gen_terms;;
open Signatures;;

open Termination_constraints;;

open Dependency_pairs_criteria;;  
open Marked_dp_criteria;;
open Hierarchical_trs;;

open Labelled_graphs;;

open Graph;;

(* selection of the graph criterion *)

let uses_all_strongly_connected_parts = ref false;;

(* [rencap] replaces all defined symbols with variables.               *)

let rencap hct defined_symbols used_vars terme =

  let rec do_rencap used_vars t = 
    match t.Hcons.node with 
	Var _ -> 
	  let x = Variables.var_outside_set used_vars
	  in (Variables.VarSet.add x used_vars,make_var_term hct x)
      | Term(f,l) ->
	  if SymbolSet.mem f defined_symbols 
	  then 
	    let x = Variables.var_outside_set used_vars
	    in (Variables.VarSet.add x used_vars,make_var_term hct x)
	  else
	    let (newvars,newl) = do_rencap_list used_vars l
	    in
	      (newvars,make_term hct f newl)
	
	
  and do_rencap_list used_vars l =
    List.fold_right 
      (fun t (vars,l) ->
	 let (newvars,newt) = do_rencap vars t
	 in (newvars,newt::l))
      l
      (used_vars,[])
	
  in
    
  match terme.Hcons.node with
    | Var _ -> terme
    | Term(f,l) -> 
	let (_,newl) = do_rencap_list used_vars l
	in make_term hct f newl
;;


let connectable sigma hct defined_symbols s t = 
  (*i
    if sigma#contains_only_free_symbols
    then
  *)
  let s = (rencap hct defined_symbols (set_of_variables t) s)
  in
  (* remainder : [Standard_unification.pre_unifiable] returns true
     whenever the terms [s] and [t] have the same ``top skeleton'',
     that is ignoring levels where there are variables or non-free
     symbols.  *)
  Standard_unification.pre_unifiable sigma s t
;;

(*
  
  Computation of the graph
  
*)

let compute_dependency_graph sigma hct defined_symbols dp =
  List.fold_right
    (fun ((s,t) as pair) g1 -> 
       (*i
	 print_string "ajout de la paire ";
	 print_term sigma (new Variables.from_string "") s;
	 print_string " ; ";
	 print_term sigma (new Variables.from_string "") t;
	 print_newline();
       *)
       let (v,g2) = add_vertex g1 pair 
       in
	 fold_left_graph
	   (fun g3 v' ->
	      let (s',t') = get_vertex_label g3 v'
	      in
	      let g4 =
		if connectable sigma hct defined_symbols t s'
	      	then add_arc g3 v v'
		else g3
	      in
		if v<>v' & (connectable sigma hct defined_symbols t' s)
	   	then add_arc g4 v' v
	   	else g4)
	   g2
	   g2)
    dp
    empty_graph
;;



(*i

version naive

let constraints_from_graph rules g =
  let v = vertices_on_cycle g
  in
  let dp' = 
    VertexSet.fold (fun v l -> (get_vertex_label g v)::l) v []
  in
    ConjTerminationConstraintSet.add
      (AtomicTerminationConstraintSet.union
	 (all_rules_weakly_decreasing rules)
	 (all_dp_strictly_decreasing dp'))
      ConjTerminationConstraintSet.empty
;;

*)


let best_vertex g =
  let (_,x) =
    fold_left_graph
      (fun ((best_nb_arc,best_v) as best) v -> 
	 let nb_arc = VertexSet.cardinal (get_successors g v)
	 in
	   if nb_arc > best_nb_arc
	   then (nb_arc,Some v)
	   else best)
      (-1,None)
      g
  in
    match x with
	Some v -> v
      | None -> raise Not_found
;;


let rec transformation_loop constraints graph =

  (* removing 1-cycles *)
  let (c,g) =
    fold_left_graph 
      (fun (c,g) v ->
	 if vertex_has_a_1_cycle g v
	 then
	   let (l,r) = get_vertex_label g v
	   in
	     (AtomicTerminationConstraintSet.add 
		{
		  left = l;
		  right = r;
		  constraint_op = Greater
		}
		c,remove_vertex_and_its_arcs g v)
	 else
	   (c,g))
      (constraints,graph)
      graph
  in

  (* deleting vertices not on cycles *)
  let (c',g') =
    VertexSet.fold 
      (fun v (c,g) ->
	 let (l,r) = get_vertex_label g v
	 in
	   (AtomicTerminationConstraintSet.add 
	      {
		left = l;
		right = r;
		constraint_op = Greater_or_equal
	      }
	      c,remove_vertex_and_its_arcs g v))
      (snd (split_cycles g))
      (c,g)
  in

    (* spliting : find the vertex with maximum number of arcs and make
       a disjunction of constraints. *)

  try
    let v = best_vertex g'
    in
    let (l,r) = 
      try get_vertex_label g' v
      with Failure _ -> failwith "ca bugue"
    in
    let c1 = AtomicTerminationConstraintSet.add 
	       {
		 left = l;
		 right = r;
		 constraint_op = Greater
	       }
	       c'
    and g1 = remove_vertex_and_its_arcs g' v
    and c2 = AtomicTerminationConstraintSet.add 
	       {
		 left = l;
		 right = r;
		 constraint_op = Greater_or_equal
	       }
	       c'
    and g2 = remove_vertex_and_link_its_arcs g' v
    in
      ConjTerminationConstraintSet.union
	(transformation_loop c1 g1)
	(transformation_loop c2 g2)
	
  with
    Not_found -> 
      ConjTerminationConstraintSet.add 
	c' ConjTerminationConstraintSet.empty
;;



(**)

module OrdPairs = 
struct 
  type 'symbol t = ('symbol Gen_terms.term * 'symbol Gen_terms.term) 
  let compare = compare 
end;;



module OrdPairList = 
struct 
  type 'symbol t = ('symbol Gen_terms.term * 'symbol Gen_terms.term) list
  let compare = compare 
end;;

module PairSet = Ordered_sets.MakePoly(OrdPairs);;
module CompSet = Ordered_sets.MakePoly(PairSet);;

module PairGraph = Graph.Make(OrdPairs);;

let compute_dep_graph sigma hct defined_symbols dp =
  List.fold_left
    (fun g1 ((s,t) as from_pair) -> 
      (*i
	 print_string "ajout de la paire ";
	 print_term sigma (new Variables.from_string "") s;
	 print_string " ; ";
	 print_term sigma (new Variables.from_string "") t;
	 print_newline();
       i*)
       let g2 =  
	 try
	   PairGraph.add_node g1 from_pair 
	 with PairGraph.NodeAlreadyInGraph -> g1
       in
	 PairGraph.fold
	   (fun ((s',t') as to_pair) g3 ->
	      let g4 =
		if connectable sigma hct defined_symbols t s'
	      	then PairGraph.add_edge g3 from_pair to_pair
		else g3
	      in
		if from_pair<>to_pair (*i  i*) 
		  & (connectable sigma hct defined_symbols t' s)
	   	then PairGraph.add_edge g4 to_pair from_pair
	   	else g4)
	   g2
	   g2)
    PairGraph.empty
    dp
;;

let set_from_list l = List.fold_left 
			(fun acc e -> PairSet.add e acc)
			PairSet.empty
			l
;;

let scc_clean_set g = List.fold_left 
		    (fun acc c ->
		     ( let c' = set_from_list c in 
			 match c with 
			   | [a] -> 
			       if PairGraph.is_edge g a a then c'::acc
			       else acc
			   | _ -> c'::acc))
			[]
			(PairGraph.scc g)
;;

let build_subgraph_from_set g s = 
  PairSet.fold
    (fun vfrom acc -> 
       let acc2 = 
	 if PairGraph.is_node acc vfrom then acc 
	 else (PairGraph.add_node acc vfrom) in	     
	 List.fold_left 
	   (fun acc2 vgoal -> 
	      if PairSet.mem vgoal s then 
		try PairGraph.add_edge acc2 vfrom vgoal
		with PairGraph.NodeNotInGraph -> 
		  PairGraph.add_edge 
		    (PairGraph.add_node acc2 vgoal) 
		    vfrom 
		    vgoal
	      else acc2)
	   acc2
	   (PairGraph.neighbours_list g vfrom)
    )
    s
    PairGraph.empty
;;


let rec scp c_set g store =
  let c = build_subgraph_from_set g c_set in
    PairGraph.fold
      (fun vfrom acc ->
	 let c' = PairGraph.del_node c vfrom  in
	 let component_list = scc_clean_set c' in
	   List.fold_left 
	     (fun acc_store component ->
		if (CompSet.mem component acc_store)
		  (* Thanks to scc_clean, there is no single-noded graph
		     without loop. *)
	      then acc_store
	      else
		let newstore = CompSet.add component acc_store in
		  scp component c' newstore) 
	   acc
	   component_list)
    (CompSet.add c_set store) 
    (* [c] is supposed to be strongly connected. *) 
    c
;;


let trans_loop rules_const comp =
  PairSet.fold
    (fun ((l,r) as node) acc -> 
       let weak = 
	 PairSet.fold 
	   (fun (l,r) acc ->
	      (AtomicTerminationConstraintSet.add 
		 {
		   left = l;
		   right = r;
		   constraint_op = Greater_or_equal
		 }
		 acc))
	   (PairSet.remove node comp)
	   rules_const	
       in
	 ConjTerminationConstraintSet.union
	 (ConjTerminationConstraintSet.add
	    (AtomicTerminationConstraintSet.add 
	       {
		 left = l;
		 right = r;
		 constraint_op = Greater
	       }
	       weak)
	    ConjTerminationConstraintSet.empty
	 )
	   acc
    )
    comp  
    ConjTerminationConstraintSet.empty
;;


let constraints_from_d_graph sigma rules g =
  let rules_constraints = all_rules_weakly_decreasing rules
  and comps = scc_clean_set g in
  let set_list =
    List.fold_left
      (fun acc e -> scp e g acc)
      CompSet.empty
      comps 
  in
    CompSet.fold
      (fun c acc -> 
	 (trans_loop 
	    rules_constraints 
	    c)::acc
      )
      set_list
      []
;;


let constraints_from_graph sigma rules g =
  let rules_constraints = all_rules_weakly_decreasing rules
  and comps = compute_strongly_connected_components false g
  in
(*  if !uses_all_strongly_connected_parts then
    assert false
  else*)
    List.map
      (fun c ->
	 (*i
	   Labelled_graphs.print (get_subgraph g c)
	   (fun (s,t) -> 
	   Gen_terms.print_term sigma Variables.default s;
	   Format.print_string " ; ";
	   Gen_terms.print_term sigma Variables.default t
	   );
	   Format.print_newline();
	 *)
	 transformation_loop rules_constraints (get_subgraph g c))
      comps
;;

let selected_constraints_from_graph sigma hct rules defined_symbols dp = 
  if !uses_all_strongly_connected_parts then
    let g = compute_dep_graph sigma hct defined_symbols dp
    in 
      constraints_from_d_graph sigma rules g
  else
    let g = compute_dependency_graph sigma hct defined_symbols dp
    in
      constraints_from_graph sigma rules g
;;
 

let modular_simple_graph_criterion htrs =
  let all_rules = htrs.htrs_all_rules
  in
  let defined_symbols = compute_defined_symbols all_rules
  and dp = Relative_dp.rdp_of_rules htrs.htrs_new_rules
  in
(*  let g = compute_dependency_graph htrs.htrs_sig htrs.hct defined_symbols dp 
  in 
    constraints_from_graph htrs.htrs_sig all_rules g *)
    selected_constraints_from_graph htrs.htrs_sig htrs.hct all_rules defined_symbols dp
;;
    
let simple_dependency_graph_criterion sigma hct r =
  let defined_symbols = compute_defined_symbols r 
  in
  let dp = compute_dependency_pairs defined_symbols r
  in
(*i
    List.iter
      (fun (s,t) ->
	  print_string "paire ";
	 print_term sigma (new Variables.from_string "") s;
	 print_string " ; ";
	 print_term sigma (new Variables.from_string "") t;
	 print_newline())
      dp;
*)
(*        
  let g = compute_dependency_graph sigma defined_symbols dp 
  in
    constraints_from_graph sigma r g*)
    selected_constraints_from_graph sigma hct r defined_symbols dp
;;


let dependency_graph_criterion_with_markings sigma hct marked_hct r = 
  let sigma_marked = signature_with_marks sigma
  and defined_symbols = compute_defined_symbols r 
  in
    
  let dp = compute_dependency_pairs_with_markings sigma hct marked_hct defined_symbols r
  and r' = (List.map (Marked_dp_criteria.copy_rule marked_hct) r) 
	   @ (remove_markings_rules sigma marked_hct defined_symbols)
  in
  let defined_marked_symbols =
    SymbolSet.fold 
      (fun f s ->
	 SymbolSet.add (Original f) s)
      defined_symbols
      SymbolSet.empty
  in
(*  let g = compute_dependency_graph sigma_marked defined_marked_symbols dp 
  in
    constraints_from_graph sigma_marked r' g*)
    selected_constraints_from_graph sigma_marked marked_hct r' defined_marked_symbols dp
;;


let modular_marked_graph_criterion htrs sigma_dupl =
  let all_rules = htrs.htrs_all_rules
  in
  let defined_symbols = compute_defined_symbols all_rules
  and mdp = 
    Relative_dp.marked_rdp_of_rules htrs.htrs_sig htrs.hct htrs.marked_hct
      htrs.htrs_new_rules
  and r' = (List.map (Marked_dp_criteria.copy_rule htrs.marked_hct) all_rules) 
 in  
  let defined_marked_symbols =
    SymbolSet.fold 
      (fun f s ->
	 SymbolSet.add (Original f) s)
      defined_symbols
      SymbolSet.empty
  in
(*
  let g = compute_dependency_graph sigma_marked defined_marked_symbols mdp 
  in constraints_from_graph sigma_marked r' g*)
    selected_constraints_from_graph sigma_dupl htrs.marked_hct r' defined_marked_symbols mdp 
;;
