(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: graph_criteria.mli,v 1.8 2002/07/01 12:01:36 contejea Exp $

***************************************************************************)

open Signatures
open Variables
open Gen_terms
open Rewrite_rules
open Dependency_pairs_criteria
open Marked_dp_criteria
open Hierarchical_trs

(*

\subsection{Dependency graph criteria}

The dependency graph is improvement of both previous criteria. The
dependency graph is made as follows :

\begin{itemize}

\item the vertices are dependency pairs ;

\item there is an arc from $(s,t)$ to $(s',t')$ if $t$ may rewrite to
$s'$ by $R$ (resp. $R'$ for dependency pairs with markings) without
steps at root.

\end{itemize}

\subsubsection{Approximated graph}

This last property is not decidable in general, so we compute an
approximation of the graph which surely contains the dependency graph.

An approximation is that there is an arc from $(s,t)$ to $(s',t')$ if
$cap(t)$ and $s'$ unify, where $cap(t)$ is the top part of $t$
containing only the root and all constructors symbols below the
root. (Remember that each leaf of $cap(t)$ is a new variable). Notice
that in the current implementation of that approximation, commutative
or AC symbols are also identified.

\subsubsection{Graph criteria}

There are two graph criteria currently implemented. The first one is
\begin{itemize}

\item each rule weakly decreases ;

\item on each cycle of the graph, each pair weakly decreses and at
least one strictly decreases.

\end{itemize}

and the second is the following, stronger but more complex to compute.

\begin{itemize}

\item each rule weakly decreases ;

\item for each part of the set of vertices, that is strongly connected
(there is a path from each vertex to each other vertex), each pair
weakly decreses and at least one strictly decreases.

\end{itemize}

The variable [uses_all_strongly_connected_parts] controls which
criterion is used. Default is [false] which means that the first
criterion is used.

*)

val uses_all_strongly_connected_parts : bool ref;;


val compute_dependency_graph :
  'symbol #Signatures.signature -> 'symbol Gen_terms.HashTerm.t ->
    'symbol Signatures.SymbolSet.t -> 
      ('symbol Gen_terms.term * 'symbol Gen_terms.term) list ->
	('symbol Gen_terms.term * 'symbol Gen_terms.term) Labelled_graphs.graph 
;;

val simple_dependency_graph_criterion :
  'symbol #Signatures.signature -> 'symbol Gen_terms.HashTerm.t ->
    'symbol Rewrite_rules.t list ->
(*i
      ('symbol Gen_terms.term * 'symbol Gen_terms.term)
      Labelled_graphs.graph *
i*)
      'symbol Termination_constraints.ConjTerminationConstraintSet.t list
;;


val dependency_graph_criterion_with_markings :
  'symbol #Signatures.signature -> 'symbol Gen_terms.HashTerm.t ->
    'symbol dupl Gen_terms.HashTerm.t -> 'symbol Rewrite_rules.t list ->
(*i
      ('symbol Marked_dp_criteria.dupl Gen_terms.term *
       'symbol Marked_dp_criteria.dupl Gen_terms.term)
      Labelled_graphs.graph *
i*)
      'symbol Marked_dp_criteria.dupl 
	Termination_constraints.ConjTerminationConstraintSet.t list
;;

val modular_simple_graph_criterion :
  Hierarchical_trs.htrs ->
(*i
    (Hierarchical_signatures.hsymbol Gen_terms.term * 
     Hierarchical_signatures.hsymbol Gen_terms.term) Labelled_graphs.graph *
i*)
    Hierarchical_signatures.hsymbol 
      Termination_constraints.ConjTerminationConstraintSet.t list
;;


val modular_marked_graph_criterion :
  Hierarchical_trs.htrs ->
    Hierarchical_signatures.hsymbol Marked_dp_criteria.dupl #Signatures.signature ->
	Hierarchical_signatures.hsymbol Marked_dp_criteria.dupl
	  Termination_constraints.ConjTerminationConstraintSet.t list
;;
    

