(***************************************************************************

Inclusion

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: include.ml,v 1.1 2000/05/16 17:38:34 monate Exp $

***************************************************************************)


(*

  modules utilitaires de bases

*)

#directory "..";;

#load "args.cmo";;
#load "messages.cmo";;
#load "orderings.cmo";;
open Format;;

#load "../compat/libutils.cma";; 

(*

  modules des termes 

*)

#directory "../terms";;
#load "libterms.cma";;

open Gen_terms;;

(*

  contraintes d'existence d'ordre

*)

#load "ordering_existence_constraints.cmo";;
open Ordering_existence_constraints;;

let print_ordering_existence_constraints sigma vars constr =
  List.iter
    (fun c ->
       Gen_terms.print_term sigma vars c.left;
       print_string (match c.constraint_op with
			 Greater -> " > "
		       | Equal -> " = "
		       | Greater_or_equal -> " >= ");
       Gen_terms.print_term sigma vars c.right;
       print_string ";";
       force_newline()) 
    constr
;;
  
(*

  critere de terminaison standard

*)

(*
#load "basic_criterion.cmo";;
*)

#load "dependency_pairs_criteria.cmo";;
open Dependency_pairs_criteria;;
(*

  module des polynomes generiques

*)

(*
#load "../term_orderings/poly_interp.cmo";;
*)
#directory "../integer_solver";;
#load "finite_domains.cmo";;

let print_fd_solution sol =
  Finite_domains.Fd_var_map.iter
    (fun x n ->
       print_string (Finite_domains.string_of_fd_var_id x);
       print_string " = ";
       print_string (Num.string_of_num n);
       print_string ";";
       force_newline())
    sol
;;

#install_printer print_fd_solution;;

#load "non_linear_solving.cmo";;

let print_non_linear_constraints c =
  List.iter 
    (fun c ->
       Non_linear_solving.print_constraint c;
       print_string ";";
       force_newline())
    c;;

#install_printer print_non_linear_constraints;;

#load "generic_polynomials.cmo";;

let print_interp sigma interp =
  Signatures.SymbolMap.iter
    (fun f p ->
       print_string (sigma#string_of_symbol f);
       print_string " -> ";
       Generic_polynomials.print_generic_polynomial p;
       print_string ";";
       force_newline())
    interp;;



(*
  
  [(time f x)] is [(f x)] but also show the execution time

*)

let time f x =
  let time_before = Unix.times ()
  in
  let show_time () =
    let time_after = Unix.times ()
    in
      print_string "Execution time : ";
      print_float (time_after.Unix.tms_utime -. time_before.Unix.tms_utime);
      print_string " sec.";
      print_newline()
      	
  in
    try
      let res = f x
      in show_time();res
    with e -> show_time();raise e      
;;

	  





(*

  definition de quelques symboles de fonctions

*)
open User_signatures;;
open Rewrite_rules;;
open User_terms;;

let my_sig = 
  new user_signature 
    "0 : constant;
     s : unary;
     m,p : binary;
     +,* : AC;";;

open Variables;;

let my_vars =
  new user_variables
    ["x";"y";"z"];;


let my_ordering_existence_constraint_printer = 
  print_ordering_existence_constraints my_sig my_vars;;

#install_printer my_ordering_existence_constraint_printer;;

let my_print_rewrite_rule_set = print_rewrite_rule_set my_sig my_vars;;

#install_printer my_print_rewrite_rule_set;;



(*

  quelques exemples de systemes de reecriture

*)


let peano =
  rule_set_of_string my_sig my_vars
    "p(x,0) -> x;
     p(x,s(y)) -> s(p(x,y));
     m(x,0) -> 0;
     m(x,s(y)) -> p(m(x,y),x);"
;;

let peano_add =
  rule_set_of_string my_sig my_vars
    "p(x,0) -> x;
     p(x,s(y)) -> s(p(x,y));
"
;;

let peano_add_ac =
  rule_set_of_string my_sig my_vars
     "x+0 -> x;
      s(x)+s(y) -> s(s(x+y));"
;;

let peano_ac = peano_add_ac @
  (rule_set_of_string my_sig my_vars
     "x * 0 -> 0;
      s(x) * s(y) -> (x*y) + x + y + s(0);")
;;

(*
let ord_constr_peano = 
  Basic_criterion.all_decreasing_constraints peano_add_ac;;

let interp,constr_ac,constr_mon,constr2 = 
  Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
    my_sig Signatures.SymbolMap.empty ord_constr_peano;;

 
time 
  (Non_linear_solving.non_linear_solve (Num.Int 10)) 
  (constr_ac@constr_mon@constr2);;
*)

let solve_dp r =
  print_string "Rewrite system : ";
  print_newline();
  my_print_rewrite_rule_set r;
  print_newline();
  print_newline();
  let ord_constr = simple_dependency_pair_criterion r
  in
    print_string "Ordering existence constraints : ";
    print_newline();
    my_ordering_existence_constraint_printer ord_constr;
    print_newline();
    print_newline();
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
      	my_sig ord_constr
    in
    let poly_constr = constr_ac@constr_dp
    in
      print_string "Polynomial ordering constraints : ";
      print_newline();
      print_non_linear_constraints poly_constr;
      print_newline();    
      time 
      	(Non_linear_solving.non_linear_solve (Num.Int 20)) 
      	poly_constr
;;
 

(*
solve_dp peano;;

solve_dp peano_add_ac;;

solve_dp peano_ac;;
*)



let my_dupl_sig = signature_with_marks my_sig;;


let my_dupl_ordering_existence_constraint_printer = 
  print_ordering_existence_constraints my_dupl_sig my_vars;;

#install_printer my_dupl_ordering_existence_constraint_printer;;

let solve_dp_markings r =
  print_string "Rewrite system : ";
  print_newline();
  my_print_rewrite_rule_set r;
  print_newline();
  print_newline();
  let ord_constr = dependency_pair_criterion_with_markings my_sig r
  in
    print_string "Ordering existence constraints : ";
    print_newline();
    my_dupl_ordering_existence_constraint_printer ord_constr;
    print_newline();
    print_newline();
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
      	my_dupl_sig ord_constr
    in
    let poly_constr = constr_ac@constr_dp
    in
      print_string "Polynomial ordering constraints : ";
      print_newline();
      print_non_linear_constraints poly_constr;
      print_newline();    
      time 
      	(Non_linear_solving.non_linear_solve (Num.Int 20)) 
      	poly_constr
;;

(* 
solve_dp_markings peano;;

solve_dp_markings peano_add_ac;;

solve_dp_markings peano_ac;;
*)


let solve_graph_dp r =
  print_string "Rewrite system : ";
  print_newline();
  my_print_rewrite_rule_set r;
  print_newline();
  print_newline();
  let ord_constr = simple_dependency_graph_criterion r
  in
    print_string "Ordering existence constraints : ";
    print_newline();
    my_ordering_existence_constraint_printer ord_constr;
    print_newline();
    print_newline();
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
      	my_sig ord_constr
    in
    let poly_constr = constr_ac@constr_dp
    in
      print_string "Polynomial ordering constraints : ";
      print_newline();
      print_non_linear_constraints poly_constr;
      print_newline();    
      time 
      	(Non_linear_solving.non_linear_solve (Num.Int 20)) 
      	poly_constr
;;
 

(*
solve_graph_dp peano;;

solve_graph_dp peano_add_ac;;

solve_graph_dp peano_ac;;
*)

let solve_graph_dp_markings r =
  print_string "Rewrite system : ";
  print_newline();
  my_print_rewrite_rule_set r;
  print_newline();
  print_newline();
  let ord_constr = dependency_graph_criterion_with_markings my_sig r
  in
    print_string "Ordering existence constraints : ";
    print_newline();
    my_dupl_ordering_existence_constraint_printer ord_constr;
    print_newline();
    print_newline();
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
      	my_dupl_sig ord_constr
    in
    let poly_constr = constr_ac@constr_dp
    in
      print_string "Polynomial ordering constraints : ";
      print_newline();
      print_non_linear_constraints poly_constr;
      print_newline();    
      time 
      	(Non_linear_solving.non_linear_solve (Num.Int 20)) 
      	poly_constr
;;

(* 
solve_graph_dp_markings peano;;

solve_graph_dp_markings peano_add_ac;;

solve_graph_dp_markings peano_ac;;
*)
