(***************************************************************************

Termination : test file #2

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: test2.ml,v 1.1 2000/05/16 17:38:34 monate Exp $

***************************************************************************)

#use "printers.ml";;

(*

  termination of binary/ternary arithmetic

*)

let my_sig = 
  new user_signature 
    "# : constant;
     0,1,j : postfix unary;
     opp : unary;
     - : infix binary;
     +,* : AC;
     abs : unary ;
     test_abs_pos, test_abs_neg : binary;
     signe, test_signe_pos, test_signe_neg : unary;
     min,min',min'' : binary ;
     test_min_pos, test_min_neg : 3 ;
     f : binary ;
     rat, irred : binary ;
     +_Q : infix commutative ;
     *_Q : infix commutative ;
";;


let my_vars =
  new user_variables
    ["x";"y";"z";"u";"v"];;

let my_ordering_existence_constraint_printer = 
  print_ordering_existence_constraints my_sig my_vars;; 

#install_printer my_ordering_existence_constraint_printer;;

let my_print_rewrite_rule_set = print_rewrite_rule_set my_sig my_vars;;

#install_printer my_print_rewrite_rule_set;;


let my_print_interp = Generic_polynomials.print_interp my_sig;;

#install_printer my_print_interp ;;

(*

  Binary arithmetic on natural numbers

*)


let bin =
  rule_set_of_string my_sig my_vars
    "(#)0 -> #;
     # + x -> x;
     (x)0 + (y)0 -> (x+y)0;
     (x)0 + (y)1 -> (x+y)1;
     (x)1 + (y)1 -> (x+y+(#)1)0;
     # * x -> #;
     (x)0 * y -> (x*y)0;
     (x)1 * y -> (x*y)0 + y;
     "
;;

(*

  Balanced ternary arithmetic on integers

*)


let tern =
  rule_set_of_string my_sig my_vars
    "(#)0 -> #;
     # + x -> x;
     (x)0 + (y)0 -> (x+y)0;
     (x)0 + (y)1 -> (x+y)1;
     (x)0 + (y)j -> (x+y)j;
     (x)1 + (y)1 -> (x+y+(#)1)j;
     (x)j + (y)j -> (x+y+(#)j)1;
     (x)1 + (y)j -> (x+y)0;
     opp(#) -> # ;
     opp((x)0) -> (opp(x))0 ;
     opp((x)1) -> (opp(x))j ;
     opp((x)j) -> (opp(x))1 ;
     x - y -> x+opp(y) ;
     # * x -> #;
     (x)0 * y -> (x*y)0;
     (x)1 * y -> (x*y)0 + y;
     (x)j * y -> (x*y)0 - y;
     "
;;


let rat =
  tern @
  rule_set_of_string my_sig my_vars
    " abs(x) -> test_abs_pos(x,x);

  test_abs_pos(#,x) -> x;
  test_abs_pos((x)0,y) -> test_abs_pos(x,y);
  test_abs_pos((x)1,y) -> test_abs_pos(x,y);
  test_abs_pos((x)j,y) -> test_abs_neg(x,y);

  test_abs_neg(#,x) -> opp(x);
  test_abs_neg((x)0,y) -> test_abs_neg(x,y);
  test_abs_neg((x)1,y) -> test_abs_pos(x,y);
  test_abs_neg((x)j,y) -> test_abs_neg(x,y);

  signe(#) -> #;
  signe((x)0) -> signe(x);
  signe((x)1) -> test_signe_pos(x);
  signe((x)j) -> test_signe_neg(x);

  test_signe_pos(#) -> (#)1;
  test_signe_pos((x)0) -> test_signe_pos(x);
  test_signe_pos((x)1) -> test_signe_pos(x);
  test_signe_pos((x)j) -> test_signe_neg(x);

  test_signe_neg(#) -> (#)j;
  test_signe_neg((x)0) -> test_signe_neg(x);
  test_signe_neg((x)1) -> test_signe_pos(x);
  test_signe_neg((x)j) -> test_signe_neg(x);

  min(x,y) -> test_min_pos(abs(y)-abs(x),x,y);
  min'(x,y) -> test_min_pos(abs((y)1)-abs((x)1),x,y);
  min''(x,y) -> test_min_pos(abs((y)j)-abs((x)j),x,y);

  test_min_pos(#,x,y) -> x;
  test_min_pos((x)0,y,z) -> test_min_pos(x,y,z);
  test_min_pos((x)1,y,z) -> test_min_pos(x,y,z);
  test_min_pos((x)j,y,z) -> test_min_neg(x,y,z);

  test_min_neg(#,x,y) -> y;
  test_min_neg((x)0,y,z) -> test_min_neg(x,y,z);
  test_min_neg((x)1,y,z) -> test_min_pos(x,y,z);
  test_min_neg((x)j,y,z) -> test_min_neg(x,y,z);

  f(#,x) -> #;
  f(x,#) -> signe(x);

  f((x)0,(y)0) -> f(x,y);
  f((x)0,(y)1) -> (f(x,(y)1))0;
  f((x)0,(y)j) -> (f(x,(y)j))0;

  f((x)1,(y)0) -> f((x)1,y);

  f((x)1,(y)1) ->
    (f(x - min'(x,y), (y)1))0 + f(min((x)1,(y)1), x - y);

  f((x)1,(y)j) -> 
    (f(x + min''(opp(x),y), (y)j))0 + f(min((x)1,(opp(y))1), x + y);

  f((x)j,(y)0) -> f((x)j,y);

  f((x)j,(y)1) ->
    (f(x + min'(opp(x),y), (y)1))0 + f(min((x)j,(opp(y))j), x + y);

  f((x)j,(y)j) ->
    (f(x - min''(x,y), (y)j))0 + f(min((x)j,(y)j), x - y);

  rat(x,y) -> irred(signe(y)*f(x,y),signe(y)*f(y,x));
 
  irred(x,y) +_Q irred(u,v) -> irred((x*v) + (u*y),y*v);

  irred(x,y) *_Q irred(u,v) -> rat(x*u,y*v);
"
;;

 

let solve_dp r =
  print_string "Rewrite system : ";
  print_newline();
  my_print_rewrite_rule_set r;
  print_newline();
  print_newline();
  let ord_constr = simple_dependency_pair_criterion r
  in
    print_string "Ordering existence constraints : ";
    print_newline();
    my_ordering_existence_constraint_printer ord_constr;
    print_newline();
    print_newline();
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
      	my_sig ord_constr
    in
    let poly_constr = constr_ac@constr_dp
    in
      print_string "Polynomial ordering constraints : ";
      print_newline();
      print_non_linear_constraints poly_constr;
      print_newline();    
      time 
      	(Non_linear_solving.non_linear_solve (Num.Int 20)) 
      	poly_constr
;;
 

(*

solve_dp bin;;

*)



let my_dupl_sig = signature_with_marks my_sig;;


let my_dupl_ordering_existence_constraint_printer = 
  print_ordering_existence_constraints my_dupl_sig my_vars;;

#install_printer my_dupl_ordering_existence_constraint_printer;;

let my_dupl_print_interp = Generic_polynomials.print_interp my_dupl_sig;;

#install_printer my_dupl_print_interp ;;


let solve_dp_markings r =
  print_string "Rewrite system : ";
  print_newline();
  my_print_rewrite_rule_set r;
  print_newline();
  print_newline();
  let ord_constr = dependency_pair_criterion_with_markings my_sig r
  in
    print_string "Ordering existence constraints : ";
    print_newline();
    my_dupl_ordering_existence_constraint_printer ord_constr;
    print_newline();
    print_newline();
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
      	my_dupl_sig ord_constr
    in
    let poly_constr = constr_ac@constr_dp
    in
      print_string "Polynomial ordering constraints : ";
      print_newline();
      print_non_linear_constraints poly_constr;
      print_newline();    
      time 
      	(Non_linear_solving.non_linear_solve (Num.Int 20)) 
      	poly_constr
;;

(* 
solve_dp_markings bin;;
*)


let solve_graph_dp r =
  print_string "Rewrite system : ";
  print_newline();
  my_print_rewrite_rule_set r;
  print_newline();
  print_newline();
  let ord_constr = simple_dependency_graph_criterion r
  in
    print_string "Ordering existence constraints : ";
    print_newline();
    my_ordering_existence_constraint_printer ord_constr;
    print_newline();
    print_newline();
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
      	my_sig ord_constr
    in
    let poly_constr = constr_ac@constr_dp
    in
      print_string "Polynomial ordering constraints : ";
      print_newline();
      print_non_linear_constraints poly_constr;
      print_newline();    
      time 
      	(Non_linear_solving.non_linear_solve (Num.Int 20)) 
      	poly_constr
;;
 

(*
solve_graph_dp bin;;
*)

let solve_graph_dp_markings r =
  print_string "Rewrite system : ";
  print_newline();
  my_print_rewrite_rule_set r;
  print_newline();
  print_newline();
  let ord_constr = dependency_graph_criterion_with_markings my_sig r
  in
    print_string "Ordering existence constraints : ";
    print_newline();
    my_dupl_ordering_existence_constraint_printer ord_constr;
    print_newline();
    print_newline();
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_orderings_existence_constraints
      	my_dupl_sig ord_constr
    in
    let poly_constr = constr_ac@constr_dp
    in
      print_string "Polynomial ordering constraints : ";
      print_newline();
      print_non_linear_constraints poly_constr;
      print_newline();    
      let sol =
        try
	  time 
            (Non_linear_solving.non_linear_solve (Num.Int 20)) 
            poly_constr
	with Not_found -> Fd_var_map.empty
      in
      let new_interp = 
	SymbolMap.fold
	(fun f p i ->
	   SymbolMap.add 
	     f (Generic_polynomials.specialize_polynomial p sol) i)
	interp
        SymbolMap.empty
      in new_interp      
;;

(* 
let i = solve_graph_dp_markings bin;;

Execution time : 83.34 sec.
val i :
  (User_signatures.symbol_id Dependency_pairs_criteria.dupl
   Signatures.SymbolMap.key, Generic_polynomials.GenericPolynomials.poly)
  Signatures.SymbolMap.t 
=
[#] = 0;
[0](X0) = X0 + 1;
[1](X0) = X0 + 2;
[+](X0,X1) = X1 + X0;
[*](X0,X1) = X0X1 + X1 + X0;
['+`](X0,X1) = X1 + X0;
['*`](X0,X1) = X0X1 + 2X1 + 2X0 + 2;

*)

(*
let i = solve_graph_dp_markings tern;;

*)
