(***************************************************************************

Main.ml for testing termination criteria

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: main.ml,v 1.20 2001/07/20 10:58:16 marche Exp $

***************************************************************************)


open Format;;
open Orderings_generalities;;

let peano_sig = User_signatures.from_string 
  "0 : constant; s : unary; +,* : AC;";;

let vars = Variables.from_string "x y z";;

let peano = User_terms.rule_set_of_string peano_sig vars 
"
x+0 -> x;
s(x)+s(y) -> s(s(x+y));
x*0 -> 0;
s(x)*s(y) -> s((x*y) + x + y);
";;


printf "=== Test sans les marques ===@.";;
Termination_expert.standard_criterion := false;;
Termination_expert.dp_criterion_uses_markings := false;;
Termination_expert.dp_criterion_uses_graph := false;;


let constr = 
  Termination_expert.compute_termination_constraints_without_marks 
    peano_sig vars peano;;

let prec = User_terms.prec_of_string peano_sig "* > + > s > 0"
;;
  
let status f = if peano_sig#is_ac f then Rpo.Multiset else Rpo.Lr_lexico;;

let order = Acrpo.acrpo peano_sig prec status;;

Termination_constraints.check_conj_constraints peano_sig vars constr order;;


printf "=== Test avec les marques ===@.";;
Termination_expert.dp_criterion_uses_markings := true;;

let constr2 =
  Termination_expert.compute_termination_constraints_with_marks 
    peano_sig vars peano;;

let peano_sig_dupl = Marked_dp_criteria.signature_with_marks peano_sig;;

let mult = peano_sig#symbol_of_string "*"
and plus = peano_sig#symbol_of_string "+"
and succ = peano_sig#symbol_of_string "s"
and zero = peano_sig#symbol_of_string "0";;

let mult = Marked_dp_criteria.Original mult
and plus = Marked_dp_criteria.Original plus
and succ = Marked_dp_criteria.Original succ
and zero = Marked_dp_criteria.Original zero
and multm = Marked_dp_criteria.Marked mult
and plusm = Marked_dp_criteria.Marked plus;;

open Finite_orderings;;

let prec = 
  let order = 
    Finite_orderings.identity_ordering 
      (compare : User_signatures.symbol_id Marked_dp_criteria.dupl -> 
	 User_signatures.symbol_id Marked_dp_criteria.dupl -> int)
  in 
  Finite_orderings. add_list order (Gt(mult,Gt(plus,Gt(succ,One(zero)))));
  Finite_orderings. add_list order (Gt(plusm,One(plus)));
  Finite_orderings. add_list order (Gt(multm,One(mult)));
  order#compare
;;

let status f = if peano_sig_dupl#is_ac f then Rpo.Multiset else Rpo.Lr_lexico;;

let order = Acrpo.acrpo peano_sig_dupl prec status;;

Termination_constraints.check_conj_constraints peano_sig_dupl vars constr2 order;;


open Signatures;;
open Variables;;
open Gen_terms;;
open User_terms;;
open Rewrite_rules;;
open Format;;
open Hierarchical_trs;;
open Graph;;
open Relative_dp;; 
open Minimal_split;;

(* Here come the graphs again, falling in the code like a tragedy... ----

module Symgraph = Graph.Make(SymbolOrd)
;;

let doo_bee graph root set =
  Signatures.SymbolSet.fold
    (fun elem acc -> 
       Symgraph.add_edge 
	 (if Symgraph.is_node acc elem 
	  then acc 
	  else Symgraph.add_node acc elem)
	 root elem)
    set
    graph
;;

let g_from_rule graph rule =
  let lhs = rule.lhs
  in
  let root = Gen_terms.root_symbol lhs
  in
    doo_bee 
      (doo_bee    
	 (if Symgraph.is_node graph root
	  then graph
	  else Symgraph.add_node graph root)
	 root 
	 (Gen_terms.set_of_symbols rule.lhs))
      root 
      (Gen_terms.set_of_symbols rule.rhs)
;;      

(* [graph_from_a_list l] returns a dependency graph of symbols of
    rules in [l] *)

let graph_from_a_list rule_list =
  List.fold_left 
    g_from_rule
    Symgraph.empty
    rule_list
;;

(* [packs_from_a_list l] returns a list of \emph{minimal} packs of
    symbols of rules in [l] according to the hierarchical dependencies. *) 

let packs_from_a_list rule_list =
  Symgraph.scc (graph_from_a_list rule_list)
;;


(* [rules_from_root sym l] returns a pair [(l1,l2)] of list of rules
   such that [l1] contains rules of [l] the root symbol of which is
   [sym] and [l2] being the list of other rules of [l]. *)

let rules_from_root sym rule_list =
  List.fold_left
    (fun (rules_of_sym, rules_left) elem ->
       if Gen_terms.root_symbol elem.lhs = sym
       then (elem::rules_of_sym, rules_left)
       else (rules_of_sym, elem::rules_left)
    )
    ([],[])
    rule_list
;;

let rules_from_roots_in_packs pack rule_list =
  List.fold_left
    (fun (rules,l2) elem -> 
       let (used,left) = rules_from_root elem l2
       in
	 (used@rules,left))
    ([],rule_list)
    pack
;;

(* [packs_and_rules_from_packs rl pl] returs a list of pairs
   [(p_i,r_i); \ldots] where [p_i] is a pack from [pl] and [r_i] a
   list of the corresponding rules in [rl].*)

let packs_and_rules_from_packs rule_list pack_list =
  fst (
    List.fold_left
	 (fun (packs_and_rules,rules) elem ->
	    let (rules_from_elem,left) = rules_from_roots_in_packs elem rules
	    in
	      (((elem,rules_from_elem)::packs_and_rules),left))
	 ([],rule_list)
	 pack_list)
;;




(* This graph is presently implemented with neighbours lists. *)

(* [contains sym pack_list] returns a pack of [pack_list] the
   first part of which contains [sym].  *)

(* PACKS ARE DISJOINT. *)
let contains sym pack_list =
  List.find
    (fun (sl,l) -> List.mem sym sl)
    pack_list
;;
(*i List.fold_left (fun acc ((sl,l) as elem) -> if List.mem sym sl 
   then elem::acc else acc) [] pack_list;; *)


  (* [depends sym sym_graph pack_list] returns a list of packs from
  [pack_list] of which symbol [sym] depends according to hierarchy
  graph of symbols [sym_graph]. *)

let depends orig_acc sym sym_graph pack_list =
  List.fold_left
    (fun acc elem -> let new_pack =  contains elem pack_list in
       if List.mem new_pack acc then acc else new_pack::acc)
    orig_acc
    (Symgraph.neighbours_list sym_graph sym)
;;


let neighbours_of_a_pr sym_graph pr_list (sl,_) =
  List.fold_left
    (fun acc elem ->
       depends acc elem sym_graph pr_list
    )
    []
    sl
;;

let clean_neighbours n_list (a1,_) = 
  List.fold_left
    (fun acc ((a2,_) as elem) -> if a1 = a2 then acc else elem::acc)
    []
    n_list
;;


let pack_graph sym_graph pr_list =
  List.fold_left
    (fun acc pr ->
       (pr,(neighbours_of_a_pr sym_graph pr_list pr))::acc)
    []
    pr_list
;;

(*--------------------------------------------------------------------*)

open Signature_syntax;;

let string_of_term sgn vars t = 

  let rec string_of_term_rec is_deep = function
      Term(f,[]) ->
      	sgn#string_of_symbol f
    | Term(f,t::l) ->
	(
	  match sgn#symbol_fix f with
	      Default | Prefix ->
      	      	  (sgn#string_of_symbol f)^"("^
      		  (string_of_term_rec false t)^
      		  (string_of_end_term_list l)
	    | Postfix ->
      		  "("^
      		  (string_of_term_rec false t)^
      		  (string_of_end_term_list l)^
      	      	  (sgn#string_of_symbol f)
	    | Infix ->
		  (if is_deep then "(" else "")^
		  (string_of_term_rec true t)^
      		  (string_of_infix_list f l)^
		  (if is_deep then ")" else "")
	)
    | Var(x) ->
	(vars#string_of_var x)
	  
  and string_of_end_term_list = function
      [] -> 
      	")"
    | t::l ->
      	","^
      	(string_of_term_rec false t)^
      	(string_of_end_term_list l)
	  
  and string_of_infix_list f = function
      [] -> ""
    | t::l ->
	" "^
      	(sgn#string_of_symbol f)^
	" "^
      	(string_of_term_rec true t)^
      	(string_of_infix_list f l)
	  
  in
    string_of_term_rec false t
;;

let string_of_arity  = function
    Prefix -> "prefix" 
  | Infix -> "infix"
  | Postfix -> "postfix"
  | Default -> ""
;;

let string_of_pack sgn p =
  List.fold_left
    (fun acc sym -> 
       (sgn#string_of_symbol sym)^" : "^(string_of_arity (sgn#symbol_fix(sym)))
       ^" "^(string_of_int(sgn#arity(sym)))^" ; "^acc)
    ""
    p
;;

let string_of_rewrite_rule sgn vars r =
  (string_of_term sgn vars r.lhs)^" ->"^(string_of_term sgn vars r.rhs)^"; "
;;

let string_of_rewrite_rule_set sgn vars rset =
  List.fold_left
    (fun acc r -> (string_of_rewrite_rule sgn vars r)^acc) 
    ""
    rset
;;

module OrdSymList = 
struct
  type 'a t =  'a SymbolOrd.t list
  let compare = compare
end
;;
  
module PackMap = Ordered_maps.MakePoly(OrdSymList)
;;

(* SI ON VEUT CALCULER LES MODULES D'UNE LISTE DE PACKS, IL FAUT
   ENCORE LE CODER DE MANIRE  NE PASSER QU'UN PR_GRAPH NON
   REFLEXIF. PLUS EFFICACE SI KEY = SYMBOLE PLUTT QUE LISTE ? *)
(* RCURSIF TERMINAL AVEC UN TRY SOME ? *)

(* [module_from_pr sym_graph pr_list sgn vars deja_vu pr] returns a pair
   [(m,mmap)] consisting of a module [m] together with a map [mmap] from 
   packs to their corresponding module.*)

let rec module_from_pr sym_graph pr_list sgn vars deja_vu ((p,r)as pr) =
  try ((PackMap.find p deja_vu), deja_vu)
  with Not_found ->
    let pack_neigh = (clean_neighbours 
			(neighbours_of_a_pr sym_graph pr_list pr) pr)
    in let (mod_neigh,new_dv) = (****)
	List.fold_left
	  (fun (acc,dv) ((l,r) as elem) ->
	     try ((PackMap.find l dv)::acc, dv)
	     with Not_found -> 
	       let (new_mod,ndv) = (****)
		 module_from_pr sym_graph pr_list sgn vars dv elem
	       in
		 (new_mod::acc, PackMap.add l new_mod ndv)) (****)
	  ([],deja_vu)
	  pack_neigh
       and mod_sgn = User_signatures.from_string (string_of_pack sgn p)  
       and mod_rules = string_of_rewrite_rule_set sgn vars r
       in
	 ((extend mod_neigh 
	     mod_sgn 
	     vars 
	     mod_rules)
	    ,new_dv)(****)
;;

let mod_map_from_list sym_graph sgn vars pr_list =
    List.fold_left
      (fun acc elem ->
	 snd (module_from_pr 
		sym_graph pr_list sgn vars 
		acc
		elem))
      PackMap.empty
      pr_list
;;
*)
(* let get_mod_of_sym_id id *)
  

(* TESTS ------------------------------------------------------------*)

let var_claves = Variables.from_string "L1 L2 L3 L4 L5 L6 X Y";;

let sig_claves = User_signatures.from_string 
"0, gt'ii'out, le'ii'out, nil : constant;
merge'iio'out, mergesort'io'in, mergesort'io'out, s, split'ioo'in,
split0'ioo'in, split1'ioo'in, split2'ioo'in, u'1'1, u'2'1, u'5'1, 
u'5'4, u'6'1, u'7'1, u'8'1 : unary;
cons, gt'ii'in, le'ii'in, merge'iio'in, split'ioo'out, split0'ioo'out,
 split1'ioo'out, split2'ioo'out, u'3'2, u'4'2, u'5'2, u'5'3 : binary;
u'9'1 : 3;
u'3'1, u'4'1 : 5;";;

let module_claves = extend []  sig_claves var_claves
"gt'ii'in(s(X), s(Y)) -> u'1'1(gt'ii'in(X, Y));
u'1'1(gt'ii'out) -> gt'ii'out;
gt'ii'in(s(X), 0) -> gt'ii'out;
le'ii'in(s(X), s(Y)) -> u'2'1(le'ii'in(X, Y));
u'2'1(le'ii'out) -> le'ii'out;
le'ii'in(0, s(Y)) -> le'ii'out;
le'ii'in(0, 0) -> le'ii'out;
merge'iio'in(nil, L1) -> merge'iio'out(L1);
merge'iio'in(L1, nil) -> merge'iio'out(L1);
merge'iio'in(cons(X, L1), cons(Y, L2)) -> u'3'1(le'ii'in(X, Y), X, L1, Y, L2);
u'3'1(le'ii'out, X, L1, Y, L2) -> u'3'2(merge'iio'in(L1, cons(Y, L2)), X);
u'3'2(merge'iio'out(L3), X) -> merge'iio'out(cons(X, L3));
merge'iio'in(cons(X, L1), cons(Y, L2)) -> u'4'1(gt'ii'in(X, Y), X, L1, Y, L2);
u'4'1(gt'ii'out, X, L1, Y, L2) -> u'4'2(merge'iio'in(cons(X, L1), L2), Y);
u'4'2(merge'iio'out(L3), Y) -> merge'iio'out(cons(Y, L3));
mergesort'io'in(nil) -> mergesort'io'out(nil);
mergesort'io'in(cons(X, nil)) -> mergesort'io'out(cons(X, nil));
mergesort'io'in(cons(X, cons(Y, L1))) -> u'5'1(split2'ioo'in(cons(X, cons(Y, L1))));
u'5'1(split2'ioo'out(L3, L4)) -> u'5'2(mergesort'io'in(L3), L4);
u'5'2(mergesort'io'out(L5), L4) -> u'5'3(mergesort'io'in(L4), L5);
u'5'3(mergesort'io'out(L6), L5) -> u'5'4(merge'iio'in(L5, L6));
u'5'4(merge'iio'out(L2)) -> mergesort'io'out(L2);
split'ioo'in(L1) -> u'6'1(split0'ioo'in(L1));
u'6'1(split0'ioo'out(L2, L3)) -> split'ioo'out(L2, L3);
split'ioo'in(L1) -> u'7'1(split1'ioo'in(L1));
u'7'1(split1'ioo'out(L2, L3)) -> split'ioo'out(L2, L3);
split'ioo'in(L1) -> u'8'1(split2'ioo'in(L1));
u'8'1(split2'ioo'out(L2, L3)) -> split'ioo'out(L2, L3);
split0'ioo'in(nil) -> split0'ioo'out(nil, nil);
split1'ioo'in(cons(X, nil)) -> split1'ioo'out(cons(X, nil), nil);
split2'ioo'in(cons(X, cons(Y, L1))) -> u'9'1(split'ioo'in(L1), X, Y);
u'9'1(split'ioo'out(L2, L3), X, Y) -> split2'ioo'out(cons(X, L2), cons(Y, L3));";;



let my_vars = Variables.from_string "x y z";;

(*let my_sig = User_signatures.from_string 
  "+ : AC ; 
   0 : constant ; 
   - : unary ;";;

let r = rule_set_of_string my_sig my_vars 
  "x+0 -> x;
   x+-(x) -> 0;
   -(0) -> 0;
   -(-(x)) -> x;
   -(x+y) -> -(x)+-(y);";;
*)
let sig_nat = User_signatures.from_string "# : constant ; 0,1 : postfix unary ;";;

let module_nat = 
  extend [] sig_nat my_vars "(#)0 -> #;";;
    
let sig_plus = User_signatures.from_string "+ : infix binary;";;

let module_plus = 
  extend [module_nat] sig_plus my_vars 
    " x + # -> x ; 
      # + x -> x; 
      (x)0 + (y)0 -> (x + y)0;
      (x)0 + (y)1 -> (x + y)1;
      (x)1 + (y)0 -> (x + y)1;
      (x)1 + (y)1 -> ((x + y) + (#)1)0;
      x + (y + z) -> (x + y) + z;";;

let sig_minus = User_signatures.from_string "- : infix binary;";;

let module_minus = 
  extend [module_nat] sig_minus my_vars
    " x - # -> x; 
      # - x -> #; 
      (x)0 - (y)0 -> (x - y)0;
      (x)0 - (y)1 -> ((x - y) - (#)1)1;
      (x)1 - (y)0 -> (x - y)1;
      (x)1 - (y)1 -> (x - y)0;";;

let sig_bool = User_signatures.from_string 
		 "true, false : constant; not : unary; if : 3;";;

let module_bool = extend [] sig_bool my_vars
		    "  not(false) -> true ; not(true) -> false ; if(true,x,y) -> x ; if(false,x,y) -> y;";;

let sig_ge = User_signatures.from_string
	       "ge: binary;";;

let module_ge = 
  extend [module_bool;module_nat] sig_ge my_vars
    " ge((x)0,(y)0) -> ge(x,y);
      ge((x)0,(y)1) -> not(ge(y,x));
      ge((x)1,(y)0) -> ge(x,y);
      ge((x)1,(y)1) -> ge(x,y);
      ge(x,#)   -> true;
      ge(#,(x)1)  -> false;
      ge(#,(x)0)  -> ge(#,x);";;

let sig_log' = User_signatures.from_string
		 "log' : unary;";;

let module_log' = 
  extend [module_ge; module_plus] sig_log' my_vars
    " log'(#) -> #;
      log'((x)0) -> if(ge(x,(#)1),log'(x) + (#)1,#);
      log'((x)1) -> log'(x) + (#)1;";;

let sig_log = User_signatures.from_string
		"log : unary;";;

let module_log = 
  extend [module_log';module_minus] sig_log my_vars
    " log(x) -> log'(x) - (#)1;";;

(*--------------------------------------------------*)
(*
let g_log = graph_from_a_list module_log.htrs_all_rules;;

(*
let print_edge trs graph =  
  List.iter
    (fun (x,y) -> let a = trs.htrs_sig#string_of_symbol x
		  and b = trs.htrs_sig#string_of_symbol y
		  in 
		    print_string ("[ "^a^" ---> "^b^" ]\n")
    )
    (Symgraph.edge_list graph)
    ;
  flush stdout
;;
*)

let packs_log = packs_from_a_list module_log.htrs_all_rules
;;
*)
let packs_claves = packs_from_a_list module_claves.htrs_all_rules
;;
(*
let packs_minus = packs_from_a_list module_minus.htrs_all_rules
;;
*)

let print_pack sgn l =
  List.iter 
    (fun x -> 
       begin 
	 print_string ((sgn#string_of_symbol x)^" ; ")
       end ) 
    l ;;
  
(*
begin
  
  print_newline() ;
  print_edge module_toto g_toto ;
  print_newline() ;

 
  List.iter
    (fun l -> begin
       List.iter 
	 (fun x -> 
	    begin 
	      print_string ((module_toto.htrs_sig#string_of_symbol x)^" ; ")
	    end ) 
	 l ;
     print_newline()
     end)
    (packs_toto)
    ;
  flush stdout
end
;;
*)


let pre_modules = 
  packs_and_rules_from_packs module_claves.htrs_all_rules packs_claves 
;;
(*
let pre_modules_minus =
  packs_and_rules_from_packs module_minus.htrs_all_rules packs_minus 
*)

let print_pack_list l =
List.iter
  (fun (a,b) -> 
     begin
       print_newline();
       print_string "=====================";  print_newline();
       print_pack module_claves.htrs_sig a;  print_newline();
       print_string "---------------------";  print_newline();
       print_rewrite_rule_set module_claves.htrs_sig var_claves b; print_newline();
       print_string "=====================";  print_newline();
       print_newline()
     end)
  l
;;

print_pack_list pre_modules;;

(*
List.iter
  (fun (p,pl) ->
     begin
       print_newline();
       print_string "##############################";  print_newline();
       print_pack module_log.htrs_sig (fst p);  print_newline();
       print_pack_list pl;
       print_newline()
     end)
  (pack_graph g_log pre_modules) 
;;  

let s_p =
  let p = fst (List.hd pre_modules) in
    (*print_string*) (string_of_pack module_log.htrs_sig p)
    (* ; print_newline()*)
;;

let sig_p = User_signatures.from_string s_p
;;


print_string "***   Modules minimaux   ***";;
print_newline();;
List.iter 
  (fun a-> 
     begin 
       print_rewrite_rule_set a.htrs_sig my_vars  a.htrs_new_rules ; 
       print_newline();
       print_string "###"; 
       print_newline() 
     end)
  (*(List.map (module_from_pr g_toto pre_modules module_toto.htrs_sig my_vars 
	       (**)PackMap.empty(**)) pre_modules)*)
   (fst(List.fold_left
    (fun (acc,dv) elem ->
       let (a1,b1) = (module_from_pr 
		      g_log pre_modules module_log.htrs_sig my_vars 
		      dv 
		      elem)
       in
	 (a1::acc,b1))
    ([],PackMap.empty)
    pre_modules))
;;
print_newline();;
*)



(*
let defined_sym_list rules =
  fst
    (List.fold_left
       (fun ((acc,dv) as accu) elem -> 
	  let e = Gen_terms.root_symbol elem.lhs
	  in 
	    if SymbolSet.mem e dv 
	    then accu
	    else (e::acc,SymbolSet.add e dv)) 
       ([],SymbolSet.empty)
       rules)
;;
*)
let defined_sym_set rules =
    (List.fold_left
       (fun dv elem -> 
	  let e = Gen_terms.root_symbol elem.lhs
	  in 
	    if SymbolSet.mem e dv 
	    then dv
	    else SymbolSet.add e dv) 
       SymbolSet.empty
       rules)
;;
(*
let defined2 rules =
  (List.fold_left
     (fun acc e -> SymbolSet.add e acc) 
	SymbolSet.empty
	rules)
;;

let r = (module_toto.htrs_all_rules)
in
  for i=1 to 100000 do
    defined_sym_set r
  done
;;    
*)
(*
print_newline();;
print_string "***   Contraintes   ***";;
print_newline();;

  (Termination_constraints.print_conj_termination_constraints
     module_log.htrs_sig
     my_vars)
  (relative_dp_criterion 
     module_log.htrs_new_rules 
     module_log.htrs_all_rules);;

print_newline();;
print_string "***   Contraintes de la dcomposition minimale ***";;
print_newline();;



(* Ce n'est pas exactement ce qu'on veut mais c'est dj un dbut. *)


List.iter
(fun modu -> 
   begin
     print_string "~~~~~~~"; print_newline();
     (Termination_constraints.print_conj_termination_constraints
     modu.htrs_sig
     my_vars
  (relative_dp_criterion 
     modu.htrs_new_rules 
     modu.htrs_all_rules))
   ;print_newline()
   end)

(fst(List.fold_left
    (fun (acc,dv) elem ->
       let (a1,b1) = (module_from_pr 
			g_log pre_modules module_log.htrs_sig my_vars 
			dv 
			elem)
       in
	 (a1::acc,b1))
    ([],PackMap.empty)
    pre_modules))
;;
*)
open Termination_expert;;
open Termination_constraints;;

let display_solution sol =
    begin
      print_string "Solution:";
      print_newline();
      Finite_domains.print_fd_solution sol;
      print_newline()
    end
;;

let build_new_interp sigma interp sol =
  SymbolMap.fold
  (fun f p i ->
     SymbolMap.add 
       f (Generic_polynomials.specialize_polynomial p sol) i)
  interp
  SymbolMap.empty
;;

let display_conj_termination_constraints sigma vars c =
    begin
      print_string "Termination constraints:";
      print_newline();
      Termination_constraints.print_conj_termination_constraints 
    	sigma vars c;
      print_string "(";
      print_int (Termination_constraints.AtomicTerminationConstraintSet.cardinal c);
      print_string " termination constraints)";
      print_newline();
      print_newline();
    end
;;


let solve_htrs my_htrs vars bound=
  let sigma = Marked_dp_criteria.signature_with_marks (my_htrs.htrs_sig)
  in
  let c =
    marked_relative_dp_criterion my_htrs
  in
    (**)display_conj_termination_constraints sigma vars c;(**)
    let interp,constr_ac,constr_mon,constr_dp = 
      Generic_polynomials.polynomial_constraints_of_termination_constraints
	sigma SymbolMap.empty c
    in
     (* display_interp sigma interp;*)
      let poly_constr = constr_ac@constr_dp
      in
	(*display_polynomial_constraints poly_constr;*)
	let sol =
	  Non_linear_solving.non_linear_solve 
	    (Num.Int bound(*!bound*)) 
	    poly_constr
	in
	  display_solution sol; my_htrs.proved_c_e_terminating := true;
	  let new_interp = build_new_interp sigma interp sol
	  in
	    Generic_polynomials.print_interp sigma new_interp      
;;


let rec solve_for_module my_htrs my_vars bounds =
(*  let my_all = my_htrs.htrs_all_rules
  in
  let g_htrs = graph_from_a_list my_all
  and pre_mod_htrs = 
    packs_and_rules_from_packs my_all (packs_from_a_list my_all) 
  in*)
  solve_htrs my_htrs my_vars bounds;
    List.iter
      (fun m -> 
	 begin 
	   if !(m.proved_c_e_terminating) 
	   then 
	     begin
	       print_string "CE-termination of";
	       print_rewrite_rule_set m.htrs_sig my_vars m.htrs_all_rules;
	       print_string "is already proven.";
	       print_newline()
	     end
	   else 
	     begin 
	       solve_for_module m my_vars bounds 
	     end ; 
	   print_newline() end)
     (my_htrs.htrs_imported)
(*
      (minimal_split my_htrs.htrs_sig my_vars my_htrs)
*)
;;


  

let rec defined_subterms defined_symbols = function
    Var(_) -> []
  | Term(f,l) as t ->
      let others = 
	Listutils.flat_map (defined_subterms defined_symbols) l
      in
      	if SymbolSet.mem f defined_symbols
      	then t::others
	else others
;;


let rdp_of_a_rule rule defined_set = 
  let lhs = rule.lhs
  in
    List.map
      (fun e -> (lhs,e))
      (defined_subterms defined_set rule.rhs)      
;;
  
let rdp_of_rules rules =
  let defined_symbols = defined_sym_set rules
  in
    List.fold_left
      (fun acc1 rule -> 
	 let lhs = rule.lhs
	 and defined_sub = defined_subterms defined_symbols rule.rhs
	 in
	   List.fold_left 
	     (fun acc2 sub -> (lhs,sub)::acc2)
	     acc1
	     defined_sub)
      []
      rules
;;

(**)
let dp_claves =
Graph_criteria.compute_dependency_graph 
  module_claves.htrs_sig 
  (defined_sym_set module_claves.htrs_new_rules)
  (rdp_of_rules module_claves.htrs_new_rules) 
;;
(**)

(*
Generic_polynomials.use_linear_interp := false;
Generic_polynomials.use_degree_two_interp := true;
*)
(*solve_for_module module_log my_vars 2;; *)

(*solve_for_module module_claves var_claves 2;;*)
(*solve_for_module module_claves var_claves 1;;*)



