(***************************************************************************

Marked dependency pairs criteria, description & specification

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: marked_dp_criteria.ml,v 1.13 2002/07/11 13:26:15 pguillon Exp $

***************************************************************************)

(*

Obsolete header:



***************************************************************************)
open Signatures;;
open Variables;;
open Gen_terms;;
open Substitution;;

open Rewrite_rules;;

open Termination_constraints;;

open Dependency_pairs_criteria;;


(*##########################################################################

                     Generating splitting substitutions

##########################################################################*)

(* 

   [(var_multiplicities t)] returns an association table from variables of [t]
   to their multiplicity.

*)


(* Actual counting. *)
let compte l = 
  List.fold_left 
    (fun acc x -> 
       match x.Hcons.node with
	   Var id -> 
	     VarMap.add 
	       id 
	       (try (VarMap.find id acc) + 1 with Not_found -> 1) 
	       acc
	 | _ -> acc)
    VarMap.empty
    l

let var_multiplicities t = 
  match t.Hcons.node with
    | Var _ -> invalid_arg "var_multiplicities"
    | Term(_,l) -> 
	let cmp = compte l 
	in VarSet.fold 
	     (fun x acc -> 
		try 
		  let _ = VarMap.find x acc in
		    acc
		with Not_found -> VarMap.add x 0 acc) 
	     (set_of_variables t)
	     cmp


(*

   [fresh_var_with_set varset n] returns a pair [(s,l)] where [l] is a
   list of [n] fresh variables as TERMS whose id are not belonging to
   [varset] and [s] is [varset] augmented with elements of [l].

*)

let fresh_var_with_set hct used_var n =
  let rec fresh_acc n  set acc =
    if n == 0 
    then (set,acc)
    else 
      let new_var_id = var_outside_set set
      in
      fresh_acc (n-1) (VarSet.add new_var_id set) ((make_var_term hct new_var_id)::acc)
  in
  fresh_acc n used_var []
;;


(* Same but l is now a list of lists of $[n],[n]-1,\ldots,1]$ fresh
distincts variables. *)

(* [sub_lists [a1;...;an]] is [[an];[an;an-1];...;[a1;...;an]] *)
let sub_lists a =
  let rec s_lists acc = function
      [] -> acc
    |a::r as l -> s_lists (l::acc) r
  in
  s_lists [] a
;; 

let all_fresh hct used_var n =
  let (varset,lvar) = fresh_var_with_set hct used_var n
  in
  (varset, sub_lists lvar)
;;

(* 
   [all_fresh_for_all t] returns for a term [t] a pair consisting of a
   new set of used variables together with a list of list of list of
   fresh variables hereafter used in definition of splitting
   substitutions.  
*)

let all_fresh_for_all hct t =
  let multset = var_multiplicities t
  and varset = set_of_variables t
  in
    (*VarSet*)List.fold_right
    (fun x (used_var,acc) -> 
       let (newvar,l) = all_fresh hct used_var ((VarMap.find x multset) + 1)
       in 
   	 (newvar,l::acc)) 
    ((**)VarSet.elements(**) varset)
    (varset,[])
;;

let compose_elt elt lsub =
  List.map (fun l -> elt::l) lsub
;;

let compose_one lelt lsub =
  List.fold_left (fun acc x -> (compose_elt x lsub) @ acc) [] lelt
;;

(* combining all possible mappings. *)
let compose_all lelt =
  List.fold_right (fun x acc -> (compose_one x acc))  lelt [[]]
;;


(* Adding right keys, building substitutions.*)
let list_of_sigma sign hct (* debug  sigma vars  debug *) t =
  let f = root_symbol t
  and var_list = VarSet.elements (set_of_variables t)
  in
    List.map 
      (fun l ->
	 List.fold_left2 
	   (fun acc id lis -> 
	      VarMap.add id 
		(match lis with 
		     [] -> assert false 
		   | [a] -> a 
		   | _ -> (make_head_flatten_term sign hct f lis)) acc)
	   VarMap.empty
	   var_list
           l)
      (compose_all (snd (all_fresh_for_all hct t)))
;;

(*##########################################################################

                     Generating M.E. normal forms

##########################################################################*)
	   
(*
   multiset are here considered as lists of pairs (id,multiplicity):
   [(x1,3);(x2,1);(x3,2)] represents multiset {x1 x1 x1 x2 x3}.

  I know this is *not* the best way, I will take care of that later so
   don't shout.
*)

(*
  [multiset_of_list l] returns a multiset representation (as defined
   above) of [l].
*)
let multiset_of_list l = 
  let multilist =
    List.fold_left (fun acc x -> (x, (try List.assoc x acc
				      with Not_found -> 0)+1)::acc)
      []
      l
  in
    List.fold_left (fun acc x -> (if List.mem_assoc x acc
				  then acc
				  else (x,List.assoc x multilist)::acc))
      []
      l
;;

let list_of_multiset l =
  let rec expand x n l = if n==0 then l else x::(expand x (n-1) l) 
  in
    List.fold_left (fun acc (id,n) -> expand id n acc) [] l
;;


(* value s_one : 'a -> int -> ('a * int) list

   [s_one id mult] returns a list of pairs $([id],m)$ for $0\le m \le
   mult$.
*)
let rec s_one id = function
    mult ->
      if mult == 0
      then [(id,0)]
      else (id,mult)::(s_one id (mult-1))
;;

let s_all l =
  List.map (fun (id,mult) -> s_one id mult) l
;;

(*
  value s_one_over_all : 'a list -> 'a list list -> 'a list
  list

   [s_one_over_all [a1;...;an] [lb1;...;lbm]] =
   [a_n::lb_1;a_n::lb_2;...;a_n::lb_m;a_{n-1}::lb_1;...;...;la_1::lb_m]]

   Look like not being optimal... I have to look at it.
*)
let s_one_over_all l1 l2 =
  List.fold_left (fun acc x -> (List.map (fun l -> x :: l) l2)@acc) [] l1
;;

(* value s_left : ('a * int) list -> ('a * int) list list

  [s_left l] returns a list of (say) left-members of all
  bi-partitions of the multiset coded as [l]. 
*)
let s_left l = List.fold_right s_one_over_all (s_all l) [[]]
;;

let clean_s_left l =
  let rec clean = function
      [] -> assert false
    | [a] -> []
    | a::reste -> a::(clean reste)
  in
    match s_left l 
    with
	[] -> raise (Invalid_argument "clean_s_left")
      | a::reste -> clean reste
;;

(*
  val complement : ('a * int) list -> ('a * int) list -> ('a * int) list
  
  [complement l2 l1] returns the complement of [l1] in [l2]. That is:
  [complement] [(x_1,M_1);...;(x_n,M_n)] [(x_1,m_1);...;(x_k,m_k)] =
  [(x_1,M_1-m_1);...;(x_n,M_n-m_n)].
*)
let complement initial lhs = 
  List.fold_left (fun acc (id,mult) -> 
		        (id, mult - (try List.assoc id lhs
				     with Not_found -> 0))
		        ::acc) [] initial
;;

(* 
   [cut_n n l] returns the [n] first elements of list [l].
*)
let rec cut_n n = function
    [] -> []
  | h::t -> if n == 0 then []
    else h::(cut_n (n-1) t)
;;

(*
  [cut_half l] returns the first half of list [l]. If length of [l] is
  odd then the middle element is taken also.
*)
let cut_half l = cut_n (((List.length l)+1) / 2) l      
;;

(********************************************************************)

(*

  [(dependency_pair_criterion_with_markings R)] returns the set of ordering
  existence constraints given by the dependency pairs
  termination criterion with markings on [R], which is the following :

  \begin{enumerate} 

  \item let $D$ be the set of \emph{defined} symbols of [R], that is
    the set of symbols occuring at root of a lefthand side of a rule
    of [R] and $D'$ a copy of $D$ (marked symbols) ;

  \item a \emph{dependency pair with markings} is any pair of terms
    $(s,t)$ such that $s$ is the lefthand side of a rule $s\rightarrow
    r$ in [R] and $t$ is any subterm of $r$ whose root symbol is
    defined, and where the root symbols of $s$ and $t$ are marked ;

  \item let $R'$ be set of rules formed by $R$ augmented with the
  rules $f'(x) \rightarrow f(x)$ for each AC symbols $f$, and the
  rules $l\rightarrow r'$ where $l\rightarrow r$ is any rule of $R$
  such that the root symbol of $r$ is AC and different from the one of
  $l$, and $r'$ is obtained form $r$ by marking thr root symbol of $r$
  ;

  \item a condition for [R] terminating is that there is a weakly
    monotonic ordering $\geq$ such that $l\geq r$ for all rules of [R']
    and $s>t$ for all dependency pair with markings of [R] ;

  \item $\geq$ must be AC-compatible.

  \end{enumerate} 

*)


type 'symbol dupl =
    Original of 'symbol
  | Marked of 'symbol
;;

let rec copy_term marked_hct t = 
  match t.Hcons.node with
    | Term(f,l) -> 
	make_term marked_hct (Original f) (List.map (copy_term marked_hct) l)
    | Var x -> make_var_term marked_hct x
;;

let mark_root_symbol hct t =
  match t.Hcons.node with
    | Term(f,l) -> make_term hct (Marked f) (List.map (copy_term hct) l)
    | Var x -> make_var_term hct x
;;

	
let rec defined_subterms defined_symbols t = 
  match t.Hcons.node with
    |  Var _ -> []
    | Term(f,l) ->
	let others = 
	  Listutils.flat_map (defined_subterms defined_symbols) l
	in
      	  if SymbolSet.mem f defined_symbols
      	  then t::others
	  else others
;;

let copy_rule marked_hct rule =
  match rule.ext with
      Some(l,r) ->
    	{
	  lhs = copy_term marked_hct rule.lhs;
	  rhs = copy_term marked_hct rule.rhs;
	  ext = Some (copy_term marked_hct l,copy_term marked_hct r);
	  is_or = rule.is_or;
	  nb = rule.nb
    	}
    | _ ->
	{
	  lhs = copy_term marked_hct rule.lhs;
	  rhs = copy_term marked_hct rule.rhs;
	  ext = None;
	  is_or = rule.is_or;
	  nb = rule.nb
    	}
;;


let ac_rhs_rules sigma marked_hct sigma_marked defined_symbols r = 
  Listutils.flat_map
    (function rule ->
       match rule.rhs.Hcons.node with
	   Term(f,l) ->
	     if (sigma#is_ac f) & (f <> root_symbol rule.lhs)
	       & (SymbolSet.mem f defined_symbols)
	     then 
	       [ make_rule sigma_marked
		   marked_hct
		   (copy_term marked_hct rule.lhs) 
		   (make_term marked_hct (Marked f) (List.map (copy_term marked_hct) l));
		 copy_rule marked_hct rule ]
	     else [copy_rule marked_hct rule]
	 | _ -> [copy_rule marked_hct rule]
    )
    r
;;

let remove_markings_rules sigma marked_hct defined_symbols =
  SymbolSet.fold
    (fun f r ->
       if (sigma#is_ac f)
       then
	 let l = 
	   List.map 
	     (fun x -> make_var_term marked_hct x) 
	     (Variables.fresh_variables 2)
	 in
	   { lhs = make_term marked_hct (Marked f) l;
	     rhs = make_term marked_hct (Original f) l;
	     ext = None;
	     is_or = true;
	     nb = 0
	   } ::r
       else r
    )
    defined_symbols
    []
;;

class ['symbol] marked_signature (orig_sig) =
  object

    val original_sig = (orig_sig : 'symbol #Signatures.signature)
			 
    method arity (f : 'symbol dupl) = 
      match f with
      	  Original f -> original_sig#arity f
	| Marked f -> original_sig#arity f
	      
    method is_ac (f : 'symbol dupl) = 
      match f with
      	  Original f -> original_sig#is_ac f
	| Marked f -> original_sig#is_ac f
	      
    method is_commutative (f : 'symbol dupl) = 
      match f with
      	  Original f -> original_sig#is_commutative f
	| Marked f -> original_sig#is_commutative f
	      
    method is_free (f : 'symbol dupl) = 
      match f with
      	  Original f -> original_sig#is_free f
	| Marked f -> original_sig#is_free f
	
    method contains_ac_symbols = original_sig#contains_ac_symbols

    method contains_only_free_symbols = original_sig#contains_only_free_symbols

    method string_of_symbol (f : 'symbol dupl) = 
      match f with
      	  Original f -> original_sig#string_of_symbol f
	| Marked f -> "'"^(original_sig#string_of_symbol f)^"`"
	      
    method symbol_fix (f : 'symbol dupl) = 
      match f with
      	  Original f -> original_sig#symbol_fix f
	| Marked f -> original_sig#symbol_fix f
	      
    method smallest_constant o = 
      let c = 
	original_sig#smallest_constant 
	  (fun f g -> o (Original f) (Original g)) in
	    (Original c)

    method hash_symbol (f : 'symbol dupl) =
      match f with
	  Original f -> original_sig#hash_symbol f
	| Marked f -> original_sig#hash_symbol f
  end



let signature_with_marks sigma = 
  new marked_signature(sigma)			    
;;
  
  



(*
[set_of_bipartitions t] returns a set of partially marked normal forms of t.
*)

module TermOrd =
  struct 
    type 'symbol t = 'symbol term 
    let compare = compare
  end
;;

module TermSet = Ordered_sets.MakePoly(TermOrd);;

let term_component_of_list hct f = function
    [] -> assert false
  | [x] -> x
  | l -> make_term hct f l
;;	

(* Sous-optimal  cause des appels  complement, on doit se debrouiller... *)
let set_of_bipartitions sigma hct marked_hct t =
  match t.Hcons.node with
    | Var _ -> invalid_arg "set_of_bipartitions"
    | Term(f,l) -> 
	if sigma#is_ac f
	then
	  let mset = multiset_of_list l in
	    List.fold_right 
	      (fun x acc -> 
		 TermSet.add 
		   (mark_root_symbol marked_hct
		      (make_term hct f 
			 [term_component_of_list hct f (list_of_multiset x);
			  term_component_of_list hct f (list_of_multiset(complement mset x))]
		      )
		   )
		   acc) 
	      (cut_half (clean_s_left mset))
	      TermSet.empty 
	else
	  TermSet.add (mark_root_symbol marked_hct t) TermSet.empty
						  

(* instantiations *)

(* instantiate all pairs of a list of dp ([dp_list]) of one rule with
substitution [subst]. *)
let instantiation_list signature hct defined_symbols l dp_list subst = 
(*  let dp_list = 
    let all_defined_subterms = 
      defined_subterms defined_symbols r
    in
      List.map (fun t -> (l,t)) all_defined_subterms
  in*)
  let u' = Substitution.substitute signature hct subst l  in
    List.fold_left 
      (fun acc (u,v) -> 
	 let v' = Substitution.substitute signature hct subst v in
	   (u',v') :: acc) 
      []
      dp_list
;;

let all_subst_for_a_dp signature hct defined_symbols l dp_list =
  List.fold_left
    (fun acc s -> 
       (instantiation_list signature hct defined_symbols l dp_list s)@acc)
    []
    (list_of_sigma signature hct l) 
;;
(*  la kusakari. *)
let pmnf_kusa sigma hct marked_hct (u,v) =
  let set_of_u = set_of_bipartitions sigma hct marked_hct u
  and set_of_v = set_of_bipartitions sigma hct marked_hct v
  in
    TermSet.fold 
      (fun u' acc ->  
	 TermSet.fold
	   (fun v' acc -> (u',v')::acc)
	   set_of_v
	   acc)
      set_of_u
      []
;;

(* Standard. *)
let pmnf sigma hct marked_hct (u,v) =
    let set_of_u = set_of_bipartitions sigma hct marked_hct u
    and v' = mark_root_symbol marked_hct v
    in
      TermSet.fold 
	(fun u' acc -> (u',v')::acc)
	set_of_u
	[]
;;

let marked_dp_of_a_rule sigma hct marked_hct defined_symbols l r =
  let dp_list = 
    List.map (fun t -> (l,t)) (defined_subterms defined_symbols r)
  in
    if sigma#is_ac (root_symbol l) 
    then
      List.fold_left
	(fun acc p -> (pmnf(*Standard*) sigma hct marked_hct p)@acc)
	[]
	(all_subst_for_a_dp sigma hct defined_symbols l dp_list)
  else
    List.map 
      (fun (a,b) -> (mark_root_symbol marked_hct a, mark_root_symbol marked_hct b)) 
      dp_list
;;
    
let compute_dependency_pairs_with_markings sigma hct marked_hct defined_symbols r =
  Listutils.flat_map
    (function rule ->
       (marked_dp_of_a_rule sigma hct marked_hct defined_symbols rule.lhs rule.rhs)
       @(match rule.ext with
	     Some(l,r) ->
	       marked_dp_of_a_rule sigma hct marked_hct defined_symbols l r
	   | None -> []))
    r
;;

let dependency_pair_criterion_with_markings sigma hct marked_hct r =
  
  let (*
sigma_marked = signature_with_marks sigma
  and *)
defined_symbols = compute_defined_symbols r 
  in
    
  let dp = compute_dependency_pairs_with_markings sigma hct marked_hct defined_symbols r
  and r' = (List.map (copy_rule marked_hct) r) 
	   @ (remove_markings_rules sigma marked_hct defined_symbols)
  in
    AtomicTerminationConstraintSet.union
      (all_rules_weakly_decreasing r')
      (all_dp_strictly_decreasing dp)
;;




