(***************************************************************************

Marked dependency pairs criteria, description & specification

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: marked_dp_criteria.mli,v 1.9 2002/07/01 12:01:36 contejea Exp $

***************************************************************************)

open Gen_terms
open Orderings_generalities

(*

  [(var_multiplicities t)] returns an association table from variables
   of [t] to their multiplicity, i.e., the number of their occurrences
   at depth 1 in [t].

*)

val var_multiplicities : 
  'symbol term -> (Variables.var_id,int) Variables.VarMap.t 


type 'symbol dupl =
  | Original of 'symbol
  | Marked of 'symbol


val mark_root_symbol :
  'symbol dupl HashTerm.t -> 'symbol term -> ('symbol dupl) term


class ['a] marked_signature : ('a #Signatures.signature) ->
  object
    method arity : 'a dupl -> int
    method is_ac : 'a dupl -> bool
    method is_commutative : 'a dupl -> bool
    method is_free : 'a dupl -> bool
    method contains_ac_symbols : bool
    method contains_only_free_symbols : bool
    method string_of_symbol : 'a dupl -> string
    method symbol_fix : 'a dupl -> Signatures.symbol_fix
    method smallest_constant : 'a dupl ordering -> 'a dupl
    method hash_symbol : 'a dupl -> int
  end

val signature_with_marks :
  'symbol #Signatures.signature -> 'symbol marked_signature
;;

val ac_rhs_rules :
    'a #Signatures.signature -> 'a dupl HashTerm.t -> 
      'a dupl #Signatures.signature -> 'a Signatures.SymbolSet.t ->
	'a Rewrite_rules.t list -> 'a dupl Rewrite_rules.t list

val copy_rule : 
  'a dupl HashTerm.t -> 'a Rewrite_rules.t -> 
    'a dupl Rewrite_rules.t


val remove_markings_rules :
  'a #Signatures.signature -> 'a dupl HashTerm.t ->
    'a Signatures.SymbolSet.t -> 'a dupl Rewrite_rules.t list




val compute_dependency_pairs_with_markings :
  ('a #Signatures.signature) -> 
    'a HashTerm.t -> 'a dupl HashTerm.t ->
      'a Signatures.SymbolSet.t ->
	'a Rewrite_rules.t list ->
	  ('a dupl term * 'a dupl term) list
;;

val dependency_pair_criterion_with_markings :
  ('symbol #Signatures.signature) -> 
    'symbol HashTerm.t -> 'symbol dupl HashTerm.t -> 
      'symbol Rewrite_rules.t list -> 
	('symbol dupl) Termination_constraints.conj_termination_constraints;;

(*i debug 
val split :
    'a #Signatures.signature ->
    Variables.user_variables -> 'b Gen_terms.term -> 'b Gen_terms.term list
(*'a Gen_terms.term -> 'a Gen_terms.term list*)
;;
 debug i*)





