(***************************************************************************

Splitting htrs in minimal modules

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: minimal_split.mli,v 1.4 2002/07/01 12:01:36 contejea Exp $

***************************************************************************)

(* A module of graphs of symbols with a special function [scc] which returns
   strongly connected components of its argument (see module Graph).*)

module Symgraph :
  sig
    type 'symbol elt = 'symbol Signatures.SymbolOrd.t
    and 'symbol t = 'symbol Graph.Make(Signatures.SymbolOrd).t
    exception NodeNotInGraph
    exception NodeAlreadyInGraph
    val empty : 'symbol t
    val add_node : 'symbol t -> 'symbol elt -> 'symbol t
    val is_node : 'symbol t -> 'symbol elt -> bool
    val add_edge : 'symbol t -> 'symbol elt -> 'symbol elt -> 'symbol t
    val is_edge : 'symbol t -> 'symbol elt -> 'symbol elt -> bool
    val del_node : 'symbol t -> 'symbol elt -> 'symbol t
    val del_edge : 'symbol t -> 'symbol elt -> 'symbol elt -> 'symbol t
    val node_list : 'symbol t -> 'symbol elt list
    val edge_list : 'symbol t -> ('symbol elt * 'symbol elt) list
    val neighbours_list : 'symbol t -> 'symbol elt -> 'symbol elt list
    val iter_node : 'symbol t -> ('symbol elt -> unit) -> unit
    val iter_edge : 'symbol t -> ('symbol elt -> 'symbol elt -> unit) -> unit
    val fold : ('symbol elt -> 'b -> 'b) -> 'b -> 'symbol t -> 'b
    val transpose : 'symbol t -> 'symbol t
    val scc : 'symbol t -> 'symbol elt list list
  end


val graph_from_a_list :
  'symbol Rewrite_rules.t list ->
    'symbol Symgraph.elt Symgraph.t
;;


val packs_from_a_list :
  'symbol Rewrite_rules.t list ->
    'symbol Symgraph.elt Symgraph.elt list list
;;


val packs_and_rules_from_packs :
  'symbol Rewrite_rules.t list ->
    'symbol list list -> ('symbol list * 'symbol Rewrite_rules.t list) list


(* [minimal_split sigma vars r] returns a list of 
   minimal modules in htrs [r].*)

val minimal_split :
  'symbol #Signatures.signature -> (*i Variables.user_variables -> i*)
    Hierarchical_trs.htrs -> Hierarchical_trs.htrs list
;;
     

