open Signatures;;
open Variables;;
open Gen_terms;;
open User_terms;;
open Rewrite_rules;;
open Termination_constraints;;
open Dependency_pairs_criteria;;
open Hierarchical_trs;;

(* [defined_sym_list rules] returns the list of defined symbols of
   list of rules [rules]. 

let defined_sym_list rules =
  fst
    (List.fold_left
       (fun ((acc,dv) as accu) elem -> 
	  let e = Gen_terms.root_symbol elem.lhs
	  in 
	    if SymbolSet.mem e dv 
	    then accu
	    else (e::acc,SymbolSet.add e dv)) 
       ([],SymbolSet.empty)
       rules)
;;
*)

(* [defined_sym_set rules] returns the set of defined symbols of
   list of rules [rules]. *)

let defined_sym_set rules =
  List.fold_left
    (fun acc elem -> 
       let e = Gen_terms.root_symbol elem.lhs
       in 
	 if SymbolSet.mem e acc 
	 then acc
	 else SymbolSet.add e acc) 
    SymbolSet.empty
    rules
;;

(* [defined_subterms def_set t] returns a list of subterms of [t] the
   root symbols of which belong to [def_set]. *)


let rec defined_subterms defined_symbols t =
  match t.Hcons.node with
    | Var(_) -> []
    | Term(f,l) ->
	let others = 
	  Listutils.flat_map (defined_subterms defined_symbols) l
	in
      	  if SymbolSet.mem f defined_symbols
      	  then t::others
	  else others
;;


let rdp_of_a_rule rule defined_set = 
  let lhs = rule.lhs
  in
    List.map
      (fun e -> (lhs,e))
      (defined_subterms defined_set rule.rhs)      
;;
  

let rdp_of_rules rules =
  let defined_symbols = defined_sym_set rules
  in
  (*i
    List.fold_left
      (fun acc1 rule -> 
	 let lhs = rule.lhs
	 and defined_sub = defined_subterms defined_symbols rule.rhs
	 in
	   List.fold_left 
	     (fun acc2 sub -> (lhs,sub)::acc2)
	     acc1
	     defined_sub)
      []
      rules
    i*)
  compute_dependency_pairs 
    (*sigma*) defined_symbols rules
;;


let marked_rdp_of_rules sigma hct marked_hct rules =
  let defined_symbols = defined_sym_set rules
  in
  (*i
    List.fold_left
      (fun acc1 rule -> 
	 let mlhs = Marked_dp_criteria.mark_root_symbol rule.lhs
	 and defined_sub = defined_subterms defined_symbols rule.rhs
	 in
	   List.fold_left 
	     (fun acc2 sub -> 
		(mlhs,Marked_dp_criteria.mark_root_symbol sub)::acc2)
	     acc1
    defined_sub)
    []
    rules
    i*)
  Marked_dp_criteria.compute_dependency_pairs_with_markings 
    sigma hct marked_hct defined_symbols rules
;;



let relative_dp_criterion htrs =
  let dp = rdp_of_rules htrs.htrs_new_rules
  in
    AtomicTerminationConstraintSet.union
      (all_rules_weakly_decreasing htrs.htrs_all_rules)
      (all_dp_strictly_decreasing dp)
;;

let marked_relative_dp_criterion htrs marked_sig = 
  let dp = 
    marked_rdp_of_rules htrs.htrs_sig htrs.hct htrs.marked_hct 
      htrs.htrs_new_rules
  and r' = 
    List.map (Marked_dp_criteria.copy_rule htrs.marked_hct)
      htrs.htrs_all_rules
  in
    AtomicTerminationConstraintSet.union
      (all_rules_weakly_decreasing r')
      (all_dp_strictly_decreasing dp)

;;


