(***************************************************************************

Definition of termination constraints

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: termination_constraints.mli,v 1.4 2001/04/30 17:25:34 urbain Exp $

***************************************************************************)


(*i*)
open Signatures;;
open Gen_terms;;
(*i*)

(*

  An atomic termination constraint is either $s=t$, $s>t$ or $s>=t$
  for two terms $s$ and $t$

*)

type constraint_op_type = Equal | Greater | Greater_or_equal;;

type 'symbol atomic_termination_constraint =
    {
      left : 'symbol term;
      right : 'symbol term;
      constraint_op : constraint_op_type;
    }
;;

(*

  a termination constraint is a disjunction of conjonction of atomic
  constraints.

*)

module AtomicTerminationConstraintSet : Ordered_sets.OrderedSet 
  with type 'symbol elt= 'symbol atomic_termination_constraint;;

type 'symbol conj_termination_constraints = 
    'symbol AtomicTerminationConstraintSet.t;;

val print_atomic_termination_constraint :
  'symbol #Signatures.signature ->
     Variables.user_variables -> 'symbol atomic_termination_constraint
       -> unit;;


module ConjTerminationConstraintSet : Ordered_sets.OrderedSet 
  with type 'symbol elt= 'symbol conj_termination_constraints;;

type 'symbol termination_constraints =
    'symbol ConjTerminationConstraintSet.t;;


val print_conj_termination_constraints : 
  'symbol #Signatures.signature ->
     Variables.user_variables -> 'symbol conj_termination_constraints
       -> unit;;

val print_termination_constraints : 
  'symbol #Signatures.signature ->
     Variables.user_variables -> 'symbol termination_constraints
       -> unit;;

(*

  [(set_of_symbols c)] returns the set of symbols occuring in the list
  of constraints [c]

*)

val set_of_symbols : 
  'symbol conj_termination_constraints -> 'symbol SymbolSet.t;;


(*

  \subsection{Checking constraints}

*)

val check_conj_constraints : 
   'symbol #Signatures.signature ->
     Variables.user_variables -> 
       'symbol conj_termination_constraints -> 
	 'symbol Gen_terms.term Orderings_generalities.ordering -> unit


