(***************************************************************************

Main function for finding termination proofs

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: termination_expert.mli,v 1.14 2002/07/01 12:01:36 contejea Exp $

***************************************************************************)

(*

  This module is the termination expert. The main function looks for a
  termination proof for a given rewrite system.

*)

(*

  \subsection{Global configuration}

  the following global variables controls which criterion the termination expert uses.


When [standard_criterion] is true, the termination criterion is the
standard one: each must decrease with respect to a monotonic
well-founded ordering. When it is false, the dependency pair criteria
are used. See module
[Dependency_pair_criteria]~\ref{dependency_pair_criteria}. default
true *)

val standard_criterion : bool ref;; 

(* When [dp_criterion_uses_markings] is true, DP criteria with marks
are used.  default true *)

val dp_criterion_uses_markings : bool ref;;

(* When [dp_criterion_uses_graph] is true, DP criteria with graph are
used.  default true *)

val dp_criterion_uses_graph : bool ref;;

(* When [enable_split] is true, relative dp criteria are used. 
   default false *)

val enable_split : bool ref;;

(* [bound] is the search bound for polynomial coefficients when
looking for an ordering defined by polynomial interpretation. Default
6 *)

val bound : int ref;;

(* The verbose level. 
  \begin{enumerate}
  \item the termination constraints to solve are displayed. 
  \item  ...
  \end{enumerate}
  Default is 0 *)

val verbose : int ref;;

val user_dhct : User_signatures.symbol_id Marked_dp_criteria.dupl Gen_terms.HashTerm.t

(*

  \subsection{Computing depencency pairs}

  [show_pairs F H MH X R] computes the dependency pairs of the
  ([F],[X])-TRS [R], and displays them. If the marked criterion is
  currently selected, then marked pairs are computed. If the graph
  criterion is currently selected, also displays the approximated
  dependency graph. [H] is an hashconsing table for standard terms
  and [MH] is an hashconsing table for marked_terms.

*)

val show_pairs : 
  ('symbol #Signatures.signature) -> 'symbol Gen_terms.HashTerm.t ->
    'symbol Marked_dp_criteria.dupl Gen_terms.HashTerm.t ->
      Variables.user_variables -> 
	('symbol Rewrite_rules.t) list -> unit


(*

  \subsection{Computing termination constraints}

*)
(*i
val compute_modular_termination_constraints_without_marks :
  Hierarchical_trs.htrs ->
    Hierarchical_signatures.hsymbol 
      Termination_constraints.conj_termination_constraints
;;

val compute_modular_termination_constraints_with_marks :
  Hierarchical_trs.htrs ->
    Hierarchical_signatures.hsymbol Marked_dp_criteria.dupl 
      Termination_constraints.conj_termination_constraints
;;
i*)
val compute_termination_constraints_without_marks :
  ('symbol #Signatures.signature) ->
    Variables.user_variables -> 
      ('symbol Rewrite_rules.t) list ->
	'symbol Termination_constraints.conj_termination_constraints
;;

val compute_termination_constraints_with_marks :
  ('symbol #Signatures.signature) -> 'symbol Marked_dp_criteria.dupl #Signatures.signature ->
    Variables.user_variables -> 
      'symbol Gen_terms.HashTerm.t ->
	'symbol Marked_dp_criteria.dupl Gen_terms.HashTerm.t ->
	  ('symbol Rewrite_rules.t) list ->
	    'symbol Marked_dp_criteria.dupl 
	      Termination_constraints.conj_termination_constraints
;;
	
(*

  \subsection{The main function}

  [expert F X R] looks for a termination proof of the ([F],[X])-TRS
  [R]. Displays the results.

*)

val expert : 
  ('symbol #Signatures.signature) ->
    Variables.user_variables -> 
      ('symbol Rewrite_rules.t) list -> unit

val modular_expert :
      Variables.user_variables -> 
	Hierarchical_trs.htrs -> 
	  (Generic_polynomials.polyinterptype * int)  list -> unit

(*i debug relative_dp*
val display_conj_termination_constraints : 
  ('symbol #Signatures.signature) ->
    Variables.user_variables -> 
	'symbol Termination_constraints.conj_termination_constraints
      -> unit
;;
i*)
