(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: gen_terms.ml,v 1.21 2002/07/16 09:29:36 contejea Exp $

***************************************************************************)


open Positions
open Signatures
open Variables
open Hcons
open Format
open Signature_syntax

type 'symbol term = 'symbol term_struct hash_consed

and 'symbol term_struct =
  | Term of 'symbol * 'symbol term list
  | Var of var_id


(*


[(print_term sigma t)] prints to standard output the term [t] over the
signature [sigma].

*)

let print_term sigma vars t = 

  let rec print_term_rec is_deep = function
      Term(f,[]) ->
      	print_string (sigma#string_of_symbol f);
    | Term(f,t::l) ->
	open_box 0;
	begin
	  match sigma#symbol_fix f with
	      Default | Prefix ->
	      	begin
      	      	  print_string (sigma#string_of_symbol f);
      		  print_string "(";
      		  print_term_rec false t.node;
      		  print_end_term_list l
	      	end
	    | Postfix ->
	      	begin
      		  print_string "(";
      		  print_term_rec false t.node;
      		  print_end_term_list l;
      	      	  print_string (sigma#string_of_symbol f);
	      	end
	    | Infix ->
	      	begin
		  if is_deep then print_string "(";
		  print_term_rec true t.node;
      		  print_infix_list f l;
		  if is_deep then print_string ")"
	      	end
	end;
	close_box();
    | Var(x) ->
	print_string (vars#string_of_var x)
	  
  and print_end_term_list = function
      [] -> 
      	print_string ")"
    | t::l ->
      	print_string ",";
      	print_term_rec false t.node;
      	print_end_term_list l
	  
  and print_infix_list f = function
      [] -> ()
    | t::l ->
	print_string " ";
      	print_string (sigma#string_of_symbol f);
	print_string " ";
      	print_term_rec true t.node;
      	print_infix_list f l
	  
  in
    print_term_rec false t.node


let hVar = 1
let hCst = 2
let hTerm = 3
let hCombine =  65599


(*
let hash_term t = 
  match t with
    | Var x -> (Variables.hash x) * hCombine + hVar
    | Term (c,[]) -> (Signatures.hash_symbol c) * hCombine + hCst
    | Term (f,l) -> 
	List.fold_left 
	  (fun acc t -> acc * hCombine + t.tag)
	  (hTerm * hCombine + (hash_symbol f))
	  l

let hash_term t =
  match t with
    | Var x -> 
	let h_x = Variables.hash x in 
	  2 * h_x
    | Term (c,[]) -> 
	let h_c = Signatures.hash_symbol c in
	  (2 * h_c) + 1
    | Term (f,l) ->
	let h_f = Signatures.hash_symbol f in
	  List.fold_left 
	    (fun acc t -> ((acc + t.hkey) * (acc + t.hkey + 1)) / 2)
	    h_f l

let hash_term t =
  match t with
    | Var x -> 
	let h_x = Variables.hash x in 
	  (h_x * (h_x + 3)) / 2
    | Term (c,[]) -> 
	let h_c = Signatures.hash_symbol c in
	  (h_c * (h_c + 1)) / 2
    | Term (f,l) -> 
	List.fold_left 
	  (fun acc t -> acc * hCombine + t.tag)
	  (hTerm * hCombine + (hash_symbol f))
	  l
*)

    
let hash_term t =
  match t with
    | Var x -> 
	let h_x = Variables.hash x in 
	  2 * h_x
    | Term (c,[]) -> 
	let h_c = Signatures.hash_symbol c in
	  (2 * h_c) + 1
    | Term (f,l) -> 
	List.fold_left 
	  (fun acc t -> acc * hCombine + t.tag)
	  (hTerm * hCombine + (hash_symbol f))
	  l

let term_eq (t1,t2 as t1_t2) = 
  match t1_t2 with
    | (Var x1), (Var x2) -> x1 == x2
    | (Term (f1,l1)), (Term (f2,l2)) -> 
	if f1 = f2
	then 
	  try 
	    List.for_all2 (fun s1 s2 -> s1.tag == s2.tag ) l1 l2
	  with _ -> false
	else false
    | _, _ -> false

module HashTerm = 
  MakePoly
    (struct
       type 'symbol t = 'symbol term_struct
       let equal (t1,t2) = term_eq (t1,t2)
       let hash t = (hash_term t) land max_int
     end)


let hash_cons h t =
  HashTerm.hashcons h t

let make_var_term h x =
  hash_cons h (Var x)

exception Bad_arity of string

(*
let make_cst_term h c =
  hash_cons h (Term(c,[]))
*)

let rec compare_terms t1 t2 =
  match (t1.node,t2.node) with
    | (Term(f1,l1),Term(f2,l2)) ->
        let x = Pervasives.compare f1 f2 in
        if x<>0 then x
	else compare_lists l1 l2 
    | (Var _,Term _) -> 1
    | (Term _ ,Var _) -> -1
    | (Var x1,Var x2) -> Pervasives.compare x1 x2

and compare_lists l1 l2 = 
  match l1,l2 with
    | ([],[]) -> 0
    | ([],_) -> 1
    | (_,[]) -> -1
    | (s1::l1,s2::l2) -> 
	let x = compare_terms s1 s2 in
	  if x<>0 then x
	  else compare_lists l1 l2

(*
let rec compare_terms t1 t2 =
  match t1.node,t2.node with
    | Var _ , Term _ -> 1
    | Term _ , Var _ -> -1
    | _ , _ -> 	Pervasives.compare (t1.tag : int) (t2.tag : int)
*)

let rec flatten_term_list f = function
  | [] -> []
  | t :: m ->
      match t.node with
	| Term (g,l) ->
	    if f == g
	    then l @ (flatten_term_list f m)
	    else t :: (flatten_term_list f m)
	| Var _ -> t :: (flatten_term_list f m)

let make_term (*i sigma i*) h f l =
(*
  if (sigma#is_ac f) && 
    (l <> (List.sort compare_terms (flatten_term_list f l)))
  then raise Exit
  else 
*)
  hash_cons h (Term(f,l))

let make_sorted_term sigma h f l =
  let list_of_subterms =
    if (sigma#is_ac f) || (sigma#is_commutative f)
    then  List.sort compare_terms l
    else l in
    hash_cons h (Term(f,list_of_subterms))

let make_head_flatten_term sigma h f l =
  let list_of_subterms =
    if sigma#is_ac f
    then List.sort compare_terms (flatten_term_list f l) 
    else 
      if sigma#is_commutative f
      then  List.sort compare_terms l
      else l in
    hash_cons h (Term(f,list_of_subterms))




exception No_subterm

let rec replace_subterm_at_position_in_term sigma hct u p t =
  match p with
      Top -> u
    | Pos (i,p') ->
	begin
	  match t.node with
	    | Var _ -> raise No_subterm
	    | Term (f,l) -> 
		let l' = 
		  Listutils.mapi 
		    (fun n s -> 
		       if n==i 
		       then replace_subterm_at_position_in_term sigma hct u p' s
		       else s) 
		    l in
		  make_head_flatten_term sigma hct f l'
	end


let hash_consing_table = (HashTerm.create 1997 : string HashTerm.t)

let rec make_term_from_string_term sigma h t =
  match t.node with
    | Term(f,l) -> 
	let g = sigma#symbol_of_string f in
	  make_head_flatten_term sigma h g 
	    (List.map (make_term_from_string_term sigma h) l)
    | Var x -> make_var_term h x


(*

  [(root_symbol t)] returns the root symbol of [t], raises [Not_found]
  if [t] is a variable

*)

let root_symbol t = 
  match t.node with
    | Var _ -> raise Not_found
    | Term (f,_) -> f

(*
  
  [(subterm_at_position p t)] returns the subterm of the term [t] at the
  position [p].
  
*)
(*
let rec subterm_at_position p t =
  match p with
      Top -> t
    | Pos (i,p') -> 
	match t with
	    Var _ -> raise No_subterm
	  | Term (_,l) ->
	      begin
		try
		  let t' = List.nth l i in
		    subterm_at_position p' t'
		with Failure "nth" -> raise No_subterm
	      end
*)
(*

  [(set_of_symbols t)] returns the set of symbols occuring in [t]

*)

let rec set_of_symbols t = 
  match t.node with 
    | Var _ -> SymbolSet.empty
    | Term(f,l) ->
	SymbolSet.add f
      	  (List.fold_left
	     (fun s t ->
		SymbolSet.union s (set_of_symbols t))
	     SymbolSet.empty
	     l)


let rec set_of_variables t = 
  match t.node with
    | Var x -> VarSet.add x VarSet.empty
    | Term (_,l) ->
	List.fold_left
	  (fun s t -> VarSet.union s (set_of_variables t))
	  VarSet.empty
	l

let rec var_occurs_in_term x t = 
  match t.node with
    | Var y -> x==y
    | Term(_,l) -> var_occurs_in_term_list x l

and var_occurs_in_term_list x l = 
  match l with
    | [] -> false
    | s::l -> 
	if var_occurs_in_term x s 
	then true 
      	else var_occurs_in_term_list x l


let rec leftify_term hct t = 
  match t.node with
    | Term (f,l) -> hash_cons hct (Term (f,List.map (leftify_term hct) l))
    | Var x -> hash_cons hct (Var (leftify_var x))

let rec rightify_term hct t = 
  match t.node with
    | Term (f,l) -> hash_cons hct (Term (f,List.map (rightify_term hct) l))
    | Var x -> hash_cons hct (Var (rightify_var x))

let rec shift_term hct v t =
  match t.node with
    | Term (f,l) -> hash_cons hct (Term (f,List.map (shift_term hct v) l))
    | Var x -> hash_cons hct (Var (shift_variable v x))

let size sigma t = 
  let rec rec_size t = 
    match t.node with
      | Var _ -> 1
      | Term(f,l) ->
	  let size_of_l = ref 0 in
	  let _ = 
	    List.iter
	      (fun s -> size_of_l := !size_of_l + (rec_size s)) l in
	    if sigma#is_ac f
	    then (pred (List.length l)) + !size_of_l
	    else 1 + !size_of_l in
    rec_size t

let check_for_right_regularity t1 t2 =
  let vars_of_t1 = set_of_variables t1
  and vars_of_t2 = set_of_variables t2 in
    VarSet.filter 
      (function v2 -> not (VarSet.mem v2 vars_of_t1))
      vars_of_t2

let build_a_term_for_a_right_regular_pair sign hct a t1 t2 =
  let vars_of_t1 = set_of_variables t1
  and vars_of_t2 = set_of_variables t2 in
  let vars =  check_for_right_regularity t1 t2 in
    if (VarSet.is_empty vars)
    then raise Exit
    else
      let rec substitute t = 
	match t.node with 
	  | Var y -> if VarSet.mem y vars then a else t
	  | Term(f,l) -> make_sorted_term sign hct f (List.map substitute l) in
	substitute t2;;



let print_equation sigma vars (s,t) =
  printf "@[<hov>";
  print_term sigma vars s;
  printf "@ =@ ";
  print_term sigma vars t;
  printf "@]"


let print_equation_list sigma vars = function
    []   -> (print_string "{}")
  | r::l ->
      let rec print_end_list n = function
	  [] -> 
	    begin
              print_string " }";
              print_break 1 0;
              print_string "(";
              print_int n;
              print_string " equation(s))"                 
	    end
	| (r::l) -> 
	    begin
              print_string  ",";
              force_newline ();
              print_equation sigma vars r;
              print_end_list (succ n) l
            end
      in 
	begin
	  print_string "{ ";
	  open_hovbox 0;
	  print_equation sigma vars r;
	  print_end_list 1 l;
	  close_box ()
	end

