(***************************************************************************

This module defines the class of hierarchical signatures

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: hierarchical_signatures.ml,v 1.6 2002/07/01 12:01:36 contejea Exp $

***************************************************************************)

open Signatures;;
open User_signatures;;

		
type hsymbol =
    {
      id : symbol_id;
      user_sig : user_signature;
    }
;;

 
class type hierarchical_signature = 
      object
	inherit [hsymbol] parseable_signature
	method is_defined_here : hsymbol -> bool
      end
;;

exception Found of hsymbol;;

class extend_signature_class 
  (sig_list : hierarchical_signature list) 
  (new_sig : #user_signature) : hierarchical_signature =
object 
    val number_of_imported_sig = List.length sig_list

    val imported_sig_table = Array.of_list sig_list

    val new_sig = (new_sig  :> user_signature)

    method arity (f : hsymbol) : int = f.user_sig#arity f.id

				     
    method is_ac (f : hsymbol) : bool = f.user_sig#is_ac f.id
		
    method is_free (f : hsymbol) : bool = f.user_sig#is_free f.id

    method is_commutative (f : hsymbol) : bool = 
      f.user_sig#is_commutative f.id
 
    method contains_ac_symbols : bool = 
      new_sig#contains_ac_symbols or
      List.exists 
	(fun s -> s#contains_ac_symbols) 
	sig_list
      
    method contains_only_free_symbols : bool = 
      new_sig#contains_only_free_symbols &&
      List.for_all 
	(fun s -> s#contains_only_free_symbols) 
	sig_list

    method string_of_symbol (f : hsymbol) : string = 
      f.user_sig#string_of_symbol f.id
	
    method symbol_of_string (name : string) : hsymbol =
      try
	let f = new_sig#symbol_of_string name in
	{ id = f ; user_sig = new_sig}
      with
	  Not_found ->
	    try
	      for i=0 to pred (Array.length imported_sig_table) do
		try
		  let f = imported_sig_table.(i)#symbol_of_string name in
		  raise (Found f)
		with Not_found -> ()
	      done;
	      raise Not_found
	    with 
		Found f -> f

    method symbol_fix (f : hsymbol) : symbol_fix = 
      f.user_sig#symbol_fix f.id

    method is_defined_here (f : hsymbol) : bool = 
       f.user_sig == new_sig

    method smallest_constant _ = ((raise Not_found) : hsymbol)

    method hash_symbol (f : hsymbol) = 
      f.user_sig#hash_symbol f.id
end 
;;

let extend_signature sig_list new_sig =
  new extend_signature_class sig_list new_sig
;;


