(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: hierarchical_trs.ml,v 1.8 2002/07/01 12:01:36 contejea Exp $

***************************************************************************)






open Hierarchical_signatures;;
open Rewrite_rules;;

type htrs = 
    {
      htrs_sig : hierarchical_signature;
      htrs_new_rules : hsymbol Rewrite_rules.t list;
      htrs_imported : htrs list;
      htrs_all_rules : hsymbol Rewrite_rules.t list;
      proved_c_e_terminating : bool ref;
      hct : hsymbol Gen_terms.HashTerm.t;
      marked_hct : hsymbol Marked_dp_criteria.dupl Gen_terms.HashTerm.t;
    }
;;


(*

  [extend imported sigma vars rules] makes a new HTRS from the list
  [imported] of known HTRS, by adding the new signature [sigma] and
  the new rules syntactically given in the string [rules], those
  being defined on [sigma] and the signatures of the [imported] htrs.

*)


let rec collect_all_rules accu traversed hlist =
  match hlist with
    | [] -> accu 
    | h::hl -> 
	if List.memq h traversed
	then collect_all_rules accu traversed hl
	else 
	  let new_accu = h.htrs_new_rules @ accu in
	  collect_all_rules new_accu (h::traversed) (h.htrs_imported@hl)
;;

exception Overriden of string;;

let extend imported sigma vars rules =
  let known_sigs = List.map (fun h -> h.htrs_sig) imported in
  let new_hsig = Hierarchical_signatures.extend_signature known_sigs sigma in
  let new_hash_table = Gen_terms.HashTerm.create 1997 in
  let new_rules = User_terms.rule_set_of_string new_hsig new_hash_table vars rules in
  let all_rules = collect_all_rules new_rules [] imported in
  let _ = List.fold_left
	    (fun _ r -> 
	       let (lambdar:hsymbol) = Gen_terms.root_symbol (r.lhs) in
		 if  not (new_hsig#is_defined_here lambdar)
		 then raise (Overriden
			       ("Function "^
				(new_hsig#string_of_symbol lambdar)^
				" is overriden.")))
	    ()
	    new_rules in
    (*i Here all is OK, we may actually build a HTRS. *)
    {
      htrs_sig = new_hsig;
      htrs_new_rules = new_rules;
      htrs_imported = imported;
      htrs_all_rules = all_rules;
      proved_c_e_terminating = ref false;
      hct = Gen_terms.HashTerm.create 1997;
      marked_hct = Gen_terms.HashTerm.create 1997;
    }
;;

let assume_ce htrs = htrs.proved_c_e_terminating := true;;
let clear_ce htrs = htrs.proved_c_e_terminating := false;;

  
