(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: main.ml,v 1.11 2001/05/01 15:02:41 urbain Exp $

***************************************************************************)


open Signatures;;
open Variables;;
open Gen_terms;;
open User_terms;;
open Rewrite_rules;;
open Format;;
open Hierarchical_trs;;
open Graph;;
open Relative_dp;; 

let my_vars = Variables.from_string "x y z";;


let my_sig = User_signatures.from_string 
  "+ : AC ; 
   0 : constant ; 
   - : unary ;";;

let t1 = term_of_string my_sig my_vars "x+y";;
let t2 = term_of_string my_sig my_vars "y+x";;

print_term my_sig my_vars t1;;
printf " =_AC ";;
print_term my_sig my_vars t2;;
printf " : %b@." (equals my_sig t1 t2);;

let r = rule_set_of_string my_sig my_vars 
  "x+0 -> x;
   x+-(x) -> 0;
   -(0) -> 0;
   -(-(x)) -> x;
   -(x+y) -> -(x)+-(y);";;

print_rewrite_rule_set my_sig my_vars r;;
print_newline();;


let sig_nat = User_signatures.from_string "0 : constant ; s : unary ;";;

let t = term_of_string sig_nat my_vars "s(s(0))";;
print_term sig_nat my_vars t;;
print_newline();;

let module_nat = 
  extend [] sig_nat my_vars "";;
    
let sig_plus = User_signatures.from_string "+ : infix binary;";;

let t = term_of_string sig_plus my_vars "x+y";;
print_term sig_plus my_vars t;;
print_newline();;

let module_plus = 
  extend [module_nat] sig_plus my_vars 
    "x+0 -> x ; x+s(y) -> s(x+y)";;

let t = term_of_string module_plus.htrs_sig my_vars "s(s(0+0)+s(0))+s(0)";;
print_term module_plus.htrs_sig my_vars t;;
print_newline();;

print_rewrite_rule_set 
  module_plus.htrs_sig my_vars module_plus.htrs_all_rules;; 
print_newline();;

let sig_mult = User_signatures.from_string "* : infix binary;";;

let module_mult = 
  extend [module_plus] sig_mult my_vars
    "x*0 -> 0 ; x*s(y) -> (x*y)+x";;

let t = term_of_string module_mult.htrs_sig my_vars "s(0+s(0))*s(s(0))";;
print_term module_mult.htrs_sig my_vars t;;
print_newline();;

print_rewrite_rule_set 
  module_mult.htrs_sig my_vars module_mult.htrs_all_rules;; 
print_newline();;


(* Here come the graphs again, falling in the code like a tragedy... ----

module Symgraph = Graph.Make(SymbolOrd)
;;

let doo_bee graph root set =
  Signatures.SymbolSet.fold
    (fun elem acc -> 
       Symgraph.add_edge 
	 (if Symgraph.is_node acc elem 
	  then acc 
	  else Symgraph.add_node acc elem)
	 root elem)
    set
    graph
;;

let g_from_rule graph rule =
  let lhs = rule.lhs
  in
  let root = Gen_terms.root_symbol lhs
  in
    doo_bee 
      (doo_bee    
	 (if Symgraph.is_node graph root
	  then graph
	  else Symgraph.add_node graph root)
	 root 
	 (Gen_terms.set_of_symbols rule.lhs))
      root 
      (Gen_terms.set_of_symbols rule.rhs)
;;      

(* [graph_from_a_list l] returns a dependency graph of symbols of
    rules in [l] *)

let graph_from_a_list rule_list =
  List.fold_left 
    g_from_rule
    Symgraph.empty
    rule_list
;;

(* [packs_from_a_list l] returns a list of \emph{minimal} packs of
    symbols of rules in [l] according to the hierarchical dependencies. *) 

let packs_from_a_list rule_list =
  Symgraph.scc (graph_from_a_list rule_list)
;;


(* [rules_from_root sym l] returns a pair [(l1,l2)] of list of rules
   such that [l1] contains rules of [l] the root symbol of which is
   [sym] and [l2] being the list of other rules of [l]. *)

let rules_from_root sym rule_list =
  List.fold_left
    (fun (rules_of_sym, rules_left) elem ->
       if Gen_terms.root_symbol elem.lhs = sym
       then (elem::rules_of_sym, rules_left)
       else (rules_of_sym, elem::rules_left)
    )
    ([],[])
    rule_list
;;

let rules_from_roots_in_packs pack rule_list =
  List.fold_left
    (fun (rules,l2) elem -> 
       let (used,left) = rules_from_root elem l2
       in
	 (used@rules,left))
    ([],rule_list)
    pack
;;

(* [packs_and_rules_from_packs rl pl] returs a list of pairs
   [(p_i,r_i); \ldots] where [p_i] is a pack from [pl] and [r_i] a
   list of the corresponding rules in [rl].*)

let packs_and_rules_from_packs rule_list pack_list =
  fst (
    List.fold_left
	 (fun (packs_and_rules,rules) elem ->
	    let (rules_from_elem,left) = rules_from_roots_in_packs elem rules
	    in
	      (((elem,rules_from_elem)::packs_and_rules),left))
	 ([],rule_list)
	 pack_list)
;;




(* This graph is presently implemented with neighbours lists. *)

(* [contains sym pack_list] returns a pack of [pack_list] the
   first part of which contains [sym].  *)

(* PACKS ARE DISJOINT. *)
let contains sym pack_list =
  List.find
    (fun (sl,l) -> List.mem sym sl)
    pack_list
;;
(*i List.fold_left (fun acc ((sl,l) as elem) -> if List.mem sym sl 
   then elem::acc else acc) [] pack_list;; *)


  (* [depends sym sym_graph pack_list] returns a list of packs from
  [pack_list] of which symbol [sym] depends according to hierarchy
  graph of symbols [sym_graph]. *)

let depends orig_acc sym sym_graph pack_list =
  List.fold_left
    (fun acc elem -> let new_pack =  contains elem pack_list in
       if List.mem new_pack acc then acc else new_pack::acc)
    orig_acc
    (Symgraph.neighbours_list sym_graph sym)
;;


let neighbours_of_a_pr sym_graph pr_list (sl,_) =
  List.fold_left
    (fun acc elem ->
       depends acc elem sym_graph pr_list
    )
    []
    sl
;;

let clean_neighbours n_list (a1,_) = 
  List.fold_left
    (fun acc ((a2,_) as elem) -> if a1 = a2 then acc else elem::acc)
    []
    n_list
;;


let pack_graph sym_graph pr_list =
  List.fold_left
    (fun acc pr ->
       (pr,(neighbours_of_a_pr sym_graph pr_list pr))::acc)
    []
    pr_list
;;

(*--------------------------------------------------------------------*)

open Signature_syntax;;

let string_of_term sgn vars t = 

  let rec string_of_term_rec is_deep = function
      Term(f,[]) ->
      	sgn#string_of_symbol f
    | Term(f,t::l) ->
	(
	  match sgn#symbol_fix f with
	      Default | Prefix ->
      	      	  (sgn#string_of_symbol f)^"("^
      		  (string_of_term_rec false t)^
      		  (string_of_end_term_list l)
	    | Postfix ->
      		  "("^
      		  (string_of_term_rec false t)^
      		  (string_of_end_term_list l)^
      	      	  (sgn#string_of_symbol f)
	    | Infix ->
		  (if is_deep then "(" else "")^
		  (string_of_term_rec true t)^
      		  (string_of_infix_list f l)^
		  (if is_deep then ")" else "")
	)
    | Var(x) ->
	(vars#string_of_var x)
	  
  and string_of_end_term_list = function
      [] -> 
      	")"
    | t::l ->
      	","^
      	(string_of_term_rec false t)^
      	(string_of_end_term_list l)
	  
  and string_of_infix_list f = function
      [] -> ""
    | t::l ->
	" "^
      	(sgn#string_of_symbol f)^
	" "^
      	(string_of_term_rec true t)^
      	(string_of_infix_list f l)
	  
  in
    string_of_term_rec false t
;;

let string_of_arity  = function
    Prefix -> "prefix" 
  | Infix -> "infix"
  | Postfix -> "postfix"
  | Default -> ""
;;

let string_of_pack sgn p =
  List.fold_left
    (fun acc sym -> 
       (sgn#string_of_symbol sym)^" : "^(string_of_arity (sgn#symbol_fix(sym)))
       ^" "^(string_of_int(sgn#arity(sym)))^" ; "^acc)
    ""
    p
;;

let string_of_rewrite_rule sgn vars r =
  (string_of_term sgn vars r.lhs)^" ->"^(string_of_term sgn vars r.rhs)^"; "
;;

let string_of_rewrite_rule_set sgn vars rset =
  List.fold_left
    (fun acc r -> (string_of_rewrite_rule sgn vars r)^acc) 
    ""
    rset
;;

module OrdSymList = 
struct
  type 'a t =  'a SymbolOrd.t list
  let compare = compare
end
;;
  
module PackMap = Ordered_maps.MakePoly(OrdSymList)
;;

(* SI ON VEUT CALCULER LES MODULES D'UNE LISTE DE PACKS, IL FAUT
   ENCORE LE CODER DE MANIRE  NE PASSER QU'UN PR_GRAPH NON
   REFLEXIF. PLUS EFFICACE SI KEY = SYMBOLE PLUTT QUE LISTE ? *)
(* RCURSIF TERMINAL AVEC UN TRY SOME ? *)

(* [module_from_pr sym_graph pr_list sgn vars deja_vu pr] returns a pair
   [(m,mmap)] consisting of a module [m] together with a map [mmap] from 
   packs to their corresponding module.*)

let rec module_from_pr sym_graph pr_list sgn vars deja_vu ((p,r)as pr) =
  try ((PackMap.find p deja_vu), deja_vu)
  with Not_found ->
    let pack_neigh = (clean_neighbours 
			(neighbours_of_a_pr sym_graph pr_list pr) pr)
    in let (mod_neigh,new_dv) = (****)
	List.fold_left
	  (fun (acc,dv) ((l,r) as elem) ->
	     try ((PackMap.find l dv)::acc, dv)
	     with Not_found -> 
	       let (new_mod,ndv) = (****)
		 module_from_pr sym_graph pr_list sgn vars dv elem
	       in
		 (new_mod::acc, PackMap.add l new_mod ndv)) (****)
	  ([],deja_vu)
	  pack_neigh
       and mod_sgn = User_signatures.from_string (string_of_pack sgn p)  
       and mod_rules = string_of_rewrite_rule_set sgn vars r
       in
	 ((extend mod_neigh 
	     mod_sgn 
	     vars 
	     mod_rules)
	    ,new_dv)(****)
;;

let mod_map_from_list sym_graph sgn vars pr_list =
    List.fold_left
      (fun acc elem ->
	 snd (module_from_pr 
		sym_graph pr_list sgn vars 
		acc
		elem))
      PackMap.empty
      pr_list
;;

(* let get_mod_of_sym_id id *)
  

(* TESTS ------------------------------------------------------------*)

let sig_toto = User_signatures.from_string "lo , li , ta : infix binary;";;

let module_toto = 
  extend [module_mult] sig_toto my_vars
    "  x lo  y -> y li x ; x ta s(y) -> (x lo y) * x ; x li 0 -> x ta 0";;

let g_toto = graph_from_a_list module_toto.htrs_all_rules;;

let print_edge trs graph =  
  List.iter
    (fun (x,y) -> let a = trs.htrs_sig#string_of_symbol x
		  and b = trs.htrs_sig#string_of_symbol y
		  in 
		    print_string ("[ "^a^" ---> "^b^" ]\n")
    )
    (Symgraph.edge_list graph)
    ;
  flush stdout
;;
  
let packs_toto = packs_from_a_list module_toto.htrs_all_rules
;;
(*
begin
  
  print_newline() ;
  print_edge module_toto g_toto ;
  print_newline() ;

 
  List.iter
    (fun l -> begin
       List.iter 
	 (fun x -> 
	    begin 
	      print_string ((module_toto.htrs_sig#string_of_symbol x)^" ; ")
	    end ) 
	 l ;
     print_newline()
     end)
    (packs_toto)
    ;
  flush stdout
end
;;
*)
let print_pack sgn pack =
  List.iter 
    (fun x -> 
       begin 
	 print_string ((sgn#string_of_symbol x)^" ; ")
       end ) 
    pack
;;

let pre_modules = 
  packs_and_rules_from_packs module_toto.htrs_all_rules packs_toto 
;;

let print_pack_list l =
List.iter
  (fun (a,b) -> 
     begin
       print_newline();
       print_string "=====================";  print_newline();
       print_pack module_toto.htrs_sig a;  print_newline();
       print_string "---------------------";  print_newline();
       print_rewrite_rule_set module_toto.htrs_sig my_vars b; print_newline();
       print_string "=====================";  print_newline();
       print_newline()
     end)
  l
;;
(*
List.iter
  (fun (p,pl) ->
     begin
       print_newline();
       print_string "##############################";  print_newline();
       print_pack module_toto.htrs_sig (fst p);  print_newline();
       print_pack_list pl;
       print_newline()
     end)
  (pack_graph g_toto pre_modules) 
;;  
*)
let s_p =
  let p = fst (List.hd pre_modules) in
    (*print_string*) (string_of_pack module_toto.htrs_sig p)
    (* ; print_newline()*)
;;

let sig_p = User_signatures.from_string s_p
;;

print_newline();
print_string " *** Printing minimal modules *** " ; print_newline();;
(*
for i = 1 to 1000 do
  List.fold_left
    (fun (acc,dv) elem ->
       let (a,b) = (module_from_pr 
		      g_toto pre_modules module_toto.htrs_sig my_vars 
		      dv 
		      elem)
       in
	 (a::acc,b))
    ([],PackMap.empty)
    pre_modules
done;;
*)
(*
  (List.map (module_from_pr g_toto pre_modules module_toto.htrs_sig my_vars 
PackMap.empty) pre_modules)
done;;
*)

(*
List.iter 
  (fun a-> 
     begin 
       print_rewrite_rule_set a.htrs_sig my_vars  a.htrs_new_rules ; 
       print_string "###"; 
       print_newline() 
     end)
  (*(List.map (module_from_pr g_toto pre_modules module_toto.htrs_sig my_vars 
	       (**)PackMap.empty(**)) pre_modules)*)
   (fst(List.fold_left
    (fun (acc,dv) elem ->
       let (a1,b1) = (module_from_pr 
		      g_toto pre_modules module_toto.htrs_sig my_vars 
		      dv 
		      elem)
       in
	 (a1::acc,b1))
    ([],PackMap.empty)
    pre_modules))
;;
*)
*)


