(***************************************************************************

Module defining rewrite rules.

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: rewrite_rules.mli,v 1.8 2002/07/18 14:48:34 contejea Exp $

***************************************************************************)

open Signatures
open Gen_terms

(*

the type for rewrite rules, defined by the lefthand side, the
righthand side, and optionally the lhs and the rhs of the AC-extension
and the size of the rule

*)

type 'symbol t = 
    {
      lhs : 'symbol term;
      rhs : 'symbol term;
      ext : ('symbol term * 'symbol term) option;
      is_or : bool;
      nb : int
    }
;;


exception Rule_with_a_var_lhs

(*

[(make_rule sigma h l r)] builds a rewrite rule over signature [sigma]
with lefthand side [l] and righthand side [r]. Determines whether the
rule needs to be AC-extended; in this case the hashconsing table [h] is
used to build the terms of the extension.

May raise the exception [Rule_with_a_var_lhs] when this is the case!

*)

val make_rule : 
  'symbol #signature -> 'symbol HashTerm.t -> 'symbol term -> 'symbol term 
    -> 'symbol t


(*

  [(make_non_oriented_rule n sigma h l r)] builds a rewrite
  rule as [(make_rule n sigma l r)], but the left-hand side
  [l] is not always greater than the right-hand side [r], it may
  depends on the instances. Only for unfailing completion purpose.

*)

val make_non_oriented_rule : 
  'symbol #signature -> 'symbol HashTerm.t -> 'symbol term -> 'symbol term 
    -> 'symbol t
(*

[(make_numbered_rule n sigma h l r)] builds a rewrite rule as [(make_rule sigma l r)], 
and this rule owns the number [n]. Only for completion purposes.

*)

val make_numbered_rule : 
  int -> 'symbol #signature -> 'symbol HashTerm.t -> 'symbol term -> 'symbol term 
    -> 'symbol t


(*

  [(make_non_oriented_numbered_rule n sigma h l r)] builds a rewrite
  rule as [(make_numbered_rule n sigma l r)], but the left-hand side
  [l] is not always greater than the right-hand side [r], it may
  depends on the instances. Only for unfailing completion purpose.

*)

val make_non_oriented_numbered_rule : 
  int -> 'symbol #signature -> 'symbol HashTerm.t -> 'symbol term -> 'symbol term 
    -> 'symbol t


(*

  printing functions

*)

val print_rewrite_rule : 
  'symbol #Signatures.signature -> Variables.user_variables -> 'symbol t -> unit

val print_rewrite_rule_set : 
  'symbol #Signatures.signature -> Variables.user_variables -> 'symbol t list -> unit


