(***************************************************************************

Signature lexer

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: signature_lexer.mll,v 1.4 2002/07/12 12:30:58 marche Exp $

***************************************************************************)


{

  open Signature_parser

  exception Invalid_char of string

(* 
   hash table of keywords
   any number is appropriate in place of 17, a prime number is better 
*)

  let keywords_table = let kt = ((Hashtbl.create 17):(string,token) Hashtbl.t)
  in List.iter  
       (function (s,k) -> Hashtbl.add kt s k)
       [
	 ("constant",	 KW_CONSTANT);
	 ("unary",	 KW_UNARY);
	 ("binary",	 KW_BINARY);
	 ("infix",	 KW_INFIX);
	 ("prefix",	 KW_PREFIX);
	 ("postfix",	 KW_POSTFIX);
	 ("commutative", KW_C);
	 ("AC",		 KW_AC)
       ];
    kt
      
      
  let keyword_or_ident s = 
    try (Hashtbl.find keywords_table s)
    with Not_found -> IDENT s
	
}

rule token = parse
    [' ''\t''\n'] 	{ token lexbuf }
  | ','            	{ COMMA }
  | ':'            	{ COLON }
  | ';'            	{ SEMICOLON }
  | ['0'-'9']+          { INT(Lexing.lexeme lexbuf) }
  | ['a'-'z''A'-'Z']+   { keyword_or_ident(Lexing.lexeme lexbuf) }
  | ['a'-'z''A'-'Z''0'-'9''_''\'']+ 
                        { IDENT(Lexing.lexeme lexbuf) }
  | ['^''+''.''&''*''-''/''!''?''@''~''#''|'':''%''$''<''=''>']+ 
                        { IDENT(Lexing.lexeme lexbuf) }
  | "(*"                { comment lexbuf; token lexbuf }
  | _                   { raise (Invalid_char (Lexing.lexeme lexbuf)) }
  | eof			{ EOF }

and comment = parse
  | "*)"                { () }
  | "(*"                { comment lexbuf; comment lexbuf }
  | _                   { comment lexbuf }  


