(***************************************************************************

Standard substitutions

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: substitution.ml,v 1.8 2002/07/11 13:26:22 pguillon Exp $

***************************************************************************)



open Gen_terms
open Rewrite_rules
open Format


type 'symbol substitution = (unit,'symbol term) Variables.VarMap.t;;

(* 

   [(merge_subst subst1 subst2)] merges substitutions [subst1] and
   [subst2], that is put them together verifying that if [subst1]
   binds a variable $x$ to a term $t_1$ and [subst2] binds the same
   variable to $t_2$, then $t_1=t_2$

   Raises Conflict if not.

   This is standard substitution merging: equality of $t_1$ and $t_2$
   is performed assuming that all symbols are free.

*)

exception Conflict;;


let rec merge_substs s1 s2 = 
  Variables.VarMap.fold
    (fun x t subst ->
       try
	 let t' = Variables.VarMap.find x subst
	 in
	   if t.Hcons.tag==t'.Hcons.tag
	   then subst
	   else raise Conflict
       with
	   Not_found ->
	     Variables.VarMap.add x t subst)
    s1 s2


let rec apply_term h t sigma = 
  match t.Hcons.node with 
    | Term(f,l) -> 
	make_term h f 
	  (List.map (function t -> apply_term h t sigma) l)
    | Var(x) -> 
	try 
	  Variables.VarMap.find x sigma
	with
	    Not_found -> t

let rec apply_sorted_term signature h t sigma = 
  match t.Hcons.node with 
    | Term(f,l) -> 
	if signature#is_free f
	then 
	  make_term h f 
	    (List.map (function t -> apply_term h t sigma) l)
	else 
	  make_sorted_term signature h f 
	    (List.map (function t -> apply_term h t sigma) l)
    | Var(x) -> 
	try 
	  Variables.VarMap.find x sigma
	with
	    Not_found -> t

let apply_rewrite_rule h r sigma = 
  let r_lhs_sigma = apply_term h r.lhs sigma
  and r_rhs_sigma = apply_term h r.rhs sigma in
  {
    lhs= r_lhs_sigma;
    rhs= r_rhs_sigma;
    ext= 
     begin
       match r.ext with 
	   None -> None
	 | Some(t1,t2) -> Some(apply_term h t1 sigma , apply_term h t2 sigma)
     end;
    is_or = r.is_or;
    nb = r.nb
  }


let print sigma vars s =
  Variables.VarMap.iter 
    (fun x t ->
       print_string (vars#string_of_var x);
       print_string " -> ";
       Gen_terms.print_term sigma vars t;
       print_newline();
    )
    s

let rec substitute signature h sigma t = 
  let res =
  match t.Hcons.node with
      Var id -> 
	begin
	  try 
	    Variables.VarMap.find id sigma
	  with Not_found -> t
	end
    | Term (f,l) -> 
	if signature#is_free f
	then 
	  make_term h f (List.map ( substitute signature h sigma) l)
	else 
	  make_head_flatten_term signature h f (List.rev_map ( substitute signature h sigma) l)
  in
(*i
  let _ =
    print_string "Appying the substitution";
    print_newline();
    print signature Variables.default sigma;
    print_newline();
    print_string " on the term ";
    Gen_terms.print_term signature Variables.default t;
    print_newline();
    print_string "result = ";
    Gen_terms.print_term signature Variables.default res;
    print_newline();
    print_flush() in
i*)
    res


let apply_subst_to_eqs signature h sigma list_of_equations = 
  List.map
    (function (s,t) -> 
       (substitute signature h sigma s, substitute signature h sigma t))
    list_of_equations


