(***************************************************************************

Term lexer

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: term_lexer.mll,v 1.14 2002/07/12 12:30:27 marche Exp $

***************************************************************************)


{

  open Term_parser
  open Term_syntax
  open Signatures

  exception Invalid_char of string

  let ident s =
    try
      match !parse_id s with
	| Postfix -> POSTFIX_IDENT s
	| Infix -> INFIX_IDENT s
	| Prefix -> PREFIX_IDENT s
	| Default -> PREFIX_IDENT s
    with 
	Not_found -> 
	  try
	    let x = !current_variables#var_of_string s
	    in VAR_IDENT x
	  with
	      Not_found ->
		raise (Syntax_error ("undefined identifier "^s))
	
}

rule token = parse
    [' ''\t''\n'] 	{ token lexbuf }
  | ','            	{ COMMA }
  | ';'            	{ SEMICOLON }
  | "->"            	{ ARROW }
  | '('            	{ OPENPAR }
  | ')'            	{ CLOSEPAR }
  | '<'                 { LT }
  | '>'                 { GT }
  | '='                 { EQ }
  | "<="                { LE }
  | ">="                { GE }
  | "<>"                { NE }
(*i
  | "\\/"               { OR }
  | "/\\"               { AND }
  | '~'                 { NOT }
 i*)
  | "mul"               { MUL }
  | "lr_lex"            { LR_LEX }
  | "rl_lex"            { RL_LEX }
  | ['a'-'z''A'-'Z''0'-'9']['a'-'z''A'-'Z''0'-'9''_''\'']*
                        { ident(Lexing.lexeme lexbuf) }
  | (['^''+''.''&''*''-''/''!''?''@''~''#''|'':''%''$''<''=''>']+) (('_'(['a'-'z''A'-'Z''0'-'9']+))*)
                        { ident(Lexing.lexeme lexbuf) }
  | "(*"                { comment lexbuf; token lexbuf }
  | _                   { raise (Invalid_char (Lexing.lexeme lexbuf)) }
  | eof			{ EOF }

and comment = parse
  | "*)"                { () }
  | "(*"                { comment lexbuf; comment lexbuf }
  | _                   { comment lexbuf }  


