(***************************************************************************

Terms defined on user signatures

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: user_terms.ml,v 1.11 2002/07/16 09:29:36 contejea Exp $

***************************************************************************)




let do_parse parse_function sigma s =
  try
    Term_syntax.parse_id := 
       (fun s -> 
	  let f = sigma#symbol_of_string s in
	  sigma#symbol_fix f);
    let b = Lexing.from_string s
    in
      parse_function Term_lexer.token b
  with
      Parsing.Parse_error -> 
	raise (Term_syntax.Syntax_error "")
    | Failure "lexing: empty token" -> 
	raise (Term_syntax.Syntax_error "invalid character")
    | Term_lexer.Invalid_char s ->
	raise (Term_syntax.Syntax_error ("invalid character '"^s^"'"))
;;

let do_full_parse parse_function sigma vars s =
  Term_syntax.current_variables := vars ;
  do_parse parse_function sigma s
;;

let term_of_string sigma h vars s =
  let t = do_full_parse Term_parser.term_eof sigma vars s in
  Gen_terms.make_term_from_string_term sigma h t
;;

let equation_of_string sigma h vars s = 
  let (u,v) = do_full_parse Term_parser.equation_eof sigma vars s in
    ((Gen_terms.make_term_from_string_term sigma h u),
     (Gen_terms.make_term_from_string_term sigma h v))
;;

let equation_set_of_string sigma h vars s = 
  let r = do_full_parse Term_parser.equation_set_eof sigma vars s in
  List.map 
    (fun (u,v) -> 
	 ((Gen_terms.make_term_from_string_term sigma h u),
	  (Gen_terms.make_term_from_string_term sigma h v)))
    r
;;

let rule_set_of_string sigma h vars s = 
  let r = do_full_parse Term_parser.rule_set_eof sigma vars s in
  List.map 
    (fun (u,v) -> 
       Rewrite_rules.make_rule sigma h
	 (Gen_terms.make_term_from_string_term sigma h u)
	 (Gen_terms.make_term_from_string_term sigma h v))
    r
;;

let order_constraint_of_string sigma h vars s = 
  let r = do_full_parse Term_parser.order_constraint_eof sigma vars s in
  Order_constraints.make_formula_from_string_formula sigma h r
;;

let prec_of_string sigma s = 
  let p = do_parse Term_parser.precedence_eof sigma s in
  let order = Finite_orderings.identity_ordering compare 
  in 
  List.iter 
    (fun prec ->
	let prec' = 
	  Finite_orderings.map_prec sigma#symbol_of_string prec
	in
	Finite_orderings.add_list order prec')
    p;
  order#compare
;;

exception Invalid_status of string

let status_of_string sigma s =
  let status = do_parse Term_parser.status_eof sigma s in
  let _ = 
    List.iter
      (function (f_string,f_status) ->
	 let f = sigma#symbol_of_string f_string in
	   if (f_status <> Rpo.Multiset) && (not (sigma#is_free f))
	   then raise (Invalid_status f_string))
      status in
  let status_function f =
    try
      List.assoc (sigma#string_of_symbol f) status 
    with 
	Not_found ->
	  if sigma#is_free f 
	  then Rpo.Lr_lexico 
	  else Rpo.Multiset
  in status_function

let user_hct = Gen_terms.HashTerm.create 1997
