(***************************************************************************

Calcul du couple_normalisant pour A(.) : corps

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: a.ml,v 1.3 1998/04/28 13:36:38 marche Exp $

***************************************************************************)

(*

Obsolete header:


Project:                Cime
Author(s):             Claude March\'e

File name:        a.ml
Created on:              03 nov 93
Last modified on: 03 nov 93

***************************************************************************)

open Symbols
open Terms



(*===============================

(\Theta_{A},\Psi_{A}

================================*)


(*==================

Calcul de Theta

==================*)

   
let union = 
  List.fold_left 
    (fun emsbl elt -> 
       if List.mem elt emsbl 
       then emsbl
       else elt::emsbl)
;;

let rec flat_map_union f = function
    ([] : term list) -> ([] : var_id list)
  | h::t -> union (f h) (flat_map_union f t)
;;

(* 
substitutions_a_interdire * t retourne toutes les variables x telles
que le motif x*? apparait dans t. 
Note: * etant associatif, il est necessairement binaire.
*)

let rec substitutions_a_interdire mult = function
    (VAR _) -> []
  | TERM(f,l) -> 
      if (f=mult)
      then substitutions_interdites mult l
      else flat_map_union (function x -> substitutions_a_interdire mult x) l

and substitutions_interdites mult = function
  [(VAR x);t] -> union [x] (substitutions_a_interdire mult t)
| [t1;t2] -> union (substitutions_a_interdire mult t1)
                   (substitutions_a_interdire mult t2) 
| _ -> failwith "A.substitutions_interdites"


(*
instancie_par_produit mult u v var_sup x retourne
le couple de termes (u,v) instancie par la substitution {x\-> x mult x'}
Le symbole mult etant seulement A, et non AC, il n'y a pas d'applatissement.
*)

let instancie_par_produit mult u v var_sup x = 
let rec instancie_terme_par_produit = function
    VAR k as v -> if x=k
                  then TERM(mult,[v;VAR var_sup])
                  else v
  | TERM(f,l) -> TERM(f,List.map instancie_terme_par_produit l) in

  (instancie_terme_par_produit u,instancie_terme_par_produit v)

(*
theta_a mult u v retourne theta(u,v) comme defini dans la completion 
normalisee, vis-a-vis de la theorie A(mult). On suppose que u>v.

theta_sym_a mult u v retourne  theta(u,v) comme defini dans la completion 
normalisee, vis-a-vis de la theorie A(mult), mais pour la completion ordonnee,
dans le cas ou u et v sont incomparables pour >.
*)

let theta_a mult u v = 
    let x = new_var (variables_du_terme u) in
    List.map (instancie_par_produit mult u v x) (substitutions_a_interdire mult u)

let theta_sym_a mult u v = 
    let x = new_var (union (variables_du_terme u) (variables_du_terme v)) in
    List.map (instancie_par_produit mult u v x) 
        (union (substitutions_a_interdire mult u)
               (substitutions_a_interdire mult v))


(*================

Calcul de Psi

================*)

let rec ajoute_variable_a_droite mult x = function
  (TERM(f,l)) as u ->  
     if f=mult 
     then match l with
                [t1;t2] -> TERM(mult,[t1;(ajoute_variable_a_droite mult x t2)])
               | _       -> failwith "A.ajoute_variable_a_droite"
     else TERM(mult,[u;VAR x])
| u -> TERM(mult,[u;VAR x])

let psi_a mult u v = 
    if (top_symbol_id u) = mult
    then let x= new_var (variables_du_terme u) in
         [(u,v); ((ajoute_variable_a_droite mult x u),
                  (ajoute_variable_a_droite mult x v))]
    else []

let psi_sym_a mult u v = 
    if ((top_symbol_id u) = mult) or ((top_symbol_id v) = mult)
    then let x= new_var (union (variables_du_terme u)
                               (variables_du_terme v)) in
         [(u,v); ((ajoute_variable_a_droite mult x u),
                  (ajoute_variable_a_droite mult x v))]
    else []



(*================

  Couple normalisant

================*)

let couple_normalisant_a mult u v = 
    (theta_a mult u v, psi_a mult u v)

let couple_normalisant_sym_a mult u v = 
    (theta_sym_a mult u v, psi_sym_a mult u v)



