(***************************************************************************

Computation of the normalizing pair for ACU(+,0) : body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: acu.ml,v 1.3 2002/05/27 12:07:51 contejea Exp $

***************************************************************************)

open Symbols
open Terms



(*===============================

(\Theta_{ACU},\Psi_{ACU}

================================*)

let rec union l1 l2 =
  match l1 with 
      [] -> l2
    | h::t -> 
	if List.mem h l2 
	then union t l2
	else h :: (union t l2)
(* 
substitutions_a_interdire plus t retourne toutes les variables x telles
que le motif x plus ? apparait dans t, modulo AC(plus). 
*)

let rec flat_map_union f = function
  ([] : term list) -> ([] : var_id list)
| h::t -> union (f h) (flat_map_union f t)

(* 
substitutions_a_interdire + t retourne toutes les variables x telles
que le motif x+? apparait dans t, modulo AC(+). 
*)


let rec substitutions_a_interdire plus = function
    (Var _) -> []
  | Term (f,l) -> 
     if (f=plus)
     then substitutions_interdites plus l
     else flat_map_union (substitutions_a_interdire plus) l

and substitutions_interdites plus = function
    [] -> []
  | (Var x)::l -> union [x] (substitutions_interdites plus l)
  | t::l -> union (substitutions_a_interdire plus t) 
                (substitutions_interdites plus l)

(*
instancie_par_zero zero u v x retourne
le couple de termes (u,v) instancie par la substitution {x\-> zero}
*)

let instancie_par_zero zero u v x = 
  let rec instancie_terme_par_zero = function
      Var k as v -> 
	if x=k
        then zero
        else v
    | Term(f,l) -> Term(f,List.map instancie_terme_par_zero l) in
    
    (instancie_terme_par_zero u,instancie_terme_par_zero v)

let theta_acu plus zero u v =
  List.map (instancie_par_zero zero u v) (substitutions_a_interdire plus u)
    
let theta_sym_acu plus zero u v =
  List.map (instancie_par_zero zero u v) 
    (union (substitutions_a_interdire plus u) (substitutions_a_interdire plus v))
    
let couple_normalisant_acu plus zero u v = 
  (theta_acu plus zero u v,[])

let couple_normalisant_sym_acu plus zero u v = 
     (theta_sym_acu plus zero u v,[])







