(***************************************************************************

  Symetrization of an equation : body

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: symmetrization.ml,v 1.8 2001/04/20 13:42:32 marche Exp $

***************************************************************************)

(*

Obsolete header:

  CiME Project - Dmons research team - LRI - Universit Paris XI

  $Id: symmetrization.ml,v 1.8 2001/04/20 13:42:32 marche Exp $

***************************************************************************)

open Terms;;
open Equations;;
open Equations_as_obj;;
open Orderings;;
open Norm_theory;;
open Normalised_rewriting;;
open Messages;;
open Symbols;;
open Format;;


module type SYMMETRIZATION = 
sig
  type equation
  exception Non_orientable;;
  val symmetrize : term ordering -> equation -> equation;;
  val weak_symmetrize : term ordering -> equation -> equation;;
end
;;

module Symmetrization =
  functor (Eqt : EQUATION) ->
struct
  
  type equation = Eqt.t
		    
exception Non_orientable;;

open Eqt;;

(*

separer_coefficient_dominant 

exemple: 
  
  separer_coefficient_dominant > + 0 - [a;b;a;(-a);a;b]

renvoie (2,a,(-b)+(-b))     si a > b

leve l'exception Non_orientable si la liste de termes
 n'admet pas d'element maximal.

Note : la fonction prend des symboles n,m,k tels que 
       + est AC, 0 et - sont libres
*)

(*
val extraire_plus_grand_terme :
      term ordering -> int * term list -> int * terme * (int * terme) list;;
*)

let extraire_plus_grand_terme order l = 
  let rec termes_maximaux = function
      [] -> [],[]
    | x::l -> 
        let rec ajoute_maxi (i,s as i_s) = function
            [], l -> [i_s],l
          | ((j,t as j_t)::l as j_t_l), m -> 
	      match order s t with
                  Greater -> 
		    let max,min = ajoute_maxi i_s (l,m) in
                      max, j_t::min
                | Less_than -> j_t_l,(i_s::m)
		| _  -> 
		    let max,min = (ajoute_maxi i_s (l,m)) in
                      (j_t::max),min 
	in  
	  ajoute_maxi x (termes_maximaux l) in 
    match 
      termes_maximaux l
    with      
	[],_ -> failwith "Cas impossible dans extraire_plus_grand_terme"
      | [i,s],l -> (i,s,l)
      | _ -> raise Non_orientable
;;


let rec fabriquer_la_combinaison_des_sous_termes minus = function
    [] -> []
  | s::l -> 
      if (not (is_a_variable s)) & (top_symbol_id s) = minus
      then 
	begin
	  match s with 
              TERM (_,[t]) ->
		let rec retire_t_de_la_liste = function 
		    [] -> [(-1,t)]
		  | (i,s)::l -> 
		      if egaux_ac s t
                      then 
			if i=1 
                        then l
                        else (i-1,t)::l
                      else (i,s)::(retire_t_de_la_liste l) in 
		  retire_t_de_la_liste  
		    (fabriquer_la_combinaison_des_sous_termes minus l)
	    | _ -> failwith "bug dans fabriquer_la_combinaison_des_sous_termes"
	end
      else 
      	let rec ajoute_s_dans_la_liste = function 
            [] -> [(1,s)]
	  | (i,t)::l -> 
	      if egaux_ac s t
              then 
		if i= -1 
              	then l
              	else (i+1,s)::l
              else (i,t)::(ajoute_s_dans_la_liste l) in 
	  ajoute_s_dans_la_liste (fabriquer_la_combinaison_des_sous_termes minus l)
;;


let changer_signe_si_necessaire (i,s,l) =
  if i>0
  then (i,s,(List.map (function (n,t) -> (-n,t)) l))
  else (-i,s,l)
;;

let separer_coefficient_dominant order plus zero opp liste =

  let fabrique_membre_droit = function
      [] -> TERM(zero,[])
    | [i,s] -> 
	if i>0 
        then 
	  if i = 1 
          then s 
          else TERM (plus,duplique i s)
        else 
	  if i = -1 
          then TERM (opp,[s]) 
          else TERM (plus,duplique (-i) (TERM(opp,[s])))
    | l -> 
        let rec fabrique_liste_membre_droit = function
            [] -> []
          | (i,s)::l -> 
	      let i_fois_s =
	      	if i>0 
              	then duplique i s
              	else duplique (-i) (TERM(opp,[s])) in
		i_fois_s @ (fabrique_liste_membre_droit l) 
	in
          TERM(plus,fabrique_liste_membre_droit l)  

  in

  let (i,s,l) = changer_signe_si_necessaire
                  (extraire_plus_grand_terme order 
                     (fabriquer_la_combinaison_des_sous_termes opp liste))


  in (i,s,(fabrique_membre_droit l))
;;


let make_symmetrization order plus zero moins u v =
  let w = normalisation_par_la_theorie_courante 
      	    (head_flatten_term (TERM(plus,[u;TERM(moins,[v])])))
  in

  let left,right =
    if (top_symbol_id w)=plus
    then
      let (i,dom,res) = (separer_coefficient_dominant order plus zero moins 
                           (liste_des_sous_termes w)) 
      in
      	if i=1 
        then dom,res
        else TERM(plus,duplique i dom),res
    else
      let ww = if (not (is_a_variable w)) & (top_symbol_id w) = moins
      then List.hd (liste_des_sous_termes w)
      else w in
      	ww,TERM(zero,[])
	  
  in 
  let eq = make_equation left right in
    set_norm_wrt_rule 0 eq;
  set_orientation true eq;
    eq
;;

let extraire_plus_grand_terme_bool order l = 
  match 
    let rec termes_maximaux = 
      function
	  [] -> ([],[])
	| x::l -> 
	    let rec ajoute_maxi s = function
		[],l -> ([s],l)
              | ((t::l),m) -> 
		  match order s t with
                      Greater -> 
			let (max,min)=(ajoute_maxi s (l,m)) in
                          (max,(t::min))
                    | Less_than -> ((t::l),(s::m))
		    | _ -> 
			let (max,min)=(ajoute_maxi s (l,m)) in
                          (t::max),min 
	    in ajoute_maxi x (termes_maximaux l) 
    in termes_maximaux l
  with      
      ([],_) -> failwith "Cas impossible dans extraire_plus_grand_terme"
    | ([s],l) -> (s,l)
    | _ -> raise Non_orientable;;


let rec fabriquer_la_combinaison_des_sous_termes_bool = function
    [] -> []
  | s::l -> 
      let rec ajoute_s_dans_la_liste = function 
          [] -> [s]
      	| t::l -> 
	    if egaux_ac s t
            then l
            else t::(ajoute_s_dans_la_liste l) 
      in
    	ajoute_s_dans_la_liste 
          (fabriquer_la_combinaison_des_sous_termes_bool l)
;;


let separer_coefficient_dominant_bool order plus zero liste =

  let fabrique_membre_droit = function
      [] -> TERM(zero,[])
    | [s] -> s
    | l -> TERM(plus,l)

  in

  extraire_plus_grand_terme_bool order 
                     (fabriquer_la_combinaison_des_sous_termes_bool liste)
;;


let make_bool_symmetrization order plus zero u v =
  let w = normalisation_par_la_theorie_courante 
      	    (head_flatten_term (TERM(plus,[u;v])))
  in

  let left,right =
    if (top_symbol_id w)=plus
    then 
      let dom,res = (separer_coefficient_dominant_bool order plus zero 
                       (liste_des_sous_termes w)) 
      in
        match res with
            t::[] -> dom,t
          | _ -> dom, TERM (plus,res) 
    else w,TERM(zero,[])

  in 
  let eq = make_equation left right in
    set_norm_wrt_rule 0 eq;
    set_orientation true eq;
    eq
;;


(*======

fonction auxiliaire: puissance modulo p

======*)

let rec puissance_modulo p x = function
    0 -> 1
  | 1 -> x
  | y -> let z = x*(puissance_modulo p x (y-1)) in z mod p;;


let make_field_symmetrization order caracteristique plus zero moins mult u v =
  let w = normalisation_par_la_theorie_courante 
      	    (head_flatten_term (TERM(plus,[u;TERM(moins,[v])])))
  in
    
  let left,right =
    if (top_symbol_id w)=plus
    then
      let (i,dom,res) = (separer_coefficient_dominant order plus zero moins 
                           (liste_des_sous_termes w)) 
      in
      	if i=1 
        then dom,res
        else dom,(normalisation_par_la_theorie_courante
		    (TERM(plus,
			  duplique 
			    (puissance_modulo
			       caracteristique 
			       i
			       (caracteristique-2)) 
			    res)))

    else
      let ww = 
	if (not (is_a_variable w)) & (top_symbol_id w) = moins
      	then List.hd (liste_des_sous_termes w)
        else w 
      in
      	ww,TERM(zero,[])

  in 
  let eq = make_equation left right in
    set_norm_wrt_rule 0 eq;
    set_orientation true eq;
    eq
;;


let make_rat_symmetrization 
  order un_z coef irred plus zero moins mult un u v = 
  
  let rec cherche_coef = function
      VAR(x) as t -> TERM(un_z,[]),TERM(un_z,[]),t
    | TERM(f,l) as t -> 
      	if f=mult
        then cherche_coef_list [] l 
      	else 
	  if f=coef
          then 
	    match l with
          	[TERM(_,[num;den])] -> num,den,TERM(un,[])
              | _ -> failwith "this case should never occur in cherche_coef"
	  else TERM(un_z,[]),TERM(un_z,[]),t

  and cherche_coef_list l1 = function
      [] -> TERM(un_z,[]),TERM(un_z,[]),TERM(mult,l1)
    | t::l2 -> 
      	if (is_a_variable t) or (top_symbol_id t) <> coef
      	then cherche_coef_list (t::l1) l2
        else 
          match liste_des_sous_termes t with
              [TERM(_,[num;den])] -> num,den,TERM(mult,l1@l2)
            | _ -> failwith "this case should never occur in cherche_coef"

  
  in 
  let w = normalisation_par_la_theorie_courante 
            (head_flatten_term (TERM(plus,[u;TERM(moins,[v])])))
  in

  let left,right =
    if (top_symbol_id w)=plus
    then
      let (i,dom,res) = (separer_coefficient_dominant order plus zero moins 
                           (liste_des_sous_termes w)) 
      in
      	if i=1 
        then 
	  let num,den,tete = cherche_coef dom
	  in 
          let queue = normalisation_par_la_theorie_courante
			(TERM(mult,[TERM(coef,[TERM(irred,[den;num])]);res]))
          in 
       	    begin
(*
             print_string "w : ";
             print_term w;
	     print_newline();
             print_string "dom : ";
             print_term dom;
	     print_newline();
             print_string "res : ";
             print_term res;
	     print_newline();
             print_string "tete : ";
             print_term tete;
	     print_newline();
             print_string "queue : ";
             print_term queue;
	     print_newline();
	     print_flush();
*)
              tete,queue
            end

        else failwith "this case should never occur in make_rat_symmetrization"
    else
      let num,den,reste = cherche_coef w
      in 
	reste,TERM(zero,[])

  in 
  let eq = make_equation left right in
    set_norm_wrt_rule 0 eq;
    set_orientation true eq;
    eq
;;


let symmetrize order eq =
  let u = first_term eq
  and v = second_term eq in
  let left,right = 
    match order u v with
      	Greater -> u,v
      | Less_than -> v,u
      | _ -> raise Non_orientable in

    try
      let sym_eq = 
      	match (norm_theory_of (top_symbol_id left)) with
  
	    AG(plus,zero,moins) -> 
	      make_symmetrization order plus zero moins left right
	  | CR(plus,zero,moins,_,_) -> 
	      make_symmetrization order plus zero moins left right
	  | BR(plus,zero,_,_) -> 
	      make_bool_symmetrization order plus zero left right
	  | FP(p,plus,zero,moins,mult,_) -> 
	      make_field_symmetrization 
	      	order p plus zero moins mult left right
	  | Int(plus_z,coef,plus,zero,moins,mult,un) ->
	      make_symmetrization order plus zero moins left right  
	  | Rat(un_z,coef,irred,plus,zero,moins,mult,un) ->
	      make_rat_symmetrization order 
	      	un_z coef irred plus zero moins mult un left right  
	  | _ -> make_equation left right
      in
	set_orientation true sym_eq;
	set_age (get_age eq) sym_eq;
	sym_eq

    with Invalid_argument _ -> 
      message "Something's wrong: variable at left-hand side";
      raise Non_orientable
;;



let make_weak_symmetrization order plus zero moins u v =
  let w = normalisation_par_la_theorie_courante 
      	    (head_flatten_term (TERM(plus,[u;TERM(moins,[v])]))) in
    
  let eq = make_equation w (TERM(zero,[])) in 
    set_norm_wrt_rule 0 eq;
    eq
;;


let make_weak_bool_symmetrization order plus zero u v =
  let w = normalisation_par_la_theorie_courante 
      	    (head_flatten_term (TERM(plus,[u;v]))) in
    
  let eq = make_equation w (TERM(zero,[])) in
    set_norm_wrt_rule 0 eq;
    eq
;;


let make_weak_field_symmetrization 
  order caracteristique plus zero moins mult u v =

  let w = normalisation_par_la_theorie_courante 
      	    (head_flatten_term (TERM(plus,[u;TERM(moins,[v])]))) in
    
  let eq = make_equation w (TERM(zero,[])) in
    set_norm_wrt_rule 0 eq;
    eq
;;



let make_weak_rat_symmetrization 
  order un_z coef irred plus zero moins mult un u v = 

  let w = normalisation_par_la_theorie_courante 
            (head_flatten_term (TERM(plus,[u;TERM(moins,[v])]))) in
    
  let eq = make_equation w (TERM(zero,[])) in
    set_norm_wrt_rule 0 eq;
    eq
;;



let weak_symmetrize order eq =
  let u = first_term eq
  and v = second_term eq in
  let left,right = 
    match order u v with
      	Greater -> u,v
      | Less_than -> v,u
      | _ -> u,v in
    
    try
      let sym_eq = 
      	match (norm_theory_of (top_symbol_id left)) with
	    AG(plus,zero,moins) -> 
	      make_weak_symmetrization order plus zero moins left right
	  | CR(plus,zero,moins,_,_) -> 
	      make_weak_symmetrization order plus zero moins left right
	  | BR(plus,zero,_,_) -> 
	      make_weak_bool_symmetrization order plus zero left right
	  | FP(p,plus,zero,moins,mult,_) -> 
	      make_weak_field_symmetrization order 
	      	p plus zero moins mult left right
	  | Int(plus_z,coef,plus,zero,moins,mult,un) ->
	      make_weak_symmetrization order plus zero moins left right  
	  | Rat(un_z,coef,irred,plus,zero,moins,mult,un) ->
	      make_weak_rat_symmetrization order 
	      	un_z coef irred plus zero moins mult un left right  
	  | _ -> (* weak_symmetrize ne doit etre appele *)
               (* qu'apres l'echec de symmetrize      *)
	      make_equation left right
      in
	set_age (get_age eq) sym_eq;
	sym_eq

    with Invalid_argument _ -> 
      message "Something's wrong: variable at left-hand side";
      let e = make_equation left right in
	set_age (get_age eq) e;
	e
;;


    end;;

module Standard_symmetrization = Symmetrization (Equation1);;
module G_symmetrization = Symmetrization (Equation_with_goal);;


let make_end_symmetrization plus left =
  if (top_symbol_id left)=plus
  then 
    let l = (liste_des_sous_termes left)
    in 
      (List.length l),(List.hd l)
  else failwith "This case should never happen, in make_end_symmetrization"
;;


let finish_symmetrization plus left =

  if (top_symbol_id left) = plus
  then
    
    match (norm_theory_of plus) with
  	
  	AG(plus,_,_)       -> make_end_symmetrization plus left
      | CR(plus,_,_,_,_)   -> make_end_symmetrization plus left
      | BR(plus,_,_,_)     -> make_end_symmetrization plus left
      | FP(_,plus,_,_,_,_) -> make_end_symmetrization plus left
      | _ -> 
	  failwith "This case should never happen, in  finish_symmetrization"

  else 1,left
;;
