(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: abstract_syntax.ml,v 1.8 2002/04/12 15:33:58 marche Exp $

***************************************************************************)




type abstract_expr =
    Integer of Numbers.t
  | String of string
  | Bool of bool
  | Var of string
  | Fun of string * abstract_expr
  | Apply of abstract_expr * abstract_expr
  | If of abstract_expr * abstract_expr * abstract_expr
  | Tuple of abstract_expr list
  | Set of abstract_expr list
;;

type abstract_command =
    Def of string * abstract_expr
  | Deffun of string * string list * abstract_expr
  | Eval of abstract_expr
  | Directive of string * string
;;

open Format;;

let rec print_expr = function

    Integer(n) -> 
      print_string "Integer(";
      print_string (Numbers.to_string n);
      print_string ")"

  | String(s) -> 
      print_string "String(";
      print_string s;
      print_string ")"

  | Bool(b) -> 
      print_string "Bool(";
      print_string (if b then "true" else "false");
      print_string ")"

  | Var(s) ->
      print_string "Var(";
      print_string s;
      print_string ")"

  | Fun(x,e) -> 
      print_string "Fun(";
      print_string x;
      print_string ",";
      print_expr e;
      print_string ")"

  | Apply(e1,e2) ->
      print_string "Apply(";
      print_expr e1;
      print_string ",";
      print_expr e2;
      print_string ")"

  | If(cond,e1,e2) ->
      print_string "If(";
      print_expr cond;
      print_string ",";
      print_expr e1;
      print_string ",";
      print_expr e2;
      print_string ")"

  | Tuple(l) -> 
      begin
	match l with
	  | [] -> assert false
	  | v::l ->
	      print_string "(";
	      print_expr v;
	      List.iter (fun v -> print_string ", "; print_expr v) l;
	      print_string ")"
      end
  | Set(l) ->
      begin
	match l with
	  | [] -> print_string "{}"
	  | v::l ->
	      print_string "{";
	      print_expr v;
	      List.iter (fun v -> print_string "; "; print_expr v) l;
	      print_string "}"
      end


;;

let print_named_expr name e = 
  print_string name;
  print_string " : ";
  print_expr e
;;


