(***************************************************************************

Lexer for toplevel input

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: toplevel_lexer.mll,v 1.14 2002/04/12 15:33:58 marche Exp $

***************************************************************************)


{

  open Toplevel_parser;;

  exception Lexical_error of string;;


}

rule token = parse
    [' ''\t''\n']  	{ token lexbuf }
  | "let"               { LET }   
  | "fun"               { FUN }   
  | "if"                { IF }
  | "then"              { THEN }
  | "else"              { ELSE }
  | "and"               { AND }
  | "or"               { OR }
  | "not"               { NOT }
  | "true"              { TRUE }
  | "false"             { FALSE }
  | "#"                 { DIRECTIVE }
  | ['A'-'Z''a'-'z']['A'-'Z''a'-'z''0'-'9''_''\'']* 
                        { IDENT(Lexing.lexeme lexbuf) }
  | ';'                 { SEMICOLON }
  | ','                 { COMMA }
  | '+'                 { PLUS }
  | '-'                 { MINUS }
  | '*'                 { STAR }
  | '^'                 { CONCAT }
  | "="                 { EQUAL }
  | "->"                { ARROW }
  | ">"                 { GT }
  | ">="                { GE }
  | "<"                 { LT }
  | "<="                { LE }
  | "<>"                { NEQ }
  | ['0'-'9']+          { INTEGER (Numbers.from_string (Lexing.lexeme lexbuf)) }
  | '('            	{ LEFTPAR }
  | ')'            	{ RIGHTPAR }
  | '{'            	{ LEFTBRACE }
  | '}'            	{ RIGHTBRACE }
  | '"'                 { STRING(string lexbuf) }
  | "(*"                { comment lexbuf }
  | _                   { raise (Lexical_error (Lexing.lexeme lexbuf)) }
  | eof			{ EOF }

and string = parse
    '"'                 { "" }
  | '\\' _              { let s=Lexing.lexeme lexbuf in s^(string lexbuf) }
  | [^'\\''"']+         { let s=Lexing.lexeme lexbuf in s^(string lexbuf) }
  | eof                 { raise (Lexical_error "string not terminated") }

and comment = parse
    "*)"                { token lexbuf }
  | [^'*']+             { comment lexbuf }
  | _                   { comment lexbuf }
  | eof                 { raise (Lexical_error "comment not terminated") }

