(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: typing.mli,v 1.23 2002/07/16 09:29:36 contejea Exp $

***************************************************************************)


open Abstract_syntax;;

(* 

   exception raised in case of typing error, with an explicit message
   as argument.

*)

exception Type_error of string;;

(*

  the type system in the toplevel

  A type is either

  \begin{itemize}

  \item a base type : int, bool, string, etc ;

  \item a dependent type : $(\Sigma,X)$ term, etc ;

  \item a type variable (without explicit name ) ;

  \item an \emph{arrow type} $t_1 \rightarrow t_2$ ; 

  \item a \emph{dependent arrow type} $(x:t_1)t_2$ ;

  \end{itemize}


  Base types are particular cases of dependent types where the list of
  dependent vars is empty.

  An arrow type $t_1 \rightarrow t_2$ is a particular case of the
  dependent arrow type $(x:t_1)t_2$ when $x$ does not occur in $t_2$,
  which is made explicit by putting $x$ as ``anonymous''.

*)

type depend = Depend_var | Anonymous | Depend_id of string ;;

type depend_name = depend ref;;

type types =
  | Base_type of depend_name list * types list * string 
  | Dep_arrow of depend_name * types * (depend_name -> types)
  | Type_variable of types option ref

and type_scheme =
    Generalized of (depend_name -> type_scheme)
  | Non_generalized of types

and typing_env_type = (string * type_scheme) list
;;

val generalize : types -> type_scheme;;

val print_type : types -> unit;;

(*i
val print_type_scheme : type_scheme -> unit;;
i*)

val unify_types : types -> types -> bool;;

val type_expr : typing_env_type -> typing_env_type -> abstract_expr -> types;;

val unit_type : types;;
val int_type : types;;
val string_type : types;;
val bool_type : types;;
val set_type : types -> types;;
val pair_type : types -> types -> types;;



val dioph_constraint_type : types;;
val signature_type : types;;
val word_signature_type : types;;
val theo_type : depend_name -> types;;
val variable_set_type : types;;
val word_type : depend_name -> types
val word_precedence_type : depend_name -> types
val precedence_type : depend_name -> types
val status_type : depend_name -> types
val word_ordering_type : depend_name -> types
val term_ordering_type : depend_name -> types
val srs_type : depend_name -> types
val position_type : types
val term_type : depend_name -> depend_name -> types
val conjecture_type : depend_name -> depend_name -> types
val equations_type : depend_name -> depend_name -> types
val trs_type : depend_name -> depend_name -> types
val htrs_type : types
val termination_constraint_type : depend_name -> depend_name -> types
val order_constraint_type : depend_name -> depend_name -> types

val parameter_set_type : types
val pword_signature_type : types
val pword_type : depend_name -> types
val psrs_type : depend_name -> types
val psubst_type : types

val arrow_type : types -> types -> types;;

val bin_arith_type : type_scheme;;
val bin_comp_type : type_scheme;;
val bin_bool_type : type_scheme;;

