(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: values.ml,v 1.34 2002/07/16 09:29:36 contejea Exp $

***************************************************************************)


open Format;;
open Abstract_syntax;;

type named_sig =
{
  mutable sig_name : string ;
  sig_val : User_signatures.user_signature
}

type named_hsig =
{
  mutable hsig_name : string ;
  hsig_val : Hierarchical_signatures.hierarchical_signature
}

type named_word_sig =
{
  mutable word_sig_name : string ;
  word_sig_val : String_signatures.user_word_signature
}


type named_parameters =
{
  mutable parameters_name : string ;
  parameters_val : Parameterized_signatures.parameter_c ;
}

type named_pword_sig =
{
  mutable pword_sig_name : string ;
  pword_sig_val : User_parameterized_signatures.user_parameterized_signature
}

type named_psubst =
{
  mutable psubst_name : string ;
  psubst_val : string -> Numbers.t
}

type named_vars =
{
  mutable vars_name : string ;
  vars_val : Variables.user_variables;
}

type values =
    Unit_value
  | Int_value of Numbers.t
  | Bool_value of bool
  | String_value of string
  | Fun_value of string * env_type * abstract_expr
  | Predef of string * values list

  | Set_value of values list
  | Prod_value of values list

  | Sig_value of named_sig
  | Word_sig_value of named_word_sig

  | Parameters_value of named_parameters
  | Pword_sig_value of named_pword_sig
  | Pword_value of 
      named_pword_sig * Parameterized_words.word
  | Psrs_value of 
      named_pword_sig * Parameterized_rewriting.srs

  | Psubst_value of named_psubst

  | Theo_value of named_sig * User_signatures.symbol_id Theory.TheorySet.t
  | Var_value of named_vars
  | Word_value of 
      named_word_sig * String_signatures.symbol_id Words.word
  | Word_precedence_value of 
      named_word_sig 
      * String_signatures.symbol_id Orderings_generalities.ordering
  | Precedence_value of 
      named_sig 
      * User_signatures.symbol_id Orderings_generalities.ordering
  | Status_value of 
      named_sig 
      * User_signatures.symbol_id Rpo.status_function
  | Word_ordering_value of 
      named_word_sig 
      * String_signatures.symbol_id Words.word Orderings_generalities.ordering
  | Term_ordering_value of 
      named_sig 
      * User_signatures.symbol_id Gen_terms.term Orderings_generalities.ordering
  | SRS_value of 
      named_word_sig * 
      String_signatures.symbol_id String_rewriting.srs * 
      (String_signatures.symbol_id String_rewriting.compiled_srs Lazy.t)
  | Position_value of Positions.position
  | Term_value of 
      named_sig *named_vars *
      User_signatures.symbol_id Gen_terms.term
  | Conjecture_value of
      named_sig * named_vars *
      (User_signatures.symbol_id Gen_terms.term * User_signatures.symbol_id Gen_terms.term) 
  | Equations_value of
      named_sig * named_vars *
      (User_signatures.symbol_id Gen_terms.term * User_signatures.symbol_id Gen_terms.term) list
  | TRS_value of 
      named_sig * named_vars *
      User_signatures.symbol_id Rewrite_rules.t list *
      User_signatures.symbol_id Naive_dnet.dnet 
  | HTRS_value of Hierarchical_trs.htrs
  | Dioph_constraint of Abstract_constraint.formula
  | Linear_constraint of Linear_constraints.formula
  | Termination_constraint of 
      named_sig * named_vars *
      User_signatures.symbol_id Termination_constraints.conj_termination_constraints
  | Order_constraint of 
      named_sig * named_vars *
      User_signatures.symbol_id Order_constraints.formula
(*
  | Automaton_value of
      User_signatures.user_signature *
      Variables.user_variables *
      Automaton.t
*)

and env_data =
    {
      (* typ : types *)
      mutable value : values
    }

and env_type = (string * env_data) list
;;


let rec print_env = function
    [] -> print_string "]"
  | (x,d)::l ->
      print_string x;
      print_string ":";
      print_value d.value;
      print_string ";";
      print_env l

and print_value = function
    Unit_value ->
      print_string "()"

  | Int_value(n) -> 
      print_string (Numbers.to_string n)

  | Bool_value(b) -> 
      print_string (if b then "true" else "false")

  | String_value(s) -> 
      print_string "\"";
      print_string s;
      print_string "\""

  | Fun_value(x,env,e) -> 
      print_string "<fun>"

  | Set_value(l) ->
      begin
	match l with
	  | [] -> print_string "{}"
	  | v::l ->
	      print_string "{";
	      print_value v;
	      List.iter (fun v -> print_string "; "; print_value v) l;
	      print_string "}"
      end

  | Prod_value(l) ->
      begin
	match l with
	  | [] -> assert false
	  | v::l ->
	      print_string "(";
	      print_value v;
	      List.iter (fun v -> print_string ", "; print_value v) l;
	      print_string ")"
      end

  | Predef(x,l) ->
      print_string "<predef>"

  | Sig_value(_) ->
      print_string "<signature>"

  | Word_sig_value(_) ->
      print_string "<word_signature>"

  | Parameters_value v ->
      v.parameters_val#print ()

  | Pword_sig_value(_) ->
      print_string "<pword_signature>"

  | Pword_value(sigma,t) ->
      User_parameterized_words.pretty_print t
	
  | Psrs_value(sigma,t) ->
      Parameterized_rewriting.pretty_print t

  | Psubst_value(_) -> 
      print_string "<psubst>"
    
  | Theo_value(sigma,t) ->
      Theory.print_theory sigma.sig_val t

  | Var_value(vars) ->
      print_string "<variable set>";
      print_newline();
(*
      vars.vars_val#print_all ()
*)

  | Word_value(sigma,t) ->
      Words.pretty_print sigma.word_sig_val t

  | Word_precedence_value(sigma,t) ->
      print_string "<word precedence>"

  | Precedence_value(sigma,t) ->
      print_string "<precedence>"

  | Status_value(sigma,s) ->
      print_string "<status>"

  | Word_ordering_value(sigma,t) ->
      print_string "<word ordering>"

  | Term_ordering_value(sigma,t) ->
      print_string "<term ordering>"

  | SRS_value(sigma,srs,compiled_srs) ->
      String_rewriting.pretty_print sigma.word_sig_val srs

  | Position_value(p) ->
      let string_of_p = Positions.string_of_position p in
	print_string "<position> : ";
	print_string string_of_p

  | Term_value(sigma,vars,t) ->
      Gen_terms.print_term sigma.sig_val vars.vars_val t

  | Conjecture_value (sigma,vars,l) ->
      Gen_terms.print_equation sigma.sig_val vars.vars_val l

  | Equations_value (sigma,vars,l) ->
      Gen_terms.print_equation_list sigma.sig_val vars.vars_val l

  | TRS_value(sigma,vars,t,d) ->
      Rewrite_rules.print_rewrite_rule_set sigma.sig_val vars.vars_val t

  | HTRS_value(h) ->
      print_string "<htrs>"

  | Dioph_constraint(c) ->
      Abstract_constraint.print_formula c

  | Linear_constraint(c) ->
      Linear_constraints.print c

  | Termination_constraint(sigma,vars,c) ->
      Termination_constraints.print_conj_termination_constraints
	sigma.sig_val vars.vars_val c

  | Order_constraint(sigma,vars,c) ->
      Order_constraints.print_formula
	sigma.sig_val vars.vars_val c
;;

exception Eval_error of string;;



