(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: values.mli,v 1.31 2002/07/16 09:29:36 contejea Exp $

***************************************************************************)


open Abstract_syntax;;

type named_sig =
{
  mutable sig_name : string ;
  sig_val : User_signatures.user_signature;
}

type named_hsig =
{
  mutable hsig_name : string ;
  hsig_val : Hierarchical_signatures.hierarchical_signature
}

type named_word_sig =
{
  mutable word_sig_name : string ;
  word_sig_val : String_signatures.user_word_signature
}

type named_parameters =
{
  mutable parameters_name : string ;
  parameters_val : Parameterized_signatures.parameter_c ;
}
    
type named_pword_sig =
{
  mutable pword_sig_name : string ;
  pword_sig_val : User_parameterized_signatures.user_parameterized_signature
}

type named_psubst =
{
  mutable psubst_name : string ;
  psubst_val : string -> Numbers.t
}

type named_vars =
{
  mutable vars_name : string ;
  vars_val : Variables.user_variables;
}

type values =
  | Unit_value
  | Int_value of Numbers.t
  | Bool_value of bool
  | String_value of string
  | Fun_value of string * env_type * abstract_expr
  | Predef of string * values list

  | Set_value of values list
  | Prod_value of values list

  | Sig_value of named_sig
  | Word_sig_value of named_word_sig

  | Parameters_value of named_parameters
  | Pword_sig_value of named_pword_sig
  | Pword_value of 
      named_pword_sig * Parameterized_words.word
  | Psrs_value of 
      named_pword_sig * Parameterized_rewriting.srs
	
  | Psubst_value of named_psubst

  | Theo_value of named_sig * User_signatures.symbol_id Theory.TheorySet.t
  | Var_value of named_vars
  | Word_value of 
      named_word_sig * String_signatures.symbol_id Words.word
  | Word_precedence_value of 
      named_word_sig 
      * String_signatures.symbol_id Orderings_generalities.ordering
  | Precedence_value of 
      named_sig 
      * User_signatures.symbol_id Orderings_generalities.ordering
  | Status_value of 
      named_sig 
      * User_signatures.symbol_id Rpo.status_function
  | Word_ordering_value of 
      named_word_sig 
      * String_signatures.symbol_id Words.word Orderings_generalities.ordering
  | Term_ordering_value of 
      named_sig 
      * User_signatures.symbol_id Gen_terms.term Orderings_generalities.ordering
  | SRS_value of 
      named_word_sig * 
      String_signatures.symbol_id String_rewriting.srs * 
      (String_signatures.symbol_id String_rewriting.compiled_srs) Lazy.t
  | Position_value of Positions.position
  | Term_value of 
      named_sig *named_vars *
      User_signatures.symbol_id Gen_terms.term
  | Conjecture_value of
      named_sig * named_vars *
      (User_signatures.symbol_id Gen_terms.term * User_signatures.symbol_id Gen_terms.term) 
  | Equations_value of
      named_sig * named_vars *
      (User_signatures.symbol_id Gen_terms.term * User_signatures.symbol_id Gen_terms.term) list
  | TRS_value of 
      named_sig * named_vars *
      User_signatures.symbol_id Rewrite_rules.t list *
      User_signatures.symbol_id Naive_dnet.dnet 
  | HTRS_value of Hierarchical_trs.htrs
  | Dioph_constraint of Abstract_constraint.formula
  | Linear_constraint of Linear_constraints.formula
  | Termination_constraint of 
      named_sig * named_vars *
      User_signatures.symbol_id Termination_constraints.conj_termination_constraints
  | Order_constraint of 
      named_sig * named_vars *
      User_signatures.symbol_id Order_constraints.formula

and env_data =
    {
      (* typ : types *)
      mutable value : values
    }

and env_type = (string * env_data) list
;;

val print_value : values -> unit;;

exception Eval_error of string;;

