(***************************************************************************

(This sentence has been added automatically,
it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: main.ml,v 1.18 2001/04/20 13:42:40 marche Exp $

***************************************************************************)

open Format;;

let test1 () =
  let sigma = String_signatures.from_string "a b c" in
  let srs = String_rewriting.compile_srs
    (User_words.srs_of_string sigma 
" a a ->  ; 
b b b ->  ; 
b a b -> a ;  
a b a -> b b;
a b b -> b a;
b b a -> a b;
a b b b -> a;
 ")
  in
    List.iter
      (fun s ->
	 let w = User_words.word_of_string sigma s in
	 let w' = String_rewriting.compiled_normalize srs w in
	 Words.pretty_print sigma w;
	 printf " -->*@ ";
	 Words.pretty_print sigma w';
	 print_newline())
      [ "" ; 
	"a" ; 
	"b" ;
	"c" ;
	"a b" ;
	"b c" ;
	"c a" ;
	"a a b" ;
	"a b c b a b c" ;
	"a b c b a b c b c b a b c b c b a b c b c a a a b b c" 
      ]
;;

(* Machine de Turing qui additionne *)

let test2 () =
  let sigma = String_signatures.from_string "0 1 q0 q1 q2" in
  let srs = String_rewriting.compile_srs
    (User_words.srs_of_string sigma 
       " 1 q0 1 -> 0 1 q1 ; 
         1 q0 0 -> 0 0 q1 ;
         1 q1 1 -> 1 1 q1 ; 
         1 q1 0 -> 1 0 q1 ;
         0 q1   -> q2 1   ; 
         1 q2   -> q2 1   ; 
         0 q2   -> 0 q0   ; 
    ")
  in
    List.iter
      (fun s ->
	 let w = User_words.word_of_string sigma s in
	 let w' = String_rewriting.compiled_normalize srs w in
	 Words.pretty_print sigma w;
	 printf " -->*@ ";
	 Words.pretty_print sigma w';
	 print_newline())
      [ "0 q0 0 0" ; 
	"0 q0 0 0" ; 
	"1 q0 0 0" ; 
	"1 q0 0 1" ; 
	"1 q0 1 1 1 0 1 1 1 1" ; 
      ]
;;


(* Machine de Turing qui additionne en laissant intact le premier argument *)

let test3 () =
  let sigma = 
    String_signatures.from_string "0 1 h q0 q1 q2 q3 q4 q5" 
  in
  let srs = 
    String_rewriting.compile_srs
      (User_words.srs_of_string sigma 
	 " 
      0 q0 0 -> 0 0 q0 ; 
      0 q0 h -> 0 0 q0 h ; 
      0 q0 1 -> 0 1 q0 ;
      1 q0 0 -> 0 0 q1 ; 
      1 q0 h -> 0 0 q1 h ; 
      1 q0 1 -> 0 1 q1 ;

      1 q1 0 -> 1 0 q1 ; 
      1 q1 h -> 1 0 q1 h ; 
      1 q1 1 -> 1 1 q1 ;
      0 q1 0 -> 0 0 q2 ; 
      0 q1 h -> 0 0 q2 h ; 
      0 q1 1 -> 0 1 q2 ;

      1 q2 0 -> 1 0 q2  ; 
      1 q2 h -> 1 0 q2 h ; 
      1 q2 1 -> 1 1 q2 ;
      0 q2   -> q3 1 ; 

      1 q3   -> q3 1 ; 
      0 q3   -> q4 0 ; 

      1 q4   -> q4 1 ; 
      0 q4 0 -> 1 0 q5 ; 
      0 q4 h -> 1 0 q5 h ; 
      0 q4 1 -> 1 1 q5 ;

      1 q5 0 -> 0 0 q1 ; 
      1 q5 h -> 0 0 q1 h ; 
      1 q5 1 -> 0 1 q1 ;

      h q0   -> h 0 q0 ; 
      h q1   -> h 0 q1 ; 
      h q2   -> h 0 q2 ; 
      h q3   -> h 0 q3 ; 
      h q4   -> h 0 q4 ; 
      h q5   -> h 0 q5 ; 
    ")
  in
    List.iter
      (fun s ->
	 let w = User_words.word_of_string sigma s in
	 let w' = String_rewriting.compiled_normalize  srs w in
	 Words.pretty_print sigma w;
	 printf " -->*@ ";
	 Words.pretty_print sigma w';
	 print_newline())
      [ 
	"h q0 1 h" ; 
	"h 1 q0 1 h" ; 
	"h q0 0 1 h" ; 
	"h 1 q0 0 1 h" ; 
	"h q0 0 0 0 1 1 1 1 1 1 0 1 1 1 1 h" ; 
	"h q0 0 0 0 1 1 1 1 1 1 1 1 1 1 0 1 1 1 1 1 1 1 1 1 h" ; 
      ]
;;

let test4 () =
  let sigma = String_signatures.from_string "a b c d e" in
  let srs = User_words.srs_of_string sigma 
       " a b a b a -> c  ; 
          c b a -> a b c ;
"
  in
    print_bool (Local_confluence.is_locally_confluent srs) ;;
      
let test5 () =
  let sigma = String_signatures.from_string "a b c d e f" in
  let w1 = User_words.word_of_string sigma "a a c d e f" in
  let w2 = User_words.word_of_string sigma "a b c d e f" in
  let prec x y = match compare x y with 
      l when l>0  -> Orderings_generalities.Less_than
    | l when l<0 -> Orderings_generalities.Greater_than
    | l -> Orderings_generalities.Equivalent 
  in
    Format.print_string (Orderings_generalities.string_of_comparison_result (Word_orderings.length_lex prec w1 w2)) ;;



let test6 () =
  let sigma = String_signatures.from_string "a b" in
  let srs = (User_words.srs_of_string sigma 
	       " a a ->  ; 
                 b b b   -> ; 
                 b a b -> a ;
                 b b a -> a b ;
                 a b b -> b a ;
                 a b a -> b b ;") in
  let prec x y = match compare x y with (* Inverse alphabetic order *)
      l when l>0  -> Orderings_generalities.Less_than
    | l when l<0 -> Orderings_generalities.Greater_than
    | l -> Orderings_generalities.Equivalent 
  in 
  let compl = assert false (*Srs_completion.complete_naive sigma srs (Word_orderings.length_lex prec)*) 
  in
    List.iter (function l,r -> 	   
		 Words.pretty_print sigma l;
		 Format.printf " -> ";
		 Words.pretty_print sigma r;
		 print_newline())
      compl
;;

let test6 () =
  let sigma = String_signatures.from_string "a" in
  let w = User_words.word_of_string sigma "a" in
  let s = 
    String_rewriting.compile_srs (User_words.srs_of_string sigma "a -> a")
  in
  Words.pretty_print sigma (String_rewriting.compiled_normalize s w)
;;

let sigma_ab = String_signatures.from_string "a b";;
let sigma_ST = String_signatures.from_string "T S";;

let srs_s3 = User_words.srs_of_string sigma_ab 
		" a a ->  ; 
                 b b b   -> ; 
                 b a b -> a ;
                 b b a -> a b ;
                 a b b -> b a ;
                 a b a -> b b ;"


let srs_s5 = User_words.srs_of_string sigma_ab 	
   "a a ->  ;
   b b b b b ->  ;
   a b a b a b -> b b b b a ;
   a b b b b a -> b a b a b ;
   b a b a b a -> a b b b b ;
   b b b b a b b b b -> a b a b a ;
   a b b a b a b b -> b b a b a b b a ;
   a b b b a b a b b a ->
       b b b b a b b b a b ;
   a b b a b b b -> b b a b b b a ;
   a b b b a b b -> b b b a b b a ;
   a b a b b a b a -> b b a b b a b b ;
   b b a b b b a b a -> a b b b a b a b ;
   a b b a b b a b b -> b a b b a b a ;
   b b a b b a b b a -> a b a b b a b ;
   a b b b a b a b b b -> b a b b b a b a ;
   a b a b b b a b a -> b b b a b a b b b ;
   b b a b a b b b a b -> a b a b b b a ;
   b a b b b a b a b b -> a b b b a b a ;
   b b b a b a b b b a -> a b a b b b a b ;
   b b b a b a b b a b -> a b b a b b a ;
   a b a b b a b b a b ->
       b a b b a b b a b a ;
   b b a b a b b a b b a ->
       a b b a b b a b a b ;
   b a b b a b b a b a b ->
       a b b a b b a b a ;"
	     
let srs_p3n1 = User_words.srs_of_string sigma_ST
"S S S S -> ;
S S T -> T S S ;
S T S T S T -> S S ;
T T T -> ;
S S T T S T T S T T S T -> T S S T T S T T S T T S ;
S S T T S T T S T T S S -> S S S T T S T T S T T S ;
S S T T S T T S T T S -> S S T S T S T S ;
"

let srs_p11n1 = User_words.srs_of_string sigma_ST
"S ^ 4 ->  ;
S ^ 2 T  -> T S ^ 2  ;
S T S T S T  -> S ^ 2  ;
T ^ 11               ->  ;
S^2 T^ 2 S T^6  S T^2 S T  -> T T T T S S T T S T T T T T T S T T S                      ;
S S T T S T T T T T T S T T S S                 -> S S S T T T T T T S T T S T T T T T T S                       ;
S S T T S T T T T T T S T T S                  -> S S T T T T T T T T T S T T T T T S T T T T T T T T T S                               ;
"

let srs_p37n2 = User_words.srs_of_string sigma_ST
"
S S S S ->  ;
S S T  -> T S S  ;
S T S T S T  -> S S  ;
T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T -> ;
S S T T S S T T S T           -> T T T T S S T T S S T T S                ;
S S T T S S T T S S           -> S S S S T T S S           ;
S S T T S S T T S            -> S S T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T S S T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T T S ;
"


let complete_srs sigma srs =
let prec x y = match compare x y with (* Inverse alphabetic order *)
      l when l>0  -> Orderings_generalities.Less_than
    | l when l<0 -> Orderings_generalities.Greater_than
    | l -> Orderings_generalities.Equivalent 
  in 
  let completed= assert false (*Srs_completion.complete_naive sigma srs 
		    (Word_orderings.length_lex prec) *)
  in
    begin
      Format.printf "Here is a completed system:\n";
      ignore 
	(List.fold_left 
	   (fun acc (l,r) -> 	   
	      begin
		Format.printf "[";
		print_int acc;
		Format.printf "] ";
		Words.pretty_print sigma l;
		Format.printf " -> ";
		Words.pretty_print sigma r;
		Format.printf ";\n";
		acc+1
	      end )
	   0
	   completed) ;
      print_int (List.length completed);
      Format.printf " rules.\n"
    end
;;
		 


let complete_srs_strategy sigma srs =
let prec x y = match compare x y with (* Inverse alphabetic order *)
      l when l>0  -> Orderings_generalities.Less_than
    | l when l<0 -> Orderings_generalities.Greater_than
    | l -> Orderings_generalities.Equivalent 
  in 
  let completed= Srs_completion.complete_srs_std_strategy
		   sigma (Word_orderings.length_lex prec) srs 

  in
    begin
      Format.printf "Here is a completed system:\n";
      Format.printf "@[<v 2>{ ";
      ignore 
	(List.fold_left 
	   (fun acc (l,r) -> 	   
	      begin
		Format.printf "[%i] " acc;
		Words.pretty_print sigma l;
		Format.printf " -> ";
		Words.pretty_print sigma r;
		Format.printf ";@,";
		acc+1
	      end )
	   0
	   completed) ;
      Format.printf "@]} (%i rules).@." (List.length completed);
    end
;;

(*let _ = 
  Printexc.catch (complete_srs_strategy (lenlex_order) sigma_ST) srs_p11n1 ;
  print_newline ();;
*)

let unif1 () =
  let sigma = String_signatures.from_string "a b c" in
  let w1 = User_words.word_of_string sigma "c a " in
  let w2 = User_words.word_of_string sigma " c a  b " in
  let unif = String_unification.unify w1 w2 in
    Format.printf "Here is a set of unifyer :\n";
    Format.printf "@[<h 5>{@\n";
    List.iter
      (function ((l1,r1),(l2,r2)) -> 
	 Words.pretty_print sigma l1;
	 Format.printf "@ ,@ ";
	 Words.pretty_print sigma r1;
	 Format.printf "@ ;@ ";
	 Words.pretty_print sigma l2;
	 Format.printf "@ ,@ ";
	 Words.pretty_print sigma r2;
	 Format.printf ".@\n")
      unif;
      
      Format.printf "}@]"
;;

(* unif1 ();;
*)
open Constraints

(* Diehedral group : two non parametric symbols [a] and [b] *)

let d2k = Parameterized_signatures.create  ["k"] 
	    [("a",[],Atom(True)) ; ("b",[],Atom(True)) ] 

let a2 = Parameterized_user_words.word_of_string d2k  
	   " b b ^ 4 + 4 - (k+k) a b b (a b) ^ 3 b ^ 2 a ^ 1";;

(*let _ = Parameterized_words.print (d2k: Parameterized_signatures.user_parameterized_signature :> ( Parameterized_signatures.parameter_id,Parameterized_signatures.symbol_id ) Parameterized_signatures.parameterized_signature ) a2;;
  *)

let _ = Parameterized_words.print d2k a2 ;;

let _  =print_newline ();;
