(***************************************************************************

This module defines the class of parameterized signatures for parameterized 
words

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: user_parameterized_signatures.ml,v 1.15 2002/04/03 13:50:31 monate Exp $

***************************************************************************)
open Format
		 
(*

  This module defines the class of parameterized signatures for parameterized 
  words

  \subsection{The user parameterized signature type}

*)

class type user_parameterized_signature =
object
  inherit Parameterized_signatures.parameterized_signature
end

class from_string (p:Parameterized_signatures.parameter_c)
  (s : Parameterized_signatures.element list) : user_parameterized_signature =
  object(self)
  val psig = let h = Hashtbl.create 17  in
    List.iter (function e -> 
		 Hashtbl.add 
		 h 
		 e.Parameterized_signatures.sig_symbol 
		 e) 
      s;
    h

  val parameters = p

  method decoration_of_symbol s = 
    Hashtbl.find psig s
      
  method to_list = assert false 
  method parameters = parameters 
  method print_element s = print_string "No Printer yet"
  method print () = Hashtbl.iter (fun s e -> self#print_element e) psig

end

let do_parse parameters s = 
  try
    let b = Lexing.from_string s
    in
      List.map 
	(function (name,formal_index,constr) -> 
	   let env,constr = 
	     Linear_constraints.from_abstract_formula 
	       parameters#parameters 
	       constr 
	   in
	   let env,linear_index = 
	     List.fold_left 
	       (fun (acc_env,acc_ind) exp -> 
		  let new_env,new_ind = 
		    Linear_constraints.from_abstract_expr 
		      acc_env exp
		  in
		    new_env,new_ind::acc_ind)
	       (env,[])
	       formal_index
	   in
	     { Parameterized_signatures.sig_symbol = name;
	       Parameterized_signatures.sig_index = linear_index ;
	       Parameterized_signatures.sig_constr = constr;
	       Parameterized_signatures.sig_env = env
	     }
	)
	(Parameterized_signatures_parser.signature_eof 
	   Parameterized_signatures_lexer.token b)
  with
      Parsing.Parse_error -> 
	raise (Parameterized_signatures_syntax.Syntax_error "remember to enclose expr. and constr. with {}")
    | Failure "lexing: empty token" -> 
	raise (Parameterized_signatures_syntax.Syntax_error 
		 "invalid character")
    | Poly_syntax.Syntax_error s -> 
	raise (Parameterized_signatures_syntax.Syntax_error s)


let from_string p s = 
  new from_string p (do_parse p s)

let instantiate_signature s = assert false


class para_from_string (s:string) =
  let token = Genlex.make_lexer [] in
  let rec dop x = 
    match Stream.peek x with 
      | Some (Genlex.Ident i) -> Stream.junk x ; i::(dop x)
      | Some _ -> Stream.junk x ; dop x
      | None -> []
  in
  let s = dop (token (Stream.of_string s)) in
object 
  val p = List.map (fun x -> x,Linear_constraints.make_var x) s
  method parameters = p
  method print () = 
    printf "\"";
    List.iter (function _,v_id -> Linear_constraints.print_expr 
		 (Linear_constraints.var v_id) ; 
		 print_string " ") p;   
    printf "\""
end

let parameters_from_string s =
  new para_from_string s
