(***************************************************************************

(This sentence has been added automatically, it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: cime-latex.ml,v 1.4 1998/04/28 13:33:08 marche Exp $

***************************************************************************)





#open "arg";;
#open "str";;
#open "printf";;

let verbose_level = ref 0;;

let filename = ref "";;

let outputfilename = ref "";;

let cimeexamplectr = ref 0;;

let args_spec =
[
("-v", Unit(function  () -> verbose_level := !verbose_level+1));
("-o", String(function s -> outputfilename := s));
];;

let get_file_name s =
  if !filename = ""
    then filename :=  
            if ((string_length s) >= 4) &
                (sub_string s ((string_length s)-4) 4 = ".tex")
              then sub_string s 0 ((string_length s)-5)
              else s 
     else raise (Bad "too many arguments")
;;

let make_input_file s =
  
  let temp_file = open_out (!outputfilename ^ ".tmpinput")
  in
  for i=0 to (string_length s)-1 do
    output_char temp_file (nth_char s i);
    output_char temp_file `\n`
  done;
  close_out temp_file
;;

let rec translate_for_write l = 
  let l1 = global_replace (regexp "{") "\\\\{" l
  in
  let l2 = global_replace (regexp "}") "\\\\}" l1
  in
  let l3 = global_replace (regexp "<ECHO>") "\\\\textnormal{\\\\sffamily\\\\bfseries " l2
  in 
  let l4 = global_replace (regexp "</ECHO>") "}" l3
  in
  l4 ^ "\n"
;;

(*
let rec translate_for_write l = 
  if (string_length l)=0
    then "\n"
    else let last = (translate_for_write (substring l 1 (string_length l)))
     in match (nth_char l 0) with
         `{` -> "\{"
       | `}` -> "\}"
;;
*)

let process_example options file =
  printf "Processing %s\n" file;
  if !verbose_level >=1
    then printf "with options %s\n" options;
  flush stdout;
  let cime_options = 
    try let pos = search_forward (regexp "options=\"\\([^\"]*\\)\"") options 0
        in (matched_group 1 options)
    with Not_found -> ""
  and cime_input = 
    try let pos = search_forward (regexp "input=\"\\([^\"]*\\)\"") options 0
        in (matched_group 1 options)            
    with Not_found -> "" 
  and from_line = 
    try let pos = search_forward (regexp "from=\"\\([^\"]*\\)\"") options 0
        in (matched_group 1 options)            
    with Not_found -> ""
  and to_line = 
    try let pos = search_forward (regexp "to=\"\\([^\"]*\\)\"") options 0
        in (matched_group 1 options)            
    with Not_found -> ""
  in 
  cimeexamplectr :=  !cimeexamplectr + 1;
  make_input_file cime_input;
  let cime_command = 
    "cat " ^ !outputfilename ^ ".tmpinput | " ^
    "cime " ^ cime_options ^ " " ^ file ^ 
    " > " ^ !outputfilename ^ ".tmpoutput"
  in
  if !verbose_level >=1
    then begin
      	 printf "[%s]\n" cime_command;
         flush stdout
	 end;
  begin
  match unix__system cime_command
  with unix__WEXITED ret -> 
         if ret <> 0 
           then raise (Bad "cannot run cime properly")
           else ()
     | _ -> raise (Bad "cannot run cime properly")
  end;
  let input_file = open_in (!outputfilename ^ ".tmpoutput")
  and output_file = open_out (!outputfilename ^ ".cime" ^ string_of_int(!cimeexamplectr))
  and state = ref 0
  and from_regexp = if from_line="" 
                      then regexp ".*"
		      else regexp from_line
  and to_regexp = if to_line="" 
                      then regexp "\\000"
		      else regexp to_line
  in
  printf "Writing %s.cime%d\n" !outputfilename !cimeexamplectr;
  try
  while true do
    let l = input_line(input_file)
    in
    if !state=0 then if string_match from_regexp l 0 then state:=1;
    if !state=1 then if string_match to_regexp l 0 then state:=2;
    if !state=1 or !state=2 then output_string output_file (translate_for_write l);
    if !state=2 then state:=3
  done

  with
    End_of_file -> begin
      	       	   close_in input_file;
                   close_out output_file
		   end
;;


let main () =

  try

  parse args_spec get_file_name;
  if !filename =""
    then raise (Bad "Missing file name");
  if !outputfilename =""
    then outputfilename := !filename;
  let input_file = open_in (!filename ^ ".tex")
  in
  printf "Reading %s.tex...\n" !filename;
  flush stdout;
  let cimeexample_regexp = 
    regexp "\\\\cimeexample\\(\\(\\[[^\\]*\\]\\)?\\)\\{\\([^\\}]*\\)\\}"
  in
  try
  while true do
    let l = input_line(input_file)
    in
    if string_match cimeexample_regexp l 0
      then process_example (matched_group 1 l) (matched_group 3 l) 
  done

  with
    End_of_file -> begin
                   close_in input_file;
      	       	   printf "Done\n"
		   end

  with
    Bad s -> begin
             printf "%s\n" s;
	     printf "usage: cime-latex [-v|-o outputfilename] inputfilename\n"
	     end
;;


printexc__f main ();
flush stdout;;
