;;; CIME mode for GNU Emacs

(defvar cime-mode-syntax-table nil
  "Syntax table in use in CIME-mode buffers.")

(defvar cime-mode-map nil
  "Keymap used in CIME mode.")

(if cime-mode-syntax-table ()
  (setq cime-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?% "<" cime-mode-syntax-table)
  (modify-syntax-entry ?' "w" cime-mode-syntax-table)
  (modify-syntax-entry ?_ "w" cime-mode-syntax-table))

(if cime-mode-map ()
  (setq cime-mode-map (make-sparse-keymap))
;  (define-key cime-mode-map "\C-x\C-l" 'cime-send-phrase)
;  (define-key cime-mode-map "\C-x\C-k" 'cime-kill-phrase)
;  (define-key cime-mode-map "\C-x-C" 'cime-comment-out-phrase)
)

(defun cime-mode ()
  "Major mode for editing CIME files."
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'cime-mode)
  (setq mode-name "cime")
  (use-local-map cime-mode-map)
  (set-syntax-table cime-mode-syntax-table)
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat "^$\\|" page-delimiter))
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate paragraph-start)
  (make-local-variable 'paragraph-ignore-fill-prefix)
  (setq paragraph-ignore-fill-prefix t)
  (make-local-variable 'indent-line-function)
  (setq indent-line-function 'cime-indent-line)
  (make-local-variable 'indent-tabs-mode)
  (setq indent-tabs-mode nil)
  (make-local-variable 'cime-tab-width)
  (setq cime-tab-width 2)
  (make-local-variable 'require-final-newline)
  (setq require-final-newline t)
  (make-local-variable 'comment-start)
  (setq comment-start "% ")
  (make-local-variable 'comment-end)
  (setq comment-end "")
  (make-local-variable 'comment-column)
  (setq comment-column 40)
  (make-local-variable 'comment-start-skip)
  (setq comment-start-skip "%")
  (make-local-variable 'comment-indent-hook)
  (setq comment-indent-hook 'c-comment-indent)
  (make-local-variable 'parse-sexp-ignore-comments)
  (setq parse-sexp-ignore-comments nil)
  (run-hooks 'cime-mode-hook))

(defun cime-insert-tab ()
  (indent-to (* cime-tab-width
		(1+ (/ (current-column) cime-tab-width)))))

(defun cime-indent-line ()
  "Indent the current line like previous line, or insert a tab if not
at beginning of line, or if previous line is not indented. Tab size is
given by the variable cime-tab-width."
  (interactive)
  (if (bolp)
      (let ((previous-indentation
	     (save-excursion
	       (forward-line -1)
	       (current-indentation))))
	(if (eq previous-indentation 0)
	    (cime-insert-tab)
	  (indent-to previous-indentation)))
    (cime-insert-tab)))


; Evaluate the current buffer

(defvar cime-command "cime ")

(defun cime-eval  ()
  "Run Cime on current buffer"
  (interactive)
  (require 'shell)
  (shell-command (concat cime-command buffer-file-name) )
)


; Hilight 
(if (x-display-color-p)
    (progn (require 'hilit19)
	   (hilit-set-mode-patterns
	    'cime-mode
	    '(
	      ("%.*$" nil comment)   
	      ("\\<\\(sort\\|operators\\|axioms\\|theory\\|unification\\|type\\|end\\|problems\\|order\\|conjectures\\)\\>" nil error)
	      ("\\<\\(as\\|constant\\|unary\\|binary\\|variable\\|infix\\|postfix\\|rpo\\|interactive\\|mapo\\|poly\\|lexico\\|AC\\|commutative\\|only\\|complete\\|plain\\|unify\\|reduce\\|compare\\|with\\|mul\\|rllex\\|lrlex\\)\\>" nil keyword)
	      )
	    )
	   )
  )



