(***************************************************************************

Interface scanner for the cime-interface-to-latex utility

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: interface_lexer.mll,v 1.4 1998/05/22 14:52:54 marche Exp $

***************************************************************************)


{


}


rule entry = parse
    [' ' '\010' '\013' '\009' '\012'] +
      { entry lexbuf }
  | "(*"'*'* 
      { print_string "\n\\subsection{Role of the module}\n\n";
	header lexbuf }
  | _ 
      { print_string "\n\\subsection{Role of the module}\n\n";
	print_string "(Not documented, this module is either missing its header, or is an automatically generated module, like CAMLYACC parser)\n\n";
       	print_string "\n\\subsection{Contents of the module}\n\n";
	filter lexbuf }

and header = parse
    "$Id" [^'$']* "$"
      { header lexbuf }
  | "CiME" [^'\n']* '\n'
      { header lexbuf }
  | _
      { print_string (Lexing.lexeme lexbuf);
	header lexbuf }
  | '*'*"*)"
      { print_string "\n\\subsection{Contents of the module}\n\n";
	filter lexbuf }

and filter = parse
    "type" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "\\begin{quote}\ntype ";
	type_or_val lexbuf; 
	filter lexbuf }
  | "val" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "\\begin{quote}\nval ";
	type_or_val lexbuf; 
	filter lexbuf }
  | "module" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "\\begin{quote}\nmodule ";
	type_or_val lexbuf; 
	filter lexbuf }
  | "class" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "\\begin{quote}\nclass ";
	type_or_val lexbuf; 
	filter lexbuf }
  | "open" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "\\begin{quote}\nopen ";
	type_or_val lexbuf; 	
	filter lexbuf }
  | "(*" '*'* "*)"
      { filter lexbuf }
  | "(*" '*'* [' ' '\010' '\013' '\009' '\012'] *
      { print_string "\n\n";
	comment lexbuf; 
	filter lexbuf }
  | _
      { filter lexbuf }
  | eof
      { print_string "\n" }

and skip = parse
    ";;"
      { filter lexbuf }
  | _ 
      { skip lexbuf  }

and type_or_val = parse
    "with" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "with ";
	type_or_val lexbuf }
  | "type" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "type ";
	type_or_val lexbuf }
  | "virtual" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "virtual ";
	type_or_val lexbuf }
  | "end" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "end ";
	type_or_val lexbuf }
  | "inherit" [' ' '\010' '\013' '\009' '\012'] +
      { print_string "inherit ";
	type_or_val lexbuf }
  | ['A'-'Z' 'a'-'z' '\192'-'\214' '\216'-'\246' '\248'-'\255' '\'' ]
    (['A'-'Z' 'a'-'z' '_' '\192'-'\214' '\216'-'\246' '\248'-'\255'
      '\'' '0'-'9' ]) * 
      { print_string "\\verb|";
        print_string (Lexing.lexeme lexbuf);
        print_string "|" ; type_or_val lexbuf }
  | "(*"
      {
 	comment lexbuf;
	type_or_val lexbuf }
  | ";;"
      { print_string "\n\\end{quote}\n";filter lexbuf }
  | "->"
      { print_string "$\\rightarrow$"; type_or_val lexbuf }
  | "*"
      { print_string "$\\times$"; type_or_val lexbuf }
  | ([' ''\t']*'\n')+ 
      { print_string "\\\\\n"; type_or_val lexbuf }
  | '{' 
      { print_string "\\{"; type_or_val lexbuf }
  | '}' 
      { print_string "\\}"; type_or_val lexbuf }
  | '#' 
      { print_string "\\verb|#|"; type_or_val lexbuf }
  | _ 
      { print_string (Lexing.lexeme lexbuf);
	type_or_val lexbuf
      }
  | eof 
      { print_string "\n\\end{quote}\n"; }

and comment = parse
    "(*" 
      { comment lexbuf;
	comment lexbuf }
  | '*'*"*)" ([' ''\t']*'\n')*
      { print_string "\n\n" }
  | "\\begin{verbatim}"
      { print_string "\\begin{verbatim}";
        verbatim lexbuf;
	comment lexbuf }
  | "\\["
      { print_string "\\[";
        comment lexbuf }
  | "\\]"
      { print_string "\\]";
        comment lexbuf }
  | "[" 
      { print_string "\\verb|"; brackets lexbuf; comment lexbuf }
  | "\""
      { print_string "\\verb|\""; string lexbuf; comment lexbuf }
  | "''"
      { print_string (Lexing.lexeme lexbuf);
	comment lexbuf }
  | "'" [^ '\\' '\''] "'"
      { print_string (Lexing.lexeme lexbuf);
	comment lexbuf }
  | "'\\" ['\\' '\'' 'n' 't' 'b' 'r'] "'"
      { print_string (Lexing.lexeme lexbuf);
	comment lexbuf }
  | "'\\" ['0'-'9'] ['0'-'9'] ['0'-'9'] "'"
      { print_string (Lexing.lexeme lexbuf);
	comment lexbuf }
  | eof
      { () }
  | _
      { print_string (Lexing.lexeme lexbuf);
	comment lexbuf }

and brackets = parse
    "]" 
      { print_string "|" }
  | _ 
      { print_string (Lexing.lexeme lexbuf);
	brackets lexbuf
      }

and string = parse
    '"'
      { print_string "\"|" }
  | '\\' ("\010" | "\013" | "\010\013") [' ' '\009'] *
      { print_string (Lexing.lexeme lexbuf);
	string lexbuf }
  | '\\' ['\\' '"' 'n' 't' 'b' 'r']
      { print_string (Lexing.lexeme lexbuf);
	string lexbuf }
  | '\\' ['0'-'9'] ['0'-'9'] ['0'-'9']
      { print_string (Lexing.lexeme lexbuf);
	string lexbuf }
  | eof
      { () }
  | _
      { print_string (Lexing.lexeme lexbuf);
	string lexbuf }
and verbatim = parse
    "\\end{verbatim}"
      { print_string "\\end{verbatim}" }
  | _
      { print_string (Lexing.lexeme lexbuf);
	verbatim lexbuf }
  | eof
      { () }

