(***************************************************************************

(This sentence has been added automatically, it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: norm_theory.ml,v 1.4 2002/05/14 14:15:54 contejea Exp $

***************************************************************************)

open Symbols;;

type axiome_connu =
    Commutativity of symbol_id
  | Associativity of symbol_id
(*
  | Left_unit of (symbol_id * symbol_id)
  | Right_unit of (symbol_id * symbol_id)
  | Idempotency of symbol_id
  | Nilpotency of (symbol_id * symbol_id * int)
  | Left_inverse of (symbol_id * symbol_id * symbol_id)
  | Right_inverse of (symbol_id * symbol_id * symbol_id)
*)
  | Left_distributivity of (symbol_id * symbol_id)
  | Right_distributivity of (symbol_id * symbol_id)
;;

type theorie_connue =
    No_theory 
  | A of symbol_id 
  | AU of (symbol_id * symbol_id)
  | ACU of (symbol_id * symbol_id)
  | ACI of symbol_id
  | ACUI of (symbol_id * symbol_id)
  | ACUN of (symbol_id * symbol_id * int)
  | G of (symbol_id * symbol_id * symbol_id)
  | AG of (symbol_id * symbol_id * symbol_id)
  | CR of (symbol_id * symbol_id * symbol_id * symbol_id * symbol_id)
(*
| BR of symbol_id * symbol_id * symbol_id * symbol_id 
| FP of int * symbol_id * symbol_id * symbol_id * symbol_id * symbol_id
*)
;;

type description_theorie_canonique =
    {
      mutable theories : theorie_connue list;
      mutable derniere_theorie_ajoutee : theorie_connue
    }
;;



open Symbols;;
open Messages;;


let theorie_canonique_courante =
  {
    theories = [];
    derniere_theorie_ajoutee = No_theory
  };;

let subtract x y= match (x,y) with 
    f, [] -> f
  | f, e  -> 
      let rec subtract_e = function
          [] -> []
       	| elem::l -> 
	    if List.mem elem e 
	    then subtract_e l 
	    else elem :: subtract_e l
      in subtract_e f
;;

let declare_theorie th = 
    let a = theorie_canonique_courante.theories in
    match th with
    CR(f,u,i,g,v) -> let a' = subtract a [AG(f,u,i);ACU(g,v)] in
                     theorie_canonique_courante.theories <- th::a';
                     theorie_canonique_courante.derniere_theorie_ajoutee <- th
  | _ ->  theorie_canonique_courante.theories <- th::a;
          theorie_canonique_courante.derniere_theorie_ajoutee <- th
;;

let groupe_abelien f =
 let rec trouve = function
     [] -> raise Not_found
   | AG(g,u,i)::l -> if (f=g) then (u,i) else trouve l
   | _::l -> trouve l in
   trouve theorie_canonique_courante.theories
;;

let unite f = 
 let rec trouve = function
     [] -> raise Not_found
   | ACU(g,u)::l -> if (f=g) then u else trouve l
   | _::l -> trouve l in
   trouve theorie_canonique_courante.theories
;;

(*
let enleve_axiomes_inutiles l = 
  match theorie_canonique_courante.derniere_theorie_ajoutee with
      No_theory -> l
    | ACU fu -> 
	subtract l [(Left_unit fu);(Right_unit fu)]
    | AG ((f,u,i) as fui) -> 
        subtract l [(Left_unit (f,u));(Right_unit (f,u));
                    (Left_inverse fui);(Right_inverse fui)]
    | CR(f,u,i,g,v) -> 
        subtract l [(Left_distributivity (f,g));(Right_distributivity (f,g))]
    | _ -> l
;; 
*)

let string_of_theorie = function
    No_theory -> "No theory"
  | ACU(n,m)  -> "ACU(" ^ (string_of_symbol_id n) ^
                 "," ^ (string_of_symbol_id m) ^ ")"
  | AU(n,m)   -> "AU(" ^ (string_of_symbol_id n) ^
                 "," ^ (string_of_symbol_id m) ^ ")"
  | A(n)      -> "A(" ^ (string_of_symbol_id n) ^ ")"
  | ACI(n)    -> "ACI(" ^ (string_of_symbol_id n) ^ ")"
  | ACUI(n,m) -> "ACUI(" ^ (string_of_symbol_id n) ^
                  "," ^ (string_of_symbol_id m) ^ ")"
  | ACUN(n,m,k) -> "AG(" ^ (string_of_symbol_id n) ^
                  "," ^ (string_of_symbol_id m) ^
                  "," ^ (string_of_int k) ^ ")"
  | G(n,m,k)  -> "G(" ^ (string_of_symbol_id n) ^
                  "," ^ (string_of_symbol_id m) ^
                  "," ^ (string_of_symbol_id k) ^ ")"
  | AG(n,m,k) -> "AG(" ^ (string_of_symbol_id n) ^
                  "," ^ (string_of_symbol_id m) ^
                  "," ^ (string_of_symbol_id k) ^ ")"
  | CR(n,m,k,j,i) -> "CR(" ^ (string_of_symbol_id n) ^
                  "," ^ (string_of_symbol_id m) ^
                  "," ^ (string_of_symbol_id k) ^ 
		  "," ^ (string_of_symbol_id j) ^
                  "," ^ (string_of_symbol_id i) ^ ")"
(*
  | BR(n,m,k,j) -> "BR(" ^ (string_of_symbol_id n) ^
                  "," ^ (string_of_symbol_id m) ^
                  "," ^ (string_of_symbol_id k) ^ 
		  "," ^ (string_of_symbol_id j) ^ ")"
  | FP(p,n,m,k,j,i) -> "FP(" ^ (string_of_int p) ^
                  "," ^ (string_of_symbol_id n) ^
                  "," ^ (string_of_symbol_id m) ^
                  "," ^ (string_of_symbol_id k) ^ 
		  "," ^ (string_of_symbol_id j) ^
                  "," ^ (string_of_symbol_id i) ^ ")"
*)
;;

let print_theorie th = 
  Format.open_hvbox 0;
  begin
  match th with
    []   -> print_string "Empty"
  | r::l -> print_string "{";
            print_string (string_of_theorie r);
  let rec print_fin_de_liste_de_theories = function
     []     -> print_string "}"
   | (r::l) -> print_string ",";
               Format.print_break 1 0;
               print_string (string_of_theorie r);
               print_fin_de_liste_de_theories l in 
    print_fin_de_liste_de_theories l
  end;
  Format.close_box ()
;;

let print_all_theories () = 
  match theorie_canonique_courante.theories with 
      [] -> ()
    | _ -> 
	message "theory";
       	Format.open_hvbox 0;
       	List.iter 
	  (function t -> 
	     print_string "  ";
             print_string (string_of_theorie t);
             Format.force_newline())
          theorie_canonique_courante.theories;
       	Format.close_box ()
;;

