(***************************************************************************

(This sentence has been added automatically, it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: symbols.ml,v 1.7 2002/07/03 16:28:11 monate Exp $

***************************************************************************)

open Messages;;
open Errors;;

type symbol_type = 
  VARIABLE | FREE | AC | C 
;;

type fix_type = 
  PREFIX | INFIX | POSTFIX | DEFAULT
;;

type symbol_id = int;;
type var_id = int;;

let not_a_symbol_id = -1;;

type type_table_de_symboles = 
    {
      mutable symboles_de_fonction : 
       	(string * symbol_type * int * fix_type * int) array;
      mutable variables : string array;
    }
;;

let table_des_symboles =
  {
    symboles_de_fonction = [||];
    variables       = [||]
  }
;;

let indice_var_courant = ref (-1);;

let first_symbol_var () = 
  indice_var_courant := 0;
  0
;;

let next_symbol_var () =
  indice_var_courant := succ(!indice_var_courant);
  !indice_var_courant;
;;

let is_free f = 
  match table_des_symboles.symboles_de_fonction.(f) with 
      (_,FREE,_,_,_) -> true
    | _ -> false;;

let is_ac f = 
  match table_des_symboles.symboles_de_fonction.(f) with 
      (_,AC,_,_,_) -> true
    | _ -> false;;

let is_c f = 
  match table_des_symboles.symboles_de_fonction.(f) with 
      (_,C,_,_,_) -> true
    | _ -> false;;

let string_of_symbol_id f = 
  if f>=0 & f < (Array.length table_des_symboles.symboles_de_fonction)
  then 
    match table_des_symboles.symboles_de_fonction.(f)
    with (s,_,_,_,_) -> s
  else failwith "Bug in symbol__string_of_symbol_id"
;;

let string_of_var_id x =
  if x>=0 & x < (Array.length table_des_symboles.variables)
  then table_des_symboles.variables.(x)
  else "V_"^string_of_int(x)
;;

let string_of_typed_symbol = function
    (VARIABLE,x) -> string_of_var_id x
  | (_,f)   -> string_of_symbol_id f
;;


exception Trouve of symbol_type * symbol_id;;

let recherche_symbole s =
  try
   
    for i=0 to (Array.length table_des_symboles.symboles_de_fonction)-1 do
      match table_des_symboles.symboles_de_fonction.(i) with
          (x,t,_,_,_) -> 
	    if s=x
	    then raise (Trouve (t,i))
    done;
    for i=0 to (Array.length table_des_symboles.variables)-1 do
      if table_des_symboles.variables.(i) = s
      then raise (Trouve (VARIABLE,i))
    done;
    raise Not_found

  with
      Trouve (t,f) -> t,f
;;


let symbol_id_of_string s =
(*
   try 
*)
  match recherche_symbole s with 
      (VARIABLE,_) -> raise Not_found
    | (_,f) -> f
(*
   with Not_found ->
   semantical_error ("symbole '" ^ s ^ "' non declare");;
*)
;;

let var_id_of_string s =
  match recherche_symbole s with 
      (VARIABLE,v) -> v
    | _ -> raise Not_found
;;


let definir_operateur t n fix s = 
  try 
     match recherche_symbole s with 
	 VARIABLE,_ -> ()
       | _,i -> 
	   table_des_symboles.symboles_de_fonction.(i) <- 
	   (match table_des_symboles.symboles_de_fonction.(i) 
	    with a,b,c,d,e -> a,b,c,d,e+1)      
	   
  with Not_found -> 
    match t with
	VARIABLE   -> (*
			printf "'%s' declare comme variable\n" s;
                      *)
          table_des_symboles.variables <-
          Array.append table_des_symboles.variables [|s|]
      | _     -> (*
		   printf "'%s' declare comme operateur " s;
		   if t=AC
                   then printf "AC\n"
                   else printf "libre d'arite %d\n" n;
		 *)
	  let fix' = 
            if fix = DEFAULT
            then match t with
		C | AC -> INFIX
              | _ -> PREFIX
            else fix
          in
            table_des_symboles.symboles_de_fonction <-
            Array.append table_des_symboles.symboles_de_fonction 
              [|(s,t,n,fix',1)|]
;;

let redeclare_operateur_c f =
  match table_des_symboles.symboles_de_fonction.(f) with
    | (s,_,n,_,i) ->
        table_des_symboles.symboles_de_fonction.(f) <- (s,C,n,INFIX,i)
;;
 
let redeclare_operateur_ac f =
  match table_des_symboles.symboles_de_fonction.(f) with
      (s,_,n,_,i) -> 
        table_des_symboles.symboles_de_fonction.(f) <- (s,AC,n,INFIX,i)
;;
 
let arity f = 
  match table_des_symboles.symboles_de_fonction.(f)
  with (_,_,a,_,_) -> a
;;

let occurences f = 
  match table_des_symboles.symboles_de_fonction.(f)
  with (_,_,_,_,n) -> n

let is_constant f = ( arity f = 0 );;

let is_unary f = ( arity f = 1 );;

let is_infix f = 
  match table_des_symboles.symboles_de_fonction.(f)
  with (_,_,_,INFIX,_) -> true
    | _ -> false
;;

let is_postfix f = 
  match table_des_symboles.symboles_de_fonction.(f)
  with (_,_,_,POSTFIX,_) -> true
    | _ -> false
;;

let get_symbol_id s =
  try symbol_id_of_string s
  with Not_found ->
    try ignore (var_id_of_string s);
      semantical_error ("variable symbol '" ^ s ^ "' cannot be used as a function symbol")
    with Not_found ->
      semantical_error ("undeclared symbol '" ^ s ^ "'")
;;


(* ======================================================================

functions that extract lists of functions symbols from the symbol table.

======================================================================= *)

let plus_grand_numero_de_symbole () =
  (Array.length table_des_symboles.symboles_de_fonction) - 1;;

let rec listof_p_fun p n =
  if (n < 0)
  then  []
  else 
    if (p n)
    then n::(listof_p_fun p (n-1))
    else (listof_p_fun p (n-1))
;;

let listof_p_functions p = listof_p_fun p (plus_grand_numero_de_symbole ());;

let listof_c_functions () = listof_p_functions is_c;;

let listof_ac_functions () = listof_p_functions is_ac;;

let listof_constant_functions () = 
  listof_p_functions (function f -> (arity f) = 0)
;;

let listof_other_functions () = 
  listof_p_functions (function f -> (is_free f) & ((arity f) > 0))
;;

let all_variables () =
  let nb_var = Array.length table_des_symboles.variables in
    match nb_var with
    	0 -> ""
      | 1 -> table_des_symboles.variables.(0)
      | _ -> 
	  let s = ref (table_des_symboles.variables.(0)) 
	  in
          for i=1 to (pred nb_var) do
            s := !s ^ " " ^ table_des_symboles.variables.(i)
          done;
	    !s
;;

let print_all_variables () = print_string (all_variables ())

let all_functions_with_arity () =
  (let lc = listof_c_functions () in
    begin
      match lc with  
	  [] -> ""
    	| f::[] -> "  "^(string_of_symbol_id f)^" : infix commutative;"
    	| f::l -> 
	    let s = ref (" " ^ (string_of_symbol_id f)) in
              List.iter
		(function x -> 
		   s := !s^("," ^ (string_of_symbol_id x)))
		l;
              !s^ " : infix commutative;"
    end)
  ^
  (let lac = listof_ac_functions () in
     begin
       match lac with  
	   [] -> ""
	 | f::[] -> "  "^(string_of_symbol_id f)^" : AC;"
	 | f::l -> 
	     let s = ref (" " ^ (string_of_symbol_id f)) in
	       List.iter 
		 (function x -> 
		    s := !s ^ "," ^ (string_of_symbol_id x))
                 l;
		!s^ " : AC;"
     end)
  ^
  (let lcst = listof_constant_functions () in
     begin
       match lcst with  
	   [] -> ""
	 | f::[] -> "  "^(string_of_symbol_id f)^" : constant;"
	 | f::l -> 
	     let s = ref (" " ^ (string_of_symbol_id f)) in
	       List.iter 
		 (function x -> 
		       s := !s^("," ^ (string_of_symbol_id x)))
                    l;
               !s^ " : constant;";
	       
     end)
    ^
  (let lo = listof_other_functions () in
	  begin
	    match lo with
		[] -> ""
	      | _ -> let s = ref "" in 
		  List.iter 
		    (fun x -> 
		       s := !s ^ 
		     ("  "^(string_of_symbol_id x)^
		      " : "^(string_of_int (arity x))^
		      ";"))
                      lo;
		    !s
	  end)
;;

let print_all_functions_with_arity () = message (all_functions_with_arity ())


let listof_functions l = 
  let s = ref "" in 
    List.iter 
      (function x -> 
	 s := !s ^ ((string_of_symbol_id x) ^ " ")) 
      l;
    !s ^"\n"


let print_listof_functions l = message (listof_functions l)
    
