(***************************************************************************

(This sentence has been added automatically, it should be replaced by a description of the module)

CiME Project - Dmons research team - LRI - Universit Paris XI

$Id: tptp2cime.ml,v 1.8 2002/07/03 16:28:11 monate Exp $

***************************************************************************)

open Arg
open Clauses
open Cime_format
open Abs_syntax
open Errors
open Format


let args_spec =
[
("-tptp-dir",
  String(function s -> cime_pb.tptp_dir <- s),"");
("-cime-dir",
  String(function s -> cime_pb.cime_dir <- s),"")
]



let get_file_name s =
  if cime_pb.source  = ""
  then cime_pb.source <- s
  else raise (Bad "too many arguments")


let enleve_quote s = 
  let l = String.length s 
  in
    if (((String.get s 0) = '\'') & ((String.get s (pred l))  = '\''))
    then String.sub s 1 (pred (pred l))
    else failwith "enleve_quote"

let lire_ligne_de_commande () =
  parse args_spec get_file_name ""


let rec read_specifications nom_fichier_courant =
  if cime_pb.tptp_dir = "" then 
    cime_pb.tptp_dir <- (Filename.concat 
			   (Filename.concat 
			      (Filename.dirname nom_fichier_courant) 
			      Filename.parent_dir_name)
			   Filename.parent_dir_name);
  let in_file = 
  try open_in nom_fichier_courant
  with Sys_error s ->
    try open_in (nom_fichier_courant ^ ".p")
    with Sys_error s ->
      try open_in (nom_fichier_courant ^ ".ax")
      with Sys_error s -> 
        raise (Arg.Bad (Printf.sprintf "Could not open file %s (%s)" nom_fichier_courant s))
in
let lexbuf = Lexing.from_channel in_file
in
  try 
    let rec collect_data = function
      	[] -> ([],[])
      | s::l -> 
      	  let (ax,conjs) = collect_data l 
	  in
      	    match s with
		Abstract_include f -> 
		  let f' = enleve_quote f 
		  in
                  let (ax',conjs') = read_specifications 
				       (Filename.concat 
					  cime_pb.tptp_dir
					  f')
		  in 
                    (ax'@ax,conjs'@conjs)
              | Abstract_clause REFL -> (ax,conjs)
              | Abstract_clause SYMM -> (ax,conjs)
              | Abstract_clause TRANS -> (ax,conjs)
              | Abstract_clause (SUB _) -> (ax,conjs)
              | Abstract_clause (AX a) -> (a::ax,conjs)
              | Abstract_clause (CONJ c) -> (ax,c::conjs)
		  
    in collect_data (Parser.donnees Lexer.token lexbuf) 
	 
  with 
    | Parsing.Parse_error ->
	printf "Syntax error while reading file %s@." nom_fichier_courant;
	printf "Characters %d-%d@." 
	  (Lexing.lexeme_start lexbuf) (Lexing.lexeme_end lexbuf);
	exit 1
	  
    | e  -> 
	printf "Unknown error while reading file %s@." nom_fichier_courant;
	raise e



let main () =
  
  try 

    lire_ligne_de_commande ();
    
    if cime_pb.source = ""   
    then raise (Arg.Bad "no file specified")
    else Printf.printf "Processing problem %s\n" cime_pb.source ;
    let (axiomes,conjectures) = read_specifications cime_pb.source 
    in
    let s = cime2_format (axiomes,conjectures) in
      print_string s;
      let ll = Lexing.from_string s in
	while true do
	  begin
	    try
	      let c = Toplevel_parser.command 
			Toplevel_lexer.token 
			ll
	      in Main.execute c
	    with
		Parsing.Parse_error ->
		  printf "Syntax error"	
	      | Toplevel_lexer.Lexical_error s ->
		  printf "Lexical error: %s" s
	      | Sys.Break ->
		  printf "Command aborted by user"
	      | Invalid_argument s -> 
		  printf "Invalid argument: %s" s
	      | Out_of_memory -> 
		  printf "Out of memory, command aborted"
	  end;
	  printf "@."
	done
	
  with
    | Sys_error s -> 
    	eprintf "System error: %s@." s;
	exit 2
    | Arg.Bad s           -> 
    	eprintf "Bad command line arguments: %s@."  s;
	exit 2
    | Exit  -> exit 0
    | e -> raise e 


let _ = Printexc.catch main ()
